!============================================================================
! I N I T I A L I S A T I O N   D E S   D O N N E E S  H 3 P                !
!============================================================================
SUBROUTINE INI_DATA_H3P_PREP

USE CONSTANTES
USE INOUTPUT
USE STR_DATA
USE PROFIL
USE PREPVAR

!---------------------------------------------------------------------------
! DECLARATIONS                                                             !
!---------------------------------------------------------------------------
IMPLICIT NONE

! Energie max --------------------------------------------------------------
! Emaxh3p_lu ! Energie max des niveaux de H3+ pris en compte pour le calcul 
             ! du .bin
             ! transmis par le module PREPVAR

! Variables locales --------------------------------------------------------
! Parametres lus dans fichier de donnees des niveaux
INTEGER       :: ndata ! nombre de niveaux dans fichier de donnees
INTEGER       :: J, K  ! J, K lus dans les donnees
REAL(KIND=DP) :: en    ! energie en cm-1 lues dans les donnees
INTEGER       :: jmax_dat ! J max dans le fichier des niveaux
INTEGER       :: kmax_dat ! K max dans le fichier des niveaux

! Parametres lus dans le fichier des transitions radiatives
INTEGER       :: ntrans_rad  ! nbre de transitions dans fichier de donnees
INTEGER       :: n           ! numero des transitions
INTEGER       :: jl,kl,ju,ku ! J, K  lus dans le fichier des transitions
REAL(KIND=DP) :: aein, etrans  ! A Einstein et E transition en cm-1

! Variable pour le tri en energies
TYPE(NIVEAUX) :: tempo
INTEGER       :: l, lmin

! Parametres utiles           
INTEGER :: i         ! compteur
INTEGER :: i_nlv     ! compteur sur les niveaux conserves
INTEGER :: ntrans    ! compteur du numero de transition

       !------------------------------------------------------------------
       ! INITIALISATION DES ENERGIES                                     !
       !------------------------------------------------------------------

       fichier = TRIM(data_dir)//'energies_h3p.dat'
       PRINT *,"Lecture de :"
       PRINT *,"   ",fichier
       OPEN(10, file = fichier, status = 'old')
       READ(10,*) ! commentaire
       READ(10,*) ! commentaire

       ! Lecture du nombre de niveaux dans le fichier de donnees
       READ(10,*) ndata
       PRINT *,"nombre de niveaux dans fichier H3+ :",ndata

       ! Calcul du nombre de niveaux conserves 
       nlvh3p = 0 ! initialisation du nombre de niveaux considere
       DO i=1,100000
          READ(10,*,END=110) J, K, en
          IF (en <= Emaxh3p_lu)  nlvh3p = nlvh3p + 1 ! Nombre de niveaux conserves
       ENDDO
110    continue
       PRINT *,"nombre de niveaux de H3+ conserves : ",nlvh3p

       REWIND(11)
       ! Dans le fichier 11, se trouvent les niveaux a conserver
       ! classes par ordre croissant d'energie

       ! Allocation et initialisation
       ALLOCATE(elevh3p(1:nlvh3p))
       elevh3p(:)%j  = 0
       elevh3p(:)%k  = 0
       elevh3p(:)%en = 0.0
       elevh3p(:)%g  = 0
       elevh3p(:)%n  = 0

       ! Lecture et sauvegarde des donnees
       REWIND(10)
       READ(10,*) ! commentaire
       READ(10,*) ! commentaire
       READ(10,*) ndata

       ! Lecture des donnees: J, K, Energies
       jmax_dat = 0
       kmax_dat = 0
       i_nlv      = 0
       DO i=1,100000
          READ(10,*,END=111) J, K, en
          en = en * xhp * clum / xk ! passage en Kelvin

          IF (en <= (Emaxh3p_lu*xhp*clum/xk) ) THEN
             i_nlv = i_nlv + 1
             elevh3p(i_nlv)%j  = J
             elevh3p(i_nlv)%k  = K
             elevh3p(i_nlv)%en = en 
             IF (mod(k,3) == 0) THEN ! niveau ortho
                elevh3p(i_nlv)%g = 2.0_dp * (2_dp * elevh3p(i_nlv)%j + 1_dp)
             ELSE ! niveau para
                elevh3p(i_nlv)%g = 2.0_dp * elevh3p(i_nlv)%j + 1_dp
             ENDIF
          ENDIF

          IF (J  > jmax_dat)  jmax_dat = J
          IF (K  > kmax_dat)  kmax_dat = K

       ENDDO
111    continue
       CLOSE(10)
       

       ! On effectue le tri en energie des niveaux
       DO i=1, nlvh3p-1
          lmin = i
          DO l=i+1, nlvh3p
            if ( elevh3p(l)%en < elevh3p(lmin)%en ) lmin = l
          ENDDO
          ! permutation
          tempo = elevh3p(i)
          elevh3p(i) = elevh3p(lmin)
          elevh3p(lmin) = tempo
       ENDDO

       ! Recherche du J max, K max et de l'energie max dans les niveaux
       ! conserves
       jmaxh3p = 0
       kmaxh3p = 0
       enmaxh3p = 0.0
       DO i=1, nlvh3p
          IF (elevh3p(i)%J  > jmaxh3p)  jmaxh3p = elevh3p(i)%J
          IF (elevh3p(i)%K  > kmaxh3p)  kmaxh3p = elevh3p(i)%K
          IF (elevh3p(i)%en > enmaxh3p) enmaxh3p = elevh3p(i)%en
       ENDDO
!      jmaxh3p  = maxval(elevh3p(:)%J)
!      kmaxh3p  = maxval(elevh3p(:)%K)


       ! Correspondance J,K -> lev
       ! Suppose que les niveaux sont lus par ordre croissant d'energie
       ALLOCATE(levh3p(0:jmax_dat,0:kmax_dat))
       levh3p(:,:) = -1
       DO i= 1, nlvh3p
          levh3p(elevh3p(i)%j,elevh3p(i)%k) = i
       ENDDO
       


       !------------------------------------------------------------------------
       ! TRANSITIONS RADIATIVES                                                !
       !------------------------------------------------------------------------

       fichier = TRIM(data_dir)//'Aeinh3p_complete.dat'
       OPEN(20,file=fichier,status="OLD",action="READ")
       READ(20,*) ! 3 lignes de commentaire
       READ(20,*)
       READ(20,*)
       READ(20,*) ntrans_rad ! nbre de transitions radiatives

       ! Lecture du fichier des coefficients d'Einstein
       ! Il faut faire tras attention a ne conserver que les transitions valables.
       ! En particulier, le fichier des A peut contenir des transitions a partir 
        ! de niveaux absents du fichier des energies et donc pour lesquels le
       ! programme ne connait pas J, K et E. A priori ces niveaux ont un levh3p negatif.
 
       ! Premiere lecture du fichier pour compter le nombre de transitions conservees
       ! Ce nombre sert a l'allocation dynamique de memoire
       ntrh3p = 0
       DO i=1, ntrans_rad
          READ(20,*,END=120) n,ju,ku,jl,kl,etrans,aein
          IF ( (levh3p(ju,ku) <= nlvh3p)                 .AND. &
               (levh3p(jl,kl) <= nlvh3p)                 .AND. &
               (levh3p(ju,ku) > 0)                       .AND. & ! supprime les transitions a partir de niveaux
               (levh3p(jl,kl) > 0)                       .AND. & ! non disponibles dans table d'energie
               (elevh3p(levh3p(ju,ku))%en <= Enmaxh3p)   .AND. &
               (elevh3p(levh3p(jl,kl))%en <= Enmaxh3p)  ) THEN

               ntrh3p = ntrh3p + 1

          ENDIF
      ENDDO
120   continue

      ! Allocation dynamique de memoire
      ALLOCATE(aijh3p(ntrh3p))  ! coefficients d'Einstein classes par numeros de transitions
      ALLOCATE(iuph3p(ntrh3p))  ! numero du niveau haut
      ALLOCATE(jloh3p(ntrh3p))  ! numero du niveau bas
      ALLOCATE(wlkh3p(ntrh3p))  ! longueur d'onde de la transition en Kelvin
      aijh3p(:) = 0.0_dp
      wlkh3p(:) = 0.0_dp
      iuph3p(:) = 0
      jloh3p(:) = 0

      ! Deuxieme lecture dans laquelle on sauve les valeurs a conserver
      REWIND(20)
      READ(20,*) ! 3 lignes de commentaire
      READ(20,*)
      READ(20,*)
      READ(20,*) 
      ntrans = 0
      DO i=1, ntrans_rad
          READ(20,*,END=121) n,ju,ku,jl,kl,etrans,aein
          IF ( (levh3p(ju,ku) <= nlvh3p)                 .AND. &
               (levh3p(jl,kl) <= nlvh3p)                 .AND. &
               (levh3p(ju,ku) > 0)                       .AND. & ! supprime les transitions a partir de niveaux
               (levh3p(jl,kl) > 0)                       .AND. & ! non disponibles dans table d'energie
               (elevh3p(levh3p(ju,ku))%en <= Enmaxh3p)   .AND. &
               (elevh3p(levh3p(jl,kl))%en <= Enmaxh3p)  ) THEN

               ntrans = ntrans + 1
               aijh3p(ntrans) = aein
               iuph3p(ntrans) = levh3p(ju,ku)
               jloh3p(ntrans) = levh3p(jl,kl)
               wlkh3p(ntrans) = etrans * xhp * clum / xk
          ENDIF
      ENDDO
121   continue
      CLOSE(20)

END SUBROUTINE INI_DATA_H3P_PREP

