
! Copyright (C) 2002 - Observatoire de Paris/Meudon
!                      Universite Paris 7
!                      CNRS (France)
!                      Contact : Jacques.Lebourlot@obspm.fr
!
! This program is free software; you can redistribute it and/or
! modify it under the terms of the GNU General Public License version 2
! as published by the Free Software Foundation.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


module INTEMI

  USE constantes
  USE PROFIL

contains

subroutine LDVInt (xmu, amx, aijein, gi, gj, nuij, xi, xj, tampon)

  USE constantes
  USE PROFIL

  implicit none

  REAL (KIND=DP), INTENT (in)                    :: xmu            !  Line of sight cosine
  REAL (KIND=DP), INTENT (in)                    :: amx            !  Mass of emiter (a.m.u.)
  REAL (KIND=DP), INTENT (in)                    :: aijein            !  Aij Einstein coef (Emission)
  REAL (KIND=DP), INTENT (in)                    :: gi             !  Statistical weight of upper level (i)
  REAL (KIND=DP), INTENT (in)                    :: gj             !  Statistical weight of lower level (j)
  REAL (KIND=DP), INTENT (inout)                 :: nuij           !  Frequency of transition
  REAL (KIND=DP), INTENT (in), DIMENSION(0:npo)  :: xi             !  Upper level density
  REAL (KIND=DP), INTENT (in), DIMENSION(0:npo)  :: xj             !  Lower level density
  CHARACTER (LEN=17), INTENT (in)                :: tampon         !  Line identity

  REAL (KIND=DP)                                 :: bij            !  Bij Einstein coef (induced emission)
  REAL (KIND=DP)                                 :: bji            !  Bji Einstein coef (absorption)

  REAL (KIND=DP)                                 :: tbg = 2.73_dp  !  Cosmic Micro-Wave background
  REAL (KIND=DP)                                 :: cdc1           !  Column density constant
  REAL (KIND=DP)                                 :: cdc2           !  Source and opacity constant
  REAL (KIND=DP)                                 :: ct2            !  Thermal width constant
  REAL (KIND=DP)                                 :: dnu            !  Frequency step in line
  INTEGER                                        :: jpm = 50       !  Number of points in line profile
  INTEGER                                        :: io, jp
  REAL (KIND=DP)                                 :: ddnu, xdop, nu
  REAL (KIND=DP)                                 :: emist, delnu

  REAL (KIND=DP), ALLOCATABLE, DIMENSION(:)      :: dopl           ! Doppler width
  REAL (KIND=DP), ALLOCATABLE, DIMENSION(:)      :: taunu          ! Optical depth in line
  REAL (KIND=DP), ALLOCATABLE, DIMENSION(:)      :: dchem          ! ds along the path
  REAL (KIND=DP), ALLOCATABLE, DIMENSION(:)      :: dtaunu         ! Optical depth step
  REAL (KIND=DP), ALLOCATABLE, DIMENSION(:)      :: sour           ! Source function
  REAL (KIND=DP), ALLOCATABLE, DIMENSION(:)      :: spec           ! Spectra
  REAL (KIND=DP), ALLOCATABLE, DIMENSION(:)      :: ibg            ! Cosmic background
  REAL (KIND=DP), ALLOCATABLE, DIMENSION(:,:)    :: tauldv         ! Total optical depth

  INTEGER, ALLOCATABLE, DIMENSION(:)             :: s_sour         ! Debug
  INTEGER, ALLOCATABLE, DIMENSION(:)             :: lim_io         ! Debug
  REAL (KIND=DP), ALLOCATABLE, DIMENSION(:,:)    :: spec_i         ! Incident intensity on partial slab
  REAL (KIND=DP), ALLOCATABLE, DIMENSION(:,:)    :: spec_e         ! Intensity built in a partial slab
  INTEGER                                        :: i1, i2, iio, nio
  REAL (KIND=DP)                                 :: k1, k2, signe, xdop1, xdop2
  REAL (KIND=DP)                                 :: fac_exp

  if (.NOT. ALLOCATED(dopl)) then
    ALLOCATE(dopl(0:npo))
    ALLOCATE(taunu(0:npo))
    ALLOCATE(dchem(0:npo))
    ALLOCATE(dtaunu(0:npo))
    ALLOCATE(sour(0:npo))
    ALLOCATE(s_sour(0:npo))
    ALLOCATE(lim_io(0:npo))
  endif

  if (.NOT. ALLOCATED(spec)) then
    ALLOCATE(spec(0:jpm))
    ALLOCATE(spec_i(0:jpm,0:3))
    ALLOCATE(spec_e(0:jpm,0:3))
    ALLOCATE(tauldv(0:jpm,0:3))
    ALLOCATE(ibg(0:jpm))
  endif

  spec = 0.0_dp
  spec_i = 0.0_dp
  spec_e = 0.0_dp
  tauldv = 0.0_dp
  ibg = 0.0_dp

! Constantes de la raie

  nuij = xk * nuij / xhp
  ct2 = 2.0_dp * xk / amu
  cdc1 = cdunit * tautav / rrr
  cdc2 = xhp / (4.0_dp*xpi * sqrt(xpi))

  bij = aijein * clum*clum / (2.0_dp*xhp * nuij**3)
  bji = bij * gi / gj

! quantite a integrer pour obtenir l'absisses curviligne le long du chemin (ds en cm)

  do io=npo,0,-1
    dchem(io) = cdc1 / (densh(io) * xmu)
  end do

! Calcul de la largeur Doppler locale (Largeur de la gaussienne
! De la fonction source locale
! et limites des inversions de population (eventuelles)

  s_sour = 0
  do io=0,npo
    dopl(io) = nuij * sqrt(vturb * vturb + ct2 * tgaz(io) / amx) / clum
    sour(io) = xi(io) * aijein / (xj(io) * bji - xi(io) * bij)
    if (sour(io) < 0.0_dp) then
      s_sour(io) = 1
      sour(io) = - sour(io)
    endif
  end do

  nio = 0
  lim_io(0) = npo
  do io=npo-1,0,-1
    if (s_sour(io+1) /= s_sour(io)) then
      nio = nio + 1
      lim_io(nio) = io + 1
    endif
  end do
  lim_io(nio+1) = 1

! Largeur caracteristique (en s-1) :

  delnu = minval(dopl)

! Pas d'integration dans le profil :

  dnu = delnu / 10.0_dp

! Initialisation de l'intensite au corps noir cosmologique

  do jp=0,jpm
    ddnu = jp * dnu
    nu = nuij + ddnu
    fac_exp = xhp*nu/(xk*tbg)
    if (fac_exp > log(HUGE(1.0_dp))*0.9_dp) then
      ibg(jp) = 0.0_dp
    else
      ibg(jp) = 2.0_dp * xhp * nu**3 / (clum*clum)
      ibg(jp) = ibg(jp) / (exp(xhp*nu/(xk*tbg)) - 1.0_dp)
    endif
    spec(jp) = 0.0_dp
  end do
  spec_i(0:jpm,0) = ibg(0:jpm)

! S'il n'y a pas d'inversion de population, on integre en une seule fois

  if (nio == 0) then

    iio = 0
    do jp=0,jpm
      ddnu = jp * dnu
      nu = nuij + ddnu

! Calcul de la profondeur optique a nu
! a - Quantite locale a integrer

      do io=npo,0,-1
        xdop = ddnu / dopl(io)
        dtaunu(io) = cdc2 * nu * (xj(io) * bji - xi(io) * bij) &
                   * exp(-xdop*xdop) / dopl(io)
      end do

! b - Integration par la methode des trapezes
! Attention : tau augmente de 0 a npo (de l'observateur vers l'infini)
!             taunu augmente de npo a 1 (du "fond" du nuage, pour revenir vers l'observateur)

      taunu = 0.0_dp
      do io=npo-1,0,-1
        taunu(io) = taunu(io+1) + (dtaunu(io+1) * dchem(io+1) + dtaunu(io) * dchem(io)) &
                                * (tau(io+1) - tau(io)) * 0.5_dp
      end do
      tauldv(jp,iio) = taunu(1)

! Integration du terme source (solution formelle)

      do io=npo-1,0,-1
        spec(jp) = spec(jp) + (sour(io+1) * exp(-(tauldv(jp,iio)-taunu(io+1))) &
                            + sour(io) * exp(-(tauldv(jp,iio)-taunu(io)))) &
                            * (taunu(io) - taunu(io+1)) * 0.5_dp
      end do
    end do

! Ajout de l'intensite incidente atenuee sur l'epaisseur, et soustraction du fond.

    do jp=0,jpm
      spec(jp) = spec(jp) - ibg(jp) * (1.0_dp - exp(-tauldv(jp,iio)))
    end do

  else

! L'integration est decoupee en plusieurs tranche, a chaque inversion de population

    do jp=0,jpm
      ddnu = jp * dnu
      nu = nuij + ddnu

! Calcul de la profondeur optique a nu
! a - Quantite locale a integrer

      taunu = 0.0_dp
      do iio=0,nio
        if (iio == 0) then
          spec_i(jp,iio) = ibg(jp)
        else
          spec_i(jp,iio) = spec(jp)
        endif
        do io=lim_io(iio),lim_io(iio+1),-1
          xdop = ddnu / dopl(io)
          signe = (-1.0_dp)**s_sour(io)
          dtaunu(io) = cdc2 * nu * (xj(io) * bji - xi(io) * bij) &
                     * exp(-xdop*xdop) * signe / dopl(io)
        end do

! b - Integration par la methode des trapezes
! Attention : tau augmente de 0 a npo (de l'observateur vers l'infini)
!             taunu augmente de npo a 1 (du "fond" du nuage, pour revenir vers l'observateur)

        do io=lim_io(iio)-1,lim_io(iio+1),-1
          taunu(io) = taunu(io+1) + (dtaunu(io+1) * dchem(io+1) + dtaunu(io)* dchem(io)) &
                                  * (tau(io+1) - tau(io)) * 0.5_dp
        end do
        tauldv(jp,iio) = taunu(lim_io(iio+1))

! Integration du terme source (solution formelle)

        spec(jp) = 0.0_dp
        do io=lim_io(iio)-1,lim_io(iio+1),-1
          if (s_sour(io) == 0) then
            spec(jp) = spec(jp) + (sour(io+1) * exp(-(tauldv(jp,iio)-taunu(io+1))) &
                                + sour(io) * exp(-(tauldv(jp,iio)-taunu(io)))) &
                                * (taunu(io) - taunu(io+1)) * 0.5_dp
          else
            spec(jp) = spec(jp) + (sour(io+1) * exp((tauldv(jp,iio)-taunu(io+1))) &
                                + sour(io) * exp((tauldv(jp,iio)-taunu(io)))) &
                                * (taunu(io) - taunu(io+1)) * 0.5_dp
          endif
        end do
        spec_e(jp,iio) = spec(jp)

! Ajout de l'intensite incidente atenuee sur l'epaisseur

        if (s_sour(lim_io(iio+1)) == 0) then
          spec(jp) = spec(jp) + spec_i(jp,iio) * exp(-(tauldv(jp,iio)-taunu(lim_io(iio))))
        else
          spec(jp) = spec(jp) + spec_i(jp,iio) * exp((tauldv(jp,iio)-taunu(lim_io(iio))))
        endif

! Integration directe de j sur l'element ou a lieu l'inversion
! Si l'on n'est pas au bord du nuage

        if (iio /= nio) then

          i1 = lim_io(iio+1)
          i2 = lim_io(iio+1)-1
          xdop1 = ddnu / dopl(i1)
          xdop2 = ddnu / dopl(i2)
          spec(jp) = spec(jp) + cdc2 * aijein * (xi(i1) * exp(-xdop1*xdop1) * dchem(i1) / dopl(i1) &
                                            + xi(i2) * exp(-xdop2*xdop2) * dchem(i2) / dopl(i2)) &
                                           * (tau(i1) - tau(i2)) * 0.5_dp

! Evaluation de l'increment de profondeur optique

          k1 = (-1.0_dp)**s_sour(i1) * (xj(i1) * bji - xi(i1) * bij) &
             * exp(-xdop1*xdop1)
          k2 = (-1.0_dp)**s_sour(i2) * (xj(i2) * bji - xi(i2) * bij) &
             * exp(-xdop2*xdop2)
          taunu(i2) = taunu(i1) + cdc2 * ((k1*k1 + k2*k2) / (k1 + k2)) &
                                * (dchem(i1) + dchem(i2)) &
                                * (tau(i1) - tau(i2)) * 0.5_dp

! On saute l'element intermediaire

          lim_io(iio+1) = lim_io(iio+1) - 1

        end if

      end do

! Reinitialisation des "bonnes" limites

      do iio=1,nio
        lim_io(iio) = lim_io(iio) + 1
      end do

    end do

! Soustraction du fond.

    do jp=0,jpm
      spec(jp) = spec(jp) - ibg(jp)
    end do

  endif

! Integration sur le profil de la raie

  emist = 0.0_dp
  do jp=0,jpm
    emist = emist + spec(jp)
  end do
  emist = (2.0_dp * emist - spec(0)) * dnu

! Sorties

  write (35, '(/,"# ",a17," nu:",f11.2," GHz (=>",f11.4," micron), I:",1pe10.3," erg cm-2 s-1 str-1")') &
         tampon, nuij*1.0e-9_dp, 1.0e4_dp*clum/nuij, emist
  do jp=0,jpm
    ddnu = jp * dnu
    nu = ddnu / nuij
!   write (35, "(1p,10e14.6)") nu, spec(jp), (tauldv(jp,iio), iio=0,nio), ibg(jp), ((spec_i(jp,iio)-ibg(jp)), iio=0,nio)
    if (nio == 0) then
      write (35, "(1p,20e14.6)") nu, spec(jp), (tauldv(jp,iio), iio=0,nio), ibg(jp)
    else
      write (35, "(1p,20e14.6)") nu, spec(jp), (tauldv(jp,iio), iio=0,nio), ibg(jp), (spec_i(jp,iio), iio=0,nio) &
          , (spec_e(jp,iio), iio=0,nio)
    endif
  end do

  print *, tampon, emist, " erg cm-2 s-1 str-1"

end subroutine LDVInt

end module INTEMI
