
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c-------------------------------------------------------------
       subroutine EVOLREACT
c-------------------------------------------------------------

       use constantes
       use prepvar
       USE H2_DATA
       USE SP_DATA
       USE CHEM_DATA
       USE PROFIL

       implicit double precision (a-h, o-z)

c-------------------------------------------------------------
c DECLARATIONS
c-------------------------------------------------------------
       character (LEN=75) :: fichier1, fichier2, fichier3
       character (LEN=8)  :: yspec

       double precision :: seuil
       double precision :: fabri(1:neqt,1:npo), destr(1:neqt,1:npo)
       double precision :: fabtot(1:npo), destot(1:npo)
       double precision :: taudest, taufabr
       double precision :: pourfabr(1:npo,1:neqt)
       double precision :: pourdest(1:npo,1:neqt)
       double precision :: pourfabr2(1:npo,1:neqt)
       double precision :: pourdest2(1:npo,1:neqt)
       double precision :: ab1(nqm), ab2(nqm), ab3(nqm)

       integer :: numdest, numfabr
       integer :: flagreact(neqt)
c------------------------------------------------------------
c xkmat(isp,k) : cste de vitesse de form (si >0), de dest (si <0)
c de l'espece isp dans la k ieme reaction ou elle intervient.


c------------------------------------------------------------
c ESPECE
c------------------------------------------------------------
       print *,'A quelle espece vous interressez-vous ?'
       read (*,5) yspec
 5     format(a8)
       do 10 jj=1,nspec
         if (speci(jj) == yspec) then
           isp = jj               !num de l'esp choisie
         endif
 10    continue

       Print *,'Entrez le seuil (pourcentage)'
       read(*,*) seuil

c------------------------------------------------------------
c INITIALISATION
c------------------------------------------------------------
       nrxi = ikmat(isp,0)      !nbre de reaction concernant isp
       DO 11 k = 1,nrxi
          flagreact(k)=0  !on met un flag a zero a toutes ces reaction
 11    CONTINUE           !qd flag vaut 1, on garde la reaction

       numfabr = 0 !nbre de reactions de fabr qu'il faudra sortir
       numdest = 0 !idem pour destruction

c----------------------------------------------------------
c CALCUL DES TAUX DE FABRICATION ET DE DESTRUCTION TOTAL.
c----------------------------------------------------------
       DO 20 iopt=0,npo
         temp = tgaz(iopt)
         dens = densh(iopt)
         av = tau(iopt) * 1.085736

         h2para = 0.0_dp
         h2orth = 0.0_dp
         do 21 lev=1,nlevh2
           jj = njlev(lev)
           if (mod(jj,2) == 0) then
             h2para = h2para + xreh2(lev,iopt)
           else
             h2orth = h2orth + xreh2(lev,iopt)
           endif
 21      continue

         call XKFILL (iopt, temp, dens, av)

         fabri(1:neqt,iopt) = 0.0d0
         destr(1:neqt,iopt) = 0.0d0


          do 22 k=1,nrxi
            ind = ikmat(isp,k)
            j1 = react(ind,1)  !premier reactif
            j2 = react(ind,2)  !second reactif
            j3 = react(ind,3)  !second reactif
            ab1(k) = abnua(j1,iopt) !abond du premier reactif
            ab2(k) = abnua(j2,iopt) !abond du second reactif
            ab3(k) = abnua(j3,iopt) !abond du troisieme reactif
 22       continue

c Calcul du taux de fabrication total et du tau de destruction total
c au point iopt.

         fabtot(iopt) = 0.0d0
         destot(iopt) = 0.0d0

         do 23 k=1,nrxi
           toto = xkmat(isp,k)       !taux de la kieme reaction
           if (toto > 0.0_dp) then
             fabri(k,iopt) = toto * ab1(k) * ab2(k) * ab3(k)
             fabtot(iopt) = fabtot(iopt) + fabri(k,iopt)
           else if (toto < 0.0d0) then
             destr(k,iopt) = -toto * ab1(k) * ab2(k) * ab3(k)
             destot(iopt) = destot(iopt) + destr(k,iopt)
           endif
 23      continue

 20    continue

c--------------------------------------------------------------------
c RECHERCHE DES REACTIONS A CONSERVER
c--------------------------------------------------------------------
c En chaque point iopt, on calcul le pourcentage que represente chaque
c reaction et si ce pourcentage est superieur au seuil on la garde.
c Les reaction qu'on garde sont associees a un indice different de 0
c dans le tableau flagreact. Si cet indice vaut 1, c'est une reaction
c de formation, s'il vaut -1, de destruction.
c Que la reaction soit conservee ou non, les pourcentages sont conserves
c dans deux tableaux: pourdest et pourfabr.

       DO 30 iopt=0, npo
          Do 31 k=1,nrxi
             toto = xkmat(isp,k)
             if (toto < 0.0_dp) then
                taudest = destr(k, iopt)
                pourdest(iopt,k) = 100.0d0 * taudest / destot(iopt)
                if (pourdest(iopt,k) > seuil) then
                   if (flagreact(k) == 0) then
                      flagreact(k)=-1
                      numdest = numdest+1
                   endif
                endif
             else if (toto > 0.0_dp) then
                taufabr = fabri(k, iopt)
                pourfabr(iopt,k) = 100.0d0 * taufabr / fabtot(iopt)
                if (pourfabr(iopt,k) > seuil) then
                   if (flagreact(k) == 0) then
                      flagreact(k)=1
                      numfabr = numfabr +1
                   endif
                endif
             endif
 31      Continue
 30   CONTINUE


c Pour les sorties, il ne faut garder dans deux tableaux que les pourcentages
c des reactions conservees. On stocke ces pourcentages dans deux nouveaux
c tableaux pourdest2 et pourfabr2.
c On releve egalement a cette occasion les nombre de reaction de formation
c et le nombre de reaction de destruction.

       DO 32 iopt=0, npo
          numfabr2 = 0
          numdest2 = 0
          Do 33 k=1,nrxi
             if (flagreact(k) == 1) then
                numfabr2 = numfabr2 +1
                pourfabr2(iopt,numfabr2)  = pourfabr(iopt,k)
             endif
             if (flagreact(k) == -1) then
                numdest2 = numdest2 +1
                pourdest2(iopt,numdest2) = pourdest(iopt,k)
             endif
 33       Continue
 32    CONTINUE

c---------------------------------------------------------------------
c SORTIE DES RESULTATS
c---------------------------------------------------------------------
       fichier1 = 'chimevold'
       fichier2 = 'chimevolf'
       fichier3 = 'chimevolr'

       OPEN(10,file=fichier1,status='unknown')
       OPEN(20,file=fichier2,status='unknown')
       OPEN(30,file=fichier3,status='unknown')

       WRITE(30,*) 'Reactions de fabrication'
       DO 40 k=1,nrxi
          if (flagreact(k) == 1) then
             ind = ikmat(isp,k)
             write(30,*) (speci(react(ind,j)), j=1,5)
          endif
 40    continue
       WRITE(30,*)
       WRITE(30,*) 'Reactions de destruction'
       DO 41 k=1,nrxi
          if (flagreact(k) == -1) then
             ind = ikmat(isp,k)
             write(30,*) (speci(react(ind,j)), j=1,5)
          endif
 41    continue

       PRINT *,'Nbre de reaction de fabrication conservees:',numfabr
       PRINT *,'Nbre de reaction de destruction conservees:',numdest
       PRINT *,'Les resultats sont repertories dans les fichiers:'
       PRINT *,' - chimevolr:    donne la liste des reactions'
       PRINT *,' - chimevolf:    taux de formation'
       PRINT *,' - chimevold:    taux de destruction'
       PRINT *,"Rem: L'ordre des colonnes des deux derniers fichiers"
       PRINT *,"     est le meme que celui dans chimevolr"
       PRINT *

       DO 42 iopt=0,npo
          av = tau(iopt) * 1.085736
          write(10,'(1p,100e12.5)') av, (pourdest2(iopt,k), k=1,numdest)
          write(20,'(1p,100e12.5)') av, (pourfabr2(iopt,k), k=1,numfabr)
 42    continue

       CLOSE(10)
       CLOSE(20)
       CLOSE(30)

c-------------------------------------------------------------------
c FIN
c-------------------------------------------------------------------

         return
         end
