
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%
       program stv
c      %%%%%%%%%%%

       use constantes
       use auxiliaries
       USE RF_TOOLS
       USE SP_DATA
       USE H2_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL
       USE CONVER
       USE transfer

c--------+---------+---------+---------+---------+---------+---------+-*-------+
c
c                         From PDR to dense clouds...
c                         ***************************
c
c                  ******************************************
c                  * jlb-gpf-er 20 november 95, version 2.0 *
c                  * jlb-cn-flp spring 2002, serious update *
c                  *     work in progress ever since...     *
c                  * jlb-jlg 2005 - Rad Transfer update     *
c                  ******************************************
c
c                  See our web site at http://aristote.obspm.fr
c
c--------+---------+---------+---------+---------+---------+---------+-*-------+
c
c       implicit double precision (a-h,o-z)
        implicit none

        integer        :: ifaf, iopt, itenua
        integer        :: i, ii, j, lev
        real (kind=dp) :: dtau, dtau1, vartg, testv
        real (kind=dp) :: xdens, xtemp
        real (kind=dp) :: vart, varn, varh, varhd, varh2, varco, varc

c--------+---------+---------+---------+---------+---------+---------+-*-------+

      print *,'********************************************************'
      print *,'*                                                      *'
      print *,'*                *****    ****     *****               *'
      print *,'*                *   *    *   *    *   *               *'
      print *,'*                *****    *   *    *****               *'
      print *,'*                *        *   *    * *                 *'
      print *,'*                *        ***      *  *                *'
      print *,'*                                                      *'
      print *,'********************************************************'


          
c                     ***************************
c                     * PART 1: INITIALISATIONS *
c                     ***************************

       call INIT1

       write(iscrn,*)
       write(iscrn,*) "---------------------------"
       write(iscrn,*) "    End of input files"
       write(iscrn,*) "---------------------------"
       write(iscrn,*)

c                     ************************
c                     * PART 2: ITERATIONS *
c                     ************************

c  MAIN LOOPS: 

c  First, global iterations - index ifaf, label 450
c  Then, through the cloud - index iopt, label 400

       do 450 ifaf=1,ifafm

       open (34, file="pdh2vJ.dat", status="unknown")
       write (34, '("#",20(i2,"-",i2," "))') (nvlev(lev)
     1                                      , njlev(lev), lev=1,20)

       if (isoneside == 1) then
         call ETIN901S
       else
         call ETIN90
       endif

       dtau = tau(1) - tau(0)
       is_imp = 0
       next_imp = 1

       do 400 iopt=0,noptm

  402  continue

         write(iscrn,*)
         write(iscrn,*) '*********************************************'
         write(iscrn,'(5x,"Two sides iteration number",i2," of",i2)')
     1               ifaf,ifafm
         write(iscrn,'(15x,"Slab number: ",i3)') iopt
         write(iscrn,*) '*********************************************'

         temp = tgaz(iopt)
c
c  convert tau to av: av = tau * 2.5 log10(e)
c
         av   = tau(iopt) * tautav

c  reinitialise species abondances (AB)

         do i=1,nspec
           ab(i) = abnua(i,iopt)
         end do

c  cosmic rays, photons and so have an abundance of 1 (numerical trick)

         ab(0) = 1.0_dp
         ab(nspec+1) = 1.0_dp
         ab(nspec+2) = 1.0_dp
         ab(nspec+3) = 1.0_dp
         ab(nspec+4) = 1.0_dp
         ab(nspec+5) = 1.0_dp

         pdish2(iopt) = 0.0_dp
         pdishd(iopt) = 0.0_dp

c  LOCAL LOOP:

c  Compute iteratively all physical quantities at a given point
c          1 - Radiative transfer
c          2 - Chemistry
c          3 - Thermal balance
c          4 - Various tests
c  index itenua - label 300

         do 300 itenua=1,miter

           write(iscrn,*) 
           write(iscrn,*)'---------------------------------------------'
           write(iscrn,1030) ifaf,iopt, itenua, av, densh(iopt), temp

c  Previous values are saved to tests

           oldhat = ab(i_h)
           oldhmo = ab(i_h2)
           if (i_hd /= 0) then
             oldhd = ab(i_hd)
           endif
           oldco = ab(i_co0)
           oldc = ab(i_c)

           fgkrf(1:mfgk) = 1.0_dp

c  1: RADIATIVE TRANSFER (FGK approximation above jfgk for H2)
c  ***********************************************************

           call FGKTR2 (iopt)

c  absorption and dissociation of H2, CO and isotopes
c  ionisation of C and S by direct integration over the radiation field
c  (to be completed with other species)

           call FGKDAT (iopt)

           if (i_hd /= 0) then
             write(iscrn,*) 'Photodissociation probabilities:'
             write(iscrn,"(1x,1p,'P(H2): ',e10.3,'  P(HD): ',e10.3,
     1                    '  P(CO): ',e10.3)")
     2                    pdish2(iopt), pdishd(iopt), pdico0(iopt)
             write(iscrn,*)
           else
             write(iscrn,*) 'Photodissociation probabilities:'
             write(iscrn,"(1x,1p,'P(H2): ',e10.3,'  P(CO): ',e10.3)")
     1                    pdish2(iopt), pdico0(iopt)
             write(iscrn,*)
           endif

c  2: CHEMISTRY
c  ************

c  Build the set of chemical equations

           call CHIMIJ (iopt, itenua)

c  Compute abundance of electrons ABELEC from charge conservation

           abelec = 0.0_dp
           do i=nn+1,nspec-1
             abelec = abelec + ab(i) * ael(i,natom)
           end do
           ab(nspec) = abelec

c  Solve the set of chemical equartions (newton-raphson scheme)
c    Inspired from (with home made tuning)
c    numerical recipes, WH Press,BP Flannery,SA Teukolsky and WT Vetterling,
c    Cambridge University Press, 1986.

           call MNEWT (iopt)

c  Give some indications to the poor waiting modelist.

           do i=1,natom-1
             if (icx(i,1) /= 0) then
               write (iscrn,"(2x,a8,':',1pe12.4,2x,a8,':',e12.4)")
     1                            speci(icx(i,1)), ab(icx(i,1))
     2                          , speci(icx(i,2)), ab(icx(i,2))
             endif
           end do

           if (i_hd /= 0) then
             write(iscrn,*)
             write(iscrn,*) 'Abondances'
             write (iscrn,1041) ab(i_h), ab(i_h2), ab(i_hd), ab(i_co0)
     1                        , errx, speci(mauvai)
           else
             write(iscrn,*)
             write(iscrn,*) 'Abondances'
             write (iscrn,1040) ab(i_h), ab(i_h2), ab(i_co0), errx
     1                        , speci(mauvai)
           endif

c  This is probably not usefull anymore, but it does no harm...

           if (ab(i_h2) == 0.0_dp) then
             ab(i_h2) = istiny
           endif
           if (ab(i_hd) == 0.0_dp) then
             ab(i_hd) = istiny
           endif
           if (ab(i_h) == 0.0_dp) then
             ab(i_h) = istiny
           endif
           if (ab(i_co0) == 0.0_dp) then
             ab(i_co0) = istiny
           endif
           if (ab(i_cs) == 0.0_dp) then
             ab(i_cs) = istiny
           endif

c  Save abundances for next iteration

           abnua(0,iopt) = 1.0_dp
           do i=1,nspec
             abnua(i,iopt) = ab(i)
           end do

c   Compute excitation of H2, CO, CS, C, C+, O, HCO+
c   Then heating and cooling rates.
c   Thermal balance only if asked for (FROID and CHAUD are used in BILTHM)

           if (ifeqth == 1) then
             call BILTHM (iopt, itenua, ifaf)
           else
             call FROID (temp, iopt, ifaf)
             call CHAUD (temp, iopt, itenua)
           endif

           write(iscrn,*) xreh2o(4,iopt)
     1                  , refh2o(iopt), reftot(iopt)
           write(iscrn,*)

c  tests :
c               - Chemical solution accuracy
c               - relative abundace of H, H2, C, CO (and maybe some other)
c               - H2 population conservation
c       ===> Leave loop 300 if ALL conditions are met
c       A Warning on screen gives the reason why another iteration is done

           if (itenua == 1) goto 300

           if (itenua <= 4 .and. ifisob == 2) goto 300

           write(iscrn,*)
           write(iscrn,*) '------------------------------------'
           write(iscrn,*) '           convergence tests        '

           if (ifeqth == 1) then
             vartg = abs(tb(itenua) - tb(itenua-1))
             if ( (abs(bthrel) > (2.0_dp * convli)) .and.
     1         (vartg > 0.1d0)) then
               print *, "   CASE 1: BT", bthrel, vartg
               goto 300
             end if
           end if

           if ((ifeqth == 1) .and.
     1         (abs(bthrel) > 1.0d-2)) then
             print *, "   CASE 2: bthrel", bthrel
             goto 300
           end if

           if (errx > 1.0d-2) then
             print *, "   CASE 3: chimie", errx
             goto 300
           end if

           testv = abs(oldhat - abnua(i_h,iopt)) / abnua(i_h,iopt)
           if (testv > convli) then
             print *, "   CASE 4: H", testv
             goto 300
           end if

           testv = abs(oldhmo - abnua(i_h2,iopt)) / abnua(i_h2,iopt)
           if (testv > convli) then
             print *, "   CASE 5: H2", testv
             goto 300
           end if

           if (i_hd /= 0) then
             testv = abs(oldhd - abnua(i_hd,iopt)) / abnua(i_hd,iopt)
             if (testv > convli) then
               print *, "   CASE 5b: HD", testv
               goto 300
             end if
           endif

           testv = abs(oldco - abnua(i_co0,iopt)) / abnua(i_co0,iopt)
           if (testv > convli) then
             print *, "   CASE 6: CO", testv
             goto 300
           end if

           testv = abs(oldc - abnua(i_c,iopt)) / abnua(i_c,iopt)
           if (testv > convli) then
             print *, "   CASE 7: C", testv
             goto 300
           end if

           if (abs(ceqh2) > 0.01d0) then
             print *, "   CASE 8: ceqh2 =", ceqh2
             goto 300
           end if

           if (i_hd /= 0) then
             if (abs(ceqhd) > 0.01d0) then
               print *, "   CASE 8b: ceqhd =", ceqhd
               goto 300
             end if
           endif

           goto 320

  300    continue

c  failure after miter iterations

         itenua = itenua - 1

  320    continue

         print *, "  O/P:", h2para, h2orth, h2orth/h2para
         write (iwrit2,"(8x,i3,3x,1pe12.6,3x,e12.6,3x,e12.6,6x,
     1                  0pf13.6,5x,i3)")
     2                  iopt, tau(iopt), av, densh(iopt), temp, itenua

c  Save new temperature

         tgaz(iopt) = temp

c  oct and dec 91: variable step size

         if (iopt == 0) then

c  No comparison possible.
c     If max iteration reached, redo the first point
c     Else, go to next point.

           if (itenua >= miter) then
             goto 402
           endif

c  fdens contains various state laws and prescriptions
c        controled by "ifisob" and "ieqth"

           call fdens(tau(iopt+1), iopt, xdens, xtemp)
           tgaz(iopt+1) = xtemp
           densh(iopt+1) = xdens

         else if (iopt >= 1) then

c  Compute relative variation between two points (thevar)

           vart = abs((tgaz(iopt) - tgaz(iopt-1)) / tgaz(iopt))
           varn = abs((densh(iopt) - densh(iopt-1)) / densh(iopt))
           varh = abs(abnua(i_h,iopt-1) - abnua(i_h,iopt))
     1          / abnua(i_h,iopt)
           varh2 = abs(abnua(i_h2,iopt-1) - abnua(i_h2,iopt))
     1           / abnua(i_h2,iopt)
           if (i_hd /= 0) then
             varhd = abs(abnua(i_hd,iopt-1) - abnua(i_hd,iopt))
     1             / abnua(i_hd,iopt)
           else
             varhd = 0.0_dp
           endif
           varco = abs(abnua(i_co0,iopt-1) - abnua(i_co0,iopt))
     1           / abnua(i_co0,iopt)
           varc = abs(abnua(i_c,iopt-1) - abnua(i_c,iopt))
     1           / abnua(i_c,iopt)
           thevar = vart + varn + varh + varhd + varh2 + varco + varc

           testv = (tau(iopt) - tau(iopt-1))
     1            / min(tau(iopt),max(tau(1),taumax-tau(iopt)))

           if ((thevar >= varmax .or. itenua >= miter)
     1        .and. (testv >= dtaumin)) then

c  If "thevar" is too large, or max number of iterations has been reached,
c     the point is discarded. We need a new point, closer to the previous one.

             if (ifisob == 5) then
               tau(iopt) = tau(iopt-1)
     1                   + 0.2_dp * (tau(iopt) - tau(iopt-1))
             else
               tau(iopt) = (tau(iopt) + tau(iopt-1)) * 0.5_dp
             endif
             if (is_imp == 1) then
               is_imp = 0
             endif
             call fdens(tau(iopt), iopt, xdens, xtemp)
             tgaz(iopt) = xtemp
             densh(iopt) = xdens
             dtau = tau(iopt) - tau(iopt-1)

c Reset abundances (use if/then/else to enforce HIP or LIP
c       solutions (if they exist...)

             do ii=1,nspec
c              if (tau(iopt) > 1.0d-1) then
c                abnua(ii,iopt) = abin(ii) * densh(iopt)
c              else
                 abnua(ii,iopt) = abnua(ii,iopt-1)
     1                        * densh(iopt) / densh(iopt-1)
c              endif
             end do

             goto 402

           else

c  Write full radiation field at "imposed tau"
c  rfield is a radiative energy density (erg cm-3 A-1)

             if (is_imp == 1 .OR.  tau(iopt) == taumax) then
               rfield(next_imp,:) =  rf_u%Val(:)
             endif

c  If end of cloud, ...finished!

             if (tau(iopt) >= taumax) goto 403

c  Else, we accept the point and step to a further one.

c  A "safety coefficient" is used to avoid to many back and fro steps.

             if (is_imp == 1 .AND. tau(iopt) /= taumax) then
               next_imp = next_imp + 1
               is_imp = 0
             endif

             if (ifisob == 5) then
               dtau1 = 0.50d0 * dtau * varmax / thevar
             else
               dtau1 = 0.90d0 * dtau * varmax / thevar
             endif
             dtau = min(dtau1,0.005_dp*taumax)
             dtau = max(dtau,tau(1))

             tau(iopt+1) = tau(iopt) + dtau

             if (tau(iopt+1) >= tau_imp(next_imp)) then
               tau(iopt+1) = tau_imp(next_imp)
               dtau = tau(iopt+1) - tau(iopt)
               is_imp = 1
             endif
             call fdens(tau(iopt+1), iopt, xdens, xtemp)
             tgaz(iopt+1) = xtemp
             densh(iopt+1) = xdens

           endif

         endif

c  Initial conditions at next point
c          - Radiation field
c          - Chemical abundances
c          - Excited levels.

c  Grains temperature

         tdust(:,iopt+1) = tdust(:,iopt)

c  Chemical species

         do j=1,nspec
           abnua(j,iopt+1) =
     1       abnua(j,iopt) * densh(iopt+1) / densh(iopt)
         end do

c  H2 levels

         do lev=1,l0h2
           xreh2(lev,iopt+1) = xreh2(lev,iopt)
         end do

c  HD levels

         if (i_hd /= 0) then
           do lev=1,l0hd
             xrehd(lev,iopt+1) = xrehd(lev,iopt)
           end do
         endif

c  CO (and isotopes) levels

         do j=1,nlvco0
           xreco0(j,iopt+1) = xreco0(j,iopt)
           xreco1(j,iopt+1) = xreco1(j,iopt)
           xreco2(j,iopt+1) = xreco2(j,iopt)
           xreco3(j,iopt+1) = xreco3(j,iopt)
         end do

c  CS levels

         do j=1,nlvcs0
           xrecs0(j,iopt+1) = xrecs0(j,iopt)
         end do

c HCO+ levels
         do j=1,nlvhcop
           xrehcop(j,iopt+1) = xrehcop(j,iopt)
         end do

c H2O levels
         do j=1,nlvh2o
           xreh2o(j,iopt+1) = xreh2o(j,iopt)
         end do

c H3+ levels
         do j=1,nlvh3p
            xreh3p(j,iopt+1) = xreh3p(j,iopt)
         end do

  400  continue

  403  continue

       npo = min(noptm, iopt)

       call REALL

c  *** end of main iteration

         write (34, "(1p,21e13.5)") av, (pdh2vJ(lev,iopt), lev=1,20)
         close (34)

c  Write partial results, so we can check convergence.
c  (binary file only -> use prep!)

         call  SORTIE1(ifaf)

  450  continue

c   Human readable output file

       call  SORTIE2

c--------+---------+---------+---------+---------+---------+---------+-*-------+

c  formats:

 1030  format (1x,'IFAF:',I2,2x,'SLAB:',i3,2x,'ITER:',i2,2x,'Av:',
     1         1pe15.8,/,
     1         ' ----------------------------------------------',/,
     2         1x,'density:',1pe13.6,2x,'temperature:',0pf10.3,/)
 1040  format (1x,'H:',1pe10.2,'  H2:',e10.2,
     1         '  CO:',e10.2,/,1x,'errx=',e10.2,2x,'mauvais: ',a8,/)
 1041  format (1x,'H:',1pe10.2,'  H2:',e10.2,'  HD:',e10.2,
     1         '  CO:',e10.2,/,1x,'errx=',e10.2,2x,'mauvais: ',a8,/)

c---------------------------------------------------------------------

       stop
       end

c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine fdens(tt, ip, xdens, xtemp)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE STR_DATA
       USE SP_DATA
       USE PROFIL

c Set density and temperature as required by the user

       implicit double precision (a-h,o-z)

       if (ifisob == 5) then
c  Fractal density profile (JLB). NO interpolation !

         do i=1,ifrac
           if (tt <= tfrac(i)) then
             ind = i
             exit
           endif
         end do
         xdens = nfrac(ind)
         xtemp = tgaz(ip)

       else if (ifisob == 3) then
c  Temperature from Sternberg & Dalgarno, Ap J S, 1995, 99, 565

         if (ifeqth == 1) then
           xtemp = tgaz(ip)
         else
           toto = max(3.5d0 - 0.8d0 * tt, 1.3d0)
           xtemp =  10.0_dp**toto
         endif
         xdens = densh(ip)

       else if (ifisob == 2) then
c  isobaric state equation

         if (ifeqth == 1) then
           xtemp = tgaz(ip)
         else
           do i=1,pr_np-1
             if (tt <= pr_tau(i+1)) then
               ind = i
               exit
             endif
           end do
           rapp   = (tt - pr_tau(ind)) / (pr_tau(ind+1) - pr_tau(ind))
           xtemp = pr_tem(ind) + (pr_tem(ind+1) - pr_tem(ind)) * rapp
         endif
         hsurh2 = abnua(i_h,ip) / abnua(i_h2,ip)
         divi = (1.0_dp + hsurh2) / (2.0_dp + hsurh2)
         xdens = (presse / xtemp) / (abtot(5) + divi)

       else if (ifisob == 1) then
c  density interpolated

         do i=0,pr_np-1
           if (tt <= pr_tau(i+1)) then
             ind = i
             exit
           endif
         end do
         rapp   = (tt - pr_tau(ind)) / (pr_tau(ind+1) - pr_tau(ind))

         if (ifeqth == 1) then
           xtemp = tgaz(ip)
         else
           xtemp = pr_tem(ind) + (pr_tem(ind+1) - pr_tem(ind)) * rapp
         endif
         xdens = pr_den(ind) + (pr_den(ind+1) - pr_den(ind)) * rapp
c  We may also use an analytic formula, such as (e.g)
c        xdens = 100.d0 * (1. + tt/ 1.086d0) **2

       else if (ifisob == 0) then
c  constant density

         xtemp = tgaz(ip)
         xdens = densh(ip)

       else

         print *, '  ifisob = ', ifisob, ' is not allowed'
         stop

       endif

       return
       end
