
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%
       subroutine INIT1
c      %%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       use inoutput
       USE RF_TOOLS
       USE SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE H2_DATA
       USE HD_DATA
       USE PROFIL
       USE TRANSFER
       USE VERIF

c--------+---------+---------+---------+---------+---------+---------+-*-------+

       implicit double precision (a-h,o-z)

       real (kind=dp)     :: coefosc

c--------+---------+---------+---------+---------+---------+---------+-*-------+

c  Compute some constants

c  From tau to Av
      tautav = 2.5_dp * log10(exp(1.0_dp))
c  From J to u (erg cm-2 s-1 A-1 sr-1 to erg cm-3 A-1)
      J_to_u = 4.0_dp * xpi / clum
c  Temperature of the CMB
      tcmb = 2.73_dp ! will eventually be modified in rf_init (stv1y.f90) if 
                     ! an external intergalactic radiation field is used. 
                     ! It will become tcmb = 2.7 * (1 + redshift)

      print *,'--------------------------------------------------------'
      print *,'                   INITIALISATION                       '
      print *,'--------------------------------------------------------'

      abnua = 0.0_dp

c  Read input datafile: pdr3.in (see short comments in input file)

       fichier = TRIM(data_dir)//'pdr3.in'
       write(iscrn,*) "Ouverture de : ", fichier
       open (iread1, file = fichier, status = 'old')
         read (iread1,"(a35)") modele
         modele = TRIM(ADJUSTL(modele))
         read (iread1,"(a35)") chimie      ! name of the chemistry file
         chimie = TRIM(ADJUSTL(chimie))
         read (iread1,*) ifafm
         read (iread1,*) Avmax
         read (iread1,*) nh_init
         read (iread1,*) radm
         read (iread1,*) radp
         read (iread1,"(a35)") srcpp
         srcpp = TRIM(ADJUSTL(srcpp))
         read (iread1,*) d_sour
         read (iread1,*) fmrc
         read (iread1,*) ifeqth
         read (iread1,*) tg_init
         read (iread1,*) ifisob
         read (iread1,*) presse

         read (iread1,*) vturb
         read (iread1,*) itrfer
         read (iread1,*) jfgkh2
         if (itrfer == 0) then
           jlyman = 0
         else if (itrfer >= 1) then
           jlyman = 1
         else
           print *, " Wrong value of itrfer:", itrfer
           stop
         endif
         if (itrfer == 0 .AND. jfgkh2 /= 0) then
           print *, ' Set jfgkh2 = 0 with itrfer = 0'
           jfgkh2 = 0
         endif
         if (itrfer == 3) then
           jfgkco0 = 5
         else
           jfgkco0 = 0
         endif

         read (iread1,*) l0h2
         read (iread1,*) ichh2

         read (iread1,"(a35)") los_ext
         read (iread1,*) rrr
         read (iread1,*) cdunit

         read (iread1,*) alb
         read (iread1,*) gg
         read (iread1,*) gratio
         read (iread1,*) rhogr
         read (iread1,*) alpgr
         read (iread1,*) rgrmin
         read (iread1,*) rgrmax
         read (iread1,*) iforh2
         read (iread1,*) istic
       close (iread1)
       NHtot = Avmax * cdunit / rrr

c Initialisation des donnees H3+ -------------------------------------------
c On a besoin de certaines de ces donnees pour faire l'allocation
c de memoire dans INIT_PROFIL ci-dessous

      CALL INI_DATA_H3P
      IF (verif_ini_data_h3p == 1) CALL SUB_VERIF_INI_DATA_H3P
      ! sont initialises :
      ! elevh3p(i)%q : donnees du niveau i de H3+
      !                q = j  : J
      !                q = k  : K 
      !                q = en : energie en Kelvin
      !                q = g  : degenerescence
      !                q = n  : numero du niveau
      !                         n = 1 est J=1, K=1
      ! levh3p(j,k)  : numero du niveau J,K
      ! aeinh3p(levh3p(ju,ku),levh3p(jl,kl)) : Coeff A d'Einstein de Ju,Ku -> Jl,Kl

      ! Allocation des tableaux de H3+ qui n'ont pas ete alloues dans 
      ! stv1u et stv1v contrairement aux autres especes
      ALLOCATE(xreh3p(nlvh3p,0:noptm))
      ALLOCATE(emih3p(ntrh3p,0:noptm))
      ALLOCATE(opdh3p(ntrh3p,0:noptm))
      ALLOCATE(elkh3p(nlvh3p))
      ALLOCATE(gsth3p(nlvh3p))
      elkh3p(:) = elevh3p(:)%en
      gsth3p(:) = elevh3p(:)%g
      
c Fin de l'initialisation des donnees H3+ --------------------------------

       call INIT_PROFIL

c  Initialize output files

       fichier = TRIM(out_dir)//TRIM(ADJUSTL(modele))//'.def'
       write(iscrn,*) "Ouverture de : ", fichier 
       open (iwrit2, file = fichier, status = 'unknown')

       file_bin = TRIM(out_dir)//TRIM(ADJUSTL(modele))//'.bin'
       print *, file_bin

       call grparam

c--------+---------+---------+---------+---------+---------+---------+-*-------+

       write (iwrit2,2000)
       write (iwrit2,2002) modele
       write (iwrit2,2003) chimie
       write (iwrit2,2010) radm, radp, srcpp, d_sour, rstar, teff, fmrc
       write (iwrit2,2005) itrfer, jfgkh2, jfgkco0-1, jlyman
       write (iwrit2,2009) ifeqth, ifisob, presse
       write (iwrit2,2011) l0h2, mjhd, l0hd, iforh2, istic
       write (iwrit2,2013) rrr, cdunit, vturb
       write (iwrit2,2014) gratio, rhogr, alpgr, rgrmin, rgrmax, dsite
       write (iwrit2,2015) lleg, alb, gg, taumax, Avmax
       write (iwrit2,2018) los_ext, cc1, cc2,cc3, cc4, xgam, y0
       write (iwrit2,2020) tgaz(0), densh(0)

c  read chemical species, initial abundances
c  and chemical equations

       call LECTUR

       call TRANSF

       close (iwrit2)
       fichier = TRIM(out_dir)//TRIM(ADJUSTL(modele))//'.def'
       write(iscrn,*) "Ouverture de : ", fichier
       open (iwrit2, file = fichier, status = 'unknown'
     1             , position='append')

       tau(2:npo) = tau_o(2:npo)
       abnua(i_h,0:npo) = abh_o
       abnua(i_h2,0:npo) = abh2_o
       do i=1,l0h2
         xreh2(i,1:npo) = xreh2(i,0)
       end do
       if (i_hd /= 0) then
         abnua(i_hd,0:npo) = 1.0e-4_dp * abnua(i_h2,0:npo)
       endif
       abnua(i_co0,0:npo) = 1.0e-4_dp * abnua(i_h2,0:npo)
       if (i_co1 /= 0) then
         abnua(i_co1,0:npo) = abnua(i_co0,0:npo) / 70.0_dp
       endif
       if (i_co2 /= 0) then
         abnua(i_co2,0:npo) = abnua(i_co0,0:npo) / 500.0_dp
       endif
       if (i_co3 /= 0) then
         abnua(i_co3,0:npo) = abnua(i_co0,0:npo) / (70.0_dp * 500.0_dp)
       endif

       call CDT

       cdth2_o = cdth2(:,0:npo)
       cdthd_o = cdthd(:,0:npo)
       cdtc0_o = cdtc0(:,0:npo)
       cdtc1_o = cdtc1(:,0:npo)
       cdtc2_o = cdtc2(:,0:npo)
       cdtc3_o = cdtc3(:,0:npo)

c--------+---------+---------+---------+---------+---------+---------+-*-------+

c  Collision rates

       if (ichh2 == 0) then
         nqc = 4
       else
         nqc = 3
       endif
       allocate (qh1h2(nqc,l0h2,l0h2))
       allocate (qheh2(3,l0h2,l0h2))
       allocate (qph2h2(3,l0h2,l0h2))
       allocate (qoh2h2(3,l0h2,l0h2))
       qh1h2  = 0.0_dp
       qheh2  = 0.0_dp
       qph2h2 = 0.0_dp
       qoh2h2 = 0.0_dp
       qh1h2(1,:,:)  = -50.0_dp
       qheh2(1,:,:)  = -50.0_dp
       qph2h2(1,:,:) = -50.0_dp
       qoh2h2(1,:,:) = -50.0_dp

       if (i_hd /= 0) then
       do i=0,njthd
         do j=0,njthd
           qh1hd(1,j,i) = -50.0d0
           qh1hd(2,j,i) = 0.0d0
           qh1hd(3,j,i) = 0.0d0
           qh1hd(4,j,i) = 0.0d0
           qhehd(1,j,i) = -50.0d0
           qhehd(2,j,i) = 0.0d0
           qhehd(3,j,i) = 0.0d0
           qhehd(4,j,i) = 0.0d0
           qph2hd(1,j,i) = -50.0d0
           qph2hd(2,j,i) = 0.0d0
           qph2hd(3,j,i) = 0.0d0
           qph2hd(4,j,i) = 0.0d0
           qoh2hd(1,j,i) = -50.0d0
           qoh2hd(2,j,i) = 0.0d0
           qoh2hd(3,j,i) = 0.0d0
           qoh2hd(4,j,i) = 0.0d0
         end do
       end do
       endif

      print *
      print *,'-------------- H2 levels ----------------'
      call lev_H2 (l0h2)
      print *,'Nb of H2 levels (mlevh2):', mlevh2

c  Read H + H2 collision rates

       if (ichh2 == 0) then
         fichier = TRIM(data_dir)//'qh1h2_MM.dat'
       else
         fichier = TRIM(data_dir)//'qh1h2_DRF.dat'
       endif
       write(iscrn,*) "Ouverture de : ", fichier
       open(iread2, file = fichier, status = 'old')
       read (iread2,*) ntnr

       do j=1,ntnr
         read (iread2,*) levu, levl, q1, q2, q3, q4
         if ((levu <= l0h2) .AND. (levl <= l0h2)) then
           qh1h2(1,levu,levl) = q1
           qh1h2(2,levu,levl) = q2
           qh1h2(3,levu,levl) = q3
           if (nqc == 4) then
             qh1h2(4,levu,levl) = q4
           endif
         endif
       end do
       close (iread2)

c  5 jan 2000 - Coarse patch for missing transitions

       do levu=3,l0h2
         iju = njlev(levu)
         ivu = nvlev(levu)
         do levl=1,levu-1
           ijl = njlev(levl)
           if (mod(iju-ijl,2) /= 0) cycle
           ivl = nvlev(levl)

           if (qh1h2(1,levu,levl) == -50.0d0) then
             if (ivl == ivu .or. ivu == 1) then
               if (ivl >= 4) then
                 lev2 = level(ivu-1,iju)
                 lev1 = level(ivl-1,ijl)
               else if (ijl <= 1) then
                 lev2 = level(ivu,iju-2)
                 lev1 = levl
               else
                 lev2 = level(ivu,iju-2)
                 lev1 = level(ivl,ijl-2)
               endif
             else if (ivu > ivl .and.
     1                (iju /= 0 .or. ijl /= 0)) then
               lev2 = level(ivu-1,iju)
               lev1 = level(max(ivl-1,0),ijl)
             else
               lev2 = levu
               lev1 = levl
             end if
             do j=1,nqc
               qh1h2(j,levu,levl) = qh1h2(j,lev2,lev1)
             end do
           end if

         end do
       end do

c  Read He + H2 collision rates

       fichier = TRIM(data_dir)//'qheh2.dat'
       write(iscrn,*) "Ouverture de : ", fichier
       open(iread2, file = fichier, status = 'old')
       read (iread2,*) ntnr

       do j=1,ntnr
         read (iread2,*) levu, levl, q1, q2, q3
         if ((levu <= l0h2) .AND. (levl <= l0h2)) then
           qheh2(1,levu,levl) = q1
           qheh2(2,levu,levl) = q2
           qheh2(3,levu,levl) = q3
         endif
       end do
       close (iread2)

c  Read o-H2 + H2 collision rates

       fichier = TRIM(data_dir)//'qoh2h2.dat'
       write(iscrn,*) "Ouverture de : ", fichier
       open(iread2, file = fichier, status = 'old')
       read (iread2,*) ntnr

       do j=1,ntnr
         read (iread2,*) levu, levl, q1, q2, q3
         if ((levu <= l0h2) .AND. (levl <= l0h2)) then
           qoh2h2(1,levu,levl) = q1
           qoh2h2(2,levu,levl) = q2
           qoh2h2(3,levu,levl) = q3
         endif
       end do
       close (iread2)

c  Read p-H2 + H2 collision rates

       fichier = TRIM(data_dir)//'qph2h2.dat'
       write(iscrn,*) "Ouverture de : ", fichier
       open(iread2, file = fichier, status = 'old')
       read (iread2,*) ntnr

       do j=1,ntnr
         read (iread2,*) levu, levl, q1, q2, q3
         if ((levu <= l0h2) .AND. (levl <= l0h2)) then
           qph2h2(1,levu,levl) = q1
           qph2h2(2,levu,levl) = q2
           qph2h2(3,levu,levl) = q3
         endif
       end do
       close (iread2)

         if (i_hd /= 0) then
c  Read He + HD collision rates
         fichier = TRIM(data_dir)//'qhehd.dat'
         write(iscrn,*) "Ouverture de : ", fichier
         open (iread2, file = fichier, status = 'old')
         read(iread2,*) ntnd

         do ntc=1,ntnd
           read(iread2,4001) levu, levl, (qhehd(i,levu,levl), i=1,4)
         end do
         close (iread2)

c  Read H + HD collision rates
         fichier = TRIM(data_dir)//'qh1hd.dat'
         write(iscrn,*) "Ouverture de : ", fichier
         open (iread2, file = fichier, status = 'old')
         read(iread2,*) ntnd

         do ntc=1,ntnd
           read(iread2,4001) levu, levl, (qh1hd(i,levu,levl), i=1,4)
         end do
         close (iread2)

c  Read o-H2 + HD collision rates
         fichier = TRIM(data_dir)//'qoh2hd.dat'
         write(iscrn,*) "Ouverture de : ", fichier
         open (iread2, file = fichier, status = 'old')
         read(iread2,*) ntnd

         do ntc=1,ntnd
           read(iread2,4001) levu, levl, (qoh2hd(i,levu,levl), i=1,4)
         end do
         close (iread2)

c  Read p-H2 + HD collision rates
         fichier = TRIM(data_dir)//'qph2hd.dat'
         write(iscrn,*) "Ouverture de : ", fichier
         open (iread2, file = fichier, status = 'old')
         read(iread2,*) ntnd

         do ntc=1,ntnd
           read(iread2,4001) levu, levl, (qph2hd(i,levu,levl), i=1,4)
         end do
         close (iread2)

         endif

c  Read H2 + CS collision rates

         fichier = TRIM(data_dir)//'ratcsh2'
         write(iscrn,*) "Ouverture de : ", fichier
         open (iread2, file = fichier, status = 'old')
         read(iread2,*) ntccs

         do nt=1,ntccs
           read(iread2,*) ji, jf, acs1, acs2, acs3, acs4
           ntc = 1 + jf + ji * (ji-1) / 2
           if (nt /= ntc) then
             print *, "  Pb coll CS!", nt, ntc
             stop
           endif
           cdocs(1,ntc) = acs1
           cdocs(2,ntc) = acs2
           cdocs(3,ntc) = acs3
           cdocs(4,ntc) = acs4
         end do

         close (iread2)

c  Read H2 + HCO+ collision rates
         fichier = TRIM(data_dir)//'rathcoph2'
         write(iscrn,*) "Ouverture de : ", fichier
         open (iread2, file = fichier, status = 'old')
         read(iread2,*) ntchcop

         do  ntc=1,ntchcop
           read(iread2,*) ju, jl, (colhcoph2(i,ju,jl), i=1,4)
         enddo
         close (iread2)

c  Reading of the collision rate H2O + He

       fichier = TRIM(data_dir)//'qh2ohe.dat'
       write(iscrn,*) "Ouverture de : ", fichier
       open(iread2, file = fichier, status = 'old')
       read (iread2,*) ntnr

       qh2ohe = -50.0_DP

       do j=1,ntnr
         read (iread2,*) levu, levl, q1, q2, q3
         if (levu > nlvh2o) cycle

           qh2ohe(1,levu,levl) = q1
           qh2ohe(2,levu,levl) = q2
           qh2ohe(3,levu,levl) = q3

       end do
       close (iread2)

c  Reading of the collision rate H2O + H2

       fichier = TRIM(data_dir)//'qh2oh2.dat'
       write(iscrn,*) "Ouverture de : ", fichier
       open(iread2, file = fichier, status = 'old')
       read (iread2,*) ntnr

       qh2oh2 = -50.0_DP

       do j=1,ntnr
         read (iread2,*) levu, levl, q1, q2, q3
         if (levu > nlvh2o) cycle

           qh2oh2(1,levu,levl) = q1
           qh2oh2(2,levu,levl) = q2
           qh2oh2(3,levu,levl) = q3

       end do
       close (iread2)

c  Inverse life times of H2 B and C states
         tmunh2bc = 0.0_dp
c  Abundances of H2 B and C states
         h2bnua = 0.0_dp
         h2cnua = 0.0_dp

c  Compute "wlth0" (Lyman threshold) here so we can filter out
c    unnecessary H2 lines.

         hiop = 2.0_dp * xpi*xpi * xqe**4 * xme / (xhp*xhp)
         wlth0 = 1.0e8_dp * xhp * clum / hiop
         print *, "  wlth0:", wlth0

c  Read H2 radiative transitions files
c
c              uvh2b29.dat: Lyman transitions (all v) J <= 29
c              uvh2c29.dat: Werner transitions (all v) J <= 29
c
c              le fichier "uvh2.dat" contient les transitions
c              radiatives de l hydrogne moleculaire (au 20 juin 88).
c              complete le 25 sept 90 pour inclure j -> 14.
c
c              les fichiers "uvh2b.dat" et "uvh2c.dat" contiennent
c              les transitions radiatives (lyman et werner)
c              de l hydrogne moleculaire (au 01 janvier 89).
c
c              Completed to J = 29 some times during the 90's...
c
c              One transition per line. Relevant data are gathered
c              into two tables
c              IRAIH2: integer parameters
c              PRAIH2: real parameters
c
c     IRAIH2:   1:       transition (1: h2, lyman; 2: h2, werner; etc)
c     IRAH2V:   2:       lower level v
c               3:       lower level J
c               4:       upper level v
c               5:       Delta J (Jup = Jlo + DelJ)
c
c     PRAIH2:   1:       oscillator strength.
c     PRAH2V:   2:       Wavelength (in Angstrom).
c               3:       Inverse radiative lifetime of upper level (s-1)
c               4:       Dissociation probability of upper level

c  Allocate various matrix

         allocate (evrdbc(l0h2,l0h2))
         allocate (evcolh(l0h2,l0h2))
         allocate (evltot(l0h2,l0h2))
         allocate (evlqua(nlevh2,nlevh2))

c  Numerical coeficient : 8 pi**2 e**2 / (me * c)   x 1e16 (for Angstrom)

         coefosc = 8.0e16_dp * xpi*xpi * xqe2 / (xme * clum)

c  First pass: count the number of lines to keep

         aeitb = 0.0_dp
         aeitc = 0.0_dp

         nrh2bt = 0
         fichier = TRIM(data_dir)//'uvh2b29.dat'
         write(iscrn,*) "Ouverture de : ", fichier
         open (iread2, file = fichier, status = 'old')
         read (iread2,4101) nraib

         do i=1,nraib
           read (iread2,4112) num, ira1, nvl, njl, nvu, ndj
     1                      , pra1, pra2, pra3, pra4
           if ((level(nvl,njl) <= l0h2) .AND.
     2         (pra2 >= wlth0)) then
             nrh2bt = nrh2bt + 1
           endif
         end do
         close (iread2)

c  Second pass: read data

         ALLOCATE (irah2b(nrh2bt,4), prah2b(nrh2bt,4))
         write(iscrn,*) "Ouverture de : ", fichier
         open (iread2, file = fichier, status = 'old')
         read (iread2,4101) nraib
         xlamax = 0.0_dp
         j = 0
         do i=1,nraib
           read (iread2,4112) num, ira1, nvl, njl, nvu, ndj
     1                     , pra1, pra2, pra3, pra4
           lev = level(nvl,njl)
           nju = njl + ndj
           if ((lev <= l0h2) .AND.
     1         (pra2 >= wlth0)) then
             dumy =  coefosc * pra1 / pra2**2
     1            * ((2.0_dp*njl+1.0_dp) / (2.0_dp*nju+1.0_dp))
             aeinb(nvu,nju,lev) = dumy
             aeitb(nvu,nju) = aeitb(nvu,nju) + dumy
           endif

           if ((lev <= l0h2) .AND.
     2         (pra2 >= wlth0)) then
             j = j + 1
             irah2b(j,1) = nvl
             irah2b(j,2) = njl
             irah2b(j,3) = nvu
             irah2b(j,4) = ndj
             prah2b(j,1) = pra1
             prah2b(j,2) = pra2
             prah2b(j,3) = pra3
             prah2b(j,4) = pra4

             xlamax = max(xlamax,pra2)
             tmunh2bc(ira1,nvu,nju)=pra3 !ira1 precise si Ly ou We
           endif
         end do
         close (iread2)

c  Now Werner band

         nrh2ct = 0
         fichier = TRIM(data_dir)//'uvh2c29.dat'
         write(iscrn,*) "Ouverture de : ", fichier
         open (iread2, file = fichier, status = 'old')
         read (iread2,4101) nraic

         do i=1,nraic
           read (iread2,4112) num, ira1, nvl, njl, nvu, ndj
     1                      , pra1, pra2, pra3, pra4
           if ((level(nvl,njl) <= l0h2) .AND.
     2         (pra2 >= wlth0)) then
             nrh2ct = nrh2ct + 1
           endif
         end do
         close (iread2)

         ALLOCATE (irah2c(nrh2ct,4), prah2c(nrh2ct,4))
         write(iscrn,*) "Ouverture de : ", fichier
         open (iread2, file = fichier, status = 'old')
         read (iread2,4101) nraic
         j = 0
         do i=1,nraic
           read (iread2,4112) num, ira1, nvl, njl, nvu, ndj
     1                     , pra1, pra2, pra3, pra4
           lev = level(nvl,njl)
           nju = njl + ndj
           if ((lev <= l0h2) .AND.
     1         (pra2 >= wlth0)) then
             dumy =  coefosc * pra1 / pra2**2
     1             * ((2.0_dp*njl+1.0_dp) / (2.0_dp*nju+1.0_dp))
             aeinc(nvu,nju,lev) = dumy
             if (ndj == 0) then
               aeitc(nvu,nju,2) = aeitc(nvu,nju,2) + dumy
             else
               aeitc(nvu,nju,1) = aeitc(nvu,nju,1) + dumy
             endif
           endif

           if ((lev <= l0h2) .AND.
     2         (pra2 >= wlth0)) then
             j = j + 1
             irah2c(j,1) = nvl
             irah2c(j,2) = njl
             irah2c(j,3) = nvu
             irah2c(j,4) = ndj
             prah2c(j,1) = pra1
             prah2c(j,2) = pra2
             prah2c(j,3) = pra3
             prah2c(j,4) = pra4

             xlamax = max(xlamax,pra2)
             tmunh2bc(ira1,nvu,nju)=pra3 !ira1 precise si Ly ou We
           endif
         end do
         close (iread2)

         evlqua = 0.0_dp

c        print *, '  nrh2bt:', nrh2bt, ' nrh2ct:', nrh2ct

         fichier = TRIM(data_dir)//'aquadh2.dat'
         write(iscrn,*) "Ouverture de : ", fichier
         open (iread2, file = fichier, status = 'old')
         read (iread2,4005)
         read (iread2,4101) nrh2q

         do i=1,nrh2q
           read (iread2,4004) nvu, nvl, nju, as, aq, ao
           if (nju > njh2) cycle
           levu = level(nvu,nju)
           njl = nju -2
           if (njl >= 0 .and. as /= 0.0_dp) then
             levl = level(nvl,njl)
             evlqua(levl,levu) = - as
             aeitq(levu) = aeitq(levu) + as
           endif
           njl = nju
           if (aq /= 0.0_dp) then
             levl = level(nvl,njl)
             evlqua(levl,levu) = - aq
             aeitq(levu) = aeitq(levu) + aq
           endif
           njl = nju + 2
           if (njl > njh2) cycle
           if (ao /= 0.0_dp) then
             levl = level(nvl,njl)
             evlqua(levl,levu) = - ao
             aeitq(levu) = aeitq(levu) + ao
           endif
         end do
         close (iread2)

         do levu=1,nlevh2
           evlqua(levu,levu) = aeitq(levu)
         end do

c  Compute cascade coefficients CASHM, BHMBX, BHMCX

         call CACOEF

c  H2 Excitation secondary electrons (from R. Gredel probably)
c     Mostly not used in usual applications

c  H2 ETL Population at 30 K is used for secondary electrons excitation

       jmax = 7
       texcit = 30.0d0
       dummy = 0.0_dp

c  Entry coefs are setin stv1w.f90
c    comment the following line if you whish to use them
       entry = 0.0_dp
       popetl = 0.0_dp

       do j=0,jmax
         lev = level(0,j)
         popetl(j) = (2.0_dp * j + 1.0_dp) * 3.0_dp**mod(j,2)
     1             * exp(-elhm(lev) / texcit)
         dummy = dummy + popetl(j)
       end do

       paretl = 0.0_dp
       ortetl = 0.0_dp

       do j=0,jmax
         popetl(j) = popetl(j) / dummy
         if (mod(j,2) == 0) then
           paretl = paretl + popetl(j)
         else
           ortetl = ortetl + popetl(j)
         endif
       end do

c      write(iscrn,*)
c      write(iscrn,*) 'H2 ortho et para'
c      write(iscrn,*) 'paretl:',paretl
c      write(iscrn,*) 'ortetl:',ortetl

       sentry = 0.0_dp
       sentro = 0.0_dp
       sentrp = 0.0_dp

       do ij=0,jmax
         do iv=0,nvxh2
           sentry = sentry + entry(iv,ij)
           if (mod(ij,2) == 0) then
             sentrp = sentrp + entry(iv,ij)
           else
             sentro = sentro + entry(iv,ij)
           endif
         end do
       end do

       if (sentry /= 0.0_dp) then
         sentrp = sentrp / sentry
         sentro = sentro / sentry
       else
         sentrp = 1.0_dp
         sentro = 1.0_dp
       endif

       do ij=0,jmax
         do iv=0,nvxh2
           if (mod(ij,2) == 0) then
             entry(iv,ij) = entry(iv,ij) * paretl / sentrp
           else
             entry(iv,ij) = entry(iv,ij) * ortetl / sentro
           endif
         end do
       end do

c--------+---------+---------+---------+---------+---------+---------+-*-------+

c  Read uvh.dat:
c
c     IRAIH:             n of lower level
c     PRAIH:    1:       oscillator strength
c               2:       Wavelength (in angstrom).
c               3:       Inverse radiative lifetime of upper level (s-1)

       fichier = TRIM(data_dir)//'uvh.dat'
       nraih = nmrh
       write(iscrn,*) "Ouverture de : ", fichier
       open (iread2, file = fichier, status = 'old')
          do i=1,nraih
          read (iread2,*) iraih(i), (praih(i,m), m = 1,3)
          end do
       close (iread2)

c--------+---------+---------+---------+---------+---------+---------+-*-------+

c  Read uvd.dat: (same format)

       fichier = TRIM(data_dir)//'uvd.dat'
       nraid = 12
       write(iscrn,*) "Ouverture de : ", fichier
       open (iread2, file = fichier, status = 'old')
       do i=1,nraid
         read (iread2,4022) iraid(i), ( praid(i,m), m = 1,3)
       end do
       close (iread2)

c--------+---------+---------+---------+---------+---------+---------+-*-------+

       nrhdbt = 0
       nrhdct = 0
       if (i_hd /= 0) then

c  Read uvhd.dat (same format as H2)
c
c     evdihd(levl,levu) : dipole and quadrupole emission probability within X
c     ahdtq(lev)        : Sum of emission prob from upper level in X over all lower levels
c     ahdnb(nvu,nju,lev): Emission probabilities B-X from (v,j) to lev
c     ahdtb(nvu,nju)    : Sum over all lower levels
c     ahdnc(nvu,nju,lev): idem C-X
c     ahdtc(nvu,nju,1:2): dem C-X   1: C+
c                                   2: C-

         print *
         print *, '-------------- HD levels ----------------'

         fichier = TRIM(data_dir)//'uvhd.dat'
         write(iscrn,*) "Ouverture de : ", fichier
         open (iread2, file = fichier, status = 'old')

         ahdtb = 0.0_dp
         ahdnb = 0.0_dp

         ahdtc = 0.0_dp
         ahdnc = 0.0_dp

c  airahd(bc) and prahd(bc) keep only lines for which j <= mjhd
c    all other transitions are neglected to ensure conservation
c    of total number of molecules

         read (iread2,4101) nrai
         do ii=1,nrai
           read (iread2,4212) ira1, nvl, njl, nvu, ndj
     1                     , pra1, pra2, pra3, pra4, pra5
           lev = levhd(nvl,njl)
           if ((njl <= mjhd) .AND.  (lev <= l0hd) .AND.
     2         (pra2 >= wlth0)) then
             if (ira1 == 1) then
               nrhdbt = nrhdbt + 1
              else
               nrhdct = nrhdct + 1
             endif
           endif
         end do
         close (iread2)

         ALLOCATE (irahdb(nrhdbt,4), prahdb(nrhdbt,4))
         ALLOCATE (irahdc(nrhdct,4), prahdc(nrhdct,4))

         ib = 0
         ic = 0
         write(iscrn,*) "Ouverture de : ", fichier
         open (iread2, file = fichier, status = 'old')
         read (iread2,4101) nrai
         do ii=1,nrai
           read (iread2,4212) ira1, nvl, njl, nvu, ndj
     1                     , pra1, pra2, pra3, pra4, pra5
           lev = levhd(nvl,njl)
           if ((njl <= mjhd) .AND.  (lev <= l0hd) .AND.
     2         (pra2 >= wlth0)) then
             if (ira1 == 1) then
               ib = ib + 1
               irahdb(ib,1) = nvl
               irahdb(ib,2) = njl
               irahdb(ib,3) = nvu
               irahdb(ib,4) = ndj
               prahdb(ib,1) = pra1
               prahdb(ib,2) = pra2
               prahdb(ib,3) = pra4
               prahdb(ib,4) = pra5
             else
               ic = ic + 1
               irahdc(ic,1) = nvl
               irahdc(ic,2) = njl
               irahdc(ic,3) = nvu
               irahdc(ic,4) = ndj
               prahdc(ic,1) = pra1
               prahdc(ic,2) = pra2
               prahdc(ic,3) = pra4
               prahdc(ic,4) = pra5
             endif

           xlamax = max(xlamax,pra2)
           endif
         end do
         close (iread2)

c        print *, 'xlamax', xlamax,'  nrhdbt:', nrhdbt,
c     1           '  nrhdct:', nrhdct

c  Read emission probabilities

        jmaxi0 = njhd(l0hd)
        fichier = TRIM(data_dir)//'ahdnb.dat'
        write(iscrn,*) "Ouverture de : ", fichier
        open(iread2,file=fichier,status='old')
        read(iread2,4005)
        do i=1,30000
          read(iread2,4400,end=310) nvu, nju, nvl, njl, dumy
          if (nvu > nvbhd .or. nju > jmaxi0+1
     1       .or. nvl > nvxhd .or. njl > mjhd) cycle
          lev = levhd(nvl,njl)
          ahdnb(nvu,nju,lev) = dumy
          ahdtb(nvu,nju) = ahdtb(nvu,nju) + dumy
        end do
 310    continue

        fichier = TRIM(data_dir)//'ahdnc1.dat'
        write(iscrn,*) "Ouverture de : ", fichier
        open(iread2,file=fichier,status='old')
        read(iread2,4005)
        do i=1,30000
          read(iread2,4400,end=311) nvu, nju, nvl, njl, dumy
          if (nvu > nvchd .or. nju > jmaxi0+1
     1       .or. nvl > nvxhd .or. njl > mjhd) cycle
          lev = levhd(nvl,njl)
          ahdnc(nvu,nju,lev) = dumy
          ahdtc(nvu,nju,1) = ahdtc(nvu,nju,1) + dumy
        end do
 311    continue

        fichier = TRIM(data_dir)//'ahdnc2.dat'
        write(iscrn,*) "Ouverture de : ", fichier
        open(iread2,file=fichier,status='old')
        read(iread2,4005)
        do i=1,30000
          read(iread2,4400,end=312) nvu, nju, nvl, njl, dumy
          if (nvu > nvchd .or. nju > jmaxi0
     1       .or. nvl > nvxhd .or. njl > mjhd) cycle
          lev = levhd(nvl,njl)
          ahdnc(nvu,nju,lev) = dumy
          ahdtc(nvu,nju,2) = ahdtc(nvu,nju,2) + dumy
        end do
 312    continue

        ahdtq = 0.0_dp
        evdihd = 0.0_dp

c  Dipole and quadrupole emission probabilities of HD

       fichier = TRIM(data_dir)//'aquadhd.dat'
       write(iscrn,*) "Ouverture de : ", fichier
       open (iread2, file = fichier, status = 'old')
       read (iread2,4005)
       read (iread2,4101) nrhdq

       do i=1,nrhdq
         read (iread2,4006) nvu, nvl, nju, as, ar, aq, ap, ao
         if (nju > mjhd) cycle
         levu = levhd(nvu,nju)
         njl = nju -2
         if (njl >= 0 .and. as /= 0.0_dp) then
           levl = levhd(nvl,njl)
           evdihd(levl,levu) = - as
           ahdtq(levu) = ahdtq(levu) + as
         endif
         njl = nju - 1
         if (njl >= 0 .and. ar /= 0.0_dp) then
           levl = levhd(nvl,njl)
           evdihd(levl,levu) = - ar
           ahdtq(levu) = ahdtq(levu) + ar
         endif
         njl = nju
         if (aq /= 0.0_dp) then
           levl = levhd(nvl,njl)
           evdihd(levl,levu) = - aq
           ahdtq(levu) = ahdtq(levu) + aq
         endif
         njl = nju + 1
         if (njl <= mjhd .and. ap /= 0.0_dp) then
           levl = levhd(nvl,njl)
           evdihd(levl,levu) = - ap
           ahdtq(levu) = ahdtq(levu) + ap
         endif
         njl = nju + 2
         if (njl <= mjhd .and. ao /= 0.0_dp) then
           levl = levhd(nvl,njl)
           evdihd(levl,levu) = - ao
           ahdtq(levu) = ahdtq(levu) + ao
         endif
       end do
       close (iread2)

       do levu=1,nlevhd
         evdihd(levu,levu) = ahdtq(levu)
       end do

c  HD cascade coefficients CASHD, BHDBX, BHDCX

       call CACOEFHD

       write(iscrn,*)
       write(iscrn,*) "Maximum wavelength:", xlamax
       write(iscrn,*) "Nb of Lyman transitions (nrhdbt):", nrhdbt
       write(iscrn,*) "      Werner            (nrhdct):", nrhdct

       endif

c--------+---------+---------+---------+---------+---------+---------+-*-------+

       nrco0 = 0
       nrco1 = 0
       nrco2 = 0
       nrco3 = 0

c  Read UVCO.DAT:
c
c     IRAICO:            lower level J
c     PRAICO:   1:       Oscillator strength
c               2:       Wavelength (in Angstrom).
c               3:       Inverse radiative lifetime of upper level (s-1)
c               4:       dissociation probability of upper level

       fichier = TRIM(data_dir)//'UVCO.NW1'
       write(iscrn,*) "Ouverture de : ", fichier
       open (iread2, file = fichier, status = 'old')

       read (iread2,4101) nrco0
       do i=1,nrco0
         read (iread2,4022) irco0(i), ( prco0(i,m), m = 1,4)
       end do
       close (iread2)

C
C  Read 13CO files if need be
C
       if (i_co1 /= 0) then

         fichier = TRIM(data_dir)//'UVC13O.NW1'
         write(iscrn,*) "Ouverture de : ", fichier
         open (iread2, file = fichier, status = 'old')

         read (iread2,4101) nrco1
         do i=1,nrco1
           read (iread2,4022) irco1(i), ( prco1(i,m), m = 1,4)
         end do
         close (iread2)

       endif
C
C  Read C18O files if need be
C
       if (i_co2 /= 0)  then

         fichier = TRIM(data_dir)//'UVCO18.NW1'
         write(iscrn,*) "Ouverture de : ", fichier
         open (iread2, file = fichier, status = 'old')

         read (iread2,4101) nrco2
         do i=1,nrco2
           read (iread2,4022) irco2(i), ( prco2(i,m), m = 1,4)
         end do
         close (iread2)

       endif
C
C  Read 13C18O files if need be
C
       if (i_co3 /= 0)  then

         fichier = TRIM(data_dir)//'UVC13O18.NW1'
         write(iscrn,*) "Ouverture de : ", fichier
         open (iread2, file = fichier, status = 'old')

         read (iread2,4101) nrco3
         do i=1,nrco3
           read (iread2,4022) irco3(i), ( prco3(i,m), m = 1,4)
         end do
         close (iread2)

       endif

c  Total number of lines included
c  Note: This is probably too large if full transfer is done for lowest
c        lying levels of H2. Can we adjust that number easily?

         mfgk = nrh2bt + nrh2ct
     1        + nrhdbt + nrhdct
     2        + nrco0
     3        + nrco1 + nrco2 + nrco3

       allocate (fgkwl(mfgk), fgkrf(mfgk), fgkrfr(mfgk), fgkgrd(mfgk))

       ik = 0
       do i=1,nrh2bt
         ik = ik + 1
         fgkwl(ik) = prah2b(i,2)
       end do
       do i=1,nrh2ct
         ik = ik + 1
         fgkwl(ik) = prah2c(i,2)
       end do
       do i=1,nrhdbt
         ik = ik + 1
         fgkwl(ik) = prahdb(i,2)
       end do
       do i=1,nrhdct
         ik = ik + 1
         fgkwl(ik) = prahdb(i,2)
       end do
       do i=1,nrco0
         ik = ik + 1
         fgkwl(ik) = prco0(i,2)
       end do
       do i=1,nrco1
         ik = ik + 1
         fgkwl(ik) = prco1(i,2)
       end do
       do i=1,nrco2
         ik = ik + 1
         fgkwl(ik) = prco2(i,2)
       end do
       do i=1,nrco3
         ik = ik + 1
         fgkwl(ik) = prco3(i,2)
       end do

c--------+---------+---------+---------+---------+---------+---------+-*-------+

!   Initialise radiation field and related quantities

       CALL  RF_INIT

c      print *, '   End of INIT1 !'
c--------+---------+---------+---------+---------+---------+---------+-*-------+

 2000 format (/,15x,'Model of Steady state Interstellar cloud (PDR)',//,
     3       15x,'JLB, GPF, ER and many others',
     4       5x,'VIII 91 to XII 05',/,15x,28('*'))
 2002 format (/,5x,'Model name:     ',a35)
 2003 format (/,5x,'Chemistry file: ',a35)
 2010 format (/,5x,'Model definition :',//,
     1   5x,'Left  side radiation field intensity = ',1pe9.2,
     2      ' x Draine field',/,
     3   5x,'Right side radiation field intensity = ',e9.2,
     4      ' x Draine field',/,
     5   5x,'Illuminating star:                     ',a15,/,
     6   5x,'Star distance                      d = ',e10.3,
     7      ' pc',/,12x,'d < 0 : left side',/,
     8   12x,'d > 0 : right side',/,
     9   12x,'d = 0 : none',//,
     a   5x,'Star radius :                          ',e9.2,/,
     b   5x,'Effective temperature :                ',e9.2,/,
     c   5x,'Cosmic rays ionisation flux: fmrc    = ',0pf9.2,
     d       ' (* 1e-17 s-1)',/)
 2009 format (
     1   5x,'Thermal balance (no: 0, yes: 1)      = ',i5,/,
     2   5x,'Equation of state: ifisob            = ',i5,/,
     3   12x,'0: Constant density nH',/,
     4   12x,'1: nH and T interpolated from a file',/,
     5   12x,'2: Isobaric (nH * T = Cte)',/,
     6   12x,'3: Analytical temperature profile',/,
     7   12x,'5: Fractal density profile (experimental)',/,
     8   5x,'external Pressure: (if needed)       = ',1pe10.2,
     9      ' K cm-3')
 2005 format (/,5x,'Radiative transfer approximation: ',/,
     1   5x,'(if itrfer = 0: pure FGK ',/,
     2   5x,'    itrfer = 1: Approximate Transfer in line',/,
     3   5x,'    itrfer = 2: Full transfer in line)',/,
     4   10x,'itrfer  =',i2,/,
     5   5x,'(jfgkh2: lowest level of H2 still computed with FGK)',/,
     6   10x,'jfgkh2 =',i2,/,
     5   5x,'(jfgkco0: highest level of CO included in transfer)',/,
     6   10x,'jfgkco =',i2,/,
     7   5x,'(if jlyman = 0, H Lyman lines use FGK)',/,
     8   10x,'jlyman =',i2,/)
c--------+---------+---------+---------+---------+---------+---------+-*-------+
 2011 format (5x,'H2 levels parameters:',/,
     1        5x,'l0h2 = ',i4,//,
     2        5x,'HD levels parameters:',/,
     3        5x,'Jmax = ',i8,5x,'l0hd = ',i4,//,
     4        5x,'H2 formation on grains: iforh2 = ',i4,/,
     5        15x,'0: Mean internal energy is 1/3 E(H2 formation)',/,
     6        15x,'1: Maxwell distribution at 1/3 E(H2 formation)',/,
     7        15x,'2: All on v = 14, J = 0 or 1 (obsolete)',/,
     8        15x,'3: Maxwell distrib at 65 K on v = 6 (obsolete)',//,
     9        5x,'H sticking on grains:    istic = ',i4,/
     a        15x,'1: Sticking read in chemistry file',/,
     b        15x,'2: Modified from Sternberg & Dalgarno',/,
     c        15x,'3: Modified from Andersson & Wannier',/,
     d        15x,'4: Modified from Flower & des Forets',/)
 2013 format (5x,'Extinction and gas to dust parameters:',/,
     1        5x,'   rrr    = ',f9.3,10x,'Av = rrr * E(B-V)',/,
     2        5x,'   cdunit = ',1pe9.2,10x,'NH = cdunit * E(B-V)',//,
     3        5x,'turbulent velocity: vturb = ',e9.3,' cm s-1')
 2014  format (/,5x,'mass ratio (gas/grains):      gratio = ',1pe10.2,/,
     1        5x,'grains density :               rhogr = ',e10.2,
     2           ' g cm-3',/,
     3        5x,'Size distribution exponent:    alpgr = ',0pf6.2,/,
     4        5x,'grains min radius:            rgrmin = ',1pe10.2,
     5           ' cm',/,
     6        5x,'grains max radius:            rgrmax = ',e10.2,
     7           ' cm',/,
     8        5x,'mean dist between adsorption sites   = ',e10.2,
     7           ' cm')
 2015 format (/,
     1        5x,'Order of Legendre expansion:    lleg = ',i3,/,
     2        5x,'Dust albedo                      alb = ',1pe10.2,/,
     3        5x,'<cos(theta)> (g)                  gg = ',e10.2,//,
     4        5x,'Total optical depth:          taumax = ',e10.2,/,
     5        5x,'                         i.e.  Avmax = ',e10.2)
 2018  format (/,5x,'Extinction curve :',a10,/,
     1           5x,'Fitzpatrick & Massa coefs:',1x,6f8.3,/)
 2020  format (5x,'Initial temperature:            ',1pe12.2,' K',/,
     1         5x,'Initial density:                ',e12.2,' cm-3')
 4001  format(1x,I2,1x,I2,4(1x,E10.3))
 4101  format (i8)
 4112  format (i8,i3,1x,2i3,1x,2i3,2x,1pd9.3,2x,1pd12.6,2(2x,1pd9.3))
 4212  format (6x,i1,1x,2i3,1x,2i3,2x,1pd9.3,2x,1pd12.6, 3x, 1pd13.7,
     1          2(2x,1pd9.3))
 4004  format (3i4,3d15.6)
 4005  format (//)
 4006  format (3i4,5d15.6)
 4022  format (2x,i3,2x,1pd9.3,2x,1pd12.6,2(2x,1pd9.3))
 4400  format (i3,5x,i3,5x,i3,5x,i3,5x,d13.6)
c--------+---------+---------+---------+---------+---------+---------+-*-------+

       return
       end

       subroutine CACOEF

       use constantes
       use auxiliaries
       USE H2_DATA
       USE STR_DATA
       USE HD_DATA
       USE PROFIL

       implicit double precision (a-h,o-z)

       dimension cscad(nlevh2,nlevh2)

c  Choose at which level of X we stop (JLB 05: ???)

       nj0 = njlev(l0h2)
       nv0 = nvlev(l0h2)
       write (iscrn,1001) l0h2, nv0, nj0, elhm(l0h2)
       write (iwrit2,1001) l0h2, nv0, nj0, elhm(l0h2)

       write(iscrn,*)
       write(iscrn,*) 'Nb of Lyman transitions (nrh2bt)', nrh2bt
       write(iscrn,*) '      Werner            (nrh2ct)', nrh2ct

       cscad = 0.0_dp

c  Cascade coefficient are computed by recurence, starting from
c    upper levels

       do levl=nlevh2,l0h2+1,-1
         njl = njlev(levl)
         cscad(levl,levl) = 1.0_dp
         do levu=levl+1,nlevh2
           nju = njlev(levu)
           total = 0.0_dp
           do k=levl+1,levu
             nj = njlev(k)
             if (aeitq(k) == 0.0_dp) cycle
             total = total - cscad(levu,k) * evlqua(levl,k)
     1                     / aeitq(k)
           end do
           cscad(levu,levl) = total
         end do
       end do

c       print *, '    cscad calcule'

c  Compute CASHM (cascades towards levels fully taken into account)

       do levl=1,l0h2
         njl = njlev(levl)
         do levu=l0h2+1,nlevh2
           nju = njlev(levu)
           total = 0.0_dp
           do k=l0h2+1,levu
             nj = njlev(k)
             if (aeitq(k) == 0.0_dp) cycle
             total = total - cscad(levu,k) * evlqua(levl,k)
     1                     / aeitq(k)
           end do
           cashm(levu,levl) = total
         end do
       end do

c  Verify CASHM:
c    Sum over lower levels must be 1 (total number of molecules is constant)

       do levu=l0h2+1,nlevh2
         nvu = nvlev(levu)
         nju = njlev(levu)
         total = 1.0_dp
         do levl=1,l0h2
           njl = njlev(levl)
           total = total - cashm(levu,levl)
         end do
         if (abs(total) > 1.0d-8 .and.
     1       total /= 1.0_dp) then
           print *, 'pb CASHM!:',nvu,nju,total,levu
         endif
       end do

c       print *, '    cashm computed'

c  Compute bhmbx (Lyman transitions)

       do nvu=0,nvbh2
         do nju=0,njh2+1
           dviei = aeitb(nvu,nju)
           if (dviei == 0.0_dp) cycle

           do levl=1,l0h2
             njl = njlev(levl)
             total = 0.0_dp
             do levu=l0h2+1,nlevh2
               nj = njlev(levu)
               total = total + aeinb(nvu,nju,levu) * cashm(levu,levl)
             end do
             total = total + aeinb(nvu,nju,levl)
             bhmbx(nvu,nju,levl) = total / dviei
           end do

         end do
       end do

c  Verify

c 510  print *, ' which BHMBX do you want? (vu, Ju, vl, Jl)'
c      read *, nvu, nju, nvl, njl
c      levl = level(nvl,njl)
c      print *, nvu, nju, levl, bhmbx(nvu, nju, levl)
c      print *, '   Repeat? (yes: 1)'
c      read *, iii
c      if (iii == 1) goto 510

c      test bhmbx

       do nvu=0,nvbh2
         do nju=0,njh2+1
           dviei = aeitb(nvu,nju)
           if (dviei == 0.0_dp) cycle
           total = 1.0_dp

           do levl=1,l0h2
             njl = njlev(levl)
             total = total - bhmbx(nvu,nju,levl)
           end do

           if (abs(total) > 1.0d-8) then
             print *, 'pb BHMBX!:',nvu,nju,total
           endif

         end do
       end do

c       print *, '    bhmbx computed'

c  Compute BHMCX (Werner transitions)
c    Beware of Lambda-doubling!
c    Each sub-level is identified only using Delta-J parity
c    This is a bit tricky (and dangerous). Should we be more systematic?

       do nvu=0,nvch2
         do nju=1,njh2+1
           dviei1 = aeitc(nvu,nju,1)
           dviei2 = aeitc(nvu,nju,2)
           if (dviei1 == 0.0_dp .and. dviei2 == 0.0_dp) cycle

           do levl=1,l0h2
             njl = njlev(levl)
             ndj0 = iabs(nju - njl)

             if (mod(ndj0,2) == 0
     1           .and. dviei2 /= 0.0_dp) then

               total = 0.0_dp
               do levu=l0h2+1,nlevh2
                 nj = njlev(levu)
                 ndj = iabs(nju - nj)
                 if (ndj /= 0) cycle
                 total = total + aeinc(nvu,nju,levu) * cashm(levu,levl)
               end do
               total = total + aeinc(nvu,nju,levl)
               bhmcx(nvu,nju,levl) = total / dviei2

             else if (mod(ndj0,2) == 1
     1                .and. dviei1 /= 0.0_dp) then

               total = 0.0_dp
               do levu=l0h2+1,nlevh2
                 nj = njlev(levu)
                 ndj = iabs(nju - nj)
                 if (ndj /= 1) cycle
                 total = total + aeinc(nvu,nju,levu) * cashm(levu,levl)
               end do
               total = total + aeinc(nvu,nju,levl)
               bhmcx(nvu,nju,levl) = total / dviei1
             endif

           end do

         end do
       end do

c      test bhmcx

       do nvu=0,nvch2
         do nju=1,njh2+1
           dviei1 = aeitc(nvu,nju,1)
           dviei2 = aeitc(nvu,nju,2)
           if (dviei1 == 0.0_dp .and. dviei2 == 0.0_dp) cycle

           total1 = 1.0_dp
           total2 = 1.0_dp

           do levl=1,l0h2
             njl = njlev(levl)
             ndj = iabs(nju - njl)

             if (mod(ndj,2) == 0) then
               total2 = total2 - bhmcx(nvu,nju,levl)
             else if (mod(ndj,2) == 1) then
               total1 = total1 - bhmcx(nvu,nju,levl)
             endif

           end do

           if ((abs(total1) > 1.0d-8
     1         .or. abs(total2) > 1.0d-8)
     2         .and. nju /= njh2+1) then
             print *, 'CX!:',nvu, nju, total1, total2
           endif

         end do
         end do

c 520  print *, ' Which BHMCX do you want? (vu, Ju, vl, Jl)'
c      read *, nvu, nju, nvl, njl
c      levl = level(nvl,njl)
c      print *, nvu, nju, levl, bhmcx(nvu, nju, levl)
c      print *, '   Repeat? (oui: 1)'
c      read *, iii
c      if (iii == 1) goto 520

c       print *, '    bhmcx computed'

c--------+---------+---------+---------+---------+---------+---------+-*-------+

 1001  format(/,1x,'Levels from 1 to ',i4, ' are included',/
     1 ,1x,'Last level is: v:',i4,' J:',i4,/
     2 ,1x,'Last level energy is:',f15.3,' K',/)
c--------+---------+---------+---------+---------+---------+---------+-*-------+

       end

c--- HD cascade coefficients --------------------------------
c      see Astron. Astrophys. 190, 215-236 (1988)

       Subroutine CACOEFHD

       use constantes
       use auxiliaries
       USE STR_DATA
       USE HD_DATA
       USE PROFIL

       implicit double precision (a-h,o-z)

       dimension cscad(nlevhd,nlevhd)

       nj0 = njhd(l0hd)
       nv0 = nvhd(l0hd)
       write (iscrn,1001) l0hd, nv0, nj0, elhd(l0hd)
       write (iwrit2,1001) l0hd, nv0, nj0, elhd(l0hd)

c   initialisation

       cscad = 0.0_dp

c  If mjhd is not large enough, some levels are not included

       do lev=1,l0hd
         nj=njhd(lev)
         if (nj > mjhd) then
           write (iwrit2,"(3x,'excluded level:',i6,' v:',i3,' J:',i3)")
     1            lev, nvhd(lev), nj
           write (iscrn,"(3x,'excluded level:',i6,' v:',i3,' J:',i3)")
     1            lev, nvhd(lev), nj
         endif
       end do

c  Same logic as for H2

       do levl=nlevhd,l0hd+1,-1
         njl = njhd(levl)
         if (njl > mjhd) cycle
         cscad(levl,levl) = 1.0_dp

         do levu=levl+1,nlevhd
           nju = njhd(levu)
           if (nju > mjhd) cycle
           total = 0.0_dp
           do k=levl+1,levu
             nj = njhd(k)
             if (nj > mjhd .or. ahdtq(k) == 0.0_dp) cycle
             total = total - cscad(levu,k) * evdihd(levl,k)
     1                     / ahdtq(k)
           end do
           cscad(levu,levl) = total
         end do
       end do

c       print *, '    cscad computed'

       do levl=1,l0hd
         njl = njhd(levl)
         if (njl > mjhd) cycle

         do levu=l0hd+1,nlevhd
           nju = njhd(levu)
           if (nju > mjhd) cycle
           total = 0.0_dp
           do k=l0hd+1,levu
             nj = njhd(k)
             if (nj > mjhd .or. ahdtq(k) == 0.0_dp) cycle
             total = total - cscad(levu,k) * evdihd(levl,k)
     1                     / ahdtq(k)
           end do
           cashd(levu,levl) = total
         end do
       end do

c  Verify CASHD:

       do levu =l0hd+1,nlevhd
         nvu = nvhd(levu)
         nju = njhd(levu)
         if (nju > mjhd) cycle
         total = 1.0_dp

         do levl=1,l0hd
           njl = njhd(levl)
           if (njl > mjhd) cycle
           total = total - cashd(levu,levl)
         end do

         if (abs(total) > 1.0d-8 .and.
     1       total /= 1.0_dp) then
           print *, 'pb CASHD!:',nvu,nju,total,levu
         endif
       end do

c      print *, '    cashd computed'

c  Compute bhdbx

       jmaxi0 = njhd(l0hd)
       do nvu=0,nvbhd
         do nju=0,jmaxi0+1
           dviei = ahdtb(nvu,nju)
           if (dviei == 0.0_dp) cycle
           do levl=1,l0hd
             njl = njhd(levl)
             if (njl > mjhd) cycle
             total = 0.0_dp
             do levu=l0hd+1,nlevhd
               nj = njhd(levu)
               if (nj > mjhd) cycle
               total = total + ahdnb(nvu,nju,levu) * cashd(levu,levl)
             end do
             total = total + ahdnb(nvu,nju,levl)
             bhdbx(nvu,nju,levl) = total / dviei
           end do
         end do
       end do

c  Verify

c 510  print *, ' Which BHDBX do you want? (vu, Ju, vl, Jl)'
c      read *, nvu, nju, nvl, njl
c      levl = levhd(nvl,njl)
c      print *, nvu, nju, levl, bhdbx(nvu, nju, levl)
c      print *, '   Repeat? (oui:1)'
c      read *, iii
c      if (iii == 1) goto 510

c      test bhdbx

       do nvu=0,nvbhd
         do nju=1,jmaxi0+1
           dviei = ahdtb(nvu,nju)
           if (dviei == 0.0_dp) cycle
           total = 1.0_dp

           do levl=1,l0hd
             njl = njhd(levl)
             if (njl > mjhd) cycle
             total = total - bhdbx(nvu,nju,levl)
           end do

           if (abs(total) > 1.0d-8) then
             print *, 'pb BHDBX!:',nvu,nju,total
           endif
         end do
       end do

c      print *, '    bhdbx computed'

c  Compute BHDCX
c    Same tick as H2 for Lamda-doubling

c  From C-

       do nvu=0,nvchd
         do nju=1,jmaxi0
           dviei2 = ahdtc(nvu,nju,2)
           if (dviei2 == 0.0_dp) cycle

           do levl=1,l0hd
             njl = njhd(levl)

             if (njl <= mjhd .and. dviei2 /= 0.0_dp) then
               total = 0.0_dp
               do levu=l0hd+1,nlevhd
                 nj = njhd(levu)
                 ndj = iabs(nju - nj)
                 if (nj > mjhd .or. ndj /= 0) cycle
                 total = total + ahdnc(nvu,nju,levu) * cashd(levu,levl)
               end do
               ndj = iabs(nju - njhd(levl))
               if (ndj == 0) total = total + ahdnc(nvu,nju,levl)
               bhdcmx(nvu,nju,levl) = total / dviei2
             endif
           end do
         end do
       end do

c  From C+

       do nvu=0,nvchd
         do nju=1,jmaxi0+1
           dviei1 = ahdtc(nvu,nju,1)
           if (dviei1 == 0.0_dp) cycle

           do levl=1,l0hd
             njl = njhd(levl)

             if (njl <= mjhd .and. dviei1 /= 0.0_dp) then
               total = 0.0_dp
               do levu=l0hd+1,nlevhd
                 nj = njhd(levu)
                 ndj = iabs(nju - nj)
                 if (nj > mjhd .or. ndj /= 1) cycle
                 total = total + ahdnc(nvu,nju,levu) * cashd(levu,levl)
               end do
               ndj = iabs(nju - njhd(levl))
               if (ndj == 1)  total = total + ahdnc(nvu,nju,levl)
               bhdcpx(nvu,nju,levl) = total / dviei1
             endif
           end do
         end do
       end do

c  test bhdcx

       do nvu=0,nvchd
         do nju=1,jmaxi0+1
           dviei1 = ahdtc(nvu,nju,1)
           dviei2 = ahdtc(nvu,nju,2)

           if (dviei1 == 0.0_dp .and. dviei2 == 0.0_dp) cycle

           total1 = 1.0_dp
           total2 = 1.0_dp

           do levl=1,l0hd
             njl = njhd(levl)
             ndj = iabs(nju - njl)

             if (njl <= mjhd) then
               total2 = total2 - bhdcmx(nvu,nju,levl)
               total1 = total1 - bhdcpx(nvu,nju,levl)
             endif
           end do

           if ((abs(total1) > 1.0d-8 .or. abs(total2) > 1.0d-8)
     1         .and. nju /= jmaxi0+1) then
             print *, 'CX!:',nvu, nju, total1 , total2
           endif
         end do
       end do

c 520  print *, ' Which BHDCX do you want? (vu, Ju, vl, Jl)'
c      read *, nvu, nju, nvl, njl
c      levl = levhd(nvl,njl)
c      print *, nvu, nju, levl, bhdcx(nvu, nju, levl)
c      print *, '   Repeat? (oui: 1)'
c      read *, iii
c      if (iii == 1) goto 520

c      print *, '    bhdcx computed'

c--------+---------+---------+---------+---------+---------+---------+-*-------+

 1001  format(/,1x,'Levels from 1 to ',i4, ' are included',/
     1 ,1x,'Last level is: v:',i4,' J:',i4,/
     2 ,1x,'Last level energy is:',f15.3,' K',/)
c--------+---------+---------+---------+---------+---------+---------+-*-------+

       end

