
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


C     *******************************
      subroutine ISOTC (nreact,idiff)
C     *******************************

      use constantes
      use auxiliaries
      USE STR_DATA
      USE SP_DATA
      USE CHEM_DATA

C *************************************************************************
C          CONSTRUCTION DE LA CHIMIE ISOTOPEE DU CARBONE
C *************************************************************************
C
C         NEQT   = NOMBRE TOTAL DE REACTIONS SANS ISOTOPE
C         NREACT = NUMERO DE LA REACTION ISOTOPEE
C         IDIFF  = NUMERO DE LA DERNIERE REACTION DIFFERENTIELLE
C         JJ     = NUMERO DE LA REACTION CORRESPONDANTE (SS ISOTOPE)
C
C *************************************************************************
C
C   CONVENTION :    DANS LA LISTE DES ESPECES, CHAQUE ESPECE ISOTOPEE DOIT
C   ----------      SUIVRE IMMEDIATEMENT l ESPECE MERE
C                   DANS LE CAS OU l ESPECE CONTIENT DE l OXYGENE ET DU
C                   CARBONE (C OU C*) ==> l ESPECE ISOTOPEE: O* SUIT C*
C                   EX :    HCO2
C                           HC*O2
C                           HCO*O
C                           HC*O*O
C
C *************************************************************************

       implicit double precision (a-h,o-z)

c 15 march 2004 - JLB - Add 3 body reactions

      do 500 jj=1,neqt

c 23 oct 2006 - JLB - skip direct photodestruction

        if (itype(jj) == 7) cycle

c  Definition des reactants et produits:

        ir0 = react(jj,0)
        ir1 = react(jj,1)
        ir2 = react(jj,2)
        ir3c = react(jj,3)
        ir3 = react(jj,4)
        ir4 = react(jj,5)
        ir5 = react(jj,6)
        ir6 = react(jj,7)

C-----------------------------------------------------------------------------
C--- IL N'Y A PAS DE CARBONE DANS LES PHOTONS OU LES CRP
C    ou les photons secondaires
C            (POUR LESQUELS AEL N'EST PAS DEFINI)

        if (ir2 == nspec+1 .or. ir2 == nspec+2
     1      .or. ir2 == nspec+3) then
          ncr2 = 0
          go to 550
        endif
C-----------------------------------------------------------------------------
C---Y-A-t IL DEJA DES ESPECES ISOTOPEES (C* OU O*) DANS LA REACTION ?
C         SI OUI, ON PASSE A LA REACTION SUIVANTE (GO TO 500)
C-----------------------------------------------------------------------------

        itest = ael(ir1,7) + ael(ir2,7)
     1        + ael(ir1,9) + ael(ir2,9)

        if (itest > 0)    then
          idiff = jj
          go to 500
        endif
C-----------------------------------------------------------------------------
C---Y-A-t IL DU CARBONE DANS LA REACTION CONSIDEREE ?
C                                        ET COMBIEN ?
C
C          NCR1, NCR2 = NOMBRE DE C DANS LES REACTANTS R1 ET R2
C          NCP1, NCP2 = NOMBRE DE C DANS LES REACTANTS P1 ET P2
C
C-----------------------------------------------------------------------------

        if (ael(ir2,1) /= ael(ir2+1,1)   .or.
     1      ael(ir2,2) /= ael(ir2+1,2)+1 .or.
     1      ael(ir2,7) /= ael(ir2+1,7)-1) then
          ncr2 = 0
        else
          ncr2 = ael(ir2,2)
        endif

  550   continue

        if (ael(ir1,1) /= ael(ir1+1,1)   .or.
     1      ael(ir1,2) /= ael(ir1+1,2)+1 .or.
     1      ael(ir1,7) /= ael(ir1+1,7)-1) then
          ncr1 = 0
        else
          ncr1 = ael(ir1,2)
        endif

        if (ael(ir3,1) /= ael(ir3+1,1)   .or.
     1      ael(ir3,2) /= ael(ir3+1,2)+1 .or.
     1      ael(ir3,7) /= ael(ir3+1,7)-1) then
          ncp1 = 0
        else
          ncp1 = ael(ir3,2)
        endif

        if (ir4 /= 0) then
          if (ael(ir4,1) /= ael(ir4+1,1)   .or.
     1        ael(ir4,2) /= ael(ir4+1,2)+1 .or.
     1        ael(ir4,7) /= ael(ir4+1,7)-1) then
            ncp2 = 0
          else
            ncp2 = ael(ir4,2)
          endif
        else
          ncp2 = 0
        endif

C--- IL N'Y A PAS DE CARBONE DANS LES PHOTONS OU LES CRP
C    ou les photons secondaires
C           (POUR LESQUELS AEL N EST PAS DEFINI)

        if (ir4 == nspec+1 .or. ir4 == nspec+2
     1  .or. ir4 == nspec+3) then
          ncp2 = 0
        endif

C-----------------------------------------------------------------------------
C--- 1) PREMIER CAS : IL Y A DU C (isotopables) DANS LES DEUX REACTANTS
C       ***********                                      ****
C
C--- trois POSSIBILITES:
C
C    A) IL Y A DU C DANS LES DEUX PRODUITS ==> 2 REACTIONS ISOTOPEES
C    B) OU BIEN DANS UN SEUL               ==> 2 REACTIONS ISOTOPEES
c    c) il n'y a pas de C isotopable       ==> On saute la reaction
C
C    ==> PAS DE RAPPORTS DE BRANCHEMENT
C
C-----------------------------------------------------------------------------

        if (ncr1 > 0 .and. ncr2 > 0) then

C-----------------------------------------------------------------------------
C------------ A) PREMIERE POSSIBILITE : IL Y A DU C DANS LES DEUX PRODUITS :
C                                                        ********
C
C  CONVENTION : LES REACTIONS DOIVENT ETRE ECRITE DANS l ORDRE :
C  ----------
C
C      AC   +  BC   -->  A'C   +  B'C
C
C                   TEL QUE  =====>      AC*  +  BC   -->  A'C   +  B'C*
C                        ET  =====>      AC   +  BC*  -->  A'C*  +  B'C
C
C                          ==> 2 REACTIONS ISOTOPEES
C-----------------------------------------------------------------------------
        if (ncp1 > 0 .and. ncp2 > 0) then
C-----------------------------------------------------------------------------
C
C--- PREMIERE REACTION ISOTOPEE:

          nreact = nreact + 1

          react (nreact,0) = ir0
          react (nreact,1) = ir1 + 1
          react (nreact,2) = ir2
          react (nreact,3) = ir3c
          react (nreact,4) = ir3
          react (nreact,5) = ir4 + 1
          react (nreact,6) = ir5
          react (nreact,7) = ir6

          gamm(nreact)  = gamm(jj)
          alpha(nreact) = alpha(jj)
          bet(nreact)   = bet(jj)
          itype(nreact) = itype(jj)

C--- DEUXIEME REACTION ISOTOPEE:

          nreact = nreact + 1

          react (nreact,0) = ir0
          react (nreact,1) = ir1
          react (nreact,2) = ir2 + 1
          react (nreact,3) = ir3c
          react (nreact,4) = ir3 + 1
          react (nreact,5) = ir4
          react (nreact,6) = ir5
          react (nreact,7) = ir6

          gamm(nreact)  = gamm(jj)
          alpha(nreact) = alpha(jj)
          bet(nreact)   = bet(jj)
          itype(nreact) = itype(jj)

C-----------------------------------------------------------------------------
C------------ B- DEUXIEME POSSIBILITE : IL Y A DU C DANS UN SEUL PRODUIT :
C                                                        *******
C
C      AC  +  BC  -->  A'C  +  B'
C  OU  AC  +  BC  -->  A'   +  B'C
C
C                            =====>      AC*  +  BC   -->  A'C*  +  B'
C                                    OU  AC*  +  BC   -->  A'    +  B'C*
C
C                        ET  =====>      AC   +  BC*  -->  A'C*  +  B
C                                    OU  AC   +  BC*  -->  A'    +  B'C*
C
C                       ==> 2 REACTIONS ISOTOPEES
C-----------------------------------------------------------------------------

        else if (ncp1 > 0 .or. ncp2 > 0) then

C-----------------------------------------------------------------------------
C
C--- PREMIERE REACTION ISOTOPEE :

          nreact = nreact + 1

          react (nreact,0) = ir0
          react (nreact,1) = ir1 + 1
          react (nreact,2) = ir2
          react (nreact,3) = ir3c

          if (ncp1 == 0) then
            react (nreact,4) = ir3
            react (nreact,5) = ir4 + 1
          else
            react (nreact,4) = ir3 + 1
            react (nreact,5) = ir4
          endif

          react (nreact,6) = ir5
          react (nreact,7) = ir6

          gamm(nreact)  = gamm(jj)
          alpha(nreact) = alpha(jj)
          bet(nreact)   = bet(jj)
          itype(nreact) = itype(jj)

C--- DEUXIEME REACTION ISOTOPEE :

          nreact = nreact + 1

          react (nreact,0) = ir0
          react (nreact,1) = ir1
          react (nreact,2) = ir2 + 1
          react (nreact,3) = ir3c

          if (ncp1 == 0) then
            react (nreact,4) = ir3
            react (nreact,5) = ir4 + 1
          else
            react (nreact,4) = ir3 + 1
            react (nreact,5) = ir4
          endif

          react (nreact,6) = ir5
          react (nreact,7) = ir6

          gamm(nreact)  = gamm(jj)
          alpha(nreact) = alpha(jj)
          bet(nreact)   = bet(jj)
          itype(nreact) = itype(jj)

        endif

C--- ON PASSE A l EXAMEN DE LA REACTION SUIVANTE (GO TO 500) :

        go to 500

      endif

C-----------------------------------------------------------------------------
C--- 2- DEUXIEME CAS : IL Y A DU C DANS UN SEUL REACTANT
C       ************                    *******
C
C---DEUX POSSIBILITES:
C
C    A- IL Y A DU C DANS LES DEUX PRODUITS ==> 2 REACTIONS ISOTOPEES
C    B- OU BIEN DANS UN SEUL               ==> 1 REACTION ISOTOPEE
c    c- Ou dans aucun                      ==> On passe
C-----------------------------------------------------------------------------

      if (ncr1 > 0 .or. ncr2 > 0) then

C-----------------------------------------------------------------------------
C
C                     LES REACTIONS s ECRIVENT ALORS :
C-----------------------------------------------------------------------------
C------------ A) PREMIERE POSSIBILITE : IL Y A DU C DANS LES DEUX PRODUITS :
C                                                        ********
C
C      AC  +  B   -->  A'C  +  B'C
C  OU  A   +  BC  -->  A'C  +  B'C
C
C                            =====>      AC*  +  B    -->  A'C*  +  B'C
C                                    OU  A    +  BC   -->  A'C*  +  B'C
C
C
C                        ET  =====>      AC*  +  B    -->  A'C   +  B'C*
C                                    OU  A    +  BC   -->  A'C   +  B'C*
C
C                          ==> 2 REACTIONS ISOTOPEES
C-----------------------------------------------------------------------------

        if (ncp1 > 0 .and. ncp2 > 0) then

C-----------------------------------------------------------------------------
C
C--- PREMIERE REACTION ISOTOPEE:

          nreact = nreact + 1

          react (nreact,0) = ir0
          if (ncr1 == 0) then
            react (nreact,1) = ir1
            react (nreact,2) = ir2 + 1
          else
            react (nreact,1) = ir1 + 1
            react (nreact,2) = ir2
          endif
          react (nreact,3) = ir3c

          react (nreact,4) = ir3 + 1
          react (nreact,5) = ir4
          react (nreact,6) = ir5
          react (nreact,7) = ir6

          gamm(nreact)  = gamm(jj)/2.0
          alpha(nreact) = alpha(jj)
          bet(nreact)   = bet(jj)
          itype(nreact) = itype(jj)

C--- DEUXIEME REACTION ISOTOPEE:

          nreact = nreact + 1

          react (nreact,0) = ir0
          if (ncr1 == 0) then
            react (nreact,1) = ir1
            react (nreact,2) = ir2 + 1
          else
            react (nreact,1) = ir1 + 1
            react (nreact,2) = ir2
          endif
          react (nreact,3) = ir3c

          react (nreact,4) = ir3
          react (nreact,5) = ir4 + 1
          react (nreact,6) = ir5
          react (nreact,7) = ir6

          gamm(nreact)  = gamm(jj)/2.0
          alpha(nreact) = alpha(jj)
          bet(nreact)   = bet(jj)
          itype(nreact) = itype(jj)

C-----------------------------------------------------------------------------
C------------ B) DEUXIEME POSSIBILITE : IL Y A DU C DANS UN SEUL PRODUIT :
C                                                        *******
C
C      AC  +  B   -->  A'C  +  B'
C  OU  A   +  BC  -->  A'C  +  B'
C  OU  AC  +  B   -->  A'   +  B'C
C  OU  A   +  BC  -->  A'   +  B'C
C
C                            =====>      AC*  +  B    -->  A'C*  +  B'
C                                    OU  A    +  BC*  -->  A'C*  +  B'
C                                    OU  AC*  +  B    -->  A'    +  B'C*
C                                    OU  A    +  BC*  -->  A'    +  B'C*
C
C                       ==> 1 REACTION ISOTOPEE
C-----------------------------------------------------------------------------
        else if (ncp1 > 0 .or. ncp2 > 0) then
C-----------------------------------------------------------------------------
C
C--- UNE SEULE REACTION ISOTOPEE :

          nreact = nreact + 1

          react (nreact,0) = ir0
          if (ncr1 == 0) then
            react (nreact,1) = ir1
            react (nreact,2) = ir2 + 1
          else
            react (nreact,1) = ir1 + 1
            react (nreact,2) = ir2
          endif
          react (nreact,3) = ir3c

          if (ncp1 == 0) then
            react (nreact,4) = ir3
            react (nreact,5) = ir4 + 1
          else
            react (nreact,4) = ir3 + 1
            react (nreact,5) = ir4
          endif

          react (nreact,6) = ir5
          react (nreact,7) = ir6

          gamm(nreact)  = gamm(jj)
          alpha(nreact) = alpha(jj)
          bet(nreact)   = bet(jj)
          itype(nreact) = itype(jj)

        endif
      endif

  500 continue

C----------------------------------------------------------------------------
C                      FIN DE l EXAMEN DES REACTIONS
C          ET DE LA CONSTRUCTION DE LA CHIMIE ISOTOPEE DU CARBONE
C----------------------------------------------------------------------------

C
      return
      end

C     *******************************
      subroutine ISOTO (nreact,idiff)
C     *******************************

      use constantes
      use auxiliaries
      USE STR_DATA
      USE SP_DATA
      USE CHEM_DATA

C *************************************************************************
C          CONSTRUCTION DE LA CHIMIE ISOTOPEE DE l OXYGENE
C *************************************************************************
C
C         NEQT   = NOMBRE TOTAL DE REACTIONS SANS ISOTOPE
C         IRC    = NOMBRE TOTAL DE REACTIONS SANS ISOTOPE DE l OXYGENE
C         NREACT = NUMERO DE LA REACTION ISOTOPEE
C         IDIFF  = NUMERO DE LA DERNIERE REACTION DIFFERENTIELLE
C         JJ     = NUMERO DE LA REACTION CORRESPONDANTE (SS ISOTOPE)
C
C *************************************************************************
C
C   CONVENTION :    DANS LA LISTE DES ESPECES, CHAQUE ESPECE ISOTOPEE DOIT
C   ----------      SUIVRE IMMEDIATEMENT l ESPECE MERE
C                   DANS LE CAS OU l ESPECE CONTIENT DE l OXYGENE ET DU
C                   CARBONE (C OU C*) ==> l ESPECE ISOTOPEE: O* SUIT C*
C                   EX :    HCO2
C                           HC*O2
C                           HCO*O
C                           HC*O*O
C
C *************************************************************************

       implicit double precision (a-h,o-z)

      irc = nreact

      do 500 jj = idiff+1,irc

c 23 oct 2006 - JLB - skip direct photodestruction

        if (itype(jj) == 7) cycle

c  reactants et produits

        ir0 = react(jj,0)
        ir1 = react(jj,1)
        ir2 = react(jj,2)
        ir3c = react(jj,3)
        ir3 = react(jj,4)
        ir4 = react(jj,5)
        ir5 = react(jj,6)
        ir6 = react(jj,7)

c-----------------------------------------------------------------------------
C--- IL N'Y A PAS d OXYGENE DANS LES PHOTONS OU LES CRP
C    ou les photons secondaires
C            (POUR LESQUELS AEL N'EST PAS DEFINI)

        if (ir2 == nspec+1 .or. ir2 == nspec+2
     1       .or. ir2 == nspec+3) then
          nor2=0
          go to 550
        endif

C-----------------------------------------------------------------------------
C---Y-A-t IL DE l OXYGENE DANS LA REACTION CONSIDEREE ?
C                                          ET COMBIEN ?
C
C          NOR1, NOR2 = NOMBRE DE O DANS LES REACTANTS R1 ET R2
C          NOP1, NOP2 = NOMBRE DE O DANS LES REACTANTS P1 ET P2
C
C---Y-A-t IL un isotope carbone a sauter? si oui, is1234 = 1
C
C          NCR1, NCR2 = NOMBRE DE C (OU C*) DANS LES REACTANTS R1 ET R2
C          NCP1, NCP2 = NOMBRE DE C (OU C*) DANS LES REACTANTS P1 ET P2
C-----------------------------------------------------------------------------

        if (ael(ir2,1) == ael(ir2+1,1)   .and.
     1      ael(ir2,2) == ael(ir2+1,2)   .and.
     1      ael(ir2,4) == ael(ir2+1,4)+1 .and.
     1      ael(ir2,9) == ael(ir2+1,9)-1) then
          nor2 = ael(ir2,4)
          is2 = 0
        else if (ael(ir2,1) == ael(ir2+2,1) .and.
     1      ael(ir2,2) == ael(ir2+2,2)   .and.
     1      ael(ir2,4) == ael(ir2+2,4)+1 .and.
     1      ael(ir2,9) == ael(ir2+2,9)-1) then
          nor2 = ael(ir2,4)
          is2 = 1
        else
          nor2 = 0
        endif

  550   continue

        if (ael(ir1,1) == ael(ir1+1,1)   .and.
     1      ael(ir1,2) == ael(ir1+1,2)   .and.
     1      ael(ir1,4) == ael(ir1+1,4)+1 .and.
     1      ael(ir1,9) == ael(ir1+1,9)-1) then
          nor1 = ael(ir1,4)
          is1 = 0
        else if (ael(ir1,1) == ael(ir1+2,1) .and.
     1      ael(ir1,2) == ael(ir1+2,2)   .and.
     1      ael(ir1,4) == ael(ir1+2,4)+1 .and.
     1      ael(ir1,9) == ael(ir1+2,9)-1) then
          nor1 = ael(ir1,4)
          is1 = 1
        else 
          nor1 = 0
        endif

        if (ael(ir3,1) == ael(ir3+1,1)   .and.
     1      ael(ir3,2) == ael(ir3+1,2)   .and.
     1      ael(ir3,4) == ael(ir3+1,4)+1 .and.
     1      ael(ir3,9) == ael(ir3+1,9)-1) then
          nop1 = ael(ir3,4)
          is3 = 0
        else if (ael(ir3,1) == ael(ir3+2,1) .and.
     1      ael(ir3,2) == ael(ir3+2,2)   .and.
     1      ael(ir3,4) == ael(ir3+2,4)+1 .and.
     1      ael(ir3,9) == ael(ir3+2,9)-1) then
          nop1 = ael(ir3,4)
          is3 = 1
        else
          nop1 = 0
        endif

        if (ir4 /= 0) then
          if (ael(ir4,1) == ael(ir4+1,1)   .and.
     1        ael(ir4,2) == ael(ir4+1,2)   .and.
     1        ael(ir4,4) == ael(ir4+1,4)+1 .and.
     1        ael(ir4,9) == ael(ir4+1,9)-1) then
            nop2 = ael(ir4,4)
            is4 = 0
          else if (ael(ir4,1) == ael(ir4+2,1) .and.
     1        ael(ir4,2) == ael(ir4+2,2)   .and.
     1        ael(ir4,4) == ael(ir4+2,4)+1 .and.
     1        ael(ir4,9) == ael(ir4+2,9)-1) then
            nop2 = ael(ir4,4)
            is4 = 1
          else
            nop2 = 0
          endif
        else
          nop2 = 0
        endif

C--- IL N'Y A PAS d OXYGENE DANS LES PHOTONS OU LES CRP
C    ou les photons secondaires
C           (POUR LESQUELS AEL N'EST PAS DEFINI)

        if (ir4 == nspec+1 .or.
     1      ir4 == nspec+2 .or. ir4 == nspec+3) then
          nop2 = 0
        endif

C-----------------------------------------------------------------------------
C--- 1) PREMIER CAS : IL Y A DE l O DANS LES DEUX REACTANTS
C       ***********                          ****
C
C--- trois POSSIBILITES:
C
C    A) IL Y A DE l O DANS LES DEUX PRODUITS ==> 2 REACTIONS ISOTOPEES
C    B) OU BIEN DANS UN SEUL                 ==> 2 REACTIONS ISOTOPEES
c    c) ou bien dans aucun                   ==> on passe
C
C    ==> PAS DE RAPPORTS DE BRANCHEMENT
C
C-----------------------------------------------------------------------------

        if (nor1 > 0 .and. nor2 > 0) then

C-----------------------------------------------------------------------------
C------------ A) PREMIERE POSSIBILITE : IL Y A DE l O DANS LES DEUX PRODUITS :
C                                                          ********
C
C  CONVENTION : LES REACTIONS DOIVENT ETRE ECRITE DANS l ORDRE (????) :
C  ----------
C
C      AO   +  BO   -->  A'O   +  B'O
C
C                   TEL QUE  =====>      AO*  +  BO   -->  A'O   +  B'O*
C                        ET  =====>      AO   +  BO*  -->  A'O*  +  B'O
C
C                          ==> 2 REACTIONS ISOTOPEES
C-----------------------------------------------------------------------------

          if (nop1 > 0 .and. nop2 > 0) then

C-----------------------------------------------------------------------------
C
C--- PREMIERE REACTION ISOTOPEE:

            nreact = nreact + 1

            react (nreact,0) = ir0
            react (nreact,1) = ir1 + 1 + is1
            react (nreact,2) = ir2
            react (nreact,3) = ir3c
            react (nreact,4) = ir3
            react (nreact,5) = ir4 + 1 + is4
            react (nreact,6) = ir5
            react (nreact,7) = ir6

            gamm(nreact)  = gamm(jj)
            alpha(nreact) = alpha(jj)
            bet(nreact)   = bet(jj)
            itype(nreact) = itype(jj)

C--- DEUXIEME REACTION ISOTOPEE:

            nreact = nreact + 1

            react (nreact,0) = ir0
            react (nreact,1) = ir1
            react (nreact,2) = ir2 + 1 + is2
            react (nreact,3) = ir3c
            react (nreact,4) = ir3 + 1 + is3
            react (nreact,5) = ir4
            react (nreact,6) = ir5
            react (nreact,7) = ir6

            gamm(nreact)  = gamm(jj)
            alpha(nreact) = alpha(jj)
            bet(nreact)   = bet(jj)
            itype(nreact) = itype(jj)

C-----------------------------------------------------------------------------
C------------ B) DEUXIEME POSSIBILITE : IL Y A DE l O DANS UN SEUL PRODUIT :
C                                                          *******
C
C      AO  +  BO  -->  A'O  +  B'
C  OU  AO  +  BO  -->  A'   +  B'O
C
C                            =====>      AO*  +  BO   -->  A'O*  +  B'
C                                    OU  AO*  +  BO   -->  A'    +  B'O*
C
C                        ET  =====>      AO   +  BO*  -->  A'O*  +  B
C                                    OU  AO   +  BO*  -->  A'    +  B'O*
C
C                       ==> 2 REACTIONS ISOTOPEES
C-----------------------------------------------------------------------------

          else if (nop1 > 0 .or. nop2 > 0) then

C-----------------------------------------------------------------------------
C
C--- PREMIERE REACTION ISOTOPEE :

            nreact = nreact + 1

            react (nreact,0) = ir0
            react (nreact,1) = ir1 + 1 + is1
            react (nreact,2) = ir2
            react (nreact,3) = ir3c

            if (nop1 == 0) then
              react (nreact,4) = ir3
              react (nreact,5) = ir4 + 1 + is4
            else
              react (nreact,4) = ir3 + 1 + is3
              react (nreact,5) = ir4
            endif

            react (nreact,6) = ir5
            react (nreact,7) = ir6

            gamm(nreact)  = gamm(jj)
            alpha(nreact) = alpha(jj)
            bet(nreact)   = bet(jj)
            itype(nreact) = itype(jj)

C--- DEUXIEME REACTION ISOTOPEE :

            nreact = nreact + 1

            react (nreact,0) = ir0
            react (nreact,1) = ir1
            react (nreact,2) = ir2 + 1 + is2
            react (nreact,3) = ir3c

            if (nop1==0) then
              react (nreact,4) = ir3
              react (nreact,5) = ir4 + 1 + is4
            else
              react (nreact,4) = ir3 + 1 + is3
              react (nreact,5) = ir4
            endif

            react (nreact,6) = ir5
            react (nreact,7) = ir6

            gamm(nreact)  = gamm(jj)
            alpha(nreact) = alpha(jj)
            bet(nreact)   = bet(jj)
            itype(nreact) = itype(jj)

          endif

C--- ON PASSE A l EXAMEN DE LA REACTION SUIVANTE (GO TO 500) :

          go to 500

        endif

C-----------------------------------------------------------------------------
C--- 2- DEUXIEME CAS : IL Y A DE l O DANS UN SEUL REACTANT
C       ************                      *******
C
C---DEUX POSSIBILITES:
C
C    A- IL Y A DE l O DANS LES DEUX PRODUITS ==> 2 REACTIONS ISOTOPEES
C    B- OU BIEN DANS UN SEUL                 ==> 1 REACTION ISOTOPEE
C-----------------------------------------------------------------------------

        if (nor1 > 0 .or. nor2 > 0) then

C-----------------------------------------------------------------------------
C
C                     LES REACTIONS s ECRIVENT ALORS :
C-----------------------------------------------------------------------------
C------------ A) PREMIERE POSSIBILITE : IL Y A DE l O DANS LES DEUX PRODUITS :
C                                                          ********
C
C      AO  +  B   -->  A'O  +  B'O
C  OU  A   +  BO  -->  A'O  +  B'O
C
C                            =====>      AO*  +  B    -->  A'O*  +  B'O
C                                    OU  A    +  BO   -->  A'O*  +  B'O
C
C
C                        ET  =====>      AO*  +  B    -->  A'O   +  B'O*
C                                    OU  A    +  BO   -->  A'O   +  B'O*
C
C                          ==> 2 REACTIONS ISOTOPEES
C-----------------------------------------------------------------------------

          if (nop1 > 0 .and. nop2 > 0) then

C-----------------------------------------------------------------------------
C
C--- PREMIERE REACTION ISOTOPEE:

            nreact = nreact + 1

            react (nreact,0) = ir0
            if (nor1==0) then
              react (nreact,1) = ir1
              react (nreact,2) = ir2 + 1 + is2
            else
              react (nreact,1) = ir1 + 1 + is1
              react (nreact,2) = ir2
            endif
            react (nreact,3) = ir3c

            react (nreact,4) = ir3 + 1 + is3
            react (nreact,5) = ir4
            react (nreact,6) = ir5
            react (nreact,7) = ir6

            gamm(nreact)  = gamm(jj)/2.0
            alpha(nreact) = alpha(jj)
            bet(nreact)   = bet(jj)
            itype(nreact) = itype(jj)

C--- DEUXIEME REACTION ISOTOPEE:

            nreact = nreact + 1

            react (nreact,0) = ir0
            if (nor1==0) then
              react (nreact,1) = ir1
              react (nreact,2) = ir2 + 1 + is2
            else
              react (nreact,1) = ir1 + 1 + is1
              react (nreact,2) = ir2
            endif
            react (nreact,3) = ir3c

            react (nreact,4) = ir3
            react (nreact,5) = ir4 + 1 + is4
            react (nreact,6) = ir5
            react (nreact,7) = ir6

            gamm(nreact)  = gamm(jj)/2.0
            alpha(nreact) = alpha(jj)
            bet(nreact)   = bet(jj)
            itype(nreact) = itype(jj)

C-----------------------------------------------------------------------------
C------------ B) DEUXIEME POSSIBILITE : IL Y A DE l O DANS UN SEUL PRODUIT :
C                                                          *******
C
C      AO  +  B   -->  A'O  +  B'
C  OU  A   +  BO  -->  A'O  +  B'
C  OU  AO  +  B   -->  A'   +  B'O
C  OU  A   +  BO  -->  A'   +  B'O
C
C                            =====>      AO*  +  B    -->  A'O*  +  B'
C                                    OU  A    +  BO*  -->  A'O*  +  B'
C                                    OU  AO*  +  B    -->  A'    +  B'O*
C                                    OU  A    +  BO*  -->  A'    +  B'O*
C
C                       ==> 1 REACTION ISOTOPEE
C-----------------------------------------------------------------------------

          else if (nop1 > 0 .or. nop2 > 0) then

C-----------------------------------------------------------------------------
C
C--- UNE SEULE REACTION ISOTOPEE :

            nreact = nreact + 1

            react (nreact,0) = ir0
            if (nor1==0) then
              react (nreact,1) = ir1
              react (nreact,2) = ir2 + 1 + is2
            else
              react (nreact,1) = ir1 + 1 + is1
              react (nreact,2) = ir2
            endif
            react (nreact,3) = ir3c

            if (nop1==0) then
              react (nreact,4) = ir3
              react (nreact,5) = ir4 + 1 + is4
            else
              react (nreact,4) = ir3 + 1 + is3
              react (nreact,5) = ir4
            endif

            react (nreact,6) = ir5
            react (nreact,7) = ir6

            gamm(nreact)  = gamm(jj)
            alpha(nreact) = alpha(jj)
            bet(nreact)   = bet(jj)
            itype(nreact) = itype(jj)

          endif
        endif
  500 continue

C----------------------------------------------------------------------------
C                     FIN DE l EXAMEN DES REACTIONS
C          ET DE LA CONSTRUCTION DE LA CHIMIE ISOTOPEE DE l OXYGENE
C----------------------------------------------------------------------------

C
      return
      end
