c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%
       subroutine LECTUR
c      %%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       use inoutput
       USE H2_DATA
       USE HD_DATA
       USE SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL
       USE RF_TOOLS

c---------------------------------------------------------------------
c  Read chemical species, initial abundances and chemical reactions
c---------------------------------------------------------------------

       implicit double precision (a-h,o-z)

       character (LEN=8) ref
       character (LEN=8) r1, r2, r3, p1, p2, p3, p4

       real (KIND=dp)                  :: msp(natom+6)
c       real (KIND=dp), dimension(nqm)  :: bilanm
c       real (KIND=dp), dimension(nqm)  :: bilanc
c       real (KIND=dp), dimension(0:np) :: charge

       REAL (kind=dp)           :: sreact
       REAL (kind=dp)           :: creact
       REAL (kind=dp)           :: sprod
       REAL (kind=dp)           :: cprod
       integer, dimension(2,21) :: bel


c              set up molecular weight of elements in amu

c If 10 : Li and 19 : k
c       data msp / 1.0d0, 12.0d0, 14.0d0, 16.0d0, 4.0d0, 2.0d0,
c    1             13.0d0, 15.0d0, 18.0d0, 6.9d0, 19.0d0, 23.0d0,
c    2             24.3d0, 27.0d0, 28.1d0, 31.0d0, 32.0d0, 35.5d0,
c    3             39.1d0, 55.8d0, 0.0d0, 0.0d0, 0.0d0 , 0.0d0,
c    4             0.0d0, 0.0d0, 0.0d0 /
c If 10 : PAH and 19 : ca
        data msp / 1.0d0, 12.0d0, 14.0d0, 16.0d0, 4.0d0, 2.0d0,
     1             13.0d0, 15.0d0, 18.0d0, 666.0d0, 19.0d0, 23.0d0,
     2             24.3d0, 27.0d0, 28.1d0, 31.0d0, 32.0d0, 35.5d0,
     3             40.08d0, 55.8d0, 0.0d0, 0.0d0, 0.0d0 , 0.0d0,
     4             0.0d0, 0.0d0, 0.0d0 /

c*********************************************************************

c              Relationship between index and element

c        1  :  h
c        2  :  c
c        3  :  n
c        4  :  o
c        5  :  he
c        6  :  d
c        7  :  c13 (=c*)
c        8  :  n15 (=n*)
c        9  :  o18 (=o*)
c       10  :  li or PAH
c       11  :  f
c       12  :  na
c       13  :  mg
c       14  :  al
c       15  :  si
c       16  :  p
c       17  :  s
c       18  :  cl
c       19  :  k ou ca
c       20  :  fe
c       21  :  charge

c      print *, iread1, iread2, iwrit1, iwrit2, iscrn

       ael(0,:) = 0
       xkmat     = 0.0_dp
       mol(0)    = 0.0_dp
       enth(0)   = 0.0_dp
       charge(0) = 0.0_dp

c   read species file and set up conservation array

       i = 1
       nn = 0
       nng = 0
       ni = 0
       nim = 0
       speci(0) = "        "

       ! Open chemistry file
       fichier = TRIM(data_dir)//chimie
       print *, "Opening: ", fichier
       open (iread1, file = fichier, status = 'old')

       read (iread1,"(/)")

       !---------------------------------------------------------------
       ! LOOP on the list of chemical species
       !---------------------------------------------------------------
   40  continue
       read (iread1,1010) l, speci(i), (ael(i,j), j=1,21),
     1            abin(i), enth(i)

c Help first iteration : 17/1/2003
       if (abin(i) == 0.0_dp) then
          abin(i) = 1.0e-20_dp
       endif

c  Identify index of all species which may be needed elsewhere

        if (speci(i) == 'h'   .or. speci(i) == 'H'   )   i_h = i
        if (speci(i) == 'd'   .or. speci(i) == 'D'   )   i_d = i
        if (speci(i) == 'h:'  .or. speci(i) == 'H:'  ) i_hgr = i
        if (speci(i) == 'd:'  .or. speci(i) == 'D:'  ) i_dgr = i
        if (speci(i) == 'h2'  .or. speci(i) == 'H2'  )  i_h2 = i
        if (speci(i) == 'hd'  .or. speci(i) == 'HD'  )  i_hd = i
        if (speci(i) == 'h2:' .or. speci(i) == 'H2:' )i_h2gr = i
        if (speci(i) == 'he'  .or. speci(i) == 'HE'  )  i_he = i
        if (speci(i) == 'c'   .or. speci(i) == 'C'   )   i_c = i
        if (speci(i) == 'n'   .or. speci(i) == 'N'   )   i_n = i
        if (speci(i) == 'o'   .or. speci(i) == 'O'   )   i_o = i
        if (speci(i) == 's'   .or. speci(i) == 'S'   )   i_s = i
        if (speci(i) == 'si'  .or. speci(i) == 'Si'  )  i_si = i
        if (speci(i) == 'c*'  .or. speci(i) == 'C*'  ) i_c13 = i
        if (speci(i) == 'o*'  .or. speci(i) == 'O*'  ) i_o18 = i
        if (speci(i) == 'h2o' .or. speci(i) == 'H2O' ) i_h2o = i
        if (speci(i) == 'oh'  .or. speci(i) == 'OH'  )  i_oh = i
        if (speci(i) == 'c2'  .or. speci(i) == 'C2'  )  i_c2 = i
        if (speci(i) == 'co'  .or. speci(i) == 'CO'  ) i_co0 = i
        if (speci(i) == 'c*o' .or. speci(i) == 'C*O' ) i_co1 = i
        if (speci(i) == 'co*' .or. speci(i) == 'CO*' ) i_co2 = i
        if (speci(i) == 'c*o*'.or. speci(i) == 'C*O*') i_co3 = i
        if (speci(i) == 'cs'  .or. speci(i) == 'CS'  )  i_cs = i

c  Positive ions

        if (ael(i,natom) >=  1 ) goto 60

c  Negative ions

        if (ael(i,natom) <= -1 ) goto 80

c  End of species list

        if (l == 888) goto 100

c  Compute molecular weight of all chemical species in amu
c  and number of species in neutral and ions fluids

c              neutrals

c  Adsorbed H2 is a special case:
c     H2: = Sum of  Si fi ds**2
c     with: Si = surface of grains of size i
c           fi = fraction occupied by H2 (between 0 et 1)
c           ds = dsite
c     The sum is from 1 to npg (i.e., same sampling as Gaussian integration)

          if (i_h2gr == i) then
            ii = i
            do mm=1,npg
              speci(i) = speci(ii)
              ael(i,:) = ael(ii,:)
              abin(i) = abin(ii)
              enth(i) = enth(ii)
              mol(i) = 2.0_dp
              charge(i) = 0.0_dp
              nng = nng + 1
              i = i + 1
            end do
            abin(ii:ii+npg-1) = abin(ii:ii+npg-1) / wwmg
          else

            la_sum = 0.0_dp
            do j=1,natom-1
               la_sum = la_sum + ael(i,j) * msp(j)
            end do
            mol(i) = la_sum
            charge(i) = ael(i,21)

            if (ni == 0) then
              nn = i
            else
              nng = nng + 1
            endif

            i = i + 1

          endif
          goto 40

c              positive ions

   60     continue
          la_sum = 0.0_dp
          do j=1,natom-1
            la_sum = la_sum + ael(i,j) * msp(j)
          end do

          if (speci(i) == 'h+'   .or. speci(i) == 'H+')     i_hp = i
          if (speci(i) == 'h3+'  .or. speci(i) == 'H3+')   i_h3p = i
          if (speci(i) == 'h2d+' .or. speci(i) == 'H2D+') i_h2dp = i
          if (speci(i) == 'c+'   .or. speci(i) == 'C+' )    i_cp = i
          if (speci(i) == 'c*+'  .or. speci(i) == 'C*+' ) i_c13p = i
          if (speci(i) == 'n+'   .or. speci(i) == 'N+')     i_np = i
          if (speci(i) == 'o+'   .or. speci(i) == 'O+')     i_op = i
          if (speci(i) == 's+'   .or. speci(i) == 'S+')     i_sp = i
          if (speci(i) == 'si+'  .or. speci(i) == 'SI+')   i_sip = i
          if (speci(i) == 'fe+'  .or. speci(i) == 'FE+')   i_fep = i
          if (speci(i) == 'hco+' .or. speci(i) == 'HCO+') i_hcop = i
          mol(i) = la_sum
          charge(i) = ael(i,21)
          ni = i - nn
          i = i + 1
          goto 40

c           negative ions

   80     continue
          la_sum = 0.0_dp
          do j=1,natom-1
            la_sum = la_sum + ael(i,j) * msp(j)
          end do
          mol(i) = la_sum
          charge(i) = ael(i,21)
          nim = i - nn - ni
          i = i + 1
          goto 40

  100     nspec = i - 1

c              positive ions

          la_sumn = 0.0_dp
          do i=nn+1,nn+ni
            la_sumn = la_sumn + abin(i)
          end do
          xni0 = la_sumn

c              negative ions

          la_sumn = 0.0_dp
          do i=nn+ni+1,nspec-nng
            la_sumn = la_sumn + abin(i)
          end do
          xnim0 = la_sumn

c     add electrons (sp(21)), photons (sp(22)), crp (sp(23)), phosec (sp(24))
c         grains (sp(25)), 2h (sp(26))
          do i=1,6
            ii = nspec + i
            jj = i + natom - 1
            speci(ii) = sp(jj)
            mol(ii) = msp(jj)
          end do
          enth(nspec+1) = 0.0_dp
          enth(nspec+2) = 13.0_dp / calev
          enth(nspec+3) = 20.0_dp / calev
          enth(nspec+4) = 13.0_dp / calev
          enth(nspec+5) = 0.0_dp
          enth(nspec+6) = 0.0_dp
          charge(nspec+1) = -1.0_dp
          charge(nspec+2) = 0.0_dp
          charge(nspec+3) = 0.0_dp
          charge(nspec+4) = 0.0_dp
          charge(nspec+5) = 0.0_dp

c n2 : nbre of (chimical species + electron) + photon + crp + phosec + grains + 2h
c      electrons are counted as a species and added to nspec
          nspec = nspec + 1

c      inital abondance of electrons.
          if (xni0 < xnim0) write (iscrn,9001) xni0, xnim0
          abin(nspec) = xni0 - xnim0
          lcon(nspec) = natom

c       total abondances of each atom

          abtot = 0.0_dp
          do i=1,nspec-1
            do l=1,natom-1
              abtot(l) = abtot(l) + ael(i,l) * abin(i)
            end do
          end do

          write (iwrit2,2000)
          do l=1,nspec
            write (iwrit2,2010) l, speci(l), mol(l), abin(l),
     1             enth(l)
          end do

c  tm1h(i) is the inverse of the time needed by species i to do one "HOP"
      tm1h = 0.0_dp
      if (i_hgr /= 0) then
        tm1h(i_hgr) = 5.0d10
      endif
      if (i_dgr /= 0) then
        tm1h(i_dgr) = 1.0d10
      endif

          write (iwrit2,2020) nn, ni, nim, nng, nspec


          CALL POST_LECTUR

c************************************************************************
c       Read chemical reactions with reordering
c************************************************************************

          PRINT *,"Read now chemical reactions"
          write (iwrit2,2050)
          i = nphot_i + 1
  200     do j=0,7
            react(i,j) = 999
          end do

          !- READ A CHEMICAL REACTION ----------------------------------------
          READ (iread1,"(A5)",ADVANCE="NO")  ref
               IF (ref == "99999") GOTO 300    ! end of chemistry
          READ(iread1,"(1x,A8)",ADVANCE="NO") r1
               IF (r1 == "        ") THEN      ! commentary line in chemistry
                  READ(iread1,*)               ! go to begining of next line
                  GOTO 200                     ! then read next line
               ENDIF
          READ(iread1,1020) r2, p1, p2, p3, p4 ! read the rest of the reaction
     1                     ,gamm(i), alpha(i), bet(i), itype(i)
c          PRINT *,i,ref,r1,r2,p1,p2,p3,p4,gamm(i), alpha(i), bet(i)
 1020  format(4a8,a7,d8.2,1x,f5.2,1x,f8.1,i4)

          ! itype = 5 : photodestruction
          if (itype(i) == 5) then
            do k=1,nphot_i-i_mandat
              if ((r1 == phdest(k)%r1 ) .AND.
     1            (p1 == phdest(k)%p1 ) .AND.
     2            (p2 == phdest(k)%p2 ) .AND.
     3            (p3 == phdest(k)%p3 ) .AND.
     4            (p4 == phdest(k)%p4 ) ) then
                goto 200
              endif
            enddo
          endif

c  Used for Pepe's chemistry to avoid duplicate reactions
c    (provided by Evelyne)
          if (ref == 'deja ')  then
            go to 200
          endif
c         if (bet(i) < 0.0_dp)  then
c           bet(i) = 0.0_dp
c         endif
c end modif

c  WARNING: speci begins at 0 !!!

          do j=0,nspec+5
            if (r1 == speci(j)) then
              react(i,1) = j
              exit
            endif
          end do
          do j=0,nspec+4
            if (r2 == speci(j)) then
              react(i,2) = j
              react(i,3) = 0
              exit
            endif
          end do
          if (r2 == speci(nspec+5)) then
            react(i,2) = i_h
            react(i,3) = i_h
          endif
          do j=0,nspec+5
            if (p1 == speci(j)) then
              react(i,4) = j
              exit
            endif
          end do
          do j=0,nspec+5
            if (p2 == speci(j)) then
              react(i,5) = j
              exit
            endif
          end do
          do j=0,nspec+5
            if (p3 == speci(j)) then
              react(i,6) = j
              exit
            endif
          end do
          do j=0,nspec+5
            if (p4 == speci(j)) then
              react(i,7) = j
              exit
            endif
          end do

          if (r2 == sp(natom+6))  react(i,2) = 0
          if (p2 == sp(natom+6))  react(i,5) = 0
          if (p3 == sp(natom+6))  react(i,6) = 0
          if (p4 == sp(natom+6))  react(i,7) = 0

c       Look for incomplete reactions

          do j=1,7
            if (react(i,j) == 999)  then
              react(i,j) = 0
              write (iwrit2,2060) i, (speci(react(i,jj)), jj=1,7)
              write (iscrn,2060) i, (speci(react(i,jj)), jj=1,7)
              goto 200
            endif
          end do

          if(react(i,5) == 0) then
            react(i,0) = 3
          else if(react(i,6) == 0) then
            react(i,0) = 4
          else if(react(i,7) == 0) then
            react(i,0) = 5
          else
            react(i,0) = 6
          endif

c  Compute sum of mass of products of each reaction
c  and net energy balance (in eV)
c  convention: positive ==> exothermique
c              negative ==> endothermique
c              ******** ==> donnees insuffisantes


      bel = 0
      do j=1,20
        bel(1,j) = ael(react(i,1),j) + ael(react(i,2),j)
     1           + ael(react(i,3),j)
        bel(2,j) = ael(react(i,4),j) + ael(react(i,5),j)
     1           + ael(react(i,6),j) + ael(react(i,7),j)
        if (bel(1,j) /= bel(2,j)) then
          print *, speci(react(i,1)), speci(react(i,2))
     1           , speci(react(i,3)), speci(react(i,4))
     2           , speci(react(i,5)), speci(react(i,6))
     3           , speci(react(i,7))
c         stop
        endif
      end do

      !--- Check wrong reactions --------------------------------------
      sreact = 0.0_dp
      creact = 0.0_dp
      sprod  = 0.0_dp
      cprod  = 0.0_dp
      sreact = sreact + mol(react(i,1)) + mol(react(i,2))
     1                + mol(react(i,3))
      creact = creact + charge(react(i,1)) + charge(react(i,2))
     1                + charge(react(i,3))
      sprod = sprod   + mol(react(i,4)) + mol(react(i,5))
     1                + mol(react(i,6)) + mol(react(i,7))
      cprod = cprod   + charge(react(i,4)) + charge(react(i,5))
     1                + charge(react(i,6)) + charge(react(i,7))

      if (react(i,5) /= nspec+1) then
        dreact = enth(react(i,1)) + enth(react(i,2))
     1         + enth(react(i,3))
        ddprod = 0.0_dp
        kj = react(i,0)
        do j=4,kj+1
          iprod = react(i,j)
          if (iprod /= nspec+1) then
            ddprod = ddprod + enth(iprod)
          endif
        end do
        de(i) = dreact - ddprod
        de(i) = de(i) * calev
      else
        de(i) = 0.0_dp
      endif
      bilanm(i) = sreact - sprod
      bilanc(i) = creact - cprod
      if (dabs(bilanm(i)) > 0.0_dp) then
        print *, i, (speci(react(i,j)),j=1,7)
        print *, i, "Mass - React:", sreact,
     1                     " Prod:", sprod, "Diff:", bilanm(i)
          print *,"Problem in mass conservation"
          stop
      endif
      if (      (speci(react(i,1)) /= "grain ")
     1    .AND. (speci(react(i,2)) /= "grain ")
     2    .AND. (speci(react(i,3)) /= "grain ") ) then
        if (dabs(bilanc(i)) . gt . 0.0_dp) then
          print *, i, (speci(react(i,j)),j=1,7)
          print *, i, "Charge - React:", creact,
     1                         " Prod:", cprod, " Diff:", bilanc(i)
          print *,"Problem in charge conservation"
          print *, react(i,1), charge(react(i,1))
          stop
        endif
      endif

c    ***********************
c----write chemical reaction
c    ***********************

          i = i + 1

c  Add HERE the "pseudo-reaction" used to limit H2 adsorption on grains
c  (not more than a single layer)
c  and multiplicate reactions invloving grain size

      if (itype(i-1) == 113) then
        gamm(i) = gamm(i-1)
        alpha(i) = alpha(i-1)
        bet(i) = bet(i-1)
        itype(i) = itype(i-1) + 10
        react(i,0) = react(i-1,0)
        react(i,1) = react(i-1,1)
        react(i,2) = i_h2gr
        react(i,3) = 0
        react(i,4) = i_h2
        react(i,5) = react(i-1,5)
        react(i,6) = react(i-1,6)
        react(i,7) = react(i-1,7)
        i = i + 1
        do mm=1,npg-1
          gamm(i) = gamm(i-2)
          alpha(i) = alpha(i-2)
          bet(i) = bet(i-2)
          itype(i) = itype(i-2)
          react(i,0) = react(i-2,0)
          react(i,1) = react(i-2,1)
          react(i,2) = react(i-2,2)
          react(i,3) = 0
          react(i,4) = react(i-2,4) + 1
          react(i,5) = react(i-2,5)
          react(i,6) = react(i-2,6)
          react(i,7) = react(i-2,7)
          i = i + 1

          gamm(i) = gamm(i-2)
          alpha(i) = alpha(i-2)
          bet(i) = bet(i-2)
          itype(i) = itype(i-2)
          react(i,0) = react(i-2,0)
          react(i,1) = react(i-2,1)
          react(i,2) = react(i-2,2) + 1
          react(i,3) = 0
          react(i,4) = react(i-2,4)
          react(i,5) = react(i-2,5)
          react(i,6) = react(i-2,6)
          react(i,7) = react(i-2,7)
          i = i + 1
        end do
      else if (itype(i-1) == 117) then
        do mm=1,npg-1
          gamm(i) = gamm(i-1)
          alpha(i) = alpha(i-1)
          bet(i) = bet(i-1)
          itype(i) = itype(i-1)
          react(i,0) = react(i-1,0)
          react(i,1) = react(i-1,1) + 1
          react(i,2) = react(i-1,2)
          react(i,3) = 0
          react(i,4) = react(i-1,4)
          react(i,5) = react(i-1,5)
          react(i,6) = react(i-1,6)
          react(i,7) = react(i-1,7)
          i = i + 1
        end do
      else if (itype(i-1) == 118) then
        do mm=1,npg-1
          gamm(i) = gamm(i-1)
          alpha(i) = alpha(i-1)
          bet(i) = bet(i-1)
          itype(i) = itype(i-1)
          react(i,0) = react(i-1,0)
          react(i,1) = react(i-1,1) + 1
          react(i,2) = react(i-1,2)
          react(i,3) = 0
          react(i,4) = react(i-1,4)
          react(i,5) = react(i-1,5)
          react(i,6) = react(i-1,6)
          react(i,7) = react(i-1,7)
          i = i + 1
        end do
      end if

          goto 200

  300     continue
          neqt = i - 1

      nreact = neqt

c *************************************************************************
c                 Build here isotopic reactions
c *************************************************************************

c   convention :    In the species list, each isotpe must follow 
c   ----------      immediately the main species    
c                   If the specy contains C and O, O* must follow C*
c                   ex :    hco2
c                           hc*o2
c                           hco*o
c                           hc*o*o

c *************************************************************************

c  Are there isotopes in the species list (C* then O*) ?

c-------------------------------------------------------------------------
c  Test variable: i_c13 > 0  ==>  There is 13C in the chemistry
c                            ==>  We need C*O excitation

c                 i_c13 = 0  ==>  No 13C

c                 i_o18 > 0  ==>  There is 18O in the chemistry
c                            ==>  We need CO* (and C*O*) excitation

c                 i_o18 = 0  ==>  No 18O
c-------------------------------------------------------------------------

c  Are there isotopes within neutral species (C* and O*) ?
c  NO ==> Reaction list is closed (I.e. go to 400)

      if ((i_c13 == 0).and.(i_o18 == 0))  goto 400

      if (i_c13 > 0) then
        call ISOTC (nreact, idiff)
      endif

      if (i_o18 > 0) then
        call ISOTO (nreact,idiff)
      endif

c----------------------------------------------------------------------------

c         neqt   = total number of reactions without isotopes (up to now)
c         nreact = Isotopic reaction number
c         idiff  = Number of the last differential reaction

c----------------------------------------------------------------------------
c  End of isotopic chemistry construction
c----------------------------------------------------------------------------

      write (iwrit2,2070)  i_c13, i_o18

      do 340 ii=neqt+1,nreact

c-----------------------------------------------------------------------------
c  Compute mass and energy balance for each reaction
c-----------------------------------------------------------------------------

      dreact = 0.0_dp
      ddprod = 0.0_dp
      do jj=1,3
        ireact = react(ii,jj)
        dreact = dreact + enth(ireact)
      end do
      kjj = react(ii,0) + 1
      do jj=4,kjj
        iprod = react(ii,jj)
        if (iprod /= nspec+1) then
          ddprod = ddprod + enth(iprod)
        endif
      end do
      de(ii) = dreact - ddprod
      de(ii) = de(ii) * calev

c-----------------------------------------------------------------------------
c  Write isotopic reactions
c-----------------------------------------------------------------------------

       r1 = speci (react(ii,1))
       r2 = speci (react(ii,2))
       r3 = speci (react(ii,3))
       p1 = speci (react(ii,4))
       p2 = speci (react(ii,5))
       p3 = speci (react(ii,6))
       p4 = speci (react(ii,7))

      ref = 'isotp   '
c-----------------------------------------------------------------------------

  340 continue

c  Total number of reactions (normal + isotopic)

  400 continue

      neqt = nreact

      write (iwrit2,2030) neqt


c  Reorder reactants in increasing species number
c  which leads to a triangular matrix

      do i=1,neqt
        if (react(i,1) > react(i,2) .and.
     1      react(i,2) /= 0)  then
          l = react(i,1)
          react(i,1) = react(i,2)
          react(i,2) = l
        endif
      end do

      close(iread1)

      call ORDON

c  Initialise ikmat: list of reactions for each species

      do i=0,nspec+5
        ikmat(i,0) = 0
      end do

c  WARNING: for pseudo-reaction 123, H2 must be taken into
c  account only ONCE: on the formation side

c JLB - 15 march 2004 - revoir le role de ikmat...

      do i=1,neqt
        ik = react(i,0)
        if (itype(i) == 123) then
          j0 = 2
        else
          j0 = 1
        endif
        do j=j0,ik+1
          iis = react(i,j)
          if (iis == 0) then
            cycle
          endif
          n = ikmat(iis,0) + 1
          ikmat(iis,n) = i
          ikmat(iis,0) = n
        end do
      end do

       write (iwrit2,2031) neqt

c*********************************************************************

c  H2 formation distribution on grains
c  O/P ratio is usually 3/4 ortho et 1/4 para, but can be changed
c  using variable ospg

c  6 XI 2003 : Let's try a different O/P on grain..

       ospg = 3.0_dp
c      ospg = 1.0_dp

       print *, '----------- de H2 and HD Formation ------------'

       if (iforh2 == 0) then

c  Excitation temperature is tuned so that the mean formation energy
c  is exactly 1.49 eV, whatever the number of H2 levels included

         eform = everg * de(ifh2gr) / (3.0_dp * xk)

         tform = eform
         tf_min = 0.0_dp
         tf_max = 2.0_dp * eform
         res = 1.0_dp

         do while ((abs(res) > 1.0e-3_dp) .AND.
     1             (abs(tform-tf_max) > 1.0e-3_dp))
           res = 0.0_dp
           do lev=1,l0h2
             ij = njlev(lev)
             if (mod(ij,2) == 0) then
               res = res + (2.0_dp*ij+1.0_dp) * (eform - elhm(lev))
     1             * exp(-elhm(lev)/tform)
             else
               res = res + (2.0_dp*ij+1.0_dp) * ospg
     1             * (eform - elhm(lev)) * exp(-elhm(lev)/tform)
             endif
           end do
           if (res < 0.0_dp) then
             tf_max = tform
             tform = 0.5_dp * (tf_min + tf_max)
           else
             tf_min = tform
             tform = 0.5_dp * (tf_min + tf_max)
           endif
         end do

         eform = tform

         write (iscrn,2101) de(ifh2gr), eform
         write (iwrit2,2101) de(ifh2gr), eform

         ech = 0.0_dp
         do lev=1,l0h2
           ij = njlev(lev)
           iv = nvlev(lev)
           if (mod(ij,2) == 0) then
             pophm(iv,ij) = (2.0_dp*ij+1.0_dp)
     1                    * exp(-elhm(lev)/eform)
           else
             pophm(iv,ij) = (2.0_dp*ij+1.0_dp) * ospg
     1                    * exp(-elhm(lev)/eform)
           endif
           ech = ech + pophm(iv,ij)
         end do

         do lev=1,l0h2
           ij = njlev(lev)
           iv = nvlev(lev)
           pophm(iv,ij) = pophm(iv,ij) / ech
         end do

         eformd = eform

         if (i_hd /= 0) then
           ech = 0.0_dp
           do ij = 0,mjhd
            do iv=0,nvxhd
              lev = levhd(iv,ij)
              if (lev == 0) cycle
              pophd(iv,ij) = (2.0_dp*ij+1.0_dp) * exp(-elhd(lev)/eformd)
              ech = ech + pophd(iv,ij)
            end do
           end do

           dummy = 0.0_dp
           do ij = 0,mjhd
            do iv = 0,nvxhd
              lev = levhd(iv,ij)
              if (lev == 0) cycle
              pophd(iv,ij) = pophd(iv,ij) / ech
            end do
           end do
         endif

       else if (iforh2 == 1) then

c  Use a Boltzmann factor of exactly de(ifh2gr)/3
c  Note that this does NOT lead to a mean energy of de(ifh2gr)/3

         eform = everg * de(ifh2gr) / (3.0_dp * xk)
         if (i_hd /= 0) then
           eformd = everg * de(ifhdgr) / (3.0_dp * xk)
         endif

c black & van dishoeck, formation phi = 1
c     apj, 322, 412 (1987), eform = 12098 cm-1

         write (iscrn,2101) de(ifh2gr), eform
         write (iwrit2,2101) de(ifh2gr), eform

         ech = 0.0_dp

         do lev=1,l0h2
           ij = njlev(lev)
           iv = nvlev(lev)
           if (mod(ij,2) == 0) then
             pophm(iv,ij) = (2.0_dp*ij+1.0_dp)
     1                    * exp(-elhm(lev)/eform)
           else
             pophm(iv,ij) = (2.0_dp*ij+1.0_dp) * 3.0_dp
     1                    * exp(-elhm(lev)/eform)
           endif
           ech = ech + pophm(iv,ij)
         end do

         do lev=1,l0h2
           ij = njlev(lev)
           iv = nvlev(lev)
           pophm(iv,ij) = pophm(iv,ij) / ech
         end do

c         print *, echp, echo, echp+echo

         if (i_hd /= 0) then
           ech = 0.0_dp
           do ij = 0,mjhd
            do iv=0,nvxhd
              lev = levhd(iv,ij)
              if (lev == 0) cycle
              pophd(iv,ij) = (2.0_dp*ij+1.0_dp) * exp(-elhd(lev)/eformd)
              ech = ech + pophd(iv,ij)
            end do
           end do

           dummy = 0.0_dp
           do ij = 0,mjhd
            do iv = 0,nvxhd
              lev = levhd(iv,ij)
              if (lev == 0) cycle
              pophd(iv,ij) = pophd(iv,ij) / ech
            end do
           end do
         endif

       else if (iforh2 == 2) then

         pophm(14,0) = 0.25_dp
         pophm(14,1) = 0.75_dp

         if (i_hd /= 0) then
           pophd(14,0) = 0.25_dp
           pophd(14,1) = 0.75_dp
         endif

       else if (iforh2 == 3) then

c  Rotational temperature of 65 K and high vibrational number (6 here)
c   Inspired by BvD (close to grain T?)

         pophm(6,0) = 0.245284557d+0
         pophm(6,1) = 0.749823809d+0
         pophm(6,2) = 0.471552089d-2
         pophm(6,3) = 0.176384812d-3
         pophm(6,4) = 0.247192595d-7
         pophm(6,5) = 0.265905215d-10
         pophm(6,6) = 0.132664312d-15
         pophm(6,7) = 0.629879296d-20

         if (i_hd /= 0) then
           pophd(6,0) = 0.245284557d+0
           pophd(6,1) = 0.749823809d+0
           pophd(6,2) = 0.471552089d-2
           pophd(6,3) = 0.176384812d-3
           pophd(6,4) = 0.247192595d-7
           pophd(6,5) = 0.265905215d-10
           pophd(6,6) = 0.132664312d-15
           pophd(6,7) = 0.629879296d-20
         endif

       else if (iforh2 == 4) then

c  Formation on a single vibrational level (ivf)
c  with prescribed rotational temperature (eform).
c  or precribed ratio (Christine's request)

c        ivf = 6
c        eform = 400.0_dp
c        ej0 = elhm(level(ivf,0))
         ech = 0.0_dp
         pophm = 0.0_dp

         pophm(6,1) = 0.18_dp
         pophm(6,2) = 0.45_dp
         pophm(6,3) = 0.10_dp

         do lev=1,l0h2
           ij = njlev(lev)
           iv = nvlev(lev)
c          if (iv /= ivf) cycle
c          if (mod(ij,2) == 0) then
c            pophm(iv,ij) = (2.0_dp*ij+1.0_dp)
c    1                    * exp(-(elhm(lev)-ej0)/eform)
c          else
c            pophm(iv,ij) = (2.0_dp*ij+1.0_dp) * ospg
c    1                    * exp(-(elhm(lev)-ej0)/eform)
c          endif
           ech = ech + pophm(iv,ij)
         end do

         do lev=1,l0h2
           ij = njlev(lev)
           iv = nvlev(lev)
           pophm(iv,ij) = pophm(iv,ij) / ech
           if (pophm(iv,ij) /= 0.0_dp) then
             print *, iv, ij, pophm(iv,ij)
           endif
         end do

       else

         write (iscrn,"(/,5x,' Bad value for iforh2')")
         stop

       endif

       dumy = 0.0_dp
       dummy = 0.0_dp
       do lev=1,l0h2
         ij = njlev(lev)
         iv = nvlev(lev)
c        print *, iv, ij, pophm(iv,ij), elhm(lev)
         dumy = dumy + pophm(iv,ij) * elhm(lev)
         dummy = dummy + pophm(iv,ij)
       end do
       emoyh2 = dumy

       do lev=l0h2+1,nlevh2
         ij = njlev(lev)
         iv = nvlev(lev)
c        print *, iv, ij, pophm(iv,ij), elhm(lev)
         dummy = dummy + pophm(iv,ij)
       end do

       write (iscrn,*) 'sum(pophm):', dummy
       write (iscrn,2102) eform, emoyh2
       write (iwrit2,2102) eform, emoyh2

       dumy = 0.0_dp
       dummy = 0.0_dp
       do lev=1,l0hd
         ij = njhd(lev)
         if (ij > mjhd) cycle
         iv = nvhd(lev)
c        print *, iv, ij, pophd(iv,ij), elhd(lev)
         dumy = dumy + pophd(iv,ij) * elhd(lev)
         dummy = dummy + pophd(iv,ij)
       end do
       emoyhd = dumy

       levhd1 = l0hd+1
       do lev=levhd1,nlevhd
         ij = njhd(lev)
         if (ij > mjhd) cycle
         iv = nvhd(lev)
c        print *, iv, ij, pophd(iv,ij), elhd(lev)
         dummy = dummy + pophd(iv,ij)
       end do

       write (iscrn,*) 'sum(pophd):', dummy
       write (iscrn,2104) eformd, emoyhd
       write (iwrit2,2104) eformd, emoyhd

c---------------------------------------------------------------------
 1010  format (i3,2x,a8,1x,2i2,3i1,1x,4i1,1x,5i1,1x,6i1,1x,i2
     1        ,1x,d10.3,f9.3,1x,a16)
 2000  format (1x,//,3x,'Included species :',/,
     1         23x,'weight   ab_init   enthalpy',/)
 2010  format (5x,i3,5x,a8,1x,f5.1,2x,1pe12.4,2x,0pf7.3,1x,a16)
 2020  format (//,3x,i3,' neutral species',
     1          /,3x,i3,' positively ionised species',
     2          /,3x,i3,' negatively ionised species',
     3          /,3x,i3,' species in grain mantles',
     4          /,3x,i3,' chemical species (with electrons)',/)
 2030  format (/,3x,i4,' chemical reactions (without photo-rxns)',/)
 2031  format (/,3x,i4,' chemical reactions (with photo-rxns)',//)
 2050  format (3x,'Chemical reactions',/,
     1         63x,'gamma',3x,'alpha',4x,'beta',3x,'deltae',/)
 2060  format (5x,'WARNING, error in reactions, line:',i4,2x,7a8)
 2070  format (//,10x,'Isotopic reactions',5x,'i_c13 =',i2,' i_o18 ='
     1         ,i2,/,10x,19('-'),/)
 2101  format (1x,'H2 formation enthalpie :',f10.4,' eV',
     1          /,1x,'including internal energy:'
     2           ,f12.2,' K')
 2102  format (1x,'eformh:',f12.2,' K  emoyh2:',f12.2,' K')
 2104  format (1x,'eformd:',f12.2,' K  emoyhd:',f12.2,' K')
 9001  format (5x,'attention, ions + :',1pd9.2,' ions - :',1pd9.2)
c---------------------------------------------------------------------

       return
       end

c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        subroutine ORDON

        use constantes
        use auxiliaries
        USE SP_DATA
        USE STR_DATA
        USE CHEM_DATA

        implicit double precision (a-h,o-z)

        character (LEN=8) r1, r2, r3, p1, p2, p3, p4

        dimension dee(nqm), lreact(nqm,0:7), ntype(nqm)
        dimension gammm(nqm), palph(nqm), bett(nqm)

c identify and re-order the chemical reactions:

c  WARNING : Type 6 reactions are treated BEFORE type 5
c            (this is because they result from a splitting of type 4)

c (0) formation of h2 and hd on grains [old style] (nh2for)
c (1) cosmic ray processes (ncrion)
c (2) secondary photon processes (npsion)
c (3) radiative association  (nasrad)
c (4) gas phase reactions (nrest)
c (6) Endothermal reactions with H2 (nh2endo) (NEW!)
c (5) photoreactions (external fit) (nphot_f)
c (7) photoreactions (direct integration) (nphot_i)
c (8) 3 body reactions
c (101...) exceptions (ncpart)

c (11) reactions on grain surface (ngsurf)
c (12) photoreactions on grain surface (ngphot)
c (13) adsorption on to grains (ngads)
c (14) neutralization (ngneut)
c (15) explosive desorption from grains (nggdes)
c (16) cosmic ray induced desorption from grains (ncrdes)
c (17) photodesorption from grains (nphdes)
c (18) evaporation from grains (ngevap)

c (111) to (118) same processes taking grain size distribution
c                into account

c -------------------------------------------------------
c the do loops are over all considered chemical reactions:
c -------------------------------------------------------

      nh2for = 0
      ncrion = 0
      npsion = 0
      nasrad = 0
      nrest = 0
      nh2endo = 0
      nphot_f = 0
      nphot_i = 0
      n3body = 0
      ncpart = 0
      ngsurf = 0
      ngphot = 0
      ngads = 0
      ngneut = 0
      nggdes = 0
      ncrdes = 0
      nphdes = 0
      ngevap = 0

      numero = 0

c (0) formation of h2 and hd on grains [old style] (nh2for)

      do i=1,neqt
        ity = itype(i)
        if (ity == 0) then
          nh2for = nh2for + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = de(i)
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (1) cosmic ray processes (ncrion)

      do i=1,neqt
        ity = itype(i)
        if (ity == 1) then
          ncrion = ncrion + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
c  Heating by cosmic rays ionisation is done in stv5c.f
          dee(numero) = 0.0_dp
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (2) secondary photon processes (npsion)

      do i=1,neqt
        ity = itype(i)
        if (ity == 2) then
          npsion = npsion + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = de(i)
          if (ael(react(i,1),10) == 1) then
            dee(numero) = 0.0_dp
          else
            dee(numero) = de(i)
          endif
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (3) radiative association  (nasrad)

      do i=1,neqt
        ity = itype(i)
        if (ity == 3) then
          nasrad = nasrad + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) =  bet(i)
c  All the energy is taken by the photon
          dee(numero) = 0.0_dp
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (4) gas phase reactions (nrest)

      do i=1,neqt
        ity = itype(i)
        if (ity == 4) then
          nrest = nrest + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = de(i)
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (6) Endothermal reactions with H2 (nh2endo)

      do i=1,neqt
        ity = itype(i)
        if (ity == 6) then
          nh2endo = nh2endo + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = de(i)
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (5) photoreactions (nphot_f)

      do i=1,neqt
        ity = itype(i)
        if (ity == 5) then
          nphot_f = nphot_f + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
c  --- guillaume dixit.
c         dee(numero) = 0.0_dp
c  --- Evelyne dixit.
          dee(numero) = de(i) * 0.1_dp
c  Special case: for PAH heating is already taken into account
          if (ael(react(i,1),10) == 1) then
            dee(numero) = 0.0_dp
          endif
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (7) photoreactions (direct integration) (nphot_i)

      do i=1,neqt
        ity = itype(i)
        if (ity == 7) then
          nphot_i = nphot_i + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
c  --- guillaume dixit.
c         dee(numero) = 0.0_dp
c  --- Evelyne dixit.
          dee(numero) = de(i) * 0.1_dp
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do

          if (react(i,1) == i_h2) then
            idh2ph = numero
          endif
          if (react(i,1) == i_hd) then
            idhdph = numero
          endif
          if (react(i,1) == i_co0) then
            idco0p = numero
          endif
          if (react(i,1) == i_co1) then
            idco1p = numero
          endif
          if (react(i,1) == i_co2) then
            idco2p = numero
          endif
          if (react(i,1) == i_co3) then
            idco3p = numero
          endif
          if (react(i,1) == i_c) then
            idcph = numero
          endif
        endif
      end do

c (8) 3 body reactions

      do i=1,neqt
        ity = itype(i)
        if (ity == 8) then
          n3body = n3body + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
c Which enthapy for 3 body?
          dee(numero) = de(i)
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (101...) exceptions (ncpart)

      do i=1,neqt
        ity = itype(i)
        if (ity > 100 .and.
     1      ity < 110) then
          ncpart = ncpart + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = de(i)
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (11) reactions on grain surface (ngsurf)

      do i=1,neqt
        ity = itype(i)
        if (ity == 11 .or. ity == 111) then
          ngsurf = ngsurf + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = 0.0_dp
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif

        if (react(i,1) == i_hgr .and.
     1      react(i,2) == i_hgr) then
          dee(numero) = de(i)
          ifh2gr = numero
        endif

        if (react(i,1) == i_hgr .and.
     1      react(i,2) == i_dgr .or.
     2      react(i,1) == i_dgr .and.
     3      react(i,2) == i_hgr) then
          dee(numero) = de(i)
          ifhdgr = numero
        endif

      end do

c (12) photoreactions on grain surface (ngphot)

      do i=1,neqt
        ity = itype(i)
        if (ity == 12 .or. ity == 112) then
          ngphot = ngphot + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = 0.0_dp
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (13) adsorption on to grains (ngads)

      do i=1,neqt
        ity = itype(i)
        if (ity == 13 .or.
     1      ity == 113 .or. ity == 123) then
          ngads = ngads + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = 0.0_dp
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do

c Sticking coef de H:
          if (react(i,1) == i_h) then
            iadhgr = numero
          endif

c Sticking coef de D:
          if (react(i,1) == i_d) then
            iaddgr = numero
          endif

        endif
      end do

c (14) neutralization (ngneut)

      do i=1,neqt
        ity = itype(i)
        if (ity == 14 .or. ity == 114) then
          ngneut = ngneut + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = 0.0_dp
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (15) explosive desorption from grains (nggdes)

      do i=1,neqt
        ity = itype(i)
        if (ity == 15 .or. ity == 115) then
          nggdes = nggdes + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = 0.0_dp
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (16) direct cosmic ray induced desorption from grains (ncrdes)

      do i=1,neqt
        ity = itype(i)
        if (ity == 16 .or. ity == 116) then
          ncrdes = ncrdes + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = 0.0_dp
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (17) photodesorption from grains (UV + PHOSEC) (nphdes)

      do i=1,neqt
        ity = itype(i)
        if (ity == 17 .or. ity == 117) then
          nphdes = nphdes + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = 0.0_dp
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (18) evaporation from grains (ngevap)

      do i=1,neqt
        ity = itype(i)
        if (ity == 18 .or. ity == 118) then
          ngevap = ngevap + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = 0.0_dp
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

      do i = 1,neqt
        gamm(i) = gammm(i)
        alpha(i) = palph(i)
        bet(i) = bett(i)
        de(i) = dee(i)
        itype(i) = ntype(i)
        do j = 0,7
          react(i,j) = lreact(i,j)
        end do
          r1=sp(natom+6)
          r2=sp(natom+6)
          r3=sp(natom+6)
          p1=sp(natom+6)
          p2=sp(natom+6)
          p3=sp(natom+6)
          p4=sp(natom+6)
        do j=1,nspec+6
          if(react(i,1) == j) r1=speci(j)
          if(react(i,2) == j) r2=speci(j)
          if(react(i,3) == j) r3=speci(j)
          if(react(i,4) == j) p1=speci(j)
          if(react(i,5) == j) p2=speci(j)
          if(react(i,6) == j) p3=speci(j)
          if(react(i,7) == j) p4=speci(j)
        end do
        if (itype(i) /= 7) then
          write(iwrit2, 2040) r1,r2,r3,p1,p2,p3,p4,
     1                 gamm(i),alpha(i),bet(i),de(i),itype(i),i
        else
          write(iwrit2, 2041) r1,r2,r3,p1,p2,p3,p4,
     1                 "   Direct integration",de(i),itype(i),i
        endif
      end do

 2040  format (2x,'     ',1x,a8,'+ ',a8,'+ ',a8,'= ',3(a8,1x),a8,
     1         1pe8.2,2x,0pf6.2,2x,f8.1,2x,f9.3,3x,i4,i5)
 2041  format (2x,'     ',1x,a8,'+ ',a8,'+ ',a8,'= ',3(a8,1x),a8,
     1         a25,f9.3,3x,i4,i5)

        return
        end
