! Copyright (C) 2002 - Observatoire de Paris/Meudon
!                      Universite Paris 7
!                      CNRS (France)
!                      Contact : Jacques.Lebourlot@obspm.fr
!
! This program is free software; you can redistribute it and/or
! modify it under the terms of the GNU General Public License version 2
! as published by the Free Software Foundation.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

!*****************************************************************************************

!*****************************************************************************************
  SUBROUTINE POST_LECTUR 
!*****************************************************************************************
  USE constantes
  USE auxiliaries
  USE inoutput
  USE H2_DATA
  USE HD_DATA
  USE SP_DATA
  USE STR_DATA
  USE CHEM_DATA
  USE PROFIL
  USE RF_TOOLS

  !=========================================================================================
  ! DECLARATIONS
  !=========================================================================================
  IMPLICIT NONE

  ! Variables internes
  INTEGER                                           :: i, j, ind
  INTEGER                                           :: garder, fact
  INTEGER                                           :: long, long2
  INTEGER                                           :: l1, l2
  CHARACTER (LEN=100)                               :: chaine, chaine_react, chaine_prdt
  CHARACTER (LEN=10), DIMENSION(:,:), ALLOCATABLE   :: esp_pres
  INTEGER, DIMENSION(:), ALLOCATABLE                :: num_react

  ! Variables declarees implicitement, utilisees dans stv1e.f: (test) pour le calcul de la 
  ! somme des masses des produits et du bilan net d'energie

  INTEGER                                           :: kj, iprod, k0
  INTEGER                                           :: nbre_react_dep, nb_exclu
  REAL (KIND=DP)                                    :: sreact, creact, sprod, cprod 
  REAL (KIND=DP)                                    :: dreact, dprod 
 
  INTEGER                                           :: k

  !==========================================================================================
  ! Recherche des reactions de photodestruction a calculer avec les sections
  !==========================================================================================

  CALL system ("./Scripts/bilan")
  ! Scripts/bilan creates 2 files : nb.dat and liste.dat

  open (10, file="../data/nb.dat", status="old")
  read (10,*) nbre_react_dep
  close (10)

  allocate (csect_files(nbre_react_dep))
  open (10, file="../data/liste.dat", status="old")
  do k=1,nbre_react_dep
     read (10,'(a15)') csect_files(k)
     csect_files(k) = TRIM(csect_files(k))
     WRITE(iscrn,*) "Search for photodestruction cross-sections of :",csect_files(k)
  end do
  close (10)

  de_nom = ""
  ns = 0
  do k = 1, nbre_react_dep
    ic = INDEX(csect_files(k), "_")     
    un_nom = csect_files(k)(1:ic-1)
    if (un_nom == de_nom) then
      cycle
    else
      ns = ns + 1
      de_nom = un_nom
    endif
  end do

  ! ns correspond au nombre d'especes intervenant 
  !dans les reactions de itype=7 (= especes presentes 
  !dans "../data/liste.dat") 
  ! nbre_react_dep correspond au nombre de reactions de 
  !itype=7 utilisees (= nombre de lignes du fichier 
  !"../data/liste.dat")

  allocate(les_esp(ns))

  de_nom = ""
  ns = 0
  do k=1,nbre_react_dep
    ic = INDEX(csect_files(k), "_")
    un_nom = csect_files(k)(1:ic-1)
    if (un_nom == de_nom) then
       cycle
    else
      ns = ns + 1
      de_nom = un_nom
      les_esp(ns) = un_nom
    endif
  end do

  ! les_esp(ns) contient les noms des ns especes 
  ! intervenant dans les reactions de itype=7
  ! csect_files(nbre_react_dep) contient les noms des 
  ! fichiers relies aux reactions de itype=7 
  ! (cf "../data/liste.dat")

  ! 2) Table of data filling up. This table 
  !    (photodest) is a  PHOTO_ESP TYPE variable

  allocate(num_react(nbre_react_dep))
  allocate(esp_pres(nbre_react_dep,7))
  nb_exclu = 0 
  
  do ind=1,nbre_react_dep
     num_react(ind) = 0 
  end do
  
  do k=1,nbre_react_dep        
     fichier = TRIM(data_dir)//'sections/'//TRIM(csect_files(k))//".dat"  
     WRITE(iscrn,*) "Ouverture de : ",fichier
     OPEN(iread2, file = fichier, status = 'old')
     READ(iread2,*)
     READ(iread2,'(a)') chaine
     
     chaine= TRIM(ADJUSTL(chaine))
     
     long= len(chaine)
     i= index(chaine,"#")
     j= index(chaine,"=")
     chaine_react= TRIM(ADJUSTL(chaine(i+1:j-1)))
     chaine_prdt= TRIM(ADJUSTL(chaine(j+1:long))) 
     
     do i=1,7
       esp_pres(k,i)=' '
     end do
  
     long= len_trim(chaine_react)
     l1= index(chaine_react," + ")
     esp_pres(k,1)=TRIM(ADJUSTL(chaine_react(1:l1-1)))
     esp_pres(k,2)=TRIM(ADJUSTL(chaine_react(l1+2:long)))
          
     do i=4,7
       long2= len_trim(chaine_prdt)
       if (long2 == 0) then
         exit
       endif
       l2= index(chaine_prdt," + ")
       if (l2 /= 0) then
         esp_pres(k,i)=TRIM(ADJUSTL(chaine_prdt(1:l2-1)))
       else
         esp_pres(k,i)=TRIM(ADJUSTL(chaine_prdt(1:long2)))
       endif
       if (l2 /= 0) then
         chaine_prdt=TRIM(ADJUSTL(chaine_prdt(l2+3:long2)))
       else
         chaine_prdt = ''
       endif
     end do

     ! Comparaison des tableaux speci(nspec+1) et 
     !esp_pres(nbre_react_dep,1:7): 
     !RQ: speci(j)=' ' existe sinon 'garder' serait = a 0

     garder = 1
     do i=1,7
        fact = 0          
        do j=0,nspec+1
            if (esp_pres(k,i) == speci(j))then
              fact = 1
           end if
        end do
        garder = garder * fact
     end do

     ! nb_exclu correspond au nombre d'especes 
     !non repertoriees par la chimie (speci(nspec+1)) 
     !et qui pourtant interviennent dans les reactions
     !de itype=7 (cf esp_pres(nbre_react_dep,1:7))

     if (garder == 0) then
        num_react(k) = 1
        nb_exclu = nb_exclu + 1
        PRINT *,"Reaction exclue :"
        PRINT *,(esp_pres(k,i),i=1,7)
     endif
     close(iread2)
  end do
 
  ! nphot_i correspond au nombre de
  !reactions de itype=7 conservees dans 
  !tout le reste du programme

  nphot_i = nbre_react_dep - nb_exclu
  i = 0
  ! Reorganisation du tableau csect_files(nbre_react_dep), 
  !qui devient maintenant csect_files(nphot_i)
  !et du tableau esp_pres(nbre_react_dep,1:7), qui devient
  !maintenant esp_pres(nphot_i,1:7)
  do k=1,nbre_react_dep
    if (num_react(k) == 0) then
      i = i + 1
      csect_files(i) = csect_files(k)
      do j=1,7
        esp_pres(i,j) = esp_pres(k,j)
      end do
    else
      cycle
    end if
  end do


  ! Count of mandatory reactions
  i_mandat = 2                ! H2 et CO
  if (i_hd /= 0) then
     i_mandat = i_mandat + 1  ! HD
  endif
  if (i_co1 /= 0) then
     i_mandat = i_mandat + 1  ! 13CO
  endif
  if (i_co2 /= 0) then
     i_mandat = i_mandat + 1  ! C18O
  endif
  if (i_co3 /= 0) then
     i_mandat = i_mandat + 1  ! 13C18O
  endif
  WRITE(iscrn,*) " "
  WRITE(iscrn,*) '-------- Photodestruction Reactions -----------'
  WRITE(iscrn,*) '        computed with cross-sections           '
  WRITE(iscrn,*) "Number of mandatory photo-reactions : ", i_mandat
  WRITE(iscrn,*) "Other photo-reactions               : ", nphot_i
  WRITE(iscrn,*) "Number of species in these reactions: ", ns
  WRITE(iscrn,*)

  ! Remplissage des tableaux & variables 
  !necessaires pour la suite du programme:
  !phdest(nphot_i+i_mandat), itype(nphot_i+i_mandat), 
  !de(nphot_i+i_mandat)... pour les nphot_i+i_mandat 
  !reactions ayant itype=7 
  !==> chgt variable "nphot_i":

  nphot_i = nphot_i + i_mandat
  WRITE(iscrn,*)  nphot_i

  allocate (phdest(nphot_i))
  ALLOCATE(pdiesp(nphot_i,0:noptm))

  do k=1,nphot_i-i_mandat
     itype(k)=7
     phdest(k)%nom= csect_files(k)

     phdest(k)%r1= esp_pres(k,1)
     phdest(k)%r2= esp_pres(k,2)
     phdest(k)%r3= esp_pres(k,3)
     phdest(k)%p1= esp_pres(k,4)
     phdest(k)%p2= esp_pres(k,5)
     phdest(k)%p3= esp_pres(k,6)
     phdest(k)%p4= esp_pres(k,7)

     do j=0,nspec+1
       if (esp_pres(k,1) == speci(j)) then
         react(k,1) = j
         exit
       endif
     end do
     react(k,2) = nspec + 1
     react(k,3) = 0
     k0 = 2
     do i=4,7
       do j=0,nspec+1
         if (esp_pres(k,i) == speci(j)) then
           react(k,i) = j
           if (j /= 0) then
             k0 = k0 + 1
           endif
           exit
         endif
       end do
     end do
     react(k,0) = k0

     if (react(k,5) /= nspec+1) then
        dreact = enth(react(k,1)) + enth(react(k,2)) &
             + enth(react(k,3))
        dprod = 0.0_dp
        kj = react(k,0)
        do j=4,kj+1
           iprod = react(k,j)
           if (iprod /= nspec+1) then
              dprod = dprod + enth(iprod)
           endif
        end do
        de(k) = dreact - dprod
        de(k) = de(k) * calev
     else
        de(k) = 0.0_dp
     endif

  end do
  
!    h2:
  react(k,0) = 4
  react(k,1) = i_h2
  react(k,2) = nspec + 1
  react(k,3) = 0
  react(k,4) = i_h
  react(k,5) = i_h
  react(k,6) = 0
  react(k,7) = 0
  itype(k) = 7
  de(k) = 1.0_dp
  phdest(k)%r1 = 'h2      '
  phdest(k)%r2 = 'photon  '
  phdest(k)%r3 = '        '
  phdest(k)%p1 = 'h       '
  phdest(k)%p2 = 'h       '
  phdest(k)%p3 = '        '
  phdest(k)%p4 = '        '

!    hd:
  if (i_hd /= 0) then
    k = k + 1
    react(k,0) = 4
    react(k,1) = i_hd
    react(k,2) = nspec + 1
    react(k,3) = 0
    react(k,4) = i_h
    react(k,5) = i_d
    react(k,6) = 0
    react(k,7) = 0
    itype(k) = 7
    de(k) = 1.0_dp
    phdest(k)%r1 = 'hd      '
    phdest(k)%r2 = 'photon  '
    phdest(k)%r3 = '        '
    phdest(k)%p1 = 'h       '
    phdest(k)%p2 = 'd       '
    phdest(k)%p3 = '        '
    phdest(k)%p4 = '        '
  endif

!    co:
  k = k + 1
  react(k,0) = 4
  react(k,1) = i_co0
  react(k,2) = nspec + 1
  react(k,3) = 0
  react(k,4) = i_c
  react(k,5) = i_o
  react(k,6) = 0
  react(k,7) = 0
  itype(k) = 7
  de(k) = 1.0_dp
  phdest(k)%r1 = 'co      '
  phdest(k)%r2 = 'photon  '
  phdest(k)%r3 = '        '
  phdest(k)%p1 = 'c       '
  phdest(k)%p2 = 'o       '
  phdest(k)%p3 = '        '
  phdest(k)%p4 = '        '

!    c*o:
  if (i_co1 /= 0) then
    k = k + 1
    react(k,0) = 4
    react(k,1) = i_co1
    react(k,2) = nspec + 1
    react(k,3) = 0
    react(k,4) = i_c13
    react(k,5) = i_o
    react(k,6) = 0
    react(k,7) = 0
    itype(k) = 7
    de(k) = 1.0_dp
    phdest(k)%r1 = 'c*o     '
    phdest(k)%r2 = 'photon  '
    phdest(k)%r3 = '        '
    phdest(k)%p1 = 'c*      '
    phdest(k)%p2 = 'o       '
    phdest(k)%p3 = '        '
    phdest(k)%p4 = '        '
  endif

!    co*:
  if (i_co2 /= 0) then
    k = k + 1
    react(k,0) = 4
    react(k,1) = i_co2
    react(k,2) = nspec + 1
    react(k,3) = 0
    react(k,4) = i_c
    react(k,5) = i_o18
    react(k,6) = 0
    react(k,7) = 0
    itype(k) = 7
    de(k) = 1.0_dp
    phdest(k)%r1 = 'co*     '
    phdest(k)%r2 = 'photon  '
    phdest(k)%r3 = '        '
    phdest(k)%p1 = 'c       '
    phdest(k)%p2 = 'o*      '
    phdest(k)%p3 = '        '
    phdest(k)%p4 = '        '
  endif

!    c*o*:
  if (i_co3 /= 0) then
    k = k + 1
    react(k,0) = 4
    react(k,1) = i_co3
    react(k,2) = nspec + 1
    react(k,3) = 0
    react(k,4) = i_c13
    react(k,5) = i_o18
    react(k,6) = 0
    react(k,7) = 0
    itype(k) = 7
    de(k) = 1.0_dp
    phdest(k)%r1 = 'c*o*    '
    phdest(k)%r2 = 'photon  '
    phdest(k)%r3 = '        '
    phdest(k)%p1 = 'c*      '
    phdest(k)%p2 = 'o*      '
    phdest(k)%p3 = '        '
    phdest(k)%p4 = '        '
  endif

  !----calcul de la somme des masses des produits de chaque reaction
  !    et du bilan net d energie de chaque reaction (en ev)
  !    convention: positif ==> exothermique
  !                negatif ==> endothermique
  !                ******* ==> donnees insuffisantes
  
  do k=1,nphot_i         
     sreact = 0.0_dp
     creact = 0.0_dp
     sprod  = 0.0_dp
     cprod  = 0.0_dp
     
     sreact = sreact + mol(react(k,1)) + mol(react(k,2)) &
                     + mol(react(k,3))
     creact = creact + charge(react(k,1)) + charge(react(k,2)) &
                     + charge(react(k,3))
     sprod = sprod   + mol(react(k,4)) + mol(react(k,5)) &
                     + mol(react(k,6)) + mol(react(k,7))
     cprod = cprod   + charge(react(k,4)) + charge(react(k,5)) &
                     + charge(react(k,6)) + charge(react(k,7))
     
     bilanm(k) = sreact - sprod
     bilanc(k) = creact - cprod
     if (dabs(bilanm(k)) > 0.0_dp) then
        print *, k, (speci(react(k,j)),j=1,7)
        print *, k, "Mass - React:", sreact, &
             " Prod:", sprod, "Diff:", bilanm(k)
        print *,"Problem in mass conservation"
        stop
     endif
     if ((speci(react(k,1)) /= "grain ") &
          .AND. (speci(react(k,2)) /= "grain ") &
          .AND. (speci(react(k,3)) /= "grain ") ) then
        if (dabs(bilanc(k)) > 0.0_dp) then
           print *, k, (speci(react(k,j)),j=1,7)
           print *, k, "Charge - React:", creact, &
                " Prod:", cprod, & 
                " Diff:", bilanc(k)
           Print *,"Problem in charge conservation"                 
        endif
     end if
  end do

  deallocate (num_react)
  deallocate (esp_pres)

end subroutine POST_LECTUR

