
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


       subroutine my_gauss

       use constantes
       use auxiliaries
       USE STR_DATA
       USE PROFIL

       implicit double precision (a-h,o-z)

       parameter (normax=npmg*2)

       real (KIND=dp) xnu(normax+1,0:normax)
       real (KIND=dp) alp(normax), btta(normax)
       real (KIND=dp) ff(1:normax+1), anu(normax)
       real (KIND=dp) a(normax/2), b(normax/2)

       xplu = 0.5_dp * (rgrmin + rgrmax)
       xmoi = 0.5_dp * (rgrmax - rgrmin)

c  Determine the number of integration points needed

       toto = (rgrmax / rgrmin) - 1.0_dp
       npg = 1 + npmg * ((1.0_dp + toto) / (4.0_dp + toto))
       npg = min(npg,npmg)

       allocate (adust(npg), wwmg(npg))

       alp(1) = xplu
       btta(1) = 0.0_dp

       nnint = npg

       nmod = 2 * nnint

       do j=2,nmod-1
         alp(j) = xplu
         btta(j) = xmoi * xmoi / (4.0_dp - 1.0_dp / ((j-1)*(j-1)))
       end do

       do k=0,nmod
         ff(k+1) = (rgrmax**(k+1-alpgr) - rgrmin**(k+1-alpgr))
     1           / (k+1-alpgr)
         xnu(1,k) = ff(k+1)
       end do

       do k=0,nmod-1
         p = xnu(1,k+1)
         q = alp(1) * xnu(1,k)
         xnu(2,k) = p - q
         do j=3,k+2
           p = xnu(j-1,k-j+3)
           q = alp(j-1) * xnu(j-1,k-j+2)
           r = btta(j-1) * xnu(j-2,k-j+2)
           xnu(j,k-j+2) = p - q - r
         end do
       end do

       do j=1,nmod
         anu(j) = xnu(j,0)
       end do

       call orthog(nnint,anu,alp,btta,a,b)

       amu0 = ff(1)

       call gaucof(nnint,a,b,amu0,adust,wwmg)

       return
       end

c-----------------------------------------------------------------------

       subroutine orthog(n,anu,alpha,beta,a,b)

       use constantes
       implicit double precision (a-h,o-z)

       integer n, nmax
       double precision a(n), alpha(2*n-1), anu(2*n), b(n), beta(2*n-1)
       parameter (nmax=64)

       integer k, l
       double precision sig(2*nmax+1,2*nmax+1)

       do l=3,2*n
         sig(1,l) = 0.0_dp
       end do

       do l=2,2*n+1
         sig(2,l) = anu(l-1)
       end do

       a(1) = alpha(1) + anu(2) / anu(1)
       b(1) = 0.0_dp

       do k=3,n+1
         do l=k,2*n-k+3
           sig(k,l) = sig(k-1,l+1) + (alpha(l-1) - a(k-2))
     1              * sig(k-1,l) - b(k-2) * sig(k-2,l)
     2              + beta(l-1) * sig(k-1,l-1)
         end do
         a(k-1) = alpha(k-1) + sig(k,k+1) / sig(k,k)
     1          - sig(k-1,k) / sig(k-1,k-1)
         b(k-1) = sig(k,k) / sig(k-1,k-1)
       end do

       return
       end

c--------------------------------------------------------------------

       subroutine gaucof(n,a,b,amu0,x,w)

       use constantes
       implicit double precision (a-h,o-z)

       integer n,nmax
       double precision amu0, a(n), b(n), w(n), x(n)
       parameter (nmax=64)

       integer i, j
       double precision z(nmax,nmax)

       do i=1,n
         if (i /= 1) b(i) = sqrt(b(i))
         do j=1,n
           if (i == j) then
             z(i,j) = 1.0_dp
           else
             z(i,j) = 0.0_dp
           endif
         end do
       end do

       call tqli(a,b,n,nmax,z)
       call eigsrt(a,z,n,nmax)

       do i=1,n
         x(i) = a(i)
         w(i) = amu0 * z(1,i)**2
       end do

       return
       end

c-----------------------------------------------------------------------

       subroutine tqli(d,e,n,nnp,z)

       use constantes
       implicit double precision (a-h,o-z)

       integer n, nnp
       double precision d(nnp), e(nnp), z(nnp,nnp)

       integer i, iter, k, l, m
       double precision b, c, dd, f, g, p, r, s, pythag

       do i=2,n
         e(i-1) = e(i)
       end do
       e(n) = 0.0_dp

       do 15 l=1,n
         iter = 0
    1    continue
         do m=l,n-1
           dd = abs(d(m)) + abs(d(m+1))
           if (abs(e(m)) + dd == dd) goto 2
         end do

         m = n

    2    if (m /= l) then
c          if (iter == 30) pause '  Too many iterations in tqli'
           iter = iter + 1
           g = (d(l+1) - d(l)) / (2.0_dp * e(l))
           r = pythag(g,1.0_dp)
           g = d(m) - d(l) + e(l) / (g + sign(r,g))
           s = 1.0_dp
           c = 1.0_dp
           p = 0.0_dp

           do 14 i=m-1,l,-1
             f = s * e(i)
             b = c * e(i)
             r = pythag(f,g)
             e(i+1) = r
             if (r == 0.0_dp) then
               d(i+1) = d(i+1) - p
               e(m) = 0.0_dp
               goto 1
             endif
             s = f / r
             c = g / r
             g = d(i+1) - p
             r = (d(i) - g) * s + 2.0_dp * c * b
             p = s * r
             d(i+1) = g + p
             g = c * r - b

c ...eigenvectors:
             do k=1,n
               f = z(k,i+1)
               z(k,i+1) = s * z(k,i) + c * f
               z(k,i) = c * z(k,i) - s * f
             end do
c ...fin eigenvectors

   14      continue

           d(l) = d(l) - p
           e(l) = g
           e(m) = 0.0_dp
           goto 1
         endif
   15  continue

       return
       end

c---------------------------------------------------------------

       double precision function pythag(a,b)

       use constantes
       double precision a, b
       double precision absa, absb

       absa = abs(a)
       absb = abs(b)

       if (absa > absb) then
         pythag = absa * sqrt(1.0_dp + (absb/absa)**2)
       else
         if (absb == 0.0_dp) then
           pythag = 0.0_dp
         else
           pythag = absb * sqrt(1.0_dp + (absa/absb)**2)
         endif
       endif

       return
       end

c----------------------------------------------------------------

       subroutine eigsrt(d,v,n,nnp)

       use constantes
       implicit double precision (a-h,o-z)

       integer n, nnp
       double precision d(nnp), v(nnp,nnp)

       integer i, j, k
       double precision p

       do i=1,n-1
         k = i
         p = d(i)
         do j=i+1,n
           if (d(j) >= p) then
             k = j
             p = d(j)
           endif
         end do

         if (k /= i) then
           d(k) = d(i)
           d(i) = p
           do j=1,n
             p = v(j,i)
             v(j,i) = v(j,k)
             v(j,k) = p
           end do
         endif
       end do

       return
       end

c----------------------------------------------------------------

