
! Copyright (C) 2002 - Observatoire de Paris/Meudon
!                      Universite Paris 7
!                      CNRS (France)
!                      Contact : Jacques.Lebourlot@obspm.fr
!
! This program is free software; you can redistribute it and/or
! modify it under the terms of the GNU General Public License version 2
! as published by the Free Software Foundation.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


MODULE STR_DATA

! Structure and transfer related quantities for:
!     CO, C*O, CO*, C*O*, CS, HCO+, H3+
!     C, N, O, S, Si, C+, N+, O+, S+, Si+

  use constantes

! Dumb quantities used in DEJONG
  INTEGER, PUBLIC, PARAMETER                 :: nlvmax = 90      ! Maximum number of levels

  REAL (kind=dp), PUBLIC, ALLOCATABLE        :: opdr(:)      ! Optical depth ("reverse": towards right side)
  REAL (kind=dp), PUBLIC, ALLOCATABLE        :: eqcol(:,:)   ! Collisional equilibrium

! Atoms and atomic ions are as follow:
!           1:   cat  : atomic carbon    (C I)
!           2:   nat  : atomic nitrogen  (N I)
!           3:   oat  : atomic oxygen    (O I)
!           4:   sat  : atomic sulfur    (S I)
!           5:   siat : atomic silicium  (Si I)
!           6:   cpl  : ionised carbon   (C II)
!           7:   npl  : ionised nitrogen (N II)
!           8:   opl  : ionised oxygen   (O II)
!           9:   spl  : ionised sulfur   (S II)
!          10:   sip  : ionised silicium (Si II)

! Cooling and heating rates

  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: reftot    ! Total cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: chotot    ! Total heating

  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refcat    ! C cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refnat    ! N cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refoat    ! O cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refsat    ! S cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refsiat   ! Si cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refcpl    ! C+ cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refnpl    ! N+ cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refopl    ! O+ cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refspl    ! S+ cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refsip    ! Si+ cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: rfjh2t    ! H2 total cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: rfjhdt    ! HD total cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refco0    ! CO cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refco1    ! 13CO cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refco2    ! C18O cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refco3    ! 13C18O cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refcs0    ! CS cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refh2o    ! H2O cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refoh     ! OH cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: reffep    ! Fe+ cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refhcop   ! HCO+ cooling
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: refh3p    ! H3+ cooling

  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: chophg    ! Photoelectric effect on grains
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: chochi    ! Chemical exothermicity
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: choeqg    ! Gaz-grains equilibrium
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: chopht    ! Direct Photoprocesses
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: chorc     ! Cosmic rays
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: chophs    ! Secondary photons
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: choh2g    ! H2 formation on grains
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: chohdg    ! HD formation on grains
  REAL (kind=dp), PUBLIC, DIMENSION (0:noptm)       :: chotur    ! Disspation of turbulence

! This is the minimum collision rate (used when nothing else is known)

  REAL (kind=dp), PUBLIC, PARAMETER                   :: colr_min = 1.0e-30_dp

!------------------------------------------------------------------------------------------
! CO 0 (C12 et O16)
!------------------------------------------------------------------------------------------
! donnees du 28 septembre 2000 a partir du JPL

  INTEGER, PUBLIC, PARAMETER                 :: nlvco0 = 31      ! Number of levels

  REAL (kind=dp), PUBLIC, DIMENSION (nlvco0) :: gstco0           ! Statistical weight
  DATA gstco0                                 /  1.0_dp,  3.0_dp,  5.0_dp,  7.0_dp,  9.0_dp &
                                              , 11.0_dp, 13.0_dp, 15.0_dp, 17.0_dp, 19.0_dp &
                                              , 21.0_dp, 23.0_dp, 25.0_dp, 27.0_dp, 29.0_dp &
                                              , 31.0_dp, 33.0_dp, 35.0_dp, 37.0_dp, 39.0_dp &
                                              , 41.0_dp, 43.0_dp, 45.0_dp, 47.0_dp, 49.0_dp &
                                              , 51.0_dp, 53.0_dp, 55.0_dp, 57.0_dp, 59.0_dp &
                                              , 61.0_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (nlvco0) :: elkco0           ! Level energy
  DATA elkco0                                 /   0.000_dp,    5.532_dp,   16.596_dp,   33.192_dp &
                                             ,   55.318_dp,   82.974_dp,  116.160_dp,  154.872_dp &
                                             ,  199.112_dp,  248.875_dp,  304.162_dp,  364.968_dp &
                                             ,  431.293_dp,  503.134_dp,  580.487_dp,  663.350_dp &
                                             ,  751.721_dp,  845.594_dp,  944.968_dp, 1049.837_dp &
                                             , 1160.198_dp, 1276.047_dp, 1397.380_dp, 1524.191_dp &
                                             , 1656.476_dp, 1794.230_dp, 1937.447_dp, 2086.122_dp &
                                             , 2240.250_dp, 2399.824_dp, 2564.839_dp /

  REAL (kind=dp), PUBLIC                     :: amco0 = 28.0_dp  ! Mass (amu)

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvco0,0:noptm) :: xreco0    ! CO levels relative abundances

  INTEGER, PUBLIC, PARAMETER                 :: ntrco0 = 30      ! Number of transitions

  REAL (kind=dp), PUBLIC, DIMENSION (ntrco0) :: aijco0           ! Transition probability
  DATA aijco0                                 / 7.165598E-08_dp, 6.878974E-07_dp, 2.487486E-06_dp &
                                              , 6.112985E-06_dp, 1.220879E-05_dp, 2.141903E-05_dp &
                                              , 3.437722E-05_dp, 5.165579E-05_dp, 7.400161E-05_dp &
                                              , 1.021826E-04_dp, 1.365310E-04_dp, 1.778280E-04_dp &
                                              , 2.266856E-04_dp, 2.837192E-04_dp, 3.495736E-04_dp &
                                              , 4.248763E-04_dp, 5.101879E-04_dp, 6.062004E-04_dp &
                                              , 7.134401E-04_dp, 8.325689E-04_dp, 9.641728E-04_dp &
                                              , 1.108853E-03_dp, 1.267150E-03_dp, 1.439707E-03_dp &
                                              , 1.627083E-03_dp, 1.829820E-03_dp, 2.048529E-03_dp &
                                              , 2.283784E-03_dp, 2.536060E-03_dp, 2.806008E-03_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (ntrco0) :: wlkco0           ! Energy of transition
  DATA wlkco0                                 /   5.532_dp,  11.064_dp,  16.596_dp,  22.126_dp,  27.656_dp &
                                              ,  33.186_dp,  38.712_dp,  44.240_dp,  49.763_dp,  55.287_dp &
                                              ,  60.806_dp,  66.325_dp,  71.841_dp,  77.353_dp,  82.863_dp &
                                              ,  88.371_dp,  93.873_dp,  99.374_dp, 104.869_dp, 110.361_dp &
                                              , 115.849_dp, 121.333_dp, 126.811_dp, 132.285_dp, 137.754_dp &
                                              , 143.217_dp, 148.675_dp, 154.128_dp, 159.574_dp, 165.015_dp /

  INTEGER, PUBLIC, DIMENSION (ntrco0)        :: iupco0           ! Upper state of transition
  DATA iupco0                                 /  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16 &
                                              , 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31 /

  INTEGER, PUBLIC, DIMENSION (ntrco0)        :: jloco0           ! Lower state of transition
  DATA jloco0                                 /  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15 &
                                              , 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30 /

  REAL (KIND=dp), PUBLIC, DIMENSION(ntrco0,0:noptm) :: emico0    ! CO Emissivity

  REAL (kind=dp), PUBLIC, DIMENSION (nlvco0)        :: eqrco0    ! CO radiative destruction
  REAL (KIND=dp), PUBLIC, DIMENSION (1:njco)        :: cdodj1    ! CO collision rate J -> J-1
  REAL (KIND=dp), PUBLIC, DIMENSION (0:njco-1)      :: cupdj1    ! CO collision rate J -> J+1
  REAL (KIND=dp), PUBLIC, DIMENSION (2:njco)        :: cdodj2    ! CO collision rate J -> J-2
  REAL (KIND=dp), PUBLIC, DIMENSION (0:njco-2)      :: cupdj2    ! CO collision rate J -> J+2
  REAL (KIND=dp), PUBLIC, DIMENSION (3:njco)        :: cdodj3    ! CO collision rate J -> J-3
  REAL (KIND=dp), PUBLIC, DIMENSION (0:njco-3)      :: cupdj3    ! CO collision rate J -> J+3
  REAL (KIND=dp), PUBLIC, DIMENSION (4:njco)        :: cdodj4    ! CO collision rate J -> J-4
  REAL (KIND=dp), PUBLIC, DIMENSION (0:njco-4)      :: cupdj4    ! CO collision rate J -> J+4

!------------------------------------------------------------------------------------------
! C*O 1 (C13 et O16)
!-------------------------------------------------------------------------------------------
! donnees du 28 septembre 2000 a partir du JPL

  REAL (kind=dp), PUBLIC, DIMENSION (nlvco0) :: gstco1           ! Statistical weight
  DATA gstco1                                 /  1.0_dp,  3.0_dp,  5.0_dp,  7.0_dp,  9.0_dp &
                                              , 11.0_dp, 13.0_dp, 15.0_dp, 17.0_dp, 19.0_dp &
                                              , 21.0_dp, 23.0_dp, 25.0_dp, 27.0_dp, 29.0_dp &
                                              , 31.0_dp, 33.0_dp, 35.0_dp, 37.0_dp, 39.0_dp &
                                              , 41.0_dp, 43.0_dp, 45.0_dp, 47.0_dp, 49.0_dp &
                                              , 51.0_dp, 53.0_dp, 55.0_dp, 57.0_dp, 59.0_dp &
                                              , 61.0_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (nlvco0) :: elkco1           ! Level energy
  DATA elkco1                                 /    0.000_dp,    5.289_dp,   15.866_dp,   31.732_dp &
                                              ,   52.885_dp,   79.325_dp,  111.051_dp,  148.062_dp &
                                              ,  190.356_dp,  237.932_dp,  290.788_dp,  348.923_dp &
                                              ,  412.333_dp,  481.017_dp,  554.973_dp,  634.197_dp &
                                              ,  718.686_dp,  808.438_dp,  903.449_dp, 1003.717_dp &
                                              , 1109.236_dp, 1220.004_dp, 1336.015_dp, 1457.267_dp &
                                              , 1583.754_dp, 1715.473_dp, 1852.417_dp, 1994.583_dp &
                                              , 2141.964_dp, 2294.556_dp, 2452.353_dp /

  REAL (kind=dp), PUBLIC, PARAMETER                     :: amco1 = 29.0_dp  ! Mass (amu)

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvco0,0:noptm) :: xreco1    ! 13CO levels relative abundances

! donnees du 28 septembre 2000 a partir de la formule de A en
! fonction du moment dipolaire et en prenant le moment dip
! relatif a CO soit 0.1098 Debyes
! (rem: JPL donne mom dip CO1=0.11046 Debyes) (FLP)
  REAL (kind=dp), PUBLIC, DIMENSION (ntrco0) :: aijco1           ! Transition probability
  DATA aijco1                                 / 6.262195E-08_dp, 6.010003E-07_dp, 2.173465E-06_dp &
                                              , 5.341467E-06_dp, 1.066814E-05_dp, 1.871462E-05_dp &
                                              , 3.004183E-05_dp, 4.520688E-05_dp, 6.477089E-05_dp &
                                              , 8.928752E-05_dp, 1.193177E-04_dp, 1.553966E-04_dp &
                                              , 1.980950E-04_dp, 2.479576E-04_dp, 3.055111E-04_dp &
                                              , 3.713075E-04_dp, 4.459030E-04_dp, 5.298093E-04_dp &
                                              , 6.235958E-04_dp, 7.277216E-04_dp, 8.427930E-04_dp &
                                              , 9.692478E-04_dp, 1.107704E-03_dp, 1.258578E-03_dp &
                                              , 1.422467E-03_dp, 1.599756E-03_dp, 1.791082E-03_dp &
                                              , 1.996801E-03_dp, 2.217525E-03_dp, 2.453662E-03_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (ntrco0) :: wlkco1           ! Energy of transition
  DATA wlkco1                                 /   5.289_dp,  10.577_dp,  15.866_dp,  21.153_dp,  26.440_dp &
                                              ,  31.726_dp,  37.011_dp,  42.294_dp,  47.576_dp,  52.856_dp &
                                              ,  58.135_dp,  63.410_dp,  68.684_dp,  73.956_dp,  79.224_dp &
                                              ,  84.489_dp,  89.752_dp,  95.011_dp, 100.268_dp, 105.519_dp &
                                              , 110.768_dp, 116.011_dp, 121.252_dp, 126.487_dp, 131.719_dp &
                                              , 136.944_dp, 142.166_dp, 147.381_dp, 152.592_dp, 157.797_dp /

  INTEGER, PUBLIC, DIMENSION (ntrco0)        :: iupco1           ! Upper state of transition
  DATA iupco1                                 /  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16 &
                                              , 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31 /

  INTEGER, PUBLIC, DIMENSION (ntrco0)        :: jloco1           ! Lower state of transition
  DATA jloco1                                 /  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15 &
                                              , 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30 /

  REAL (KIND=dp), PUBLIC, DIMENSION(ntrco0,0:noptm) :: emico1    ! 13CO Emissivity
  REAL (kind=dp), PUBLIC, DIMENSION (nlvco0)        :: eqrco1    ! 13CO radiative destruction

!------------------------------------------------------------------------------------------
! CO* 2 (C12 et O18)
!------------------------------------------------------------------------------------------
! donnees du 28 septembre 2000 a partir du JPL

  REAL (kind=dp), PUBLIC, DIMENSION (nlvco0) :: gstco2           ! Statistical weight
  DATA gstco2                                 /  1.0_dp,  3.0_dp,  5.0_dp,  7.0_dp,  9.0_dp &
                                              , 11.0_dp, 13.0_dp, 15.0_dp, 17.0_dp, 19.0_dp &
                                              , 21.0_dp, 23.0_dp, 25.0_dp, 27.0_dp, 29.0_dp &
                                              , 31.0_dp, 33.0_dp, 35.0_dp, 37.0_dp, 39.0_dp &
                                              , 41.0_dp, 43.0_dp, 45.0_dp, 47.0_dp, 49.0_dp &
                                              , 51.0_dp, 53.0_dp, 55.0_dp, 57.0_dp, 59.0_dp &
                                              , 61.0_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (nlvco0) :: elkco2           ! Level energy
  DATA elkco2                                 /    0.000_dp,    5.269_dp,   15.806_dp,   31.611_dp &
                                              ,   52.684_dp,   79.024_dp,  110.629_dp,  147.499_dp &
                                              ,  189.632_dp,  237.027_dp,  289.683_dp,  347.596_dp &
                                              ,  410.765_dp,  479.189_dp,  552.863_dp,  631.786_dp &
                                              ,  715.955_dp,  805.366_dp,  900.017_dp,  999.903_dp &
                                              , 1105.022_dp, 1215.369_dp, 1330.941_dp, 1451.733_dp &
                                              , 1577.741_dp, 1708.960_dp, 1845.386_dp, 1987.013_dp &
                                              , 2133.837_dp, 2285.851_dp, 2443.051_dp /

  REAL (kind=dp), PUBLIC                     :: amco2 = 30.0_dp  ! Mass (amu)

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvco0,0:noptm) :: xreco2    ! C18O levels relative abundances

! donnees du 28 septembre 2000 en utilisant la formule de A
! a partir du moment dipolaire. Le moment dipolaire employe est celui
! de CO - (Le JPL done toutefois pour C12O18 0.11079 Debyes) (FLP)
  REAL (kind=dp), PUBLIC, DIMENSION (ntrco0) :: aijco2           ! Transition probability
  DATA aijco2                                 / 6.191423E-08_dp, 5.942074E-07_dp, 2.148493E-06_dp &
                                              , 5.281092E-06_dp, 1.054755E-05_dp, 1.850131E-05_dp &
                                              , 2.969979E-05_dp, 4.469257E-05_dp, 6.403444E-05_dp &
                                              , 8.827779E-05_dp, 1.179560E-04_dp, 1.536315E-04_dp &
                                              , 1.958539E-04_dp, 2.451320E-04_dp, 3.020421E-04_dp &
                                              , 3.671045E-04_dp, 4.408399E-04_dp, 5.238097E-04_dp &
                                              , 6.164956E-04_dp, 7.194771E-04_dp, 8.332198E-04_dp &
                                              , 9.582861E-04_dp, 1.095145E-03_dp, 1.244333E-03_dp &
                                              , 1.406330E-03_dp, 1.581671E-03_dp, 1.770787E-03_dp &
                                              , 1.974247E-03_dp, 2.192421E-03_dp, 2.425918E-03_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (ntrco0) :: wlkco2           ! Energy of transition
  DATA wlkco2                                 /   5.269_dp,  10.537_dp,  15.805_dp,  21.073_dp,  26.340_dp &
                                              ,  31.605_dp,  36.870_dp,  42.133_dp,  47.395_dp,  52.656_dp &
                                              ,  57.913_dp,  63.169_dp,  68.424_dp,  73.674_dp,  78.923_dp &
                                              ,  84.169_dp,  89.411_dp,  94.651_dp,  99.886_dp, 105.119_dp &
                                              , 110.347_dp, 115.572_dp, 120.792_dp, 126.008_dp, 131.219_dp &
                                              , 136.426_dp, 141.627_dp, 146.824_dp, 152.014_dp, 157.200_dp /

  INTEGER, PUBLIC, DIMENSION (ntrco0)        :: iupco2           ! Upper state of transition
  DATA iupco2                                 /  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16 &
                                              , 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31 /

  INTEGER, PUBLIC, DIMENSION (ntrco0)        :: jloco2           ! Lower state of transition
  DATA jloco2                                 /  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15 &
                                              , 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30 /

  REAL (KIND=dp), PUBLIC, DIMENSION(ntrco0,0:noptm) :: emico2    ! C18O Emissivity
  REAL (kind=dp), PUBLIC, DIMENSION (nlvco0)        :: eqrco2    ! C18O radiative destruction

!------------------------------------------------------------------------------------------
! C*O* 3 (C13 et O18)
!------------------------------------------------------------------------------------------
! donnees du 28 septembre 2000 avec le moment dipol de CO

  REAL (kind=dp), PUBLIC, DIMENSION (nlvco0) :: gstco3           ! Statistical weight
  DATA gstco3                                 /  1.0_dp,  3.0_dp,  5.0_dp,  7.0_dp,  9.0_dp &
                                              , 11.0_dp, 13.0_dp, 15.0_dp, 17.0_dp, 19.0_dp &
                                              , 21.0_dp, 23.0_dp, 25.0_dp, 27.0_dp, 29.0_dp &
                                              , 31.0_dp, 33.0_dp, 35.0_dp, 37.0_dp, 39.0_dp &
                                              , 41.0_dp, 43.0_dp, 45.0_dp, 47.0_dp, 49.0_dp &
                                              , 51.0_dp, 53.0_dp, 55.0_dp, 57.0_dp, 59.0_dp &
                                              , 61.0_dp /

! donnees du 28 septembre 2000 a partir des coeff de Dunham
! ref: Guelachvili, J Molecular Spect, 98, 64 (1983)
  REAL (kind=dp), PUBLIC, DIMENSION (nlvco0) :: elkco3           ! Level energy
  DATA elkco3                                 /    0.000_dp,    5.025_dp,   15.076_dp,   30.151_dp &
                                              ,   50.251_dp,   75.374_dp,  105.520_dp,  140.687_dp &
                                              ,  180.875_dp,  226.082_dp,  276.307_dp,  331.547_dp &
                                              ,  391.801_dp,  457.067_dp,  527.342_dp,  602.624_dp &
                                              ,  682.911_dp,  768.199_dp,  858.486_dp,  953.769_dp &
                                              , 1054.044_dp, 1159.307_dp, 1269.556_dp, 1384.786_dp &
                                              , 1504.993_dp, 1630.173_dp, 1760.322_dp, 1895.436_dp &
                                              , 2035.509_dp, 2180.536_dp, 2330.513_dp /

  REAL (kind=dp), PUBLIC                     :: amco3 = 31.0_dp  ! Mass (amu)

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvco0,0:noptm) :: xreco3    ! 13C18O levels relative abundances

  REAL (kind=dp), PUBLIC, DIMENSION (ntrco0) :: aijco3           ! Transition probability
  DATA aijco3                                 / 5.370492E-08_dp, 5.157212E-07_dp, 1.864328E-06_dp &
                                              , 4.582820E-06_dp, 9.152062E-06_dp, 1.605551E-05_dp &
                                              , 2.577151E-05_dp, 3.878443E-05_dp, 5.556907E-05_dp &
                                              , 7.660687E-05_dp, 1.023654E-04_dp, 1.333294E-04_dp &
                                              , 1.699682E-04_dp, 2.127452E-04_dp, 2.621381E-04_dp &
                                              , 3.186170E-04_dp, 3.826237E-04_dp, 4.546462E-04_dp &
                                              , 5.351338E-04_dp, 6.245271E-04_dp, 7.232779E-04_dp &
                                              , 8.318812E-04_dp, 9.507226E-04_dp, 1.080268E-03_dp &
                                              , 1.220961E-03_dp, 1.373242E-03_dp, 1.537550E-03_dp &
                                              , 1.714248E-03_dp, 1.903793E-03_dp, 2.106651E-03_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (ntrco0) :: wlkco3           ! Energy of transition
  DATA wlkco3                                 /   5.025_dp,  10.051_dp,  15.075_dp,  20.100_dp,  25.123_dp &
                                              ,  30.146_dp,  35.167_dp,  40.188_dp,  45.207_dp,  50.225_dp &
                                              ,  55.240_dp,  60.254_dp,  65.266_dp,  70.275_dp,  75.282_dp &
                                              ,  80.287_dp,  85.288_dp,  90.287_dp,  95.283_dp, 100.275_dp &
                                              , 105.263_dp, 110.249_dp, 115.230_dp, 120.207_dp, 125.180_dp &
                                              , 130.149_dp, 135.114_dp, 140.073_dp, 145.027_dp, 149.977_dp /

  INTEGER, PUBLIC, DIMENSION (ntrco0)        :: iupco3           ! Upper state of transition
  DATA iupco3                                 /  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16 &
                                              , 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31 /

  INTEGER, PUBLIC, DIMENSION (ntrco0)        :: jloco3           ! Lower state of transition
  DATA jloco3                                 /  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15 &
                                              , 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30 /

  REAL (KIND=dp), PUBLIC, DIMENSION(ntrco0,0:noptm) :: emico3    ! 13C18O Emissivity
  REAL (kind=dp), PUBLIC, DIMENSION (nlvco0)        :: eqrco3    ! 13C18O radiative destruction

!------------------------------------------------------------------------------------------
! CS
!------------------------------------------------------------------------------------------

  INTEGER, PUBLIC, PARAMETER                 :: nlvcs0 = 21      ! Number of levels

  REAL (kind=dp), PUBLIC, DIMENSION (nlvcs0) :: gstcs0           ! Statisctical weight
  DATA gstcs0                                 /  1.0_dp,  3.0_dp,  5.0_dp,  7.0_dp,  9.0_dp &
                                              , 11.0_dp, 13.0_dp, 15.0_dp, 17.0_dp, 19.0_dp &
                                              , 21.0_dp, 23.0_dp, 25.0_dp, 27.0_dp, 29.0_dp &
                                              , 31.0_dp, 33.0_dp, 35.0_dp, 37.0_dp, 39.0_dp &
                                              , 41.0_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (nlvcs0) :: elkcs0           ! Level energy
  DATA elkcs0                                 /  0.000_dp,  2.355_dp,  7.066_dp, 14.132_dp, 23.553_dp &
                                              , 35.330_dp, 49.462_dp, 65.950_dp, 84.793_dp, 105.99_dp &
                                              , 129.54_dp, 155.45_dp, 183.72_dp, 214.33_dp, 247.31_dp &
                                              , 282.64_dp, 320.33_dp, 360.37_dp, 381.57_dp, 447.52_dp &
                                              , 494.62_dp /

  REAL (kind=dp), PUBLIC                     :: amcs0 = 44.1_dp  ! Mass (amu)

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvcs0,0:noptm) :: xrecs0    ! CS levels relative abundances

  INTEGER, PUBLIC, PARAMETER                 :: ntrcs0 = 20      ! Number of transitions

  REAL (kind=dp), PUBLIC, DIMENSION (ntrcs0) :: aijcs0           ! Transition probability
  DATA aijcs0                                 / 1.753e-6_dp, 1.683e-5_dp, 6.087e-5_dp, 1.496e-4_dp &
                                              , 2.989e-4_dp, 5.244e-4_dp, 8.420e-4_dp, 1.267e-3_dp &
                                              , 1.816e-3_dp, 2.505e-3_dp, 3.348e-3_dp, 4.363e-3_dp &
                                              , 5.564e-3_dp, 6.968e-3_dp, 8.590e-3_dp, 1.045e-2_dp &
                                              , 1.255e-2_dp, 1.492e-2_dp, 1.758e-2_dp, 2.053e-2_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (ntrcs0) :: wlkcs0           ! Energy of transition
  DATA wlkcs0                                 /  2.355_dp,  4.711_dp,  7.066_dp,  9.421_dp, 11.777_dp &
                                              , 14.132_dp, 16.487_dp, 18.843_dp, 21.198_dp, 23.554_dp &
                                              , 25.909_dp, 28.264_dp, 30.620_dp, 32.975_dp, 35.330_dp &
                                              , 37.686_dp, 40.041_dp, 42.396_dp, 44.752_dp, 47.107_dp /

  INTEGER, PUBLIC, DIMENSION (ntrcs0)        :: iupcs0           ! Upper state of transition
  DATA iupcs0                                 /  2,  3,  4,  5,  6,  7,  8,  9, 10, 11 &
                                              , 12, 13, 14, 15, 16, 17, 18, 19, 20, 21 /

  INTEGER, PUBLIC, DIMENSION (ntrcs0)        :: jlocs0           ! Lower state of transition
  DATA jlocs0                                 /  1,  2,  3,  4,  5,  6,  7,  8,  9, 10 &
                                              , 11, 12, 13, 14, 15, 16, 17, 18, 19, 20 /

  REAL (KIND=dp), PUBLIC, DIMENSION(ntrcs0,0:noptm)   :: emics0    ! CS Emissivity
  REAL (KIND=dp), PUBLIC, DIMENSION(4,210)   :: cdocs            ! CS collision rates
  INTEGER, PUBLIC                            :: ntccs            ! number of CS collisional transitions included

!------------------------------------------------------------------------------------------
! HCO+
!------------------------------------------------------------------------------------------
  INTEGER, PUBLIC, PARAMETER                  :: nlvhcop = 21      ! Number of levels

  REAL (kind=dp), PUBLIC, DIMENSION (nlvhcop) :: gsthcop           ! Statisctical weight
  DATA gsthcop                                 /  1.0_dp,  3.0_dp,  5.0_dp,  7.0_dp,  9.0_dp &
                                               , 11.0_dp, 13.0_dp, 15.0_dp, 17.0_dp, 19.0_dp &
                                               , 21.0_dp, 23.0_dp, 25.0_dp, 27.0_dp, 29.0_dp &
                                               , 31.0_dp, 33.0_dp, 35.0_dp, 37.0_dp, 39.0_dp &
                                               , 41.0_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (nlvhcop) :: elkhcop           ! Level energy
  DATA elkhcop                                 /   0.000_dp,   4.280_dp,  12.841_dp,  25.682_dp &
                                               ,  42.802_dp,  64.202_dp,  89.881_dp, 119.838_dp &
                                               , 154.073_dp, 192.584_dp, 235.372_dp, 282.435_dp &
                                               , 333.772_dp, 389.382_dp, 449.263_dp, 513.415_dp &
                                               , 581.836_dp, 654.524_dp, 731.477_dp, 812.695_dp &
                                               , 898.174_dp /

  REAL (kind=dp), PUBLIC                      :: amhcop = 29.0_dp  ! Mass (amu)

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvhcop,0:noptm)     :: xrehcop   ! HCO+ levels relative abundances

  INTEGER, PUBLIC, PARAMETER                  :: ntrhcop = 20      ! Number of transitions

  REAL (kind=dp), PUBLIC, DIMENSION (ntrhcop) :: aijhcop           ! Transition probability
  DATA aijhcop                                 / 2.887e-5_dp, 2.771e-4_dp, 1.002e-3_dp, 2.462e-3_dp &
                                               , 4.917e-3_dp, 8.625e-3_dp, 1.384e-2_dp, 2.083e-2_dp &
                                               , 2.983e-2_dp, 4.110e-2_dp, 5.490e-2_dp, 7.149e-2_dp &
                                               , 9.109e-2_dp, 0.114_dp, 0.140_dp, 0.170_dp, 0.205_dp &
                                               , 0.243_dp, 0.286_dp, 0.333_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (ntrhcop) :: wlkhcop           ! Energy of transition
  DATA wlkhcop                                 /  4.280_dp,  8.561_dp, 12.841_dp, 17.120_dp &
                                               , 21.400_dp, 25.679_dp, 29.957_dp, 34.236_dp &
                                               , 38.512_dp, 42.788_dp, 47.063_dp, 51.337_dp &
                                               , 55.610_dp, 59.881_dp, 64.152_dp, 68.421_dp &
                                               , 72.688_dp, 76.954_dp, 81.217_dp, 85.479_dp /

  INTEGER, PUBLIC, DIMENSION (ntrhcop)        :: iuphcop           ! Upper state of transition
  DATA iuphcop                                 /  2,  3,  4,  5,  6,  7,  8,  9, 10, 11 &
                                               , 12, 13, 14, 15, 16, 17, 18, 19, 20, 21/

  INTEGER, PUBLIC, DIMENSION (ntrhcop)        :: jlohcop           ! Lower state of transition
  DATA jlohcop                                 /  1,  2,  3,  4,  5,  6,  7,  8,  9, 10 &
                                               , 11, 12, 13, 14, 15, 16, 17, 18, 19, 20/

  REAL (KIND=dp), PUBLIC, DIMENSION(ntrhcop,0:noptm)     :: emihcop   ! HCO+ Emissivity
  REAL (KIND=dp), PUBLIC, DIMENSION(4,0:njhcop,0:njhcop) :: colhcoph2 ! HCO+ collision rates (with H2)

!-----------------------------------------------------------------------------------------
! H2O
!-----------------------------------------------------------------------------------------
! Dates taken from Kyro's 1981 paper.

  INTEGER, PUBLIC, PARAMETER                 :: nlvh2o = 90      ! Number of levels
  INTEGER, PUBLIC                            :: nh2o_u = 45      ! Number of levels effectively used

  REAL (kind=dp), PUBLIC, DIMENSION (nlvh2o) :: gsth2o           ! Statistical weight
  DATA gsth2o                                 /  1.0_dp,  9.0_dp,  3.0_dp,  9.0_dp,  5.0_dp  &
                                              , 15.0_dp,  5.0_dp, 15.0_dp,  5.0_dp, 21.0_dp  &
                                              ,  7.0_dp, 21.0_dp,  7.0_dp, 21.0_dp,  9.0_dp  &
                                              , 27.0_dp,  9.0_dp,  7.0_dp, 21.0_dp, 27.0_dp  &
                                              ,  9.0_dp, 33.0_dp, 11.0_dp, 27.0_dp,  9.0_dp  &
                                              , 33.0_dp, 11.0_dp, 33.0_dp, 13.0_dp, 39.0_dp  &
                                              , 27.0_dp,  9.0_dp, 11.0_dp, 33.0_dp, 13.0_dp  &
                                              , 39.0_dp, 45.0_dp, 15.0_dp, 13.0_dp, 11.0_dp  &
                                              , 33.0_dp, 39.0_dp, 13.0_dp, 45.0_dp, 15.0_dp  &
                                              , 11.0_dp, 33.0_dp, 17.0_dp, 51.0_dp, 39.0_dp  &
                                              , 13.0_dp, 45.0_dp, 15.0_dp, 45.0_dp, 17.0_dp  &
                                              , 51.0_dp, 39.0_dp, 13.0_dp, 57.0_dp, 19.0_dp  &
                                              , 15.0_dp, 45.0_dp, 17.0_dp, 51.0_dp, 39.0_dp  &
                                              , 13.0_dp, 17.0_dp, 15.0_dp, 45.0_dp, 57.0_dp  &
                                              , 19.0_dp, 21.0_dp, 63.0_dp, 51.0_dp, 17.0_dp  &
                                              , 57.0_dp, 15.0_dp, 45.0_dp, 19.0_dp, 51.0_dp  &
                                              , 17.0_dp, 57.0_dp, 21.0_dp, 63.0_dp, 69.0_dp  &
                                              , 23.0_dp, 19.0_dp, 57.0_dp, 15.0_dp, 45.0_dp  /

  REAL (kind=dp), PUBLIC, DIMENSION (nlvh2o) :: elkh2o           ! Level energy (en Kelvin)
  DATA elkh2o                                 /    0.0000_dp,   34.2361_dp,   53.4341_dp,   60.9659_dp  &
                                              ,  100.8489_dp,  114.3820_dp,  136.9421_dp,  194.1014_dp  &
                                              ,  195.9176_dp,  196.7774_dp,  204.7149_dp,  249.4444_dp  &
                                              ,  296.8334_dp,  305.2575_dp,  319.4974_dp,  323.5051_dp  &
                                              ,  396.3947_dp,  410.3844_dp,  410.6710_dp,  432.1713_dp  &
                                              ,  454.3540_dp,  468.1223_dp,  469.9602_dp,  550.3788_dp  &
                                              ,  552.2861_dp,  574.7541_dp,  598.8556_dp,  642.4550_dp  &
                                              ,  642.7233_dp,  643.5227_dp,  702.3072_dp,  702.3453_dp  &
                                              ,  725.1268_dp,  732.0964_dp,  781.1533_dp,  795.5487_dp  &
                                              ,  843.5088_dp,  843.8476_dp,  867.2923_dp,  877.8546_dp  &
                                              ,  878.1808_dp,  933.7735_dp,  951.8598_dp, 1013.2499_dp  &
                                              , 1021.0103_dp, 1067.7200_dp, 1067.7246_dp, 1070.5862_dp  &
                                              , 1070.7286_dp, 1088.8028_dp, 1090.3214_dp, 1125.7602_dp  &
                                              , 1175.0890_dp, 1212.0128_dp, 1270.3358_dp, 1274.2344_dp  &
                                              , 1278.5476_dp, 1278.5963_dp, 1323.9720_dp, 1324.0320_dp  &
                                              , 1334.8716_dp, 1339.8977_dp, 1414.2499_dp, 1447.6362_dp  &
                                              , 1503.6643_dp, 1503.6647_dp, 1511.0051_dp, 1524.6585_dp  &
                                              , 1524.9301_dp, 1552.6196_dp, 1554.4983_dp, 1603.6296_dp  &
                                              , 1603.6551_dp, 1615.3938_dp, 1628.4400_dp, 1729.3688_dp  &
                                              , 1749.8973_dp, 1749.9041_dp, 1749.9579_dp, 1805.9801_dp  &
                                              , 1807.0519_dp, 1845.9105_dp, 1860.4413_dp, 1861.3274_dp  &
                                              , 1909.4936_dp, 1909.5045_dp, 1929.3137_dp, 1957.1563_dp  &
                                              , 2006.9067_dp, 2006.9068_dp  /

  REAL (kind=dp), PUBLIC                     :: amh2o = 18.02_dp ! Mass (amu)

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvh2o,0:noptm) :: xreh2o    ! H2O levels relative abundances

  INTEGER, PUBLIC, PARAMETER                 :: ntrh2o = 328      ! Number of transitions

  REAL (kind=dp), PUBLIC, DIMENSION (ntrh2o) :: aijh2o            ! Transition probability
  DATA aijh2o                                 /  1.8431396e-2_dp,  3.4584114e-3_dp,  5.8468348e-3_dp  &
                            ,  5.5966476e-2_dp,  7.0512566e-3_dp,  2.5728746e-1_dp,  3.0642985e-2_dp  &
                            ,  2.6161190e-1_dp,  1.8753346e-2_dp,  5.0523430e-2_dp,  1.2520834e-1_dp  &
                            ,  3.6123919e-6_dp,  2.6608877e-3_dp,  1.6451164e-2_dp,  3.5377082e-1_dp  &
                            ,  5.2422876e-2_dp,  3.3382819e-1_dp,  2.2810784e-2_dp,  1.7268960e-1_dp  &
                            ,  2.4655891e-1_dp,  3.0473553e-5_dp,  3.3409401e-2_dp,  3.7256186e-2_dp  &
                            ,  2.0672921e-2_dp,  1.2335249e+0_dp,  7.9020035e-2_dp,  1.6254850e-4_dp  &
                            ,  1.2548056e+0_dp,  8.4147888e-3_dp,  6.6708821e-2_dp,  4.8602119e-1_dp  &
                            ,  8.0607115e-2_dp,  5.3878493e-5_dp,  3.8432831e-1_dp,  2.8420752e-2_dp  &
                            ,  5.6648302e-4_dp,  3.9033854e-1_dp,  4.4653747e-1_dp,  1.1556061e-5_dp  &
                            ,  9.1821234e-2_dp,  1.3876074e+0_dp,  1.2308250e-1_dp,  2.5066363e-4_dp  &
                            ,  1.4657913e+0_dp,  2.4012034e-2_dp,  8.1980791e-2_dp,  1.5715750e-1_dp  &
                            ,  7.5905498e-2_dp,  6.7151180e-1_dp,  1.2002486e-1_dp,  8.9458785e-4_dp  &
                            ,  4.1875550e-1_dp,  1.1459250e-2_dp,  4.2789006e-2_dp,  7.1373007e-1_dp  &
                            ,  7.5016726e-1_dp,  1.9796816e-9_dp,  1.7133910e-2_dp,  2.6020049e-3_dp  &
                            ,  3.4469478e+0_dp,  1.5526979e-1_dp,  1.6385053e-4_dp,  8.4150100e-3_dp  &
                            ,  6.7107165e-3_dp,  3.4555928e+0_dp,  1.0810037e-4_dp,  1.5040192e-1_dp  &
                            ,  2.1428863e-1_dp,  1.5350799e+0_dp,  1.6457361e-1_dp,  3.0387307e-4_dp  &
                            ,  4.7441073e-5_dp,  1.6577179e+0_dp,  3.7445105e-2_dp,  8.1083289e-2_dp  &
                            ,  1.0890614e-4_dp,  4.5255712e-1_dp,  1.3614098e-1_dp,  9.4371779e-1_dp  &
                            ,  1.7340670e-1_dp,  2.3005331e-3_dp,  1.1580753e+0_dp,  1.1795035e+0_dp  &
                            ,  4.5769387e-1_dp,  7.0264293e-2_dp,  7.4724291e-2_dp,  1.3460579e-5_dp  &
                            ,  8.6552016e-2_dp,  7.6542756e-3_dp,  3.7794673e+0_dp,  2.3934853e-1_dp  &
                            ,  2.9093901e-4_dp,  2.6901296e-2_dp,  3.8339411e+0_dp,  2.8155855e-2_dp  &
                            ,  2.6167651e-4_dp,  2.1397997e-1_dp,  3.5595090e-1_dp,  1.7099986e+0_dp  &
                            ,  2.1618144e-1_dp,  3.8164919e-4_dp,  1.3959614e-3_dp,  1.7685784e+0_dp  &
                            ,  4.2030982e-2_dp,  8.0329573e-2_dp,  3.5959680e-3_dp,  9.4997600e-1_dp  &
                            ,  2.1595443e-1_dp,  1.3430994e+0_dp,  2.4228043e-1_dp,  2.2680139e-3_dp  &
                            ,  6.5256145e-4_dp,  1.5502087e-2_dp,  3.7466332e-3_dp,  1.7678137e-5_dp  &
                            ,  7.2686525e+0_dp,  1.3627939e-4_dp,  2.5502713e-1_dp,  1.4023057e-2_dp  &
                            ,  2.6645806e-4_dp,  4.5139639e-3_dp,  7.2719821e+0_dp,  1.4080880e-4_dp  &
                            ,  2.5385952e-1_dp,  1.7418118e+0_dp,  1.7539435e+0_dp,  2.7007568e-1_dp  &
                            ,  1.3671563e-2_dp,  4.0641847e+0_dp,  2.9875401e-1_dp,  2.8638975e-4_dp  &
                            ,  2.7726190e-5_dp,  5.0674913e-2_dp,  4.2428698e+0_dp,  6.6960990e-2_dp  &
                            ,  3.9882247e-4_dp,  2.2709803e-1_dp,  3.4292616e-5_dp,  5.1846021e-1_dp  &
                            ,  2.6055915e-1_dp,  1.3592472e-1_dp,  1.0569410e-4_dp,  4.9401816e-1_dp  &
                            ,  1.9629554e+0_dp,  2.8574003e-1_dp,  5.1039830e-4_dp,  6.6840958e-3_dp  &
                            ,  1.8032588e+0_dp,  3.9336539e-2_dp,  2.5981257e-2_dp,  9.0405159e-2_dp  &
                            ,  1.6293275e+0_dp,  3.1063049e-1_dp,  1.9035323e+0_dp,  3.2691716e-1_dp  &
                            ,  1.0567487e-3_dp,  2.9478202e-3_dp,  6.5951488e-2_dp,  1.3047565e-2_dp  &
                            ,  5.7088501e-5_dp,  7.8436326e+0_dp,  4.0431762e-1_dp,  3.3137760e-4_dp  &
                            ,  5.0087815e-2_dp,  9.9856811e-4_dp,  1.9322575e-2_dp,  7.8611523e+0_dp  &
                            ,  4.0107022e-4_dp,  3.9601561e-1_dp,  2.4852606e+0_dp,  2.4920654e+0_dp  &
                            ,  6.1099767e-1_dp,  1.9084357e-2_dp,  4.2344637e+0_dp,  3.5519621e-1_dp  &
                            ,  2.4904067e-4_dp,  1.1653473e-3_dp,  6.9393355e-2_dp,  4.6015684e+0_dp  &
                            ,  1.1086011e-1_dp,  4.8517373e-4_dp,  2.1229342e-1_dp,  1.5286734e-3_dp  &
                            ,  6.0916853e-1_dp,  7.0697581e-1_dp,  2.3754264e-1_dp,  3.1923748e-4_dp  &
                            ,  6.2472224e-1_dp,  2.3472065e+0_dp,  3.7877946e-1_dp,  7.0199587e-4_dp  &
                            ,  1.4771556e-2_dp,  4.2160594e-4_dp,  2.1075600e-5_dp,  1.7226391e-2_dp  &
                            ,  3.7892574e-3_dp,  1.7970803e-5_dp,  1.3014439e+1_dp,  1.1694578e-4_dp  &
                            ,  3.6890456e-1_dp,  5.8688543e-5_dp,  1.7179625e-2_dp,  2.2176832e-4_dp  &
                            ,  1.5463530e-6_dp,  3.7854684e-3_dp,  1.3012023e+1_dp,  1.3092670e-4_dp  &
                            ,  3.6870773e-1_dp,  1.8269023e+0_dp,  3.4442489e-2_dp,  1.0828249e-1_dp  &
                            ,  1.2144684e-1_dp,  6.8911977e-3_dp,  1.9606811e-1_dp,  2.7579183e-2_dp  &
                            ,  1.0649377e-4_dp,  8.4374954e+0_dp,  5.0121371e-1_dp,  3.9498923e-4_dp  &
                            ,  2.1148409e-5_dp,  1.1210221e-1_dp,  2.0449490e-3_dp,  8.5167353e+0_dp  &
                            ,  5.3098346e-2_dp,  6.9943366e-4_dp,  4.6878586e-1_dp,  2.1991038e-5_dp  &
                            ,  2.4746655e+0_dp,  4.1768271e-1_dp,  2.6488245e+0_dp,  4.2691939e-1_dp  &
                            ,  2.1844016e-4_dp,  3.4078933e+0_dp,  3.4108827e+0_dp,  1.0655872e+0_dp  &
                            ,  2.3042546e-2_dp,  4.3393847e+0_dp,  4.2220952e-1_dp,  7.2939971e-3_dp  &
                            ,  2.5314448e-4_dp,  7.3942249e-2_dp,  4.8190197e+0_dp,  1.4020100e-1_dp  &
                            ,  5.0277348e-4_dp,  1.9157837e-1_dp,  1.1448759e-2_dp,  7.2671655e-1_dp  &
                            ,  1.5054736e+0_dp,  3.8287706e-1_dp,  6.5179749e-4_dp,  3.0442404e-3_dp  &
                            ,  8.5173074e-5_dp,  6.3294630e-2_dp,  1.4121202e-2_dp,  7.9532989e-5_dp  &
                            ,  1.3870720e+1_dp,  3.3812469e-4_dp,  6.0389618e-1_dp,  2.6204692e-4_dp  &
                            ,  6.1692088e-2_dp,  1.2579775e-3_dp,  5.7254839e-6_dp,  1.4691706e-2_dp  &
                            ,  1.3873496e+1_dp,  4.1627662e-4_dp,  6.0194289e-1_dp,  7.5211028e-1_dp  &
                            ,  2.9192655e+0_dp,  4.9871404e-1_dp,  9.6015418e-4_dp,  1.8868697e-2_dp  &
                            ,  1.1402499e-2_dp,  4.8952552e-1_dp,  4.4978185e-2_dp,  1.5129915e-4_dp  &
                            ,  8.9245387e+0_dp,  5.7314919e-1_dp,  2.8797975e-4_dp,  9.3271519e-4_dp  &
                            ,  1.9274699e-1_dp,  3.0257835e-3_dp,  9.1703371e+0_dp,  1.1338365e-1_dp  &
                            ,  9.7494682e-4_dp,  4.8496341e-1_dp,  9.8943865e-4_dp,  1.9061203e+0_dp  &
                            ,  3.0450373e-2_dp,  3.3684051e-1_dp,  1.8681123e-1_dp,  3.4924797e+0_dp  &
                            ,  5.3634780e-1_dp,  3.5963755e+0_dp,  5.4143982e-1_dp,  6.1058007e-6_dp  &
                            ,  4.5284575e+0_dp,  4.5301085e+0_dp,  1.5428351e+0_dp,  2.5468566e-2_dp  &
                            ,  4.5097597e+0_dp,  5.1004116e-1_dp,  2.2654467e-2_dp,  3.2314431e-4_dp  &
                            ,  6.4511754e-2_dp,  4.8485708e+0_dp,  1.4600810e-1_dp,  4.5627514e-4_dp  &
                            ,  4.7655733e-2_dp,  1.8197606e-1_dp,  2.6601101e-6_dp,  1.4786533e-4_dp  &
                            ,  2.4112685e-5_dp,  6.5213799e-8_dp,  1.9479150e-2_dp,  3.5151237e-3_dp  &
                            ,  7.6477539e-6_dp,  2.0917223e+1_dp,  1.0486111e-4_dp,  4.8828311e-1_dp  &
                            ,  6.8046747e-5_dp,  8.9350873e-7_dp,  1.9488568e-2_dp,  7.7410623e-5_dp  &
                            ,  1.8018152e-6_dp,  3.4934138e-3_dp,  2.0920964e+1_dp,  1.1040854e-4_dp  &
                            ,  4.8825266e-1_dp  /

! Manquent 65 valeurs !
  REAL (kind=dp), PUBLIC, DIMENSION (ntrh2o) :: wlkh2o           ! Energy of transition (en Kelvin)
  DATA wlkh2o                /   53.432_dp,   26.729_dp,   47.413_dp,   80.143_dp,   36.092_dp  &
                             ,  133.130_dp,   79.716_dp,  142.477_dp,   58.972_dp,   82.392_dp  &
                             ,  103.862_dp,    8.797_dp,   55.341_dp,   52.665_dp,  159.885_dp  &
                             ,   92.115_dp,  190.868_dp,   55.811_dp,  114.777_dp,  126.723_dp  &
                             ,   18.247_dp,   99.557_dp,   76.894_dp,  309.522_dp,  214.457_dp  &
                             ,  113.545_dp,   90.883_dp,  216.560_dp,  213.884_dp,  105.408_dp  &
                             ,  182.719_dp,  108.662_dp,   21.501_dp,  249.629_dp,   57.957_dp  &
                             ,   43.969_dp,  144.611_dp,  150.457_dp,   15.605_dp,  353.587_dp  &
                             ,  245.111_dp,  118.202_dp,   82.253_dp,  255.442_dp,  232.779_dp  &
                             ,   97.927_dp,  142.576_dp,  106.627_dp,  202.453_dp,  128.890_dp  &
                             ,   46.568_dp,  318.937_dp,   92.073_dp,   67.699_dp,  172.755_dp  &
                             ,  175.393_dp,    1.067_dp,  452.843_dp,  378.785_dp,  291.624_dp  &
                             ,  151.921_dp,  127.548_dp,  497.609_dp,  305.937_dp,  291.949_dp  &
                             ,  232.375_dp,  150.052_dp,  405.613_dp,  270.761_dp,  126.266_dp  &
                             ,   82.401_dp,   22.781_dp,  299.912_dp,  263.963_dp,   89.638_dp  &
                             ,   29.789_dp,  182.289_dp,  138.423_dp,  220.786_dp,  152.020_dp  &
                             ,   63.449_dp,  199.977_dp,  201.116_dp,  397.315_dp,  142.159_dp  &
                             ,   86.136_dp,   23.444_dp,  481.439_dp,  407.877_dp,  325.555_dp  &
                             ,  152.721_dp,   96.698_dp,  554.652_dp,  327.788_dp,  303.414_dp  &
                             ,  234.648_dp,  146.078_dp,  465.633_dp,  291.306_dp,  138.219_dp  &
                             ,   90.262_dp,   55.591_dp,  352.990_dp,  309.124_dp,   84.565_dp  &
                             ,   74.003_dp,  217.691_dp,  169.733_dp,  239.847_dp,  177.155_dp  &
                             ,   69.147_dp,  748.192_dp,  613.340_dp,  468.845_dp,  424.980_dp  &
                             ,  365.361_dp,  200.421_dp,  189.859_dp,  635.527_dp,  599.578_dp  &
                             ,  425.252_dp,  365.404_dp,  272.166_dp,  189.537_dp,  226.729_dp  &
                             ,  227.210_dp,  514.027_dp,  445.261_dp,  356.691_dp,  155.022_dp  &
                             ,   75.551_dp,   21.076_dp,  620.335_dp,  365.179_dp,  309.156_dp  &
                             ,  246.464_dp,  138.455_dp,   22.599_dp,  482.217_dp,  191.977_dp  &
                             ,  112.506_dp,   55.029_dp,  532.343_dp,  307.784_dp,  154.073_dp  &
                             ,  104.499_dp,   84.764_dp,  416.447_dp,  368.489_dp,  123.205_dp  &
                             ,   86.250_dp,  249.314_dp,  199.740_dp,  260.974_dp,  203.497_dp  &
                             ,   62.218_dp,  810.391_dp,  636.065_dp,  482.979_dp,  435.021_dp  &
                             ,  400.350_dp,  189.737_dp,  152.782_dp,  679.712_dp,  635.847_dp  &
                             ,  411.287_dp,  400.725_dp,  257.576_dp,  188.267_dp,  253.233_dp  &
                             ,  253.435_dp,  553.695_dp,  491.003_dp,  382.995_dp,  159.776_dp  &
                             ,   64.534_dp,   56.272_dp,  696.345_dp,  406.107_dp,  326.635_dp  &
                             ,  269.158_dp,  127.879_dp,   61.347_dp,  570.377_dp,  239.150_dp  &
                             ,  143.908_dp,   90.214_dp,  604.102_dp,  321.863_dp,  173.395_dp  &
                             ,  123.659_dp,  107.734_dp,  928.874_dp,  860.108_dp,  771.537_dp  &
                             ,  569.869_dp,  490.398_dp,  435.924_dp,  291.642_dp,  225.110_dp  &
                             , 1033.664_dp,  778.507_dp,  722.485_dp,  659.792_dp,  551.784_dp  &
                             ,  435.928_dp,  328.565_dp,  225.061_dp,  489.973_dp,  440.400_dp  &
                             ,  176.126_dp,   96.752_dp,  881.897_dp,  657.338_dp,  503.627_dp  &
                             ,  454.053_dp,  434.318_dp,  189.778_dp,  110.404_dp,   20.989_dp  &
                             ,  729.349_dp,  681.392_dp,  436.108_dp,  399.153_dp,  250.685_dp  &
                             ,  185.024_dp,   21.262_dp,  278.373_dp,  228.638_dp,  284.151_dp  &
                             ,  230.456_dp,   43.491_dp,  279.585_dp,  279.671_dp,  602.119_dp  &
                             ,  544.642_dp,  403.364_dp,  167.751_dp,   90.460_dp,   62.772_dp  &
                             ,  784.559_dp,  453.332_dp,  358.090_dp,  304.395_dp,  117.430_dp  &
                             ,  103.777_dp,  658.612_dp,  281.720_dp,  176.742_dp,  125.708_dp  &
                             ,  968.704_dp,  906.012_dp,  798.003_dp,  574.784_dp,  479.543_dp  &
                             ,  471.282_dp,  238.883_dp,  225.230_dp, 1106.335_dp,  816.096_dp  &
                             ,  736.625_dp,  679.148_dp,  537.869_dp,  471.336_dp,  302.256_dp  &
                             ,  224.966_dp,  679.343_dp,  335.695_dp,  195.452_dp,  146.322_dp  &
                             ,  121.513_dp,  962.430_dp,  680.191_dp,  531.723_dp,  481.987_dp  &
                             ,  466.061_dp,  190.578_dp,   76.608_dp,   56.072_dp,  786.007_dp  &
                             ,  736.434_dp,  472.159_dp,  392.785_dp,  252.543_dp,  178.604_dp  &
                             ,   57.151_dp,  571.653_dp,  521.917_dp,  230.507_dp,  116.537_dp  &
                             ,  305.931_dp,  256.801_dp,  308.696_dp,  257.662_dp,   15.416_dp  &
                             ,  305.826_dp,  305.862_dp,  658.951_dp,  605.257_dp,  418.290_dp  &
                             ,  179.349_dp,  122.258_dp,   68.870_dp,  886.390_dp,  509.496_dp  &
                             ,  404.520_dp,  353.486_dp,  151.170_dp,  111.241_dp, 1364.129_dp  &
                             , 1139.569_dp,  985.858_dp,  936.284_dp,  916.549_dp,  672.009_dp  &
                             ,  592.636_dp,  503.220_dp,  378.453_dp,  257.000_dp, 1211.309_dp  &
                             , 1163.351_dp,  918.067_dp,  881.112_dp,  732.644_dp,  666.983_dp  &
                             ,  503.220_dp,  391.499_dp,  256.994_dp  /

  INTEGER, PUBLIC, DIMENSION (ntrh2o)        :: iuph2o           ! Upper state of transition
  DATA iuph2o                                 /   3,  4,  5,  6,  7,  8,  8,  9,  9, 10, 11, 11, 12, 12, 13  &
                                              ,  13, 14, 14, 15, 16, 16, 17, 17, 18, 18, 18, 18, 19, 19, 19  &
                                              ,  20, 20, 20, 21, 21, 21, 22, 23, 23, 24, 24, 24, 24, 25, 25  &
                                              ,  25, 26, 26, 27, 27, 27, 28, 28, 28, 29, 30, 30, 31, 31, 31  &
                                              ,  31, 31, 32, 32, 32, 32, 32, 33, 33, 33, 33, 33, 34, 34, 34  &
                                              ,  34, 35, 35, 36, 36, 36, 37, 38, 39, 39, 39, 39, 40, 40, 40  &
                                              ,  40, 40, 41, 41, 41, 41, 41, 42, 42, 42, 42, 42, 43, 43, 43  &
                                              ,  43, 44, 44, 45, 45, 45, 46, 46, 46, 46, 46, 46, 46, 47, 47  &
                                              ,  47, 47, 47, 47, 48, 49, 50, 50, 50, 50, 50, 50, 51, 51, 51  &
                                              ,  51, 51, 51, 52, 52, 52, 52, 53, 53, 53, 53, 53, 54, 54, 54  &
                                              ,  54, 55, 55, 56, 56, 56, 57, 57, 57, 57, 57, 57, 57, 58, 58  &
                                              ,  58, 58, 58, 58, 59, 60, 61, 61, 61, 61, 61, 61, 62, 62, 62  &
                                              ,  62, 62, 62, 63, 63, 63, 63, 64, 64, 64, 64, 64, 65, 65, 65  &
                                              ,  65, 65, 65, 65, 65, 66, 66, 66, 66, 66, 66, 66, 66, 67, 67  &
                                              ,  67, 67, 68, 68, 68, 68, 68, 68, 68, 68, 69, 69, 69, 69, 69  &
                                              ,  69, 69, 70, 70, 71, 71, 71, 72, 73, 74, 74, 74, 74, 74, 74  &
                                              ,  75, 75, 75, 75, 75, 75, 76, 76, 76, 76, 77, 77, 77, 77, 77  &
                                              ,  77, 77, 77, 78, 78, 78, 78, 78, 78, 78, 78, 79, 79, 79, 79  &
                                              ,  79, 80, 80, 80, 80, 80, 80, 80, 80, 81, 81, 81, 81, 81, 81  &
                                              ,  81, 82, 82, 82, 82, 83, 83, 84, 84, 84, 85, 86, 87, 87, 87  &
                                              ,  87, 87, 87, 88, 88, 88, 88, 88, 88, 89, 89, 89, 89, 89, 89  &
                                              ,  89, 89, 89, 89, 90, 90, 90, 90, 90, 90, 90, 90, 90/

  INTEGER, PUBLIC, DIMENSION (ntrh2o)        :: jloh2o           ! Lower state of transition
  DATA jloh2o                                 /   1,  2,  3,  2,  5,  4,  6,  3,  7,  6,  5,  9,  8, 10,  7  &
                                              ,  11,  6, 12, 11, 10, 14, 13, 15,  5,  9, 13, 15,  8, 10, 14  &
                                              ,  12, 16, 19, 11, 17, 18, 16, 15, 21, 10, 14, 20, 22, 13, 15  &
                                              ,  21, 20, 22, 17, 23, 25, 16, 24, 26, 23, 22, 28, 12, 16, 19  &
                                              ,  24, 26, 11, 17, 18, 23, 25, 15, 21, 27, 29, 32, 20, 22, 28  &
                                              ,  31, 27, 29, 26, 30, 34, 30, 29, 23, 33, 35, 38, 17, 23, 25  &
                                              ,  33, 35, 16, 24, 26, 30, 34, 22, 28, 36, 37, 41, 27, 29, 39  &
                                              ,  40, 36, 37, 35, 38, 43, 15, 21, 27, 29, 32, 39, 40, 20, 22  &
                                              ,  28, 31, 36, 41, 38, 37, 26, 30, 34, 42, 44, 47, 23, 33, 35  &
                                              ,  38, 43, 46, 30, 42, 44, 49, 29, 39, 45, 48, 51, 36, 37, 50  &
                                              ,  52, 45, 48, 44, 49, 54, 22, 28, 36, 37, 41, 50, 52, 27, 29  &
                                              ,  39, 40, 45, 51, 49, 48, 35, 38, 43, 53, 55, 58, 30, 42, 44  &
                                              ,  49, 54, 57, 38, 53, 55, 60, 37, 52, 56, 59, 62, 26, 30, 34  &
                                              ,  42, 44, 47, 54, 57, 23, 33, 35, 38, 43, 46, 53, 58, 45, 48  &
                                              ,  61, 63, 29, 39, 45, 48, 51, 61, 63, 66, 36, 37, 50, 52, 56  &
                                              ,  62, 65, 56, 59, 55, 60, 67, 60, 59, 44, 49, 54, 64, 69, 70  &
                                              ,  38, 53, 55, 60, 67, 68, 49, 64, 70, 73, 35, 38, 43, 53, 55  &
                                              ,  58, 67, 68, 30, 42, 44, 49, 54, 57, 64, 69, 48, 63, 71, 72  &
                                              ,  75, 37, 52, 56, 59, 62, 74, 76, 78, 45, 48, 61, 63, 71, 75  &
                                              ,  77, 56, 59, 74, 76, 71, 72, 70, 73, 82, 73, 72, 55, 60, 67  &
                                              ,  79, 81, 83, 49, 64, 70, 73, 80, 82, 29, 39, 45, 48, 51, 61  &
                                              ,  63, 66, 75, 77, 36, 37, 50, 52, 56, 62, 65, 74, 78/

  REAL (KIND=dp), PUBLIC, DIMENSION (ntrh2o,0:noptm)    :: emih2o     ! H2O Emissivity
  REAL (KIND=dp), PUBLIC, DIMENSION (3,nlvh2o,nlvh2o)   :: qh2ohe     ! H2O collision rate with Helium
  REAL (KIND=dp), PUBLIC, DIMENSION (3,nlvh2o,nlvh2o)   :: qh2oh2     ! H2O collision rate with H2

!------------------------------------------------------------------------------------------
! C
!------------------------------------------------------------------------------------------
! Energy levels.
!        1 - 3P J = 0
!        2 - 3P J = 1
!        3 - 3P J = 2
!        4 - 1D J = 2
!        5 - 1S J = 0

  INTEGER, PUBLIC, PARAMETER                 :: nlvcat = 5       ! Number of levels

  REAL (kind=dp), PUBLIC, DIMENSION (nlvcat) :: gstcat           ! Statistical weight
  DATA gstcat                                 / 1.0_dp, 3.0_dp, 5.0_dp, 5.0_dp, 1.0_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (nlvcat) :: elkcat           ! Level energy
  DATA elkcat                                 / 0.0_dp, 23.5969_dp, 62.4454_dp &
                                              , 14665.507_dp, 31147.902_dp /

  REAL (kind=dp), PUBLIC                     :: amcat = 12.0_dp  ! Mass (amu)

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvcat,0:noptm) :: xrecat    ! C levels relative abundances

! Transitions
!       1: 2-1 -> 609.75 micron
!       2: 3-2 -> 370.37 micron
!       3: 3-1 -> 230.41 micron
!       4: 4-3 -> 9850.26 Angstrom
!       5: 4-2 -> 9824.13 Angstrom
!       6: 4-1 -> 9808.32 Angstrom
!       7: 5-4 -> 8727.13 Angstrom
!       8: 5-3 -> 4627.346 Angstrom
!       9: 5-2 -> 4621.570 Angstrom

  INTEGER, PUBLIC, PARAMETER                 :: ntrcat = 9       ! Number of transitions

  REAL (kind=dp), PUBLIC, DIMENSION (ntrcat) :: wlkcat           ! Energy of transition
  DATA wlkcat                                 /    23.5969_dp,    38.8485_dp,    62.4454_dp &
                                              , 14603.0616_dp, 14641.9101_dp, 14665.507_dp &
                                              , 16482.395_dp,  31085.4566_dp, 31124.3051_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (ntrcat) :: aijcat           ! Transition probability
  DATA aijcat                                 / 7.93e-8_dp, 2.65e-7_dp, 1.71e-14_dp &
                                              , 2.44e-4_dp, 8.21e-5_dp, 7.77e-8_dp &
                                              , 5.28e-1_dp, 2.00e-5_dp, 2.71e-3_dp /

  INTEGER, PUBLIC, DIMENSION (ntrcat)        :: iupcat           ! Upper state of transition
  DATA iupcat                                 / 2, 3, 3, 4, 4, 4, 5, 5, 5 /

  INTEGER, PUBLIC, DIMENSION (ntrcat)        :: jlocat           ! Lower state of transition
  DATA jlocat                                 / 1, 2, 1, 3, 2, 1, 4, 3, 2 /

  REAL (KIND=dp), PUBLIC, DIMENSION(ntrcat,0:noptm) :: emicat    ! C Emissivity

!------------------------------------------------------------------------------------------
! N
!------------------------------------------------------------------------------------------
! Energy levels.
!        1 - 4So J = 3/2
!        2 - 2Do J = 5/2
!        3 - 2Do J = 3/2
!        4 - 2Po J = 1/2
!        5 - 2Po J = 3/2

  INTEGER, PUBLIC, PARAMETER                 :: nlvnat = 5       ! Number of levels

  REAL (kind=dp), PUBLIC, DIMENSION (nlvnat) :: gstnat           ! Statistical weight
  DATA gstnat                                 / 4.0_dp, 6.0_dp, 4.0_dp, 2.0_dp, 4.0_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (nlvnat) :: elknat           ! Level energy
  DATA elknat                                 / 0.0_dp, 27660.82_dp, 27673.36_dp &
                                              , 41494.431_dp, 41494.986_dp /

  REAL (kind=dp), PUBLIC                     :: amnat = 14.0_dp  ! Mass (amu)

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvnat,0:noptm) :: xrenat    ! N levels relative abundances

! Transitions
!       1: 3-2 -> 1.147 mm (261.21 GHz)
!       2: 4-3 -> 1.04104 micron
!       3: 5-3 -> 1.04100 micron
!       4: 4-2 -> 1.04010 micron
!       5: 5-2 -> 1.04006 micron
!       6: 2-1 -> 5200.257 Angstrom
!       7: 3-1 -> 5197.902 Angstrom
!       8: 4-1 -> 3466.543 Angstrom
!       9: 5-1 -> 3466.497 Angstrom

  INTEGER, PUBLIC, PARAMETER                 :: ntrnat = 9       ! Number of transitions

  REAL (kind=dp), PUBLIC, DIMENSION (ntrnat) :: wlknat           ! Energy of transition
  DATA wlknat                                 /    12.540_dp, 13821.071_dp, 13821.626_dp &
                                              , 13833.611_dp, 13834.166_dp, 27660.82_dp &
                                              , 27673.360_dp, 41494.431_dp, 41494.986_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (ntrnat) :: aijnat           ! Transition probability
  DATA aijnat                                 / 1.27e-8_dp, 5.29e-2_dp, 2.76e-2_dp &
                                              , 3.45e-2_dp, 6.14e-2_dp, 7.27e-6_dp &
                                              , 2.02e-5_dp, 2.71e-3_dp, 6.58e-3_dp /

  INTEGER, PUBLIC, DIMENSION (ntrnat)        :: iupnat           ! Upper state of transition
  DATA iupnat                                 / 3, 4, 5, 4, 5, 2, 3, 4, 5 /

  INTEGER, PUBLIC, DIMENSION (ntrnat)        :: jlonat           ! Lower state of transition
  DATA jlonat                                 / 2, 3, 3, 2, 2, 1, 1, 1, 1 /

  REAL (KIND=dp), PUBLIC, DIMENSION(ntrnat,0:noptm) :: eminat    ! N Emissivity

!------------------------------------------------------------------------------------------
! O
!------------------------------------------------------------------------------------------
! 9 VII 2003 - First attempt at unifying atomic datas with the shock model
! Added 2 levels to O structure:

! Energy levels.
!        1 - 3P J = 2
!        2 - 3P J = 1
!        3 - 3P J = 0
!        4 - 1D J = 2
!        5 - 1S J = 0

  INTEGER, PUBLIC, PARAMETER                 :: nlvoat = 5       ! Number of levels

  REAL (kind=dp), PUBLIC, DIMENSION (nlvoat) :: gstoat           ! Statistical weight
  DATA gstoat                                 / 5.0_dp, 3.0_dp, 1.0_dp, 5.0_dp, 1.0_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (nlvoat) :: elkoat           ! Level energy
  DATA elkoat                                 / 0.0_dp, 227.717_dp, 326.582_dp &
                                              , 22831.226_dp, 48621.932_dp /

  REAL (kind=dp), PUBLIC                     :: amoat = 16.0_dp  ! Mass (amu)

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvoat,0:noptm) :: xreoat    ! O levels relative abundances

! Transitions
!       1: 3-2 -> 145.53 micron
!       2: 2-1 -> 63.19  micron
!       3: 3-1 -> 44.06  micron
!       4: 4-3 -> 6391.73 Angstrom
!       5: 4-2 -> 6363.78 Angstrom
!       6: 4-1 -> 6300.30 Angstrom
!       7: 5-4 -> 5577.34 Angstrom
!       8: 5-2 -> 2972.29 Angstrom
!       9: 5-1 -> 2958.37 Angstrom

  INTEGER, PUBLIC, PARAMETER                 :: ntroat = 9       ! Number of transitions

  REAL (kind=dp), PUBLIC, DIMENSION (ntroat) :: wlkoat           ! Energy of transition
  DATA wlkoat                                 / 98.865_dp, 227.717_dp, 326.582_dp, &
                                                22504.644_dp, 22603.509_dp, 22831.226_dp, &
                                                25790.706_dp, 48394.215_dp, 48621.932_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (ntroat) :: aijoat           ! Transition probability
  DATA aijoat                                 / 1.66e-5_dp, 8.46e-5_dp, 1.10e-10_dp, &
                                                1.20e-6_dp, 2.17e-3_dp, 6.71e-3_dp, &
                                                1.02e-0_dp, 7.83e-2_dp, 2.87e-4_dp /

  INTEGER, PUBLIC, DIMENSION (ntroat)        :: iupoat           ! Upper state of transition
  DATA iupoat                                 / 3, 2, 3, 4, 4, 4, 5, 5, 5 /

  INTEGER, PUBLIC, DIMENSION (ntroat)        :: jlooat           ! Lower state of transition
  DATA jlooat                                 / 2, 1, 1, 3, 2, 1, 4, 2, 1 /

  REAL (KIND=dp), PUBLIC, DIMENSION(ntroat,0:noptm) :: emioat    ! O Emissivity

!------------------------------------------------------------------------------------------
! S
!-------------------------------------------------------------------------------------
! Energy levels.
!        1 - 3P J = 2
!        2 - 3P J = 1
!        3 - 3P J = 0
!        4 - 1D J = 2
!        5 - 1S J = 0

  INTEGER, PUBLIC, PARAMETER                 :: nlvsat = 5       ! Number of levels

  REAL (kind=dp), PUBLIC, DIMENSION (nlvsat) :: gstsat           ! Statistical weight
  DATA gstsat                                 / 5.0_dp, 3.0_dp, 1.0_dp, 5.0_dp, 1.0_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (nlvsat) :: elksat           ! Level energy
  DATA elksat                                 /     0.0_dp,    569.86_dp, 825.37_dp &
                                              , 13292.83_dp, 31913.28_dp /

  REAL (kind=dp), PUBLIC                     :: amsat = 32.0_dp  ! Mass (amu)

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvsat,0:noptm) :: xresat    ! S levels relative abundances

! Transitions
!       1: 3-2 -> 56.31 micron
!       2: 2-1 -> 25.25 micron
!       3: 3-1 -> 17.43 micron
!       4: 4-3 -> 1.15407 micron
!       5: 4-2 -> 1.13089 micron
!       6: 4-1 -> 1.08241 micron
!       7: 5-4 -> 7725.05 Angstrom
!       8: 5-2 -> 4589.26 Angstrom
!       9: 5-1 -> 4507.31 Angstrom

  INTEGER, PUBLIC, PARAMETER                 :: ntrsat = 9       ! Number of transitions

  REAL (kind=dp), PUBLIC, DIMENSION (ntrsat) :: wlksat           ! Energy of transition
  DATA wlksat                                 /   255.51_dp,   569.86_dp,   825.37_dp &
                                              , 12467.46_dp, 12722.97_dp, 13292.83_dp &
                                              , 18620.45_dp, 31343.42_dp, 31913.28_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (ntrsat) :: aijsat           ! Transition probability
  DATA aijsat                                 / 3.02e-4_dp, 1.39e-3_dp, 6.71e-8_dp &
                                              , 3.84e-6_dp, 8.16e-3_dp, 2.78e-2_dp &
                                              , 1.53e-0_dp, 3.50e-1_dp, 8.23e-3_dp /

  INTEGER, PUBLIC, DIMENSION (ntrsat)        :: iupsat           ! Upper state of transition
  DATA iupsat                                 / 3, 2, 3, 4, 4, 4, 5, 5, 5 /

  INTEGER, PUBLIC, DIMENSION (ntrsat)        :: jlosat           ! Lower state of transition
  DATA jlosat                                 / 2, 1, 1, 3, 2, 1, 4, 2, 1 /

  REAL (KIND=dp), PUBLIC, DIMENSION(ntrsat,0:noptm) :: emisat    ! S Emissivity

!-------------------------------------------------------------------------------------
! Si
!-------------------------------------------------------------------------------------
! Energy levels.
!        1 - 3P J = 0
!        2 - 3P J = 1
!        3 - 3P J = 2
!        4 - 1D J = 2
!        5 - 1S J = 0

  INTEGER, PUBLIC, PARAMETER                  :: nlvsiat = 5       ! Number of levels

  REAL (kind=dp), PUBLIC, DIMENSION (nlvsiat) :: gstsiat           ! Statistical weight
  DATA gstsiat                                 / 1.0_dp, 3.0_dp, 5.0_dp, 5.0_dp, 1.0_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (nlvsiat) :: elksiat           ! Level energy
  DATA elksiat                                 /    0.000_dp,   110.951_dp, 321.086_dp &
                                               , 9062.998_dp, 22149.939_dp /

  REAL (kind=dp), PUBLIC                     :: amsiat = 28.0_dp  ! Mass (amu)

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvsiat,0:noptm) :: xresiat    ! Si levels relative abundances

! Transitions
!       1: 2-1 -> 129.682 micron
!       2: 3-2 -> 68.472 micron
!       3: 3-1 -> 44.812 micron
!       4: 4-3 -> 1.6459 micron
!       5: 4-2 -> 1.6073 micron
!       6: 4-1 -> 1.5876 micron
!       7: 5-4 -> 1.0994 micron
!       8: 5-3 -> 6589.61 Angstrom
!       9: 5-2 -> 6526.78 Angstrom

  INTEGER, PUBLIC, PARAMETER                  :: ntrsiat = 9       ! Number of transitions

  REAL (kind=dp), PUBLIC, DIMENSION (ntrsiat) :: wlksiat           ! Energy of transition
  DATA wlksiat                                 /   110.951_dp,   210.135_dp,   321.086_dp &
                                               ,  8741.912_dp,  8952.047_dp,  9062.998_dp &
                                               , 13086.941_dp, 21828.853_dp, 22038.988_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (ntrsiat) :: aijsiat           ! Transition probability
  DATA aijsiat                                 / 8.25e-6_dp, 4.21e-5_dp, 3.56e-13_dp &
                                               , 2.25e-3_dp, 7.93e-4_dp, 4.70e-7_dp &
                                               , 1.14e-0_dp, 9.02e-4_dp, 3.13e-2_dp /

  INTEGER, PUBLIC, DIMENSION (ntrsiat)        :: iupsiat           ! Upper state of transition
  DATA iupsiat                                 / 2, 3, 3, 4, 4, 4, 5, 5, 5 /

  INTEGER, PUBLIC, DIMENSION (ntrsiat)        :: jlosiat           ! Lower state of transition
  DATA jlosiat                                 / 1, 2, 1, 3, 2, 1, 4, 3, 2 /

  REAL (KIND=dp), PUBLIC, DIMENSION(ntrsiat,0:noptm) :: emisiat    ! Si Emissivity

!-------------------------------------------------------------------------------------
! C+
!-------------------------------------------------------------------------------------
! Energy levels.
!        1 - 2P J = 1/2
!        2 - 2P J = 3/2
!        3 - 4P J = 1/2
!        4 - 4P J = 3/2
!        5 - 4P J = 5/2

  INTEGER, PUBLIC, PARAMETER                 :: nlvcpl = 5       ! Number of levels

  REAL (kind=dp), PUBLIC, DIMENSION (nlvcpl) :: gstcpl           ! Statistical weight
  DATA gstcpl                                 / 2.0_dp, 4.0_dp, 2.0_dp, 4.0_dp, 6.0_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (nlvcpl) :: elkcpl           ! Level energy
  DATA elkcpl                                 / 0.0_dp, 91.25_dp, 61874.63_dp, 61906.28_dp &
                                              , 61947.00_dp /

  REAL (kind=dp), PUBLIC                     :: amcpl = 12.0_dp  ! Mass (amu)

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvcpl,0:noptm) :: xrecpl    ! C+ levels relative abundance

! Transitions
!       1: 2-1 -> 157.68 micron
!       2: 3-2 -> 2328.12 Angstrom
!       3: 4-2 -> 2326.93 Angstrom
!       4: 5-2 -> 2325.40 Angstrom
!       5: 3-1 -> 2324.69 Angstrom
!       6: 4-1 -> 2323.50 Angstrom

  INTEGER, PUBLIC, PARAMETER                 :: ntrcpl = 6       ! Number of transitions

  REAL (kind=dp), PUBLIC, DIMENSION (ntrcpl) :: aijcpl           ! Transition probability
  DATA aijcpl                                 / 2.29e-6_dp, 6.55e+1_dp, 5.24e+0_dp, &
                                                4.32e+1_dp, 5.53e+1_dp, 1.71e-0_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (ntrcpl) :: wlkcpl           ! Energy of transition
  DATA wlkcpl                                 / 91.25_dp, 61783.38_dp, 61815.03_dp, &
                                                61855.75_dp, 61874.63_dp, 61906.28_dp /

  INTEGER, PUBLIC, DIMENSION (ntrcpl)        :: iupcpl           ! Upper state of transition
  DATA iupcpl                                 / 2, 3, 4, 5, 3, 4 /

  INTEGER, PUBLIC, DIMENSION (ntrcpl)        :: jlocpl           ! Lower state of transition
  DATA jlocpl                                 / 1, 2, 2, 2, 1, 1 /

  REAL (KIND=dp), PUBLIC, DIMENSION(ntrcpl,0:noptm) :: emicpl    ! C+ Emissivity

!-------------------------------------------------------------------------------------
! N+
!-------------------------------------------------------------------------------------
! Energy levels.
!        1 - 3P J = 0
!        2 - 3P J = 1
!        3 - 3P J = 2
!        4 - 1D J = 2
!        5 - 1S J = 0

  INTEGER, PUBLIC, PARAMETER                 :: nlvnpl = 5       ! Number of levels

  REAL (kind=dp), PUBLIC, DIMENSION (nlvnpl) :: gstnpl           ! Statistical weight
  DATA gstnpl                                 / 1.0_dp, 3.0_dp, 5.0_dp, 5.0_dp, 1.0_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (nlvnpl) :: elknpl           ! Level energy
  DATA elknpl                                 /     0.0_dp,     70.07_dp, 188.20_dp &
                                              , 22037.48_dp, 47033.77_dp /

  REAL (kind=dp), PUBLIC                     :: amnpl = 14.0_dp  ! Mass (amu)

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvnpl,0:noptm) :: xrenpl    ! N+ levels relative abundances

! Transitions
!       1: 2-1 -> 205.34 micron
!       2: 3-2 -> 121.80 micron
!       3: 3-1 -> 76.45 micron
!       4: 4-3 -> 6583.45 Angstrom
!       5: 4-2 -> 6548.05 Angstrom
!       6: 4-1 -> 6527.23 Angstrom
!       7: 5-4 -> 5754.59 Angstrom
!       8: 5-3 -> 3070.55 Angstrom
!       9: 5-2 -> 3062.83 Angstrom

  INTEGER, PUBLIC, PARAMETER                 :: ntrnpl = 9       ! Number of transitions

  REAL (kind=dp), PUBLIC, DIMENSION (ntrnpl) :: wlknpl           ! Energy of transition
  DATA wlknpl                                 /    70.07_dp,   118.13_dp,   188.20_dp &
                                              , 21849.28_dp, 21967.41_dp, 22037.48_dp &
                                              , 24996.29_dp, 46845.57_dp, 46963.70_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (ntrnpl) :: aijnpl           ! Transition probability
  DATA aijnpl                                 / 2.08e-6_dp, 7.46e-6_dp, 1.16e-12_dp &
                                              , 2.99e-3_dp, 1.01e-3_dp, 5.35e-7_dp &
                                              , 1.12e-0_dp, 1.51e-4_dp, 3.38e-2_dp /

  INTEGER, PUBLIC, DIMENSION (ntrnpl)        :: iupnpl           ! Upper state of transition
  DATA iupnpl                                 / 2, 3, 3, 4, 4, 4, 5, 5, 5 /

  INTEGER, PUBLIC, DIMENSION (ntrnpl)        :: jlonpl           ! Lower state of transition
  DATA jlonpl                                 / 1, 2, 1, 3, 2, 1, 4, 3, 2 /

  REAL (KIND=dp), PUBLIC, DIMENSION(ntrnpl,0:noptm) :: eminpl    ! N+ Emissivity

!-------------------------------------------------------------------------------------
! O+
!-------------------------------------------------------------------------------------
! Energy levels.
!        1 - 4So J = 3/2
!        2 - 2Do J = 5/2
!        3 - 2Do J = 3/2
!        4 - 2Po J = 3/2
!        5 - 2Po J = 1/2

  INTEGER, PUBLIC, PARAMETER                 :: nlvopl = 5       ! Number of levels

  REAL (kind=dp), PUBLIC, DIMENSION (nlvopl) :: gstopl           ! Statistical weight
  DATA gstopl                                 / 4.0_dp, 6.0_dp, 4.0_dp, 4.0_dp, 2.0_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (nlvopl) :: elkopl           ! Level energy
  DATA elkopl                                 /     0.0_dp,  38575.94_dp, 38604.75_dp &
                                              , 58226.77_dp, 58229.63_dp /

  REAL (kind=dp), PUBLIC                     :: amopl = 16.0_dp  ! Mass (amu)

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvopl,0:noptm) :: xreopl    ! O+ levels relative abundances

! Transitions
!       1: 5-4 -> 5.03 mm (59.66 GHz)
!       2: 3-2 -> 0.4995 mm (600.18 GHz)
!       3: 4-3 -> 7330.73 Angstrom
!       4: 5-3 -> 7329.67 Angstrom
!       5: 4-2 -> 7319.99 Angstrom
!       6: 5-2 -> 7318.92 Angstrom
!       7: 2-1 -> 3728.815 Angstrom
!       8: 3-1 -> 3726.032 Angstrom
!       9: 4-1 -> 2470.341 Angstrom
!      10: 5-1 -> 2470.219 Angstrom

  INTEGER, PUBLIC, PARAMETER                 :: ntropl = 10      ! Number of transitions

  REAL (kind=dp), PUBLIC, DIMENSION (ntropl) :: wlkopl           ! Energy of transition
  DATA wlkopl                                 / 2.86_dp, 28.81_dp, 19622.02_dp &
                                              , 19624.88_dp, 19650.83_dp, 19653.69_dp &
                                              , 38575.94_dp, 38604.75_dp, 58226.77_dp &
                                              , 58229.63_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (ntropl) :: aijopl           ! Transition probability
  DATA aijopl                                 / 2.08e-11_dp, 1.20e-7_dp, 6.14e-2_dp &
                                              , 1.02e-1_dp, 1.17e-1_dp, 6.15e-2_dp &
                                              , 3.82e-5_dp, 1.65e-4_dp, 5.64e-2_dp &
                                              , 2.32e-2_dp /

  INTEGER, PUBLIC, DIMENSION (ntropl)        :: iupopl           ! Upper state of transition
  DATA iupopl                                 / 5, 3, 4, 5, 4, 5, 2, 3, 4, 5 /

  INTEGER, PUBLIC, DIMENSION (ntropl)        :: jloopl           ! Lower state of transition
  DATA jloopl                                 / 4, 2, 3, 3, 2, 2, 1, 1, 1, 1 /

  REAL (KIND=dp), PUBLIC, DIMENSION(ntropl,0:noptm) :: emiopl    ! O+ Emissivity

!-------------------------------------------------------------------------------------
! S+
!-------------------------------------------------------------------------------------
! Energy levels.
!        1 - 4So J = 3/2
!        2 - 2Do J = 3/2
!        3 - 2Do J = 5/2
!        4 - 2Po J = 1/2
!        5 - 2Po J = 3/2

  INTEGER, PUBLIC, PARAMETER                 :: nlvspl = 5       ! Number of levels

  REAL (kind=dp), PUBLIC, DIMENSION (nlvspl) :: gstspl           ! Statistical weight
  DATA gstspl                                 / 4.0_dp, 4.0_dp, 6.0_dp, 2.0_dp, 4.0_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (nlvspl) :: elkspl           ! Level energy
  DATA elkspl                                 / 0.0_dp, 21370.92_dp, 21416.66_dp &
                                              , 35287.17_dp, 35354.38_dp /

  REAL (kind=dp), PUBLIC                     :: amspl = 32.0_dp  ! Mass (amu)

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvspl,0:noptm) :: xrespl    ! C+ levels relative abundance

! Transitions
!       1: 3-2 -> 0.31456 mm (953.04 GHz)
!       2: 5-4 -> 0.21409 mm (1400.33 GHz)
!       3: 4-3 -> 1.0373 micron
!       4: 4-2 -> 1.0339 micron
!       5: 5-3 -> 1.0323 micron
!       6: 5-2 -> 1.0290 micron
!       7: 2-1 -> 6730.82 Angstrom
!       8: 3-1 -> 6716.44 Angstrom
!       9: 4-1 -> 4076.35 Angstrom
!      10: 5-1 -> 4068.60 Angstrom

  INTEGER, PUBLIC, PARAMETER                 :: ntrspl = 10      ! Number of transitions

  REAL (kind=dp), PUBLIC, DIMENSION (ntrspl) :: wlkspl           ! Energy of transition
  DATA wlkspl                                 / 45.74_dp, 67.21_dp, 13870.51_dp &
                                              , 13916.25_dp, 13937.72_dp, 13983.46_dp &
                                              , 21370.92_dp, 21416.66_dp, 35287.17_dp &
                                              , 35354.38_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (ntrspl) :: aijspl           ! Transition probability
  DATA aijspl                                 / 3.35e-7_dp, 1.03e-6_dp, 7.79e-2 &
                                              , 1.63e-1_dp, 1.79e-1_dp, 1.33e-1_dp &
                                              , 8.82e-4_dp, 2.60e-4_dp, 9.06e-2_dp &
                                              , 2.25e-1_dp /

  INTEGER, PUBLIC, DIMENSION (ntrspl)        :: iupspl           ! Upper state of transition
  DATA iupspl                                 / 3, 5, 4, 4, 5, 5, 2, 3, 4, 5 /

  INTEGER, PUBLIC, DIMENSION (ntrspl)        :: jlospl           ! Lower state of transition
  DATA jlospl                                 / 2, 4, 3, 2, 3, 2, 1, 1, 1, 1 /

  REAL (KIND=dp), PUBLIC, DIMENSION(ntrspl,0:noptm) :: emispl    ! C+ Emissivity

!-------------------------------------------------------------------------------------
! Si+
!-------------------------------------------------------------------------------------
! Energy levels.
!        1 - 2P J = 1/2
!        2 - 2P J = 3/2
!        3 - 4P J = 1/2
!        4 - 4P J = 3/2
!        5 - 4P J = 5/2

  INTEGER, PUBLIC, PARAMETER                 :: nlvsip = 5       ! Number of levels

  REAL (kind=dp), PUBLIC, DIMENSION (nlvsip) :: gstsip           ! Statistical weight
  DATA gstsip                                 / 2.0_dp, 4.0_dp, 2.0_dp, 4.0_dp, 6.0_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (nlvsip) :: elksip           ! Level energy
  DATA elksip                                 /     0.0_dp,    413.29_dp, 61617.06_dp &
                                              , 61772.93_dp, 62025.14_dp /

  REAL (kind=dp), PUBLIC                     :: amsip = 28.0_dp  ! Mass (amu)

! Transitions
!       1: 2-1 -> 34.814 micron
!       2: 3-2 -> 2350.17 Angstrom
!       3: 4-2 -> 2344.20 Angstrom
!       4: 5-2 -> 2334.60 Angstrom
!       5: 3-1 -> 2334.40 Angstrom
!       6: 4-1 -> 2328.52 Angstrom

  REAL (KIND=dp), PUBLIC, DIMENSION(nlvsip,0:noptm) :: xresip    ! Si+ levels relative abundances

  INTEGER, PUBLIC, PARAMETER                 :: ntrsip = 6       ! Number of transitions

  REAL (kind=dp), PUBLIC, DIMENSION (ntrsip) :: wlksip           ! Energy of transition
  DATA wlksip                                 /   413.29_dp, 61203.77_dp, 61359.64_dp &
                                              , 61611.85_dp, 61617.06_dp, 61772.93_dp /

  REAL (kind=dp), PUBLIC, DIMENSION (ntrsip) :: aijsip           ! Transition probability
  DATA aijsip                                 / 2.13e-4_dp, 4.9e+3_dp, 1.7e+3_dp &
                                              , 2.7e+3_dp,  6.3e+3_dp, 2.0e+1_dp /

  INTEGER, PUBLIC, DIMENSION (ntrsip)        :: iupsip           ! Upper state of transition
  DATA iupsip                                 / 2, 3, 4, 5, 3, 4 /

  INTEGER, PUBLIC, DIMENSION (ntrsip)        :: jlosip           ! Lower state of transition
  DATA jlosip                                 / 1, 2, 2, 2, 1, 1 /

  REAL (KIND=dp), PUBLIC, DIMENSION(ntrsip,0:noptm) :: emisip    ! Si+ Emissivity

!------------------------------------------------------------------------------------------
! H3+                                                                                     !
!------------------------------------------------------------------------------------------
! Contrairement aux autres especes les niveaux d'energie et les coefficients d'Einstein 
! ne sont pas definis ici. Ils seront lus dans des fichiers dans stv1b.f au moment de 
! l'initialisation

Type niveaux !------------------------------------------
   REAL (KIND=dp)   :: en  ! Energie en Kelvin         !
   INTEGER          :: J   ! Nombre quantique J        !
   INTEGER          :: K   ! Nombre quantique K        !
   REAL (KIND=dp)   :: g   ! Degenerescence            !
   INTEGER          :: n   ! Numero du niveau          !
end type niveaux !--------------------------------------

REAL (KIND=dp), PUBLIC, PARAMETER    :: amh3p = 3.0_dp      ! masse de H3p

! Emax est ce qui definit le dernier niveau conserve
!REAL (KIND=dp), PUBLIC, PARAMETER  :: Emaxh3p = 17000.00_dp  ! Emax en cm-1
!REAL (KIND=dp), PUBLIC, PARAMETER  :: Emaxh3p = 2320.372_dp  ! Emax en cm-1 (7,0)
 REAL (KIND=dp), PUBLIC, PARAMETER  :: Emaxh3p = 2396.426_dp  ! Emax en cm-1 (9,8)
!REAL (KIND=dp), PUBLIC, PARAMETER  :: Emaxh3p =  169.295_dp  ! Emax en cm-1 (2,2) 3 niveaux
!REAL (KIND=dp), PUBLIC, PARAMETER  :: Emaxh3p =  237.356_dp  ! Emax en cm-1 (2,1) 4 niveaux

REAL (KIND=dp), PUBLIC     :: enmaxh3p  ! Energie(K) max consideree
                                        ! eventuellement diff de Emax
                                        ! si Emax n'est pas exactement
                                        ! l'energie d'un niveau
INTEGER, PUBLIC            :: jmaxh3p   ! J max considere
INTEGER, PUBLIC            :: kmaxh3p   ! K max considere

! Donnees sur les niveaux
INTEGER, PUBLIC                      :: nlvh3p      ! Nombre de niveaux conserves
TYPE(niveaux), PUBLIC, ALLOCATABLE   :: elevh3p(:)  ! Donnees sur les niveaux  
INTEGER, PUBLIC, ALLOCATABLE         :: levh3p(:,:) ! Correspondance J,K -> lev
! Donnees sur les niveaux ranges dans tes tableaux sous le meme modele que les
! autres especes. Elles sont redondantes avec elevh3p mais ce sont les autres
! especes qu'il faudrait ranger sur le modele de la structure elevh3p
REAL (KIND=dp), PUBLIC, ALLOCATABLE  :: elkh3p(:)
REAL (KIND=dp), PUBLIC, ALLOCATABLE  :: gsth3p(:)



! Transitions radiatives
INTEGER, PUBLIC                      :: ntrh3p      ! nombre de transitions
REAL (KIND=dp), PUBLIC, ALLOCATABLE  :: wlkh3p(:)   ! longueur d'onde en Kelvin
REAL (KIND=dp), PUBLIC, ALLOCATABLE  :: aijh3p(:)   ! Coefficients d'Einstein
INTEGER       , PUBLIC, ALLOCATABLE  :: iuph3p(:)   ! numero du niveau haut
INTEGER       , PUBLIC, ALLOCATABLE  :: jloh3p(:)   ! numero du niveau bas

! Populations
REAL (KIND=dp), PUBLIC, ALLOCATABLE  :: xreh3p(:,:) ! Abondance relative des niveaux
REAL (KIND=dp), PUBLIC, ALLOCATABLE  :: emih3p(:,:) ! Emissivite des transitions


END MODULE STR_DATA
