
! Copyright (C) 2002 - Observatoire de Paris/Meudon
!                      Universite Paris 7
!                      CNRS (France)
!                      Contact : Jacques.Lebourlot@obspm.fr
!
! This program is free software; you can redistribute it and/or
! modify it under the terms of the GNU General Public License version 2
! as published by the Free Software Foundation.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

MODULE CONVER

  use constantes

! Convergence variables

  REAL (KIND=dp), PUBLIC                              :: varmax       ! convergence tolerance, target
  REAL (KIND=dp), PUBLIC                              :: thevar       ! convergence tolerance, done
  REAL (KIND=dp), PUBLIC                              :: dtaumin      ! min relative variation of tau
  REAL (KIND=dp), PUBLIC                              :: errx         ! relative error on abundances
  INTEGER                                             :: mauvai       ! identity of worse species
  REAL (KIND=dp), PUBLIC                              :: ceqh2        ! relative error on H2 levels
  REAL (KIND=dp), PUBLIC                              :: ceqhd        ! relative error on HD levels
  REAL (KIND=dp), PUBLIC                              :: bthrel       ! relative error on thermal balance

! Local variables (used for current iteration)

  REAL (KIND=dp), PUBLIC                              :: temp         ! Temperature
  REAL (KIND=dp), PUBLIC                              :: av           ! Av
  REAL (KIND=dp), PUBLIC                              :: abelec       ! Electron abundance

! Backup variables (for relative variations)

  REAL (KIND=dp), PUBLIC                              :: oldhat       ! H
  REAL (KIND=dp), PUBLIC                              :: oldhmo       ! H2
  REAL (KIND=dp), PUBLIC                              :: oldhd        ! HD
  REAL (KIND=dp), PUBLIC                              :: oldco        ! CO
  REAL (KIND=dp), PUBLIC                              :: oldc         ! C
END MODULE CONVER

MODULE PROFIL

! Cloud profile related data

  use constantes
  use STR_DATA
  use CONVER

!  Variables used in numerical algorithms (sort of...)

  INTEGER, PUBLIC                                     :: isoneside    ! 1 if one side, 0 if two sides
  INTEGER, PUBLIC                                     :: ifafm =  20  ! Number of global iteration on whole structure
  REAL (KIND=dp), PUBLIC, DIMENSION(:), ALLOCATABLE   :: tau_imp      ! Mandatory points in optical depth grid
  INTEGER, PUBLIC                                     :: is_imp = 0   ! 1: we are currently computing a mandatory point
  INTEGER, PUBLIC                                     :: next_imp = 1 ! next mandatory point

!  Model defining variables

  CHARACTER (LEN=35), PUBLIC                          :: modele       ! Root name of output files
  CHARACTER (LEN=35), PUBLIC                          :: chimie       ! Name of chemistry file
  CHARACTER (LEN=35), PUBLIC                          :: los_ext      ! Line of sight extinction curve
  CHARACTER (LEN=35), PUBLIC                          :: srcpp        ! Radiation source (star)
  REAL (KIND=dp), PUBLIC                              :: d_sour       ! Distance of radiation source (0.0 -> infinity)
  REAL (KIND=dp), PUBLIC                              :: rstar        ! Radius of emitting source (read in rsun, converted to cm)
  REAL (KIND=dp), PUBLIC                              :: teff         ! Effective temperature (K)
  REAL (KIND=dp), PUBLIC                              :: taumax       ! Maximal optical depth computed
  REAL (KIND=dp), PUBLIC                              :: Avmax        ! Maximal AV computed (taumax * 2.5 log10(e))
  REAL (KIND=dp), PUBLIC                              :: NHtot        ! Total proton column density considered
  REAL (KIND=dp), PUBLIC                              :: opdinf       ! Optical depth for 1 sided clouds
  REAL (KIND=dp), PUBLIC                              :: nh_init      ! Initial density
  REAL (KIND=dp), PUBLIC                              :: tg_init      ! Initial temperature
  REAL (KIND=dp), PUBLIC                              :: presse       ! External pressure (if ifisob = 2)
  REAL (KIND=dp), PUBLIC                              :: fmrc         ! Cosmic rays inisation scaling
  REAL (KIND=dp), PUBLIC                              :: zeta         ! Cosmic rays ionisatin rate (zeta = 1.0e-17 * fmrc)
  REAL (KIND=dp), PUBLIC                              :: fphsec       ! Secondary photons flux
  REAL (KIND=dp), PUBLIC                              :: vturb        ! Turbulent velocity (Doppler broadening only)
  REAL (KIND=dp), PUBLIC                              :: radm         ! Radiation field intensity on left side
  REAL (KIND=dp), PUBLIC                              :: radp         ! Radiation field intensity on right side
  REAL (KIND=dp), PUBLIC                              :: radm0        ! Radiation field intensity at infinity on left side
  REAL (KIND=dp), PUBLIC                              :: radp0        ! Radiation field intensity at infinity on right side
  REAL (KIND=dp), PUBLIC                              :: cdunit       ! Av to gaz Column Density convertion
  REAL (KIND=dp), PUBLIC                              :: rrr          ! Ratio Av / E(B-V)
  REAL (KIND=dp), PUBLIC                              :: alb          ! Dust albedo
  REAL (KIND=dp), PUBLIC                              :: gg           ! Anisotropy (<cos(theta)>)
  REAL (KIND=dp), PUBLIC                              :: gratio       ! Grain/gaz mass ratio
  REAL (KIND=dp), PUBLIC                              :: rhogr        ! Grain density
  REAL (KIND=dp), PUBLIC                              :: alpgr        ! Grain size distribution index
  REAL (KIND=dp), PUBLIC                              :: rgrmin       ! Minimum grain size
  REAL (KIND=dp), PUBLIC                              :: rgrmax       ! Maximum grain size
  REAL (KIND=dp), PUBLIC                              :: dsite = 2.6e-8_dp ! Mean distance between sites on grains
  REAL (KIND=dp), PUBLIC                              :: cc1, cc2, cc3  !  Fitzpatrick fit parameters
  REAL (KIND=dp), PUBLIC                              :: cc4, xgam, y0  !  Fitzpatrick fit parameters
  INTEGER, PUBLIC                                     :: ifeqth       ! 1 : Temperature from thermal balance
  INTEGER, PUBLIC                                     :: ifisob       ! 2 : Isobaric state equation
  INTEGER, PUBLIC                                     :: l0h2         ! Highest H2 level included
  INTEGER, PUBLIC                                     :: ichh2        ! H + H2 collisions
  INTEGER, PUBLIC, PARAMETER                          :: l0hd = 9     ! Highest HD level included
  INTEGER, PUBLIC                                     :: itrfer       ! Which Radiative transfer approximation?
  INTEGER, PUBLIC                                     :: iforh2       ! H2 formation mechanism
  INTEGER, PUBLIC                                     :: istic        ! H Sticking coefficient
  INTEGER, PUBLIC                                     :: jlyman = 0   ! Include (1) or not (0) H Lyman lines
  INTEGER, PUBLIC                                     :: jfgkh2 = 0   ! Lowest J treated by FGK approximation
  INTEGER, PUBLIC                                     :: jfgkco0 = 0  ! Number of CO level included in full transfer
  INTEGER, PUBLIC, PARAMETER                          :: mjhd = 8     ! Upper J level of HD included
  INTEGER, PUBLIC                                     :: ifrac        ! Number of slice in fractal grid
  REAL (KIND=dp), PUBLIC, DIMENSION(:), ALLOCATABLE   :: tfrac        ! Tau grid for fractal model
  REAL (KIND=dp), PUBLIC, DIMENSION(:), ALLOCATABLE   :: nfrac        ! nH grid for fractal model
  INTEGER, PUBLIC                                     :: pr_np        ! Number of points in interpolation grid
  REAL (KIND=dp), PUBLIC, DIMENSION(:), ALLOCATABLE   :: pr_tau       ! Tau grid for interpolation
  REAL (KIND=dp), PUBLIC, DIMENSION(:), ALLOCATABLE   :: pr_tem       ! Temperature grid for interpolation
  REAL (KIND=dp), PUBLIC, DIMENSION(:), ALLOCATABLE   :: pr_den       ! Density grid for interpolation

!  Variables used for current iteration

  REAL (KIND=dp), PUBLIC, DIMENSION(0:noptm)          :: tau          ! Optical depth profile
  REAL (KIND=dp), PUBLIC, DIMENSION(0:noptm)          :: densh        ! Density profile
  REAL (KIND=dp), PUBLIC, DIMENSION(0:noptm)          :: tgaz         ! Temperature profile
  REAL (KIND=dp), PUBLIC, DIMENSION(0:nes,0:noptm)    :: abnua        ! species abundances
  REAL (KIND=dp), PUBLIC, DIMENSION(nlevh2,0:noptm)   :: xreh2        ! H2 levels relative abundances (X level)
  REAL (KIND=dp), PUBLIC, DIMENSION(0:nvbc,0:njbc,0:noptm) :: h2bnua  ! H2 populations (B level)
  REAL (KIND=dp), PUBLIC, DIMENSION(0:nvbc,0:njbc,0:noptm) :: h2cnua  ! H2 populations (C level)
  REAL (KIND=dp), PUBLIC, DIMENSION(0:noptm)          :: timh21       ! Formation and destruction of H2
  REAL (KIND=dp), PUBLIC, DIMENSION(0:noptm)          :: timh22       ! Formation and destruction of H2
  REAL (KIND=dp), PUBLIC, DIMENSION(0:noptm)          :: timh23       ! Formation and destruction of H2
  REAL (KIND=dp), PUBLIC, DIMENSION(0:noptm)          :: timh24       ! Formation and destruction of H2
  REAL (KIND=dp), PUBLIC, DIMENSION(npmg,0:noptm)     :: t_25         ! Formation of H2 on grains
  REAL (KIND=dp), PUBLIC, DIMENSION(nlevhd,0:noptm)   :: xrehd        ! HD levels relative abundances
  REAL (KIND=dp), PUBLIC, DIMENSION(0:noptm)          :: timhd1       ! Formation and destruction of HD
  REAL (KIND=dp), PUBLIC, DIMENSION(0:noptm)          :: timhd2       ! Formation and destruction of HD
  REAL (KIND=dp), PUBLIC, DIMENSION(0:noptm)          :: timhd3       ! Formation and destruction of HD
  REAL (KIND=dp), PUBLIC, DIMENSION(0:noptm)          :: timhd4       ! Formation and destruction of HD
  REAL (KIND=dp), PUBLIC, DIMENSION(0:noptm)          :: timco1       ! Formation and destruction of CO
  REAL (KIND=dp), PUBLIC, DIMENSION(0:noptm)          :: timco2       ! Formation and destruction of CO
  REAL (KIND=dp), PUBLIC, DIMENSION(0:noptm)          :: timco3       ! Formation and destruction of CO

  REAL (KIND=dp), PUBLIC, DIMENSION(nlevh2,0:noptm)   :: cdth2        ! H2 weighted column density (by T)
  REAL (KIND=dp), PUBLIC, DIMENSION(nlevhd,0:noptm)   :: cdthd        ! HD weighted column density (by T)
  REAL (KIND=dp), PUBLIC, DIMENSION(nlvco0,0:noptm)   :: cdtc0        ! CO0 weighted column density (by T)
  REAL (KIND=dp), PUBLIC, DIMENSION(nlvco0,0:noptm)   :: cdtc1        ! CO1 weighted column density (by T)
  REAL (KIND=dp), PUBLIC, DIMENSION(nlvco0,0:noptm)   :: cdtc2        ! CO2 weighted column density (by T)
  REAL (KIND=dp), PUBLIC, DIMENSION(nlvco0,0:noptm)   :: cdtc3        ! CO3 weighted column density (by T)

  REAL (KIND=dp), PUBLIC, DIMENSION(0:np)             :: ab           ! abundances at current point (temporary variable)
  REAL (KIND=dp), PUBLIC, DIMENSION(0:miter)          :: tb           ! temperature iteration at point iopt

!  Variables saved from previous iteration

  INTEGER, PUBLIC                                     :: npo          ! Number of points in previous grid
  INTEGER, PUBLIC                                     :: n_imp        ! Number of imposed point in Av
  REAL (KIND=dp), PUBLIC, DIMENSION(:), ALLOCATABLE   :: tau_o        ! previous optical depth profile (dust at V)
  REAL (KIND=dp)                                      :: abhm
  REAL (KIND=dp)                                      :: abhp
  REAL (KIND=dp), PUBLIC, DIMENSION(:), ALLOCATABLE   :: abh_o        ! previous atomic hydrogen profile
  REAL (KIND=dp), PUBLIC, DIMENSION(:), ALLOCATABLE   :: abh2_o       ! previous molecular hydrogen profile
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: xh2_o        ! previous H2(J) fractional abundance
  REAL (KIND=dp), PUBLIC, DIMENSION(:), ALLOCATABLE   :: abco0_o      ! previous molecular hydrogen profile
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: xco0_o       ! previous H2(J) fractional abundance
  REAL (KIND=dp), PUBLIC, DIMENSION(:), ALLOCATABLE   :: tgaz_o       ! previous temperature profile
  REAL (KIND=dp), PUBLIC, DIMENSION(:), ALLOCATABLE   :: dens_o       ! previous density profile
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: cdth2_o      ! previous integrated weighted H2 profile
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: cdthd_o      ! previous integrated weighted HD profile
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: cdtc0_o      ! previous integrated weighted CO0 profile
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: cdtc1_o      ! previous integrated weighted CO1 profile
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: cdtc2_o      ! previous integrated weighted CO2 profile
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: cdtc3_o      ! previous integrated weighted CO3 profile

  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdcat_o     ! previous optical depth in lines of C
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdnat_o     ! previous optical depth in lines of N
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdoat_o     ! previous optical depth in lines of O
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdsat_o     ! previous optical depth in lines of S
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdsiat_o    ! previous optical depth in lines of Si
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdcpl_o     ! previous optical depth in lines of C+
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdnpl_o     ! previous optical depth in lines of N+
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdopl_o     ! previous optical depth in lines of O+
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdspl_o     ! previous optical depth in lines of S+
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdsip_o     ! previous optical depth in lines of Si+
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdco0_o     ! previous optical depth in lines of CO0
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdco1_o     ! previous optical depth in lines of CO1
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdco2_o     ! previous optical depth in lines of CO2
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdco3_o     ! previous optical depth in lines of CO3
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdcs0_o     ! previous optical depth in lines of CS0
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdhcop_o    ! previous optical depth in lines of HCO+
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdh2o_o     ! previous optical depth in lines of H2O
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdh3p_o     ! previous optical depth in lines of H3+  

  REAL (KIND=dp), PUBLIC, DIMENSION(ntrcat,0:noptm)   :: opdcat       ! optical depth in lines of C
  REAL (KIND=dp), PUBLIC, DIMENSION(ntrnat,0:noptm)   :: opdnat       ! optical depth in lines of N
  REAL (KIND=dp), PUBLIC, DIMENSION(ntroat,0:noptm)   :: opdoat       ! optical depth in lines of O
  REAL (KIND=dp), PUBLIC, DIMENSION(ntrsat,0:noptm)   :: opdsat       ! optical depth in lines of S
  REAL (KIND=dp), PUBLIC, DIMENSION(ntrsiat,0:noptm)  :: opdsiat      ! optical depth in lines of Si
  REAL (KIND=dp), PUBLIC, DIMENSION(ntrcpl,0:noptm)   :: opdcpl       ! optical depth in lines of C+
  REAL (KIND=dp), PUBLIC, DIMENSION(ntrnpl,0:noptm)   :: opdnpl       ! optical depth in lines of S+
  REAL (KIND=dp), PUBLIC, DIMENSION(ntropl,0:noptm)   :: opdopl       ! optical depth in lines of O+
  REAL (KIND=dp), PUBLIC, DIMENSION(ntrspl,0:noptm)   :: opdspl       ! optical depth in lines of S+
  REAL (KIND=dp), PUBLIC, DIMENSION(ntrsip,0:noptm)   :: opdsip       ! optical depth in lines of Si+
  REAL (KIND=dp), PUBLIC, DIMENSION(ntrco0,0:noptm)   :: opdco0       ! optical depth in lines of CO0
  REAL (KIND=dp), PUBLIC, DIMENSION(ntrco0,0:noptm)   :: opdco1       ! optical depth in lines of CO1
  REAL (KIND=dp), PUBLIC, DIMENSION(ntrco0,0:noptm)   :: opdco2       ! optical depth in lines of CO2
  REAL (KIND=dp), PUBLIC, DIMENSION(ntrco0,0:noptm)   :: opdco3       ! optical depth in lines of CO3
  REAL (KIND=dp), PUBLIC, DIMENSION(ntrcs0,0:noptm)   :: opdcs0       ! optical depth in lines of CS0
  REAL (KIND=dp), PUBLIC, DIMENSION(ntrhcop,0:noptm)  :: opdhcop      ! optical depth in lines of HCO+
  REAL (KIND=dp), PUBLIC, DIMENSION(ntrh2o,0:noptm)   :: opdh2o       ! optical depth in lines of H2O
  REAL (KIND=dp), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdh3p       ! optical depth in lines of H3+



CONTAINS

  subroutine INIT_PROFIL

    use constantes
    use inoutput
    USE STR_DATA

    implicit none

!   REAL (KIND=dp)                   :: pas = 1.0e-8_dp, t
    REAL (KIND=dp)                   :: pas = 1.0e-6_dp, t
    REAL (KIND=dp)                   :: tauc, tt, toto
    REAL (KIND=dp)                   :: rap_0, rap_n
!   INTEGER, PARAMETER               :: n_obl = 46
    INTEGER, PARAMETER               :: n_obl = 36
    REAL (KIND=dp), DIMENSION(n_obl) :: pt_obl
    INTEGER                          :: npp, i
    INTEGER                          :: n_o
    INTEGER                          :: n_int, n, j

!   data pt_obl / 1.0e-8_dp, 2.0e-8_dp, 3.0e-8_dp, 5.0e-8_dp, 7.0e-8_dp &
!               , 1.0e-7_dp, 2.0e-7_dp, 3.0e-7_dp, 5.0e-7_dp, 7.0e-7_dp &
    data pt_obl / 1.0e-6_dp, 2.0e-6_dp, 3.0e-6_dp, 5.0e-6_dp, 7.0e-6_dp &
                , 1.0e-5_dp, 2.0e-5_dp, 3.0e-5_dp, 5.0e-5_dp, 7.0e-5_dp &
                , 1.0e-4_dp, 2.0e-4_dp, 3.0e-4_dp, 5.0e-4_dp, 7.0e-4_dp &
                , 1.0e-3_dp, 2.0e-3_dp, 3.0e-3_dp, 5.0e-3_dp, 7.0e-3_dp &
                , 1.0e-2_dp, 2.0e-2_dp, 3.0e-2_dp, 5.0e-2_dp, 7.0e-2_dp &
                , 0.1_dp, 0.2_dp, 0.3_dp, 0.5_dp, 0.7_dp &
                , 1.0_dp, 2.0_dp, 3.0_dp, 5.0_dp, 7.0_dp, 1.0e10_dp /

!  Control of variable step size (in tau)

       if (ifisob == 5) then
         varmax = 1.0_dp
         dtaumin = 1.0e-3_dp
       else
         varmax = 0.5_dp
         dtaumin = 1.0e-2_dp
       endif

    pt_obl = pt_obl / tautav

!  If l0h2 < 40, it is not possible to have 1/3 of H2 formation
!     energy in internal energy

    if (iforh2 == 0 .AND. l0h2 < 40) then
      print *, "   WARNING !: iforh2 changed to 1 (not enough internal energy)"
      iforh2 = 1
    endif

!  If radp = 0.0 then radiation field is on one side only
!     and there is no need to iterate. Else, iteration is mandatory

    radm0 = radm
    radp0 = radp

    if (radp == 0.0_dp) then
      isoneside = 1
    else
      isoneside = 0
    endif

    if (isoneside == 1) then
      opdinf = 1.0e99_dp
    else
      opdinf = 0.0e0_dp
    endif

    taumax = Avmax / tautav
    pas = pas / tautav
    tau(0) = 0.0_dp
    tau(1) = pas * 0.5_dp

!  30 mai2002, linear interpolation in temperature and density grid

!   if (abs(ifisob) == 1 .OR. abs(ifisob) == 2) then
    if (abs(ifisob) == 1) then
      fichier = TRIM(data_dir)//TRIM(modele)//'.pfl'
      open (66, file=fichier, status="old")
      read (66,*) pr_np
      if (ifisob < 0) then
        ALLOCATE (pr_tau(0:2*pr_np))
        ALLOCATE (pr_tem(0:2*pr_np))
        ALLOCATE (pr_den(0:2*pr_np))
      else
        ALLOCATE (pr_tau(0:pr_np))
        ALLOCATE (pr_tem(0:pr_np))
        ALLOCATE (pr_den(0:pr_np))
      endif
      do i=0,pr_np
        read (66,*) pr_tau(i), pr_tem(i), pr_den(i)
        pr_tau(i) = pr_tau(i) / tautav
      end do
      nh_init = pr_den(0)
      tg_init = pr_tem(0)
      close (66)
      if (ifisob < 0) then
        do i=0,pr_np-1
          pr_tau(2*pr_np-i) = 2.0_dp * pr_tau(pr_np) - pr_tau(i)
          pr_tem(2*pr_np-i) = pr_tem(i)
          pr_den(2*pr_np-i) = pr_den(i)
        end do
        ifisob = -ifisob
        pr_np = pr_np * 2
      endif

    endif

!  3 mai 2002, Fractal model grid

    if (ifisob == 5) then

      print *, "  Warning : ifisob =", ifisob

      fichier = TRIM(data_dir)//TRIM(modele)//'.pfl'
      open (66, file=fichier, status="old")
      read (66,*) ifrac
      ALLOCATE (tfrac(0:ifrac))
      ALLOCATE (nfrac(0:ifrac))
      do i=0,ifrac
        read (66,*) tfrac(i), nfrac(i)
        tfrac(i) = taumax * tfrac(i)
        nfrac(i) = nfrac(i) * nh_init
      end do
      close (66)

      if (isoneside == 1) then

        npp = int(log10(tfrac(1)/pas)) + 1
        npo = npp + ifrac
        ALLOCATE(tau_imp(0:npo))
        n_imp = npo
        tau_imp(0) = 0.0_dp
        do i=1,npp
          tau_imp(i) = pas
          pas = pas * 10.0_dp
        end do
        tau_imp(npp+1:npp+ifrac) = tfrac(1:ifrac)
        do i=1,npo
          print *, i, tau_imp(i), tau_imp(i) * tautav
        end do

      else

        npp = int(log10(tfrac(1)/pas))
        npo = 2 * npp + ifrac
        ALLOCATE(tau_imp(0:npo))
        n_imp = npo
        tau_imp(0) = 0.0_dp
        do i=1,npp
          tau_imp(i) = pas
          tau_imp(npo-i) = taumax - pas
          pas = pas * 10.0_dp
        end do
        tau_imp(npp+1:npp+ifrac-1) = tfrac(1:ifrac-1)
        tau_imp(npo) = taumax
        do i=1,npo
          print *, i, tau_imp(i), tau_imp(i) * tautav
        end do
      endif
    else
!  Build "mandatory" points
!  8 oct 2003 : same points whether or not we are "two-sided"

      tauc = 0.5_dp * taumax

      do i=1,n_obl
        if (tauc <= pt_obl(i)) then
          n_o = i - 1
          exit
        endif
      end do
      n_int = 2
!     n_int = 1

      npo = 2 * (n_o + n_int * n_o) + 2
      ALLOCATE(tau_imp(0:npo))
      tau_imp(0) = 0.0_dp
      tau_imp(npo/2) = tauc
      tau_imp(npo) = taumax

      do i=1,n_o-1
        n = 1 + (i-1) * (n_int+1)
        tau_imp(n) = pt_obl(i)
        tau_imp(npo-n) = taumax - pt_obl(i)
        tt = (pt_obl(i+1) / pt_obl(i))**(1.0/(n_int+1))
        do j=1,n_int
          tau_imp(n+j) = pt_obl(i) * tt**j
          tau_imp(npo-n-j) = taumax - pt_obl(i) * tt**j
        end do
      end do
      n = 1 + (n_o-1) * (n_int+1)
      tau_imp(n) = pt_obl(n_o)
      tau_imp(npo-n) = taumax - pt_obl(n_o)
      tt = (tauc / pt_obl(i))**(1.0/(n_int+1))
      do j=1,n_int
        tau_imp(n+j) = pt_obl(i) * tt**j
        tau_imp(npo-n-j) = taumax - pt_obl(i) * tt**j
      end do

      n_imp = npo
      do i=0,npo
        print *, i, tau_imp(i), tau_imp(i) * tautav
      end do
    endif

!  Initial integrated weighted molecular profiles and optical depth

    ALLOCATE (tau_o(0:npo))
    ALLOCATE (abh_o(0:npo))
    ALLOCATE (abh2_o(0:npo))
    if (jfgkh2 > 0) then
      ALLOCATE (xh2_o(0:jfgkh2-1,0:npo))
    endif
    ALLOCATE (abco0_o(0:npo))
    if (jfgkco0 > 0) then
      ALLOCATE (xco0_o(0:jfgkco0-1,0:npo))
    endif
    ALLOCATE (tgaz_o(0:npo))
    ALLOCATE (dens_o(0:npo))
    ALLOCATE (cdth2_o(nlevh2,0:npo))
    ALLOCATE (cdthd_o(nlevhd,0:npo))
    ALLOCATE (cdtc0_o(nlvco0,0:npo))
    ALLOCATE (cdtc1_o(nlvco0,0:npo))
    ALLOCATE (cdtc2_o(nlvco0,0:npo))
    ALLOCATE (cdtc3_o(nlvco0,0:npo))

    ALLOCATE (opdcat_o(ntrcat,0:npo))
    ALLOCATE (opdnat_o(ntrnat,0:npo))
    ALLOCATE (opdoat_o(ntroat,0:npo))
    ALLOCATE (opdsat_o(ntrsat,0:npo))
    ALLOCATE (opdsiat_o(ntrsiat,0:npo))
    ALLOCATE (opdcpl_o(ntrcpl,0:npo))
    ALLOCATE (opdnpl_o(ntrnpl,0:npo))
    ALLOCATE (opdopl_o(ntropl,0:npo))
    ALLOCATE (opdspl_o(ntrspl,0:npo))
    ALLOCATE (opdsip_o(ntrsip,0:npo))
    ALLOCATE (opdco0_o(ntrco0,0:npo))
    ALLOCATE (opdco1_o(ntrco0,0:npo))
    ALLOCATE (opdco2_o(ntrco0,0:npo))
    ALLOCATE (opdco3_o(ntrco0,0:npo))
    ALLOCATE (opdcs0_o(ntrcs0,0:npo))
    ALLOCATE (opdhcop_o(ntrhcop,0:npo))
    ALLOCATE (opdh2o_o(ntrh2o,0:npo))
    ALLOCATE (opdh3p_o(ntrh3p,0:npo))

    tau_o = tau_imp
    do i=1,npo-1
      if (tau_o(i) == 0.0_dp) then
        abhm = 1.0_dp
      else
        abhm = 0.5_dp - atan(10.0_dp * log10(1.0e4*tau_o(i)/radm)) / xpi
      endif
      if (isoneside == 1) then
        t = tau_o(i)
        abhp = 0.0_dp
      else
        t = min(tau_o(i),taumax-tau_o(i))
        if (t == 0.0_dp) then
          abhp = 1.0_dp
        else
          abhp = 0.5_dp - atan(10.0_dp &
               * log10(1.0e4*(taumax-tau_o(i))/radp)) / xpi
        endif
      endif
!     abh_o(i) = nh_init * exp(-5.0_dp*t) + 1.0_dp
      abh_o(i) = nh_init * max(abhm, abhp)
      abh2_o(i) = (nh_init - abh_o(i)) * 0.5_dp + 1.0_dp
      toto = nh_init / (abh_o(i) + 2.0_dp * abh2_o(i))
      abh_o(i) = abh_o(i) * toto
      abh2_o(i) = (nh_init - abh_o(i)) * 0.5_dp
      tgaz_o(i) = tg_init * exp(-3.0_dp*t) + 10.0_dp
    end do

!  Extrapolate to edges

    rap_0 = tau_o(1) / (tau_o(1) - tau_o(2))
    rap_n = (tau_o(npo) - tau_o(npo-2)) / (tau_o(npo-1) - tau_o(npo-2))

    abh_o(0) = abh_o(1) + (abh_o(2) - abh_o(1)) * rap_0
    abh2_o(0) = abh2_o(1) + (abh2_o(2) - abh2_o(1)) * rap_0
    tgaz_o(0) = tgaz_o(1) + (tgaz_o(2) - tgaz_o(1)) * rap_0
    abh_o(npo) = abh_o(npo-2) + (abh_o(npo-1) - abh_o(npo-2)) * rap_n
    abh2_o(npo) = abh2_o(npo-2) + (abh2_o(npo-1) - abh2_o(npo-2)) * rap_n
    tgaz_o(npo) = tgaz_o(npo-2) + (tgaz_o(npo-1) - tgaz_o(npo-2)) * rap_n

    if (jfgkh2 > 0) then
      xh2_o(0,:) = 0.25_dp
    endif
    if (jfgkh2 > 1) then
      xh2_o(1,:) = 0.75_dp
    endif
    if (jfgkh2 > 2) then
      do i=2,jfgkh2-1
        xh2_o(i,:) = 10.0_dp**(-i)
      end do
    endif
    abco0_o = 1.0e-10_dp
    if (jfgkco0 > 0) then
      xco0_o(:,:) = 1.0_dp / jfgkco0
    endif
    dens_o = nh_init

    opdcat_o = SPREAD(tau_imp, 1, ntrcat) * 1.0e-3_dp
    opdnat_o = SPREAD(tau_imp, 1, ntrnat) * 1.0e-3_dp
    opdoat_o = SPREAD(tau_imp, 1, ntroat) * 1.0e-3_dp
    opdsat_o = SPREAD(tau_imp, 1, ntrsat) * 1.0e-3_dp
    opdsiat_o = SPREAD(tau_imp, 1, ntrsiat) * 1.0e-3_dp
    opdcpl_o = SPREAD(tau_imp, 1, ntrcpl) * 1.0e-3_dp
    opdnpl_o = SPREAD(tau_imp, 1, ntrnpl) * 1.0e-3_dp
    opdopl_o = SPREAD(tau_imp, 1, ntropl) * 1.0e-3_dp
    opdspl_o = SPREAD(tau_imp, 1, ntrspl) * 1.0e-3_dp
    opdsip_o = SPREAD(tau_imp, 1, ntrsip) * 1.0e-3_dp
    opdco0_o = SPREAD(tau_imp, 1, ntrco0) * 1.0e-3_dp
    opdco1_o = SPREAD(tau_imp, 1, ntrco0) * 1.0e-3_dp
    opdco2_o = SPREAD(tau_imp, 1, ntrco0) * 1.0e-3_dp
    opdco3_o = SPREAD(tau_imp, 1, ntrco0) * 1.0e-3_dp
    opdcs0_o = SPREAD(tau_imp, 1, ntrcs0) * 1.0e-3_dp
    opdhcop_o = SPREAD(tau_imp, 1, ntrhcop) * 1.0e-3_dp
    opdh2o_o = SPREAD(tau_imp, 1, ntrh2o) * 1.0e-3_dp
    opdh3p_o = SPREAD(tau_imp, 1, ntrh3p) * 1.0e-3_dp

!  Initialisation of various other profiles

    if (ifisob == 5) then
      densh = nfrac(1)
    else
      densh = nh_init
    endif
    tgaz  = tg_init
    zeta = 1.0e-17_dp * fmrc

!  Secondary photons flux due to cosmic rays
!  1E3 cm-2 s-1 (Roberge, in Dalgarno's book)

   fphsec = 1.0e3_dp * fmrc

!  conversion of turbulent velocity from km s-1 to cm s-1

   vturb = vturb * 1.0e5_dp

!  Extinction curve

   call FM_PAR

  END subroutine INIT_PROFIL

!      %%%%%%%%%%%%%%%%
       subroutine REALL
!      %%%%%%%%%%%%%%%%

       use constantes
       use STR_DATA
       use SP_DATA

       implicit none

       integer :: i

       DEALLOCATE (tau_o)
       DEALLOCATE (abh_o)
       DEALLOCATE (abh2_o)
       if (jfgkh2 > 0) then
         DEALLOCATE (xh2_o)
       endif
       DEALLOCATE (abco0_o)
       if (jfgkco0 > 0) then
         DEALLOCATE (xco0_o)
       endif
       DEALLOCATE (tgaz_o)
       DEALLOCATE (dens_o)
       DEALLOCATE (cdth2_o)
       DEALLOCATE (cdthd_o)
       DEALLOCATE (cdtc0_o)
       DEALLOCATE (cdtc1_o)
       DEALLOCATE (cdtc2_o)
       DEALLOCATE (cdtc3_o)

       DEALLOCATE (opdcat_o)
       DEALLOCATE (opdnat_o)
       DEALLOCATE (opdoat_o)
       DEALLOCATE (opdsat_o)
       DEALLOCATE (opdsiat_o)
       DEALLOCATE (opdcpl_o)
       DEALLOCATE (opdnpl_o)
       DEALLOCATE (opdopl_o)
       DEALLOCATE (opdspl_o)
       DEALLOCATE (opdsip_o)
       DEALLOCATE (opdco0_o)
       DEALLOCATE (opdco1_o)
       DEALLOCATE (opdco2_o)
       DEALLOCATE (opdco3_o)
       DEALLOCATE (opdcs0_o)
       DEALLOCATE (opdhcop_o)
       DEALLOCATE (opdh2o_o)
       DEALLOCATE (opdh3p_o)

       ALLOCATE (tau_o(0:npo))
       ALLOCATE (abh_o(0:npo))
       ALLOCATE (abh2_o(0:npo))
       if (jfgkh2 > 0) then
         ALLOCATE (xh2_o(0:jfgkh2-1,0:npo))
       endif
       ALLOCATE (abco0_o(0:npo))
       if (jfgkco0 > 0) then
         ALLOCATE (xco0_o(0:jfgkco0-1,0:npo))
       endif
       ALLOCATE (tgaz_o(0:npo))
       ALLOCATE (dens_o(0:npo))
       ALLOCATE (cdth2_o(nlevh2,0:npo))
       ALLOCATE (cdthd_o(nlevhd,0:npo))
       ALLOCATE (cdtc0_o(nlvco0,0:npo))
       ALLOCATE (cdtc1_o(nlvco0,0:npo))
       ALLOCATE (cdtc2_o(nlvco0,0:npo))
       ALLOCATE (cdtc3_o(nlvco0,0:npo))

       ALLOCATE (opdcat_o(ntrcat,0:npo))
       ALLOCATE (opdnat_o(ntrnat,0:npo))
       ALLOCATE (opdoat_o(ntroat,0:npo))
       ALLOCATE (opdsat_o(ntrsat,0:npo))
       ALLOCATE (opdsiat_o(ntrsiat,0:npo))
       ALLOCATE (opdcpl_o(ntrcpl,0:npo))
       ALLOCATE (opdnpl_o(ntrnpl,0:npo))
       ALLOCATE (opdopl_o(ntropl,0:npo))
       ALLOCATE (opdspl_o(ntrspl,0:npo))
       ALLOCATE (opdsip_o(ntrsip,0:npo))
       ALLOCATE (opdco0_o(ntrco0,0:npo))
       ALLOCATE (opdco1_o(ntrco0,0:npo))
       ALLOCATE (opdco2_o(ntrco0,0:npo))
       ALLOCATE (opdco3_o(ntrco0,0:npo))
       ALLOCATE (opdcs0_o(ntrcs0,0:npo))
       ALLOCATE (opdhcop_o(ntrhcop,0:npo))
       ALLOCATE (opdh2o_o(ntrh2o,0:npo))
       ALLOCATE (opdh3p_o(ntrh3p,0:npo))

       call CDT

       tau_o = tau(0:npo)
       dens_o = densh(0:npo)
       tgaz_o = tgaz(0:npo)
       abh_o = abnua(i_h,0:npo)
       abh2_o = abnua(i_h2,0:npo)
       do i=0,jfgkh2-1
         xh2_o(i,:) = xreh2(i+1,0:npo)
       end do
       abco0_o = abnua(i_co0,0:npo)
       do i=0,jfgkco0-1
         xco0_o(i,:) = xreco0(i+1,0:npo)
       end do
       cdth2_o = cdth2(:,0:npo)
       cdthd_o = cdthd(:,0:npo)
       cdtc0_o = cdtc0(:,0:npo)
       cdtc1_o = cdtc1(:,0:npo)
       cdtc2_o = cdtc2(:,0:npo)
       cdtc3_o = cdtc3(:,0:npo)

       opdcat_o = opdcat(:,0:npo)
       opdnat_o = opdnat(:,0:npo)
       opdoat_o = opdoat(:,0:npo)
       opdsat_o = opdsat(:,0:npo)
       opdsiat_o = opdsiat(:,0:npo)
       opdcpl_o = opdcpl(:,0:npo)
       opdnpl_o = opdnpl(:,0:npo)
       opdopl_o = opdopl(:,0:npo)
       opdspl_o = opdspl(:,0:npo)
       opdsip_o = opdsip(:,0:npo)
       opdco0_o = opdco0(:,0:npo)
       opdco1_o = opdco1(:,0:npo)
       opdco2_o = opdco2(:,0:npo)
       opdco3_o = opdco3(:,0:npo)
       opdcs0_o = opdcs0(:,0:npo)
       opdhcop_o = opdhcop(:,0:npo)
       opdh2o_o = opdh2o(:,0:npo)
       opdh3p_o = opdh3p(:,0:npo)

       return
       end subroutine REALL

!      %%%%%%%%%%%%%%
       subroutine CDT
!      %%%%%%%%%%%%%%

       use constantes
!      use auxiliaries
!      USE RF_TOOLS
       USE SP_DATA
       USE H2_DATA
       USE HD_DATA
       USE STR_DATA

        implicit none

        REAL (KIND=dp)  :: ct2, ct3, deltau
        REAL (KIND=dp)  :: b0sb_p, b0sb_i, vt2, cden
        INTEGER         :: iopt, jopt, lev, j

!     *********************************************************************
!               vturb donne en cm s-1
!               ct2= 2k/mua   = 1.662902e8  en cgs
!               bdop:  dimension vitesse cm s-1 a
!      *****************************************************************

!  CDUNIT gas to dust ratio : NH = CDUNIT * E(B-V)
!  RRR extinction to color index ratio : Av = RRR * E(B-V)

    ct2 = 2.0_dp * xk / amu
    ct3 = cdunit * tautav / rrr
    vt2 = vturb * vturb

    do iopt=0,npo

       if (iopt > 0) then
         jopt = iopt - 1
       else
         jopt = 0
       endif
         deltau = tau(iopt) - tau(jopt)

!  Doppler width weighted column density of H2 for each level

       b0sb_p = vturb / sqrt(vt2 + ct2 * tgaz(jopt) / amh2)
       b0sb_i = vturb / sqrt(vt2 + ct2 * tgaz(iopt) / amh2)

       do lev=1,l0h2
         cden = ct3 * deltau * 0.5_dp &
              * ((abnua(i_h2,iopt) * xreh2(lev,iopt) * b0sb_i / densh(iopt)) &
              + (abnua(i_h2,jopt) * xreh2(lev,jopt) * b0sb_p / densh(jopt)))
         if (cden < 0.0_dp) then
           print *, "CDH2", lev, jopt, xreh2(lev,jopt)
           print *, "CDH2", lev, iopt, xreh2(lev,iopt)
!          stop
           cden = 0.0_dp
         endif
         cdth2(lev,iopt) = cdth2(lev,jopt) + cden
       end do

!  Doppler width weighted column density of HD for each level

       if (i_hd /= 0) then
         b0sb_p = vturb / sqrt(vt2 + ct2 * tgaz(jopt) / amhd)
         b0sb_i = vturb / sqrt(vt2 + ct2 * tgaz(iopt) / amhd)

         do lev=1,l0hd
           cden = ct3 * deltau * 0.5_dp &
                * ((abnua(i_hd,iopt) * xrehd(lev,iopt) * b0sb_i / densh(iopt)) &
                + (abnua(i_hd,jopt) * xrehd(lev,jopt) * b0sb_p / densh(jopt)))
           cdthd(lev,iopt) = cdthd(lev,jopt) + cden
         end do
       endif

!  Doppler width weighted column density of CO for each level

       b0sb_p = vturb / sqrt(vt2 + ct2 * tgaz(jopt) / amco0)
       b0sb_i = vturb / sqrt(vt2 + ct2 * tgaz(iopt) / amco0)

       do j=1,nlvco0
         cden = ct3 * deltau * 0.5_dp &
          * ((abnua(i_co0,iopt) * xreco0(j,iopt) * b0sb_i / densh(iopt)) &
          + (abnua(i_co0,jopt) * xreco0(j,jopt) * b0sb_p / densh(jopt)))
         cdtc0(j,iopt) = cdtc0(j,jopt) + cden
       end do

!  Doppler width weighted column density of C*O for each level

       if (i_c13 /= 0) then

         b0sb_p = vturb / sqrt(vt2 + ct2 * tgaz(jopt) / amco1)
         b0sb_i = vturb / sqrt(vt2 + ct2 * tgaz(iopt) / amco1)

         do j=1,nlvco0
           cden = ct3 * deltau * 0.5_dp &
             * ((abnua(i_co1,iopt) * xreco1(j,iopt) * b0sb_i / densh(iopt)) &
             + (abnua(i_co1,jopt) * xreco1(j,jopt) * b0sb_p / densh(jopt)))
           cdtc1(j,iopt) = cdtc1(j,jopt) + cden
         end do

       endif

!  Doppler width weighted column density of CO* for each level

       if (i_o18 /= 0) then

         b0sb_p = vturb / sqrt(vt2 + ct2 * tgaz(jopt) / amco2)
         b0sb_i = vturb / sqrt(vt2 + ct2 * tgaz(iopt) / amco2)

         do j=1,nlvco0
           cden = ct3 * deltau * 0.5_dp &
             * ((abnua(i_co2,iopt) * xreco2(j,iopt) * b0sb_i / densh(iopt)) &
             + (abnua(i_co2,jopt) * xreco2(j,jopt) * b0sb_p / densh(jopt)))
           cdtc2(j,iopt) = cdtc2(j,jopt) + cden
         end do

       endif

!  Doppler width weighted column density of C*O* for each level

       if (i_c13 /= 0 .and. i_o18 /= 0) then

         b0sb_p = vturb / sqrt(vt2 + ct2 * tgaz(jopt) / amco3)
         b0sb_i = vturb / sqrt(vt2 + ct2 * tgaz(iopt) / amco3)

         do j=1,nlvco0
           cden = ct3 * deltau * 0.5_dp &
             * ((abnua(i_co3,iopt) * xreco3(j,iopt) * b0sb_i / densh(iopt)) &
             + (abnua(i_co3,jopt) * xreco3(j,jopt) * b0sb_p / densh(jopt)))
           cdtc3(j,iopt) = cdtc3(j,jopt) + cden
         end do

       endif

    end do

    return
    end subroutine CDT

!   %%%%%%%%%%%%%%%%%
    subroutine FM_PAR
!   %%%%%%%%%%%%%%%%%

!  Fitzpatrick and Massa data base of extinction curves
      USE CONSTANTES
      USE INOUTPUT
      IMPLICIT NONE
      
    INTEGER           :: i
    INTEGER           :: los_found ! flag=1 if line of sight found in data
    CHARACTER(len=20) :: com_dummy ! informations not used in the program
    CHARACTER(len=15) :: los       ! name of the line of sight
    REAL(KIND=dp)     :: rrr_true  ! Rv found in litterature for this l.o.s.

    write(iscrn,*) los_ext

       fichier = TRIM(ADJUSTL(data_dir))//"line_of_sight.dat"
       OPEN(10,file=fichier,status="old",action="read")
       READ(10,*) ! commentaire
       READ(10,*) ! commentaire
       los_found = 0
       DO i = 1, 10000
          READ(10,*,END=110) los, rrr_true, y0, xgam, cc1, cc2, cc3, cc4, com_dummy 
          los = TRIM(ADJUSTL(los))
          IF (los == "End_of_file") EXIT
          IF (los == TRIM(ADJUSTL(los_ext))) THEN
             los_found = 1
             IF (rrr /= rrr_true) THEN
                WRITE(iscrn,*)"**** WARNING ****"
                WRITE(iscrn,*)"Line of sight : ",TRIM(ADJUSTL(los_ext))
                WRITE(iscrn,*)"Entered value for Rv is :    ",rrr
                WRITE(iscrn,*)"Proper  value seems to be :  ",rrr_true
                WRITE(iscrn,*)"Program continues with Rv =  ",rrr
!                WRITE(iscrn,*)"Type enter to continue"
!                PAUSE
             ENDIF
             EXIT
          ENDIF
      ENDDO
110   continue
      CLOSE(10)
      
      IF (los_found == 0) THEN
         WRITE(ISCRN,*) "**** WARNING ****"
         WRITE(ISCRN,*) "Line of sight not found in 'line_of_sight.dat': "
         WRITE(ISCRN,*) "Line of sight searched : ",los_ext
         WRITE(ISCRN,*) "Automatic switch to 'GALAXY' extinction curve"
         WRITE(ISCRN,*) "Automatic switch to Rv = 3.1"
!         WRITE(ISCRN,*) "Type Enter to continue"
!         PAUSE
          y0   =  4.59_dp    
          xgam =  1.05_dp
          cc1  = -0.38_dp
          cc2  =  0.74_dp    
          cc3  =  3.96_dp
          cc4  =  0.26_dp
          rrr  =  3.1_dp
      ENDIF
      
    return
    end subroutine FM_PAR

END MODULE PROFIL

