
! Copyright (C) 2002 - Observatoire de Paris/Meudon
!                      Universite Paris 7
!                      CNRS (France)
!                      Contact : Jacques.Lebourlot@obspm.fr
! 
! This program is free software; you can redistribute it and/or
! modify it under the terms of the GNU General Public License version 2
! as published by the Free Software Foundation.
! 
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
! 
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


MODULE H2_DATA

! H2 structure related quantities
! This module replaces old stvbld1.f

  use constantes

! H2 levels
!    ordered by increasing energy

  REAL (kind=dp), PUBLIC                                      :: amh2 = 2.0_dp  ! Mass (amu)
  REAL (kind=dp), PUBLIC                                      :: h2para ! Total abundance of para-H2
  REAL (kind=dp), PUBLIC                                      :: h2orth ! Total abundance of ortho-H2
  REAL (kind=dp), PUBLIC, DIMENSION (nlevh2)                  :: elhm   ! Energies inside electonic state X (K)
  INTEGER, PUBLIC, DIMENSION (nlevh2)                         :: njlev  ! Rotational number J
  INTEGER, PUBLIC, DIMENSION (nlevh2)                         :: nvlev  ! Vibrational number v
  INTEGER, PUBLIC, DIMENSION (0:nvxh2,0:njh2)                 :: level  ! Reverse indexing (Level number from (v,J))
  REAL (kind=dp), PUBLIC, DIMENSION (0:nvbh2,0:njh2+1,nlevh2) :: aeinb  ! Prob of emission B-X
  REAL (kind=dp), PUBLIC, DIMENSION (0:nvch2,0:njh2+1,nlevh2) :: aeinc  ! Prob of emission C-X
  REAL (kind=dp), PUBLIC, DIMENSION (0:nvbh2,0:njh2+1)        :: aeitb  ! Total prob of emission B-X to bound states
  REAL (kind=dp), PUBLIC, DIMENSION (0:nvch2,0:njh2+1,2)      :: aeitc  ! Total prob of emission B-X to bound states
  REAL (kind=dp), PUBLIC, DIMENSION (nlevh2)                  :: aeitq  ! Total quadrupole prob of emission 
  REAL (kind=dp), PUBLIC, DIMENSION (1:2,0:nvbh2,0:njh2)      :: tmunh2bc ! Inverse life time of H2 B and C states

  REAL (KIND=dp), PUBLIC, DIMENSION (0:nvxh2,0:njh2)          :: pophm    ! Probability to form H2(v,J) on grain
  REAL (KIND=dp), PUBLIC, DIMENSION (nlevh2,nlevh2)           :: cashm    ! H2 cascade coeficients
  REAL (KIND=dp), PUBLIC, DIMENSION (0:nvbh2,0:njh2+1,nlevh2) :: bhmbx    ! H2 B-X cascade coeficients
  REAL (KIND=dp), PUBLIC, DIMENSION (0:nvch2,0:njh2+1,nlevh2) :: bhmcx    ! H2 C-X cascade coeficients
  REAL (KIND=dp), PUBLIC                                      :: emoyh2   ! Mean energy of H2 formed on grain

  REAL (KIND=dp), PUBLIC, DIMENSION (0:nvbh2,0:njh2+1,nlevh2) :: escpbx   ! H2 B-X escape probability
  REAL (KIND=dp), PUBLIC, DIMENSION (0:nvch2,0:njh2+1,nlevh2) :: escpcx   ! H2 C-X escape probability
  REAL (KIND=dp), PUBLIC, DIMENSION (0:nvbh2,0:njh2+1)        :: pbstbx   ! Probability of downward transition without reabsorptio
  REAL (KIND=dp), PUBLIC, DIMENSION (0:nvch2,0:njh2+1,0:1)    :: pbstcx   ! Probability of downward transition without reabsorptio

! 21 VIII 2003 - reduce size by using allocatable area

  REAL (kind=dp), PUBLIC, ALLOCATABLE, DIMENSION (:,:,:)      :: qh1h2  ! Collision cross section H + H2
  REAL (kind=dp), PUBLIC, ALLOCATABLE, DIMENSION (:,:,:)      :: qheh2  ! Collision cross section He + H2
  REAL (kind=dp), PUBLIC, ALLOCATABLE, DIMENSION (:,:,:)      :: qph2h2 ! Collision cross section pH2 + H2
  REAL (kind=dp), PUBLIC, ALLOCATABLE, DIMENSION (:,:,:)      :: qoh2h2 ! Collision cross section oH2 + H2
 
  REAL (kind=dp), PUBLIC, ALLOCATABLE, DIMENSION (:,:)        :: evltot ! H2 equilibrium
  REAL (kind=dp), PUBLIC, ALLOCATABLE, DIMENSION (:,:)        :: evcolh ! H2 collisional equilibrium
  REAL (kind=dp), PUBLIC, ALLOCATABLE, DIMENSION (:,:)        :: evrdbc ! H2 radiative equilibrium (B+C)
  REAL (kind=dp), PUBLIC, ALLOCATABLE, DIMENSION (:,:)        :: evlqua ! H2 quadrupole transitions

  DATA (njlev(nnh2), nnh2=1,120) / &
        0,  1,  2,  3,  4,  5,  6,  7,  8,  0,  1,  2,  3,  9,  4, &
        5, 10,  6, 11,  7,  8,  0,  1, 12,  2,  3,  9,  4, 13,  5, &
       10,  6, 14, 11,  7,  8,  0,  1, 12,  2, 15,  3,  9,  4, 13, &
        5, 16, 10,  6, 14, 11,  7, 17,  8,  0,  1,  2, 12, 15,  3, &
        9,  4, 18, 13,  5, 10, 16,  6, 19, 14,  7, 11, 17,  0,  8, &
        1,  2, 12, 15,  3, 20,  9,  4, 18,  5, 13, 10, 16,  6, 21, &
        7, 14, 11, 19, 17,  0,  8,  1,  2, 12,  3, 15, 22,  9,  4, &
       20, 18,  5, 13, 10, 16,  6, 23, 21,  7, 11, 14, 19,  0, 17 /

  DATA (njlev(nnh2), nnh2=121,240) / &
        8,  1,  2, 12,  3, 15,  9,  4, 22, 24, 20,  5, 18, 13, 10, &
        6, 16,  7, 11, 23, 14, 21, 25, 19,  0,  1,  8, 17,  2,  3, &
       12, 15,  9,  4, 22,  5, 20, 24, 26, 18, 13, 10,  6, 16,  7, &
       11, 14, 21, 23,  0,  1, 19,  8, 25,  2, 17,  3, 27, 12,  4, &
        9, 15,  5, 22, 20, 13, 10, 24,  6, 18, 26, 16,  7, 28, 11, &
       14,  0,  1,  8,  2, 21, 19, 23,  3, 17, 12, 25,  4,  9, 15, &
        5, 27,  6, 10, 13, 29, 20, 22, 18, 24,  7, 16,  0, 11,  1, &
       26,  2, 14,  8,  3, 28,  4, 19, 12, 21,  9, 17, 23,  5, 15 /

  DATA (njlev(nnh2), nnh2=241,318) / &
       25,  6, 10, 13, 27,  7,  0,  1, 18, 20,  2, 11, 16, 22,  8, &
        3, 14, 29, 24,  4,  9,  5, 12, 26,  6, 19, 17, 15, 21, 10, &
        7,  0,  1, 13, 28, 23,  2,  3,  8, 11,  4, 25,  5, 16, 18, &
        9, 14, 20,  6, 12,  0,  1,  2,  7,  3, 22, 27, 10,  4, 15, &
       17, 24, 19, 29, 21, 26, 18, 23, 20, 28, 25, 22, 24, 27, 26, &
       29, 28, 29 /

   DATA (nvlev(nnh2), nnh2=1,120) / &
        0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  1,  1,  0,  1, &
        1,  0,  1,  0,  1,  1,  2,  2,  0,  2,  2,  1,  2,  0,  2, &
        1,  2,  0,  1,  2,  2,  3,  3,  1,  3,  0,  3,  2,  3,  1, &
        3,  0,  2,  3,  1,  2,  3,  0,  3,  4,  4,  4,  2,  1,  4, &
        3,  4,  0,  2,  4,  3,  1,  4,  0,  2,  4,  3,  1,  5,  4, &
        5,  5,  3,  2,  5,  0,  4,  5,  1,  5,  3,  4,  2,  5,  0, &
        5,  3,  4,  1,  2,  6,  5,  6,  6,  4,  6,  3,  0,  5,  6, &
        1,  2,  6,  4,  5,  3,  6,  0,  1,  6,  5,  4,  2,  7,  3 /

   DATA (nvlev(nnh2), nnh2=121,240) / &
        6,  7,  7,  5,  7,  4,  6,  7,  1,  0,  2,  7,  3,  5,  6, &
        7,  4,  7,  6,  1,  5,  2,  0,  3,  8,  8,  7,  4,  8,  8, &
        6,  5,  7,  8,  2,  8,  3,  1,  0,  4,  6,  7,  8,  5,  8, &
        7,  6,  3,  2,  9,  9,  4,  8,  1,  9,  5,  9,  0,  7,  9, &
        8,  6,  9,  3,  4,  7,  8,  2,  9,  5,  1,  6,  9,  0,  8, &
        7, 10, 10,  9, 10,  4,  5,  3, 10,  6,  8,  2, 10,  9,  7, &
       10,  1, 10,  9,  8,  0,  5,  4,  6,  3, 10,  7, 11,  9, 11, &
        2, 11,  8, 10, 11,  1, 11,  6,  9,  5, 10,  7,  4, 11,  8 /

   DATA (nvlev(nnh2), nnh2=241,318) / &
        3, 11, 10,  9,  2, 11, 12, 12,  7,  6, 12, 10,  8,  5, 11, &
       12,  9,  1,  4, 12, 11, 12, 10,  3, 12,  7,  8,  9,  6, 11, &
       12, 13, 13, 10,  2,  5, 13, 13, 12, 11, 13,  4, 13,  9,  8, &
       12, 10,  7, 13, 11, 14, 14, 14, 13, 14,  6,  3, 12, 14, 10, &
       9,  5,  8,  2,   7,  4,  9,  6,  8,  3,  5,  7,  6,  4,  5, &
       3,  4,  4 /

  DATA (elhm(nnh2), nnh2=1,40) / &
              0.000,   170.502,   509.850,  1015.153,  1681.678, &
           2503.870,  3474.434,  4586.377,  5829.758,  5987.124, &
           6149.208,  6471.607,  6951.587,  7196.995,  7584.601, &
           8365.297,  8677.322,  9286.624, 10261.792, 10341.518, &
          11521.835, 11635.733, 11789.515, 11940.306, 12095.238, &
          12550.500, 12817.518, 13150.723, 13703.177, 13890.700, &
          14221.112, 14764.143, 15540.247, 15722.289, 15763.382, &
          16880.549, 16952.836, 17098.373, 17311.221, 17387.795 /

  DATA (elhm(nnh2), nnh2=41,80) / &
          17443.910, 17818.695, 18107.412, 18386.518, 18979.717, &
          19086.494, 19403.453, 19434.906, 19912.254, 20717.855, &
          20853.826, 20856.531, 21411.676, 21911.713, 21942.777, &
          22080.041, 22353.262, 22355.348, 22517.115, 22759.832, &
          23069.986, 23295.639, 23459.896, 23930.797, 23955.805, &
          24322.619, 24368.402, 24734.037, 25539.627, 25570.533, &
          25623.812, 25659.885, 26263.691, 26606.766, 26617.039, &
          26735.959, 26992.889, 27074.184, 27266.252, 27375.230 /

  DATA (elhm(nnh2), nnh2=81,120) / &
          27644.508, 27706.766, 27878.793, 28194.861, 28498.986, &
          28556.613, 28883.758, 29009.398, 29229.539, 29766.611, &
          30064.307, 30097.846, 30139.904, 30154.623, 30792.355, &
          30943.201, 30995.520, 31064.123, 31304.639, 31466.506, &
          31662.332, 31690.260, 31900.314, 32015.752, 32133.217, &
          32134.541, 32606.393, 32712.721, 32855.164, 33116.461, &
          33324.887, 33394.957, 34037.586, 34129.527, 34173.625, &
          34289.781, 34296.805, 34445.047, 34946.219, 34994.133 /

  DATA (elhm(nnh2), nnh2=121,160) / &
          35041.168, 35058.781, 35282.359, 35527.305, 35614.730, &
          35784.195, 35990.469, 36051.891, 36131.387, 36174.594, &
          36299.789, 36589.641, 36689.840, 36820.027, 37013.016, &
          37221.918, 37308.078, 37942.629, 38101.133, 38134.469, &
          38159.793, 38164.746, 38304.844, 38405.027, 38605.531, &
          38709.152, 38744.379, 38861.109, 38915.137, 39221.090, &
          39246.109, 39538.586, 39619.730, 39623.188, 40032.648, &
          40117.109, 40131.270, 40133.152, 40421.898, 40434.516 /

  DATA (elhm(nnh2), nnh2=161,200) / &
          40439.551, 40560.828, 40696.930, 40947.719, 41356.547, &
          41559.379, 41672.746, 41862.758, 41897.305, 41904.672, &
          41998.988, 42021.488, 42088.828, 42120.223, 42186.219, &
          42379.039, 42464.160, 42523.281, 42607.152, 42828.875, &
          42885.941, 42937.391, 43276.066, 43591.270, 43613.586, &
          43694.766, 43740.203, 43752.191, 43799.785, 43823.902, &
          44091.062, 44224.586, 44393.922, 44602.437, 44642.695, &
          44814.273, 44820.293, 44904.406, 45050.965, 45071.078 /

  DATA (elhm(nnh2), nnh2=201,240) / &
          45203.754, 45274.934, 45310.113, 45318.832, 45526.270, &
          45585.129, 45590.184, 45642.844, 45763.332, 45955.801, &
          46039.023, 46039.270, 46501.262, 46522.500, 46558.371, &
          46651.246, 46722.871, 46783.262, 46832.426, 47011.863, &
          47023.129, 47110.781, 47319.051, 47319.797, 47391.961, &
          47405.453, 47536.273, 47552.187, 47597.180, 47749.867, &
          47952.945, 48028.355, 48134.426, 48145.086, 48159.504, &
          48214.391, 48268.770, 48345.066, 48367.199, 48556.676 /

  DATA (elhm(nnh2), nnh2=241,280) / &
          48690.359, 48759.910, 48866.586, 48988.832, 49192.242, &
          49199.504, 49355.098, 49415.141, 49419.375, 49421.691, &
          49533.586, 49542.867, 49561.629, 49574.621, 49677.516, &
          49707.934, 49839.281, 49846.375, 49879.855, 49933.695, &
          50184.500, 50205.566, 50232.484, 50335.230, 50516.559, &
          50550.988, 50554.293, 50684.164, 50684.266, 50709.660, &
          50857.828, 50864.664, 50909.195, 50922.578, 50937.527, &
          50958.477, 50996.547, 51123.570, 51219.555, 51239.895 /

  DATA (elhm(nnh2), nnh2=281,318) / &
          51284.961, 51377.781, 51474.129, 51508.043, 51519.969, &
          51588.758, 51597.535, 51649.160, 51681.449, 51759.258, &
          51759.645, 51784.309, 51831.215, 51894.137, 51895.676, &
          51906.203, 51939.012, 51948.453, 51967.906, 52236.187, &
          52291.715, 52296.707, 52439.984, 52644.312, 52696.355, &
          52824.656, 53002.297, 53072.582, 53282.969, 53491.012, &
          53575.293, 53662.430, 54154.066, 54207.992, 54764.906, &
          54972.961, 55490.238, 56641.590 /

   DATA (level(ivh2,0), ivh2=0,14) /    1,  10,  22,  37,  55, &
          74,  96, 119, 145, 170, 197, 223, 247, 272, 291 /

   DATA (level(ivh2,1), ivh2=0,14) /    2,  11,  23,  38,  56, &
          76,  98, 122, 146, 171, 198, 225, 248, 273, 292 /

   DATA (level(ivh2,2), ivh2=0,14) /    3,  12,  25,  40,  57, &
          77,  99, 123, 149, 175, 200, 227, 251, 277, 293 /

   DATA (level(ivh2,3), ivh2=0,14) /    4,  13,  26,  42,  60, &
          80, 101, 125, 150, 177, 204, 230, 256, 278, 295 /

   DATA (level(ivh2,4), ivh2=0,14) /    5,  15,  28,  44,  62, &
          83, 105, 128, 154, 180, 208, 232, 260, 281, 299 /

   DATA (level(ivh2,5), ivh2=0,14) /    6,  16,  30,  46,  65, &
          85, 108, 132, 156, 183, 211, 239, 262, 283,   0 /

   DATA (level(ivh2,6), ivh2=0,14) /    7,  18,  32,  49,  68, &
          89, 112, 136, 163, 189, 213, 242, 265, 289,   0 /

   DATA (level(ivh2,7), ivh2=0,14) /    8,  20,  35,  52,  71, &
          91, 115, 138, 165, 193, 221, 246, 271, 294,   0 /

   DATA (level(ivh2,8), ivh2=0,14) /    9,  21,  36,  54,  75, &
          97, 121, 147, 173, 199, 229, 255, 279,   0,   0 /

   DATA (level(ivh2,9), ivh2=0,14) /   14,  27,  43,  61,  82, &
         104, 127, 153, 181, 209, 236, 261, 286,   0,   0 /

   DATA (level(ivh2,10), ivh2=0,14) /  17,  31,  48,  66,  87, &
         110, 135, 162, 187, 214, 243, 270, 298,   0,   0 /

   DATA (level(ivh2,11), ivh2=0,14) /  19,  34,  51,  72,  93, &
         116, 139, 166, 195, 224, 252, 280,   0,   0,   0 /

   DATA (level(ivh2,12), ivh2=0,14) /  24,  39,  58,  78, 100, &
         124, 151, 179, 206, 234, 263, 290,   0,   0,   0 /

   DATA (level(ivh2,13), ivh2=0,14) /  29,  45,  64,  86, 109, &
         134, 161, 186, 215, 244, 274,   0,   0,   0,   0 /

   DATA (level(ivh2,14), ivh2=0,14) /  33,  50,  70,  92, 117, &
         141, 167, 196, 228, 257, 287,   0,   0,   0,   0 /

   DATA (level(ivh2,15), ivh2=0,14) /  41,  59,  79, 102, 126, &
         152, 182, 210, 240, 268, 300,   0,   0,   0,   0 /

   DATA (level(ivh2,16), ivh2=0,14) /  47,  67,  88, 111, 137, &
         164, 192, 222, 253, 284,   0,   0,   0,   0,   0 /

   DATA (level(ivh2,17), ivh2=0,14) /  53,  73,  95, 120, 148, &
         176, 205, 237, 267, 301,   0,   0,   0,   0,   0 /

   DATA (level(ivh2,18), ivh2=0,14) /  63,  84, 107, 133, 160, &
         190, 219, 249, 285, 307,   0,   0,   0,   0,   0 /

   DATA (level(ivh2,19), ivh2=0,14) /  69,  94, 118, 144, 172, &
         202, 233, 266, 303,   0,   0,   0,   0,   0,   0 /

   DATA (level(ivh2,20), ivh2=0,14) /  81, 106, 131, 157, 185, &
         217, 250, 288, 309,   0,   0,   0,   0,   0,   0 /

   DATA (level(ivh2,21), ivh2=0,14) /  90, 114, 142, 168, 201, &
         235, 269, 305,   0,   0,   0,   0,   0,   0,   0 /

   DATA (level(ivh2,22), ivh2=0,14) / 103, 129, 155, 184, 218, &
         254, 296, 312,   0,   0,   0,   0,   0,   0,   0 /

   DATA (level(ivh2,23), ivh2=0,14) / 113, 140, 169, 203, 238, &
         276, 308,   0,   0,   0,   0,   0,   0,   0,   0 /

   DATA (level(ivh2,24), ivh2=0,14) / 130, 158, 188, 220, 259, &
         302, 313,   0,   0,   0,   0,   0,   0,   0,   0 /

   DATA (level(ivh2,25), ivh2=0,14) / 143, 174, 207, 241, 282, &
         311,   0,   0,   0,   0,   0,   0,   0,   0,   0 /

   DATA (level(ivh2,26), ivh2=0,14) / 159, 191, 226, 264, 306, &
         315,   0,   0,   0,   0,   0,   0,   0,   0,   0 /

   DATA (level(ivh2,27), ivh2=0,14) / 178, 212, 245, 297, 314, &
           0,   0,   0,   0,   0,   0,   0,   0,   0,   0 /

   DATA (level(ivh2,28), ivh2=0,14) / 194, 231, 275, 310, 317, &
           0,   0,   0,   0,   0,   0,   0,   0,   0,   0 /

   DATA (level(ivh2,29), ivh2=0,14) / 216, 258, 304, 316, 318, &
           0,   0,   0,   0,   0,   0,   0,   0,   0,   0 /

!  Entry rates (Gredel and Dalgarno)

   REAL (kind=dp), PUBLIC, DIMENSION (0:nvxh2,0:njh2)   :: entry
   REAL (kind=dp), PUBLIC, DIMENSION (0:njh2)           :: popetl
   REAL (kind=dp), PUBLIC                               :: sentry
   REAL (kind=dp), PUBLIC                               :: paretl
   REAL (kind=dp), PUBLIC                               :: ortetl

!  Introduction des Entry rates d'Alex Dalgarno
!    (excitation par les electrons secondaires)

!  Pour l'instant, on ne fait rien...

         DATA entry / 450*0.0_dp /

!  Calcul fait par R. Gredel, pour:
!        Un taux d'ionisation de 1e-8
!        Des electrons secondaires de 30 eV
!        Une temperature d'excitation de H2 de 30 K (ETL)
!        J = 0 a jmax = 7, v = 0 a 14
!       le deuxieme jeu de valeurs est relatif a T = 100K

!    DATA (entry(0,j), j=0,7) / 0.00e+00, 0.00e+00, 0.66e+00, 0.11e-01 &
!                             , 0.59e-07, 0.12e-13, 0.35e-25, 0.15e-36 /
 
!    DATA (entry(0,j), j=0,7) / 0.00e+00, 0.00e+00, 0.35e+00, 0.32e+00 &
!                             , 0.45e-02, 0.12e-03, 0.00e-00, 0.00e-00 /

!    DATA (entry(1,j), j=0,7) / 0.27e+01, 0.82e-01, 0.27e+01, 0.82e-01 &
!                             , 0.12e-06, 0.38e-13, 0.41e-23, 0.20e-34 /
 
!    DATA (entry(1,j), j=0,7) / 0.10e+01, 0.17e+01, 0.10e+01, 0.17e+01 &
!                             , 0.69e-02, 0.28e-03, 0.17e-06, 0.18e-09 /

!    DATA (entry(2,j), j=0,7) / 0.20e-00, 0.61e-02, 0.20e+00, 0.60e-02 &
!                             , 0.99e-08, 0.29e-14, 0.31e-24, 0.15e-35 /

!    DATA (entry(2,j), j=0,7) / 0.76e-01, 0.12e-00, 0.77e-01, 0.12e-00 &
!                             , 0.55e-03, 0.21e-04, 0.13e-07, 0.13e-10 /
 
!    DATA (entry(3,j), j=0,7) / 0.78e-02, 0.37e-03, 0.11e-01, 0.19e-03 &
!                             , 0.11e-08, 0.21e-15, 0.00e+00, 0.00e+00 /
 
!    DATA (entry(3,j), j=0,7) / 0.31e-02, 0.76e-02, 0.42e-02, 0.40e-02 &
!                             , 0.64e-04, 0.16e-05, 0.83e-09, 0.79e-12 /

!    DATA (entry(4,j), j=0,7) / 0.74e-02, 0.35e-03, 0.97e-02, 0.18e-03 &
!                             , 0.10e-08, 0.20e-15, 0.00e+00, 0.00e+00 /
 
!    DATA (entry(4,j), j=0,7) / 0.29e-02, 0.71e-02, 0.39e-02, 0.37e-02 &
!                             , 0.59e-04, 0.15e-05, 0.76e-09, 0.73e-12 /

!    DATA (entry(5,j), j=0,7) / 0.72e-02, 0.34e-03, 0.96e-02, 0.18e-03 &
!                             , 0.10e-08, 0.19e-15, 0.00e+00, 0.00e+00 /
 
!    DATA (entry(5,j), j=0,7) / 0.28e-02, 0.70e-02, 0.38e-02, 0.36e-02 &
!                             , 0.58e-04, 0.14e-05, 0.75e-09, 0.72e-12 /

!    DATA (entry(6,j), j=0,7) / 0.67e-02, 0.32e-03, 0.93e-02, 0.17e-03 &
!                             , 0.99e-09, 0.19e-15, 0.00e+00, 0.00e+00 /
 
!    DATA (entry(6,j), j=0,7) / 0.26e-02, 0.66e-02, 0.37e-02, 0.35e-02 &
!                             , 0.56e-04, 0.14e-05, 0.73e-09, 0.70e-12 /

!    DATA (entry(7,j), j=0,7) / 0.60e-02, 0.29e-03, 0.89e-02, 0.16e-03 &
!                             , 0.95e-09, 0.18e-15, 0.00e+00, 0.00e+00 /
 
!    DATA (entry(7,j), j=0,7) / 0.23e-02, 0.60e-02, 0.35e-02, 0.34e-02 &
!                             , 0.54e-04, 0.13e-05, 0.70e-09, 0.67e-12 /

!    DATA (entry(8,j), j=0,7) / 0.54e-02, 0.27e-03, 0.87e-02, 0.16e-03 &
!                             , 0.93e-09, 0.17e-15, 0.00e+00, 0.00e+00 /
 
!    DATA (entry(8,j), j=0,7) / 0.21e-02, 0.56e-02, 0.34e-02, 0.33e-02 &
!                             , 0.53e-04, 0.13e-05, 0.68e-09, 0.65e-12 /

!    DATA (entry(9,j), j=0,7) / 0.48e-02, 0.25e-03, 0.81e-02, 0.15e-03 &
!                             , 0.87e-09, 0.16e-15, 0.00e+00, 0.00e+00 /
 
!    DATA (entry(9,j), j=0,7) / 0.19e-02, 0.51e-02, 0.32e-02, 0.31e-02 &
!                             , 0.49e-04, 0.12e-05, 0.64e-09, 0.61e-12 /

!    DATA (entry(10,j), j=0,7) / 0.48e-02, 0.25e-03, 0.85e-02, 0.16e-03 &
!                              , 0.91e-09, 0.17e-15, 0.00e+00, 0.00e+00 /
 
!    DATA (entry(10,j), j=0,7) / 0.19e-02, 0.52e-02, 0.34e-02, 0.32e-02 &
!                              , 0.52e-04, 0.13e-05, 0.67e-09, 0.64e-12 /

!    DATA (entry(11,j), j=0,7) / 0.44e-02, 0.24e-03, 0.82e-02, 0.15e-03 &
!                              , 0.88e-09, 0.17e-15, 0.00e+00, 0.00e+00 /
 
!    DATA (entry(11,j), j=0,7) / 0.17e-02, 0.49e-02, 0.33e-02, 0.31e-02 &
!                              , 0.50e-04, 0.12e-05, 0.65e-09, 0.62e-12 /

!    DATA (entry(12,j), j=0,7) / 0.46e-02, 0.25e-03, 0.88e-02, 0.16e-03 &
!                              , 0.94e-09, 0.18e-15, 0.00e+00, 0.00e+00 /
 
!    DATA (entry(12,j), j=0,7) / 0.18e-02, 0.51e-02, 0.35e-02, 0.33e-02 &
!                              , 0.53e-04, 0.13e-05, 0.69e-09, 0.66e-12 /

!    DATA (entry(13,j), j=0,7) / 0.37e-02, 0.20e-03, 0.71e-02, 0.13e-03 &
!                              , 0.76e-09, 0.14e-15, 0.00e+00, 0.00e+00 /
 
!    DATA (entry(13,j), j=0,7) / 0.14e-02, 0.41e-02, 0.28e-02, 0.27e-02 &
!                              , 0.43e-04, 0.11e-05, 0.56e-09, 0.53e-12 /

!    DATA (entry(14,j), j=0,7) / 0.25e-02, 0.14e-03, 0.50e-02, 0.91e-04 &
!                              , 0.53e-09, 0.10e-15, 0.81e-26, 0.36e-37 /

!    DATA (entry(14,j), j=0,7) / 0.97e-03, 0.28e-02, 0.20e-02, 0.19e-02 &
!                              , 0.30e-04, 0.75e-06, 0.39e-09, 0.37e-12 /

END MODULE H2_DATA

MODULE SP_DATA

! Species related quantities
! This module replaces part of stvbld2.f

  use constantes

  CHARACTER (LEN=8), PUBLIC, DIMENSION (natom+6)  :: sp             ! List of atomic species + reaction partners

!  DATA sp / 'h       ', 'c       ', 'n       ', 'o       ', &
!            'he      ', 'd       ', 'c*      ', 'n*      ', &
!            'o*      ', 'li      ', 'f       ', 'na      ', &
!            'mg      ', 'al      ', 'si      ', 'p       ', &
!            's       ', 'cl      ', 'k       ', 'fe      ', &
!            'electr  ', 'photon  ', 'crp     ', 'phosec  ', &
!            'grain   ', '2h      ', '        '/

  DATA sp / 'h       ', 'c       ', 'n       ', 'o       ', &
            'he      ', 'd       ', 'c*      ', 'n*      ', &
            'o*      ', 'pah     ', 'f       ', 'na      ', &
            'mg      ', 'al      ', 'si      ', 'p       ', &
            's       ', 'cl      ', 'ca      ', 'fe      ', &
            'electr  ', 'photon  ', 'crp     ', 'phosec  ', &
            'grain   ', '2h      ', '        '/

! index of specific species that require some particular attention

  INTEGER, PUBLIC  :: i_h    = 0
  INTEGER, PUBLIC  :: i_d    = 0
  INTEGER, PUBLIC  :: i_hgr  = 0               ! H on grains
  INTEGER, PUBLIC  :: i_dgr  = 0               ! D on gr
  INTEGER, PUBLIC  :: i_h2   = 0
  INTEGER, PUBLIC  :: i_hd   = 0
  INTEGER, PUBLIC  :: i_h2gr = 0               ! H2 on grains
  INTEGER, PUBLIC  :: i_he   = 0
  INTEGER, PUBLIC  :: i_c    = 0
  INTEGER, PUBLIC  :: i_n    = 0
  INTEGER, PUBLIC  :: i_o    = 0
  INTEGER, PUBLIC  :: i_s    = 0
  INTEGER, PUBLIC  :: i_si   = 0
  INTEGER, PUBLIC  :: i_c13  = 0
  INTEGER, PUBLIC  :: i_o18  = 0
  INTEGER, PUBLIC  :: i_h2o  = 0
  INTEGER, PUBLIC  :: i_oh   = 0
  INTEGER, PUBLIC  :: i_c2   = 0
  INTEGER, PUBLIC  :: i_co0  = 0               ! 12C16O
  INTEGER, PUBLIC  :: i_co1  = 0               ! 13C16O
  INTEGER, PUBLIC  :: i_co2  = 0               ! 12C18O
  INTEGER, PUBLIC  :: i_co3  = 0               ! 13C18O
  INTEGER, PUBLIC  :: i_cs   = 0
  INTEGER, PUBLIC  :: i_hp   = 0
  INTEGER, PUBLIC  :: i_cp   = 0
  INTEGER, PUBLIC  :: i_np   = 0
  INTEGER, PUBLIC  :: i_op   = 0
  INTEGER, PUBLIC  :: i_sp   = 0
  INTEGER, PUBLIC  :: i_sip  = 0
  INTEGER, PUBLIC  :: i_fep  = 0
  INTEGER, PUBLIC  :: i_h3p  = 0
  INTEGER, PUBLIC  :: i_h2dp = 0
  INTEGER, PUBLIC  :: i_hcop = 0

  CHARACTER (LEN=8), DIMENSION(0:np)         :: speci   ! List of species in chemistry
  REAL (KIND=dp), PUBLIC, DIMENSION(nes)     :: abin    ! Initial abundances od species

  REAL (KIND=dp), PUBLIC, DIMENSION(0:np)    :: mol     ! Molecular weight
  REAL (KIND=dp), PUBLIC, DIMENSION(0:natom) :: abtot   ! Total relative abundances of atoms
  INTEGER, PUBLIC, DIMENSION(0:nes,natom+1)  :: ael     ! Chemical composition of species
  INTEGER, PUBLIC, DIMENSION(np)             :: lcon    ! Conservation of species
                                                        ! if lcon(i) = 0 then species i is not used for conservation
                                                        ! if lcon(i) = j /= 0 then atom j is conserved thru species i
  INTEGER, PUBLIC, DIMENSION(0:natom,2)      :: icx     ! Conservation of atoms
                                                        ! icx(i,1) : index of species used for conservation of i
                                                        ! icx(i,2) : index of second most abundant species containing i

END MODULE SP_DATA

MODULE HD_DATA

! HD structure related quantities
! This module replaces old stvbld4.f

  use constantes

  REAL (kind=dp), PUBLIC                       :: amhd = 3.0_dp  ! Mass (amu)
  REAL (kind=dp), PUBLIC, DIMENSION (nlevhd)   :: elhd           ! Energies inside electonic state X (K)
  INTEGER, PUBLIC, DIMENSION (nlevhd)          :: njhd           ! Rotational number J
  INTEGER, PUBLIC, DIMENSION (nlevhd)          :: nvhd           ! Vibrational number v
  INTEGER, PUBLIC, DIMENSION (0:nvxhd,0:njthd) :: levhd          ! Reverse indexing (Level number from (v,J))
  REAL (kind=dp), PUBLIC, DIMENSION (0:nvbhd,0:njthd+1,nlevhd) :: ahdnb  ! Prob of emission B-X
  REAL (kind=dp), PUBLIC, DIMENSION (0:nvchd,0:njthd+1,nlevhd) :: ahdnc  ! Prob of emission C-X
  REAL (kind=dp), PUBLIC, DIMENSION (0:nvbhd,0:njthd+1)        :: ahdtb  ! Total prob of emission B-X to bound states
  REAL (kind=dp), PUBLIC, DIMENSION (0:nvchd,0:njthd+1,2)      :: ahdtc  ! Total prob of emission B-X to bound states
  REAL (kind=dp), PUBLIC, DIMENSION (nlevhd)                   :: ahdtq  ! Total quadrupole prob of emission 

  REAL (KIND=dp), PUBLIC, DIMENSION (0:nvxhd,0:njthd)          :: pophd    ! Probability to form HD(v,J) on grain
  REAL (KIND=dp), PUBLIC, DIMENSION (nlevhd,nlevhd)            :: cashd    ! HD cascade coeficients
  REAL (KIND=dp), PUBLIC, DIMENSION (0:nvbhd,0:njthd+1,nlevhd) :: bhdbx    ! HD B-X cascade coeficients
  REAL (KIND=dp), PUBLIC, DIMENSION (0:nvchd,0:njthd+1,nlevhd) :: bhdcmx   ! HD (C-)-X cascade coeficients
  REAL (KIND=dp), PUBLIC, DIMENSION (0:nvchd,0:njthd+1,nlevhd) :: bhdcpx   ! HD (C+)-X cascade coeficients
  REAL (KIND=dp), PUBLIC                                       :: emoyhd   ! Mean energy of HD formed on grain

  REAL (KIND=dp), PUBLIC, DIMENSION (0:nvbhd,0:njthd+1,nlevhd) :: eshdbx   ! HD B-X exape probability
  REAL (KIND=dp), PUBLIC, DIMENSION (0:nvchd,0:njthd+1,nlevhd) :: esdcmx   ! HD (C-)-X exape probability
  REAL (KIND=dp), PUBLIC, DIMENSION (0:nvchd,0:njthd+1,nlevhd) :: esdcpx   ! HD (C+)-X exape probability
  REAL (KIND=dp), PUBLIC, DIMENSION (0:nvbhd,0:njthd+1)        :: pthdbx   ! Probability of downward transition without reabsorptio
  REAL (KIND=dp), PUBLIC, DIMENSION (0:nvchd,0:njthd+1,0:1)    :: pthdcx   ! Probability of downward transition without reabsorptio

  REAL (KIND=dp), PUBLIC, DIMENSION (nlevhd,nlevhd)            :: evdihd   ! HD dipole and quadrupole transitions
  REAL (KIND=dp), PUBLIC, DIMENSION (nlevhd,nlevhd)            :: evcohd   ! HD collisional equilibrium
  REAL (KIND=dp), PUBLIC, DIMENSION (nlevhd,nlevhd)            :: evrdhd   ! HD radiative equilibrium (B+C)

  REAL (kind=dp), PUBLIC, DIMENSION (4,0:njthd,0:njthd)        :: qh1hd  ! Collision cross section H + HD
  REAL (kind=dp), PUBLIC, DIMENSION (4,0:njthd,0:njthd)        :: qhehd  ! Collision cross section He + HD
  REAL (kind=dp), PUBLIC, DIMENSION (4,0:njthd,0:njthd)        :: qph2hd ! Collision cross section pH2 + HD
  REAL (kind=dp), PUBLIC, DIMENSION (4,0:njthd,0:njthd)        :: qoh2hd ! Collision cross section oH2 + HD

    data (njhd(nnh2), nnh2=1,182) / &
        0,  1,  2,  3,  4,  5,  6,  7,  8,  0,  1,  9,  2,  3,  4, &
       10,  5,  6,  7,  8,  0,  1,  9,  2,  3,  4, 10,  5,  6,  7, &
        8,  0,  1,  9,  2,  3,  4, 10,  5,  6,  7,  8,  0,  1,  9, &
        2,  3,  4, 10,  5,  6,  7,  8,  0,  1,  2,  9,  3,  4, 10, &
        5,  6,  7,  8,  0,  1,  2,  9,  3,  4, 10,  5,  6,  7,  8, &
        0,  1,  2,  9,  3,  4, 10,  5,  6,  7,  8,  0,  1,  2,  9, &
        3,  4, 10,  5,  6,  7,  8,  0,  1,  2,  9,  3,  4, 10,  5, &
        6,  7,  8,  0,  1,  2,  9,  3,  4,  5, 10,  6,  7,  8,  0, &
        1,  2,  3,  9,  4,  5, 10,  6,  7,  8,  0,  1,  2,  3,  9, &
        4,  5, 10,  6,  7,  8,  0,  1,  2,  3,  9,  4,  5, 10,  6, &
        7,  0,  8,  1,  2,  3,  4,  9,  5, 10,  6,  7,  0,  1,  2, &
        8,  3,  4,  9,  5,  6, 10,  7,  0,  1,  2,  3,  8,  4,  5, &
        0,  1 /

    data (nvhd(nnh2), nnh2=1,182) / &
        0,  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  0,  1,  1,  1, &
        0,  1,  1,  1,  1,  2,  2,  1,  2,  2,  2,  1,  2,  2,  2, &
        2,  3,  3,  2,  3,  3,  3,  2,  3,  3,  3,  3,  4,  4,  3, &
        4,  4,  4,  3,  4,  4,  4,  4,  5,  5,  5,  4,  5,  5,  4, &
        5,  5,  5,  5,  6,  6,  6,  5,  6,  6,  5,  6,  6,  6,  6, &
        7,  7,  7,  6,  7,  7,  6,  7,  7,  7,  7,  8,  8,  8,  7, &
        8,  8,  7,  8,  8,  8,  8,  9,  9,  9,  8,  9,  9,  8,  9, &
        9,  9,  9, 10, 10, 10,  9, 10, 10, 10,  9, 10, 10, 10, 11, &
       11, 11, 11, 10, 11, 11, 10, 11, 11, 11, 12, 12, 12, 12, 11, &
       12, 12, 11, 12, 12, 12, 13, 13, 13, 13, 12, 13, 13, 12, 13, &
       13, 14, 13, 14, 14, 14, 14, 13, 14, 13, 14, 14, 15, 15, 15, &
       14, 15, 15, 14, 15, 15, 14, 15, 16, 16, 16, 16, 15, 16, 16, &
       17, 17 /

    data ( elhd(nnh2), nnh2=1,182) / &
        0.000_dp,   128.381_dp,   384.262_dp,   765.892_dp,  1270.704_dp,  1895.363_dp,  2635.847_dp, &
     3487.530_dp,  4445.265_dp,  5226.717_dp,  5349.551_dp,  5503.488_dp,  5594.363_dp,  5959.453_dp, &
     6442.331_dp,  6656.306_dp,  7039.761_dp,  7747.842_dp,  8562.088_dp,  9477.516_dp, 10198.072_dp, &
    10315.467_dp, 10488.732_dp, 10549.423_dp, 10898.292_dp, 11359.656_dp, 11590.038_dp, 11930.373_dp, &
    12606.659_dp, 13384.166_dp, 14258.058_dp, 14919.248_dp, 15031.289_dp, 15223.114_dp, 15254.564_dp, &
    15587.468_dp, 16027.647_dp, 16273.808_dp, 16572.051_dp, 17217.000_dp, 17958.266_dp, 18791.162_dp, &
    19393.439_dp, 19500.184_dp, 19710.611_dp, 19712.885_dp, 20029.980_dp, 20449.184_dp, 20711.258_dp, &
    20967.527_dp, 21581.434_dp, 22286.787_dp, 23079.023_dp, 23622.861_dp, 23724.314_dp, 23926.459_dp, &
    23953.207_dp, 24227.773_dp, 24626.029_dp, 24904.119_dp, 25118.334_dp, 25701.211_dp, 26370.656_dp, &
    27122.219_dp, 27607.855_dp, 27704.002_dp, 27895.549_dp, 27951.094_dp, 28181.012_dp, 28558.211_dp, &
    28852.184_dp, 29024.332_dp, 29575.971_dp, 30209.219_dp, 30919.740_dp, 31346.574_dp, 31437.357_dp, &
    31618.186_dp, 31702.834_dp, 31887.609_dp, 32243.500_dp, 32553.520_dp, 32683.086_dp, 33203.043_dp, &
    33799.527_dp, 34468.285_dp, 34835.043_dp, 34920.340_dp, 35090.207_dp, 35204.707_dp, 35343.227_dp, &
    35677.301_dp, 36003.895_dp, 36089.715_dp, 36577.191_dp, 37135.945_dp, 37761.770_dp, 38066.832_dp, &
    38146.445_dp, 38304.969_dp, 38450.105_dp, 38540.992_dp, 38852.453_dp, 39196.109_dp, 39236.672_dp, &
    39690.402_dp, 40209.902_dp, 40790.988_dp, 41032.313_dp, 41105.977_dp, 41252.602_dp, 41429.109_dp, &
    41470.789_dp, 41758.492_dp, 42113.047_dp, 42119.402_dp, 42531.215_dp, 43009.254_dp, 43542.977_dp, &
    43717.910_dp, 43785.246_dp, 43919.219_dp, 44118.422_dp, 44127.781_dp, 44380.801_dp, 44703.672_dp, &
    44758.746_dp, 45083.770_dp, 45517.305_dp, 46000.008_dp, 46105.332_dp, 46165.832_dp, 46286.105_dp, &
    46464.742_dp, 46527.164_dp, 46699.629_dp, 46988.023_dp, 47093.656_dp, 47326.563_dp, 47711.320_dp, &
    48137.844_dp, 48169.938_dp, 48222.875_dp, 48328.008_dp, 48483.852_dp, 48601.148_dp, 48688.219_dp, &
    48938.227_dp, 49095.762_dp, 49230.305_dp, 49560.262_dp, 49879.242_dp, 49923.270_dp, 49923.629_dp, &
    50011.594_dp, 50141.555_dp, 50311.117_dp, 50313.805_dp, 50517.137_dp, 50725.648_dp, 50755.656_dp, &
    51021.914_dp, 51190.188_dp, 51224.555_dp, 51292.391_dp, 51310.270_dp, 51391.863_dp, 51520.211_dp, &
    51614.031_dp, 51673.691_dp, 51847.453_dp, 51925.141_dp, 52035.348_dp, 52045.645_dp, 52067.699_dp, &
    52110.629_dp, 52172.023_dp, 52229.363_dp, 52248.016_dp, 52332.797_dp, 52377.703_dp, 52382.195_dp /

  data (levhd(ivh2, 0), ivh2=0,17) / &
         1,  10,  21,  32,  43,  54,  65,  76,  87,  98, 109, 120, 131, 142, 152, 163, 174, 181 /

  data (levhd(ivh2, 1), ivh2=0,17) / &
         2,  11,  22,  33,  44,  55,  66,  77,  88,  99, 110, 121, 132, 143, 154, 164, 175, 182 /

  data (levhd(ivh2, 2), ivh2=0,17) / &
         3,  13,  24,  35,  46,  56,  67,  78,  89, 100, 111, 122, 133, 144, 155, 165, 176,   0 /

  data (levhd(ivh2, 3), ivh2=0,17) / &
         4,  14,  25,  36,  47,  58,  69,  80,  91, 102, 113, 123, 134, 145, 156, 167, 177,   0 /

  data (levhd(ivh2, 4), ivh2=0,17) / &
         5,  15,  26,  37,  48,  59,  70,  81,  92, 103, 114, 125, 136, 147, 157, 168, 179,   0 /

  data (levhd(ivh2, 5), ivh2=0,17) / &
         6,  17,  28,  39,  50,  61,  72,  83,  94, 105, 115, 126, 137, 148, 159, 170, 180,   0 /

  data (levhd(ivh2, 6), ivh2=0,17) / &
         7,  18,  29,  40,  51,  62,  73,  84,  95, 106, 117, 128, 139, 150, 161, 171,   0,   0 /

  data (levhd(ivh2, 7), ivh2=0,17) / &
         8,  19,  30,  41,  52,  63,  74,  85,  96, 107, 118, 129, 140, 151, 162, 173,   0,   0 /

  data (levhd(ivh2, 8), ivh2=0,17) / &
         9,  20,  31,  42,  53,  64,  75,  86,  97, 108, 119, 130, 141, 153, 166, 178,   0,   0 /

  data (levhd(ivh2, 9), ivh2=0,17) / &
        12,  23,  34,  45,  57,  68,  79,  90, 101, 112, 124, 135, 146, 158, 169,   0,   0,   0 /

  data (levhd(ivh2,10), ivh2=0,17) / &
        16,  27,  38,  49,  60,  71,  82,  93, 104, 116, 127, 138, 149, 160, 172,   0,   0,   0 /

END MODULE HD_DATA

MODULE CHEM_DATA

! Chemistry related quantities
! Introduced 30 aug 2002 - JLB

  use constantes

! Different type of chemical reactions

  INTEGER, PUBLIC                            :: nh2for   ! type 0 : H2 and HD formation on grains
                                                         ! Obsolete old style ! DON'T USE ANY MORE !
  INTEGER, PUBLIC                            :: ncrion   ! type 1 : Cosmic rays processes
  INTEGER, PUBLIC                            :: npsion   ! type 2 : Secondary photons processes
  INTEGER, PUBLIC                            :: nasrad   ! type 3 : Radiative association
  INTEGER, PUBLIC                            :: nrest    ! type 4 : Ordinary gas phase reactions
  INTEGER, PUBLIC                            :: nh2endo  ! type 6 : Endothermal reactions with H2 (NEW!)
  INTEGER, PUBLIC                            :: nphot_f  ! type 5 : Photoreactions (external fit)
  INTEGER, PUBLIC                            :: nphot_i  ! type 7 : Photoreactions (direct integration)
  INTEGER, PUBLIC                            :: i_mandat ! type 7 : Photoreactions (mandatory rxns -> not in sections)
  INTEGER, PUBLIC                            :: n3body   ! type 8 : 3 body reactions

  INTEGER, PUBLIC                            :: ncpart   ! type 101, ... : Exceptions and special cases

  INTEGER, PUBLIC                            :: ngsurf   ! type 11 : Reactions on grain surface
  INTEGER, PUBLIC                            :: ngphot   ! type 12 : Photoreactions on grain surface
  INTEGER, PUBLIC                            :: ngads    ! type 13 : Adsorption on to grains
  INTEGER, PUBLIC                            :: ngneut   ! type 14 : Neutralization
  INTEGER, PUBLIC                            :: nggdes   ! type 15 : Explosive desorption from grains
  INTEGER, PUBLIC                            :: ncrdes   ! type 16 : Cosmic rays induced desorption
  INTEGER, PUBLIC                            :: nphdes   ! type 17 : Photodesorption from grains
  INTEGER, PUBLIC                            :: ngevap   ! type 18 : Evaporation from grains

  INTEGER, PUBLIC                            :: nn       ! Number of neutral species
  INTEGER, PUBLIC                            :: nng      ! Number of species on grains
  INTEGER, PUBLIC                            :: ni       ! Number of ionised species
  INTEGER, PUBLIC                            :: nim      ! Number of negatively charged species
  INTEGER, PUBLIC                            :: nspec    ! Number chemical species (total)
  INTEGER, PUBLIC                            :: neqt     ! Number of chemical equations
  INTEGER, PUBLIC, PARAMETER                 :: nbrm = 3 ! Max number of reactants
  INTEGER, PUBLIC, PARAMETER                 :: nbpm = 4 ! Max number of products
  INTEGER, PUBLIC, PARAMETER                 :: nbsm = 7 ! Max number of species in a reaction

  REAL (kind=dp), PUBLIC, DIMENSION (0:np)   :: enth    ! Recation enthalpies
  INTEGER, PUBLIC, DIMENSION(nqm,0:nbsm)     :: react   ! Chemical reactions
  REAL (kind=dp), PUBLIC, DIMENSION (nes)    :: tm1h    ! Jump inverse life time on grains
  REAL (kind=dp), PUBLIC                     :: xkhydr  ! Hydrogenation rate on grains

  INTEGER, PUBLIC, DIMENSION(0:np,0:nqm)            :: ikmat  ! Full reaction rates (index)
  REAL (kind=dp), PUBLIC, DIMENSION (0:nes,nqm)     :: xkmat  ! Full reaction rates (values)
  REAL (kind=dp), PUBLIC, DIMENSION (0:nes,0:noptm) :: rxigr  ! Recombinatin rate of ions on grains

  INTEGER, PUBLIC                            :: iadhgr ! H adsorption on grains reaction number
  INTEGER, PUBLIC                            :: iaddgr ! D adsorption on grains reaction number
  INTEGER, PUBLIC                            :: ifh2gr ! H2 formation on grains reaction number
  INTEGER, PUBLIC                            :: ifhdgr ! HD formation on grains reaction number
  INTEGER, PUBLIC                            :: idh2ph ! H2 destruction by photons reaction number
  INTEGER, PUBLIC                            :: idhdph ! HD destruction by photons reaction number
  INTEGER, PUBLIC                            :: idco0p ! CO destruction by photons reaction number
  INTEGER, PUBLIC                            :: idco1p ! C*O destruction by photons reaction number
  INTEGER, PUBLIC                            :: idco2p ! CO* destruction by photons reaction number
  INTEGER, PUBLIC                            :: idco3p ! C*O* destruction by photons reaction number
  INTEGER, PUBLIC                            :: idcph  ! C destruction by photons reaction number

  REAL (kind=dp), PUBLIC, DIMENSION (nqm)    :: gamm   ! Arrhenius coefficient
  REAL (kind=dp), PUBLIC, DIMENSION (nqm)    :: alpha  ! Arrhenius coefficient
  REAL (kind=dp), PUBLIC, DIMENSION (nqm)    :: bet    ! Arrhenius coefficient
  REAL (kind=dp), PUBLIC, DIMENSION (nqm)    :: de     ! Endothermicity
  INTEGER, PUBLIC, DIMENSION (nqm)           :: itype  ! Type of reaction

  ! Tabular used to check conservation in chemical reactions
  REAL (KIND=dp), dimension(nqm)             :: bilanm ! bilan masse
  REAL (KIND=dp), dimension(nqm)             :: bilanc ! bilan charge
  REAL (KIND=dp), dimension(0:np)            :: charge ! charge of species 

END MODULE CHEM_DATA
