
! Copyright (C) 2002 - Observatoire de Paris/Meudon
!                      Universite Paris 7
!                      CNRS (France)
!                      Contact : Jacques.Lebourlot@obspm.fr
!
! This program is free software; you can redistribute it and/or
! modify it under the terms of the GNU General Public License version 2
! as published by the Free Software Foundation.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.



! module "constantes" define all sorts of parameter data that are known
!   before hand and do not change during a run

module constantes


! "kind parameter"  for double precision

  integer, public, parameter :: dp = selected_real_kind(P=15)

! usual numbers

  real (kind=dp), public, parameter :: istiny = 1.0e-30_dp

! Input-Output units:

  integer, public, parameter :: iread1 =  1
  integer, public, parameter :: iread2 =  2
  integer, public, parameter :: iwrit1 = 11
  integer, public, parameter :: iwrit2 = 12
  integer, public, parameter :: iscrn  =  6

! Physical constants (Partly from  rev. mod. phys. 59 , 1121 (1987) )
! Revised from physics.nist.gov/constants

  real (kind=dp), public, parameter :: amu  = 1.66053873e-24_dp        ! Atomic mass unit (g)
  real (kind=dp), public, parameter :: xmh  = 1.6735e-24_dp            ! H atom mass (g)
  real (kind=dp), public, parameter :: xmp  = 1.67262158e-24_dp        ! proton mass (g)
  real (kind=dp), public, parameter :: xme  = 9.10938188e-28_dp        ! electron mass (g)
  real (kind=dp), public, parameter :: xqe  = 4.80325e-10_dp           ! electric charge (esu)
  real (kind=dp), public, parameter :: xqe2 = 2.30712e-19_dp           ! xqe * xqe
  real (kind=dp), public, parameter :: e2smc2 = 2.817939e-13_dp        ! r0 = e2 / (m*c**2) (cm) - Lang
  real (kind=dp), public, parameter :: xk   = 1.3806503e-16_dp         ! Boltzman constant k (erg deg-1)
  real (kind=dp), public, parameter :: xhp  = 6.62606876e-27_dp        ! Planck constant h (erg s)
  real (kind=dp), public, parameter :: xr   = 8.314472e7_dp            ! Perfect gaz constant R (erg deg-1 mole-1)
  real (kind=dp), public, parameter :: clum = 2.99792458e10_dp         ! Celerity of light c (cm s-1)
  real (kind=dp), public, parameter :: xpi  = 3.1415926535897932384_dp ! Pi (!)
  real (kind=dp), public, parameter :: sqpi = 1.77245385090551602_dp   ! Sqrt(pi)
  real (kind=dp), public            :: hiop                            ! Ionisation potential of H
                                                                       ! = 2 pi**2 e**4 m / h**2 (Lang p 244)
! cosmologicall Black Body
  real (kind=dp), public            :: tcmb

! Solor radius
  real (kind=dp)                    :: rsun = 6.9598e10_dp             ! cm, Lang, p565

! Conversion factors

  real (kind=dp), public            :: tautav                          ! From Optical depth (tau) to Av (in mag)
  real (kind=dp), public            :: J_to_u                          ! From mean Intensity to energy density
                                                                       !      J_to_u = 4 pi / c
  real (kind=dp), public, parameter :: hcsurk  = 1.43883442658354_dp   ! From Wave number (cm-1) to Temperature (Kelvin)
  real (kind=dp), public, parameter :: everg   = 1.6022e-12_dp         ! From Electron-Volt to Erg
  real (kind=dp), public, parameter :: calev   = 4.3363e-2_dp          ! From Cal mole-1 to Ev
  real (kind=dp), public, parameter :: pccm    = 3.086e18_dp           ! From parsec to cm
  real (kind=dp), public, parameter :: ryd2ang = 911.267101235_dp      ! Rydberg to Angstrom conversion

! Numerical iterations and precision related constants

  integer, public, parameter        :: miter  = 50                 ! Max global iterations for a slab
  real (kind=dp), public, parameter :: convli = 1.0e-3_dp          ! Relative error in global iterations

! Various initial or maximum dimensions

  integer, public, parameter        :: nes    = 420                ! Max number of chemical species (220)
  integer, public, parameter        :: np     = 422                ! Max number of chemical species + photons + CR (222)
  integer, public, parameter        :: natom  = 21                 ! Number of chemical elements (atoms)
  integer, public, parameter        :: nqm    = 6000               ! Max number of chemical reactions (2500)
  integer, public, parameter        :: noptm  = 1000               ! Max number of optical depth points (500)

! H2 related dimensions

  integer, public, parameter        :: nlevh2 = 318                ! Number of H2 rovibrational levels (X)
  integer, public, parameter        :: njh2   = 29                 ! Highest rotational level of H2 included
  integer, public, parameter        :: nvxh2  = 14                 ! Highest vibrational level (X)
  integer, public, parameter        :: nvbh2  = 37                 ! Highest vibrational level (B)
  integer, public, parameter        :: nvch2  = 13                 ! Highest vibrational level (C)
  integer, public, parameter        :: nvbc   = 5                  ! Highest vibrational level (B and C) for wich pop is saved
  integer, public, parameter        :: njbc  = 15                  ! Highest rotational level (B and C) for wich pop is saved

! HD related dimensions

  integer, public, parameter        :: nlevhd = 182                ! Number of HD rovibrational levels (X)
  integer, public, parameter        :: njthd  = 10                 ! Highest rotational level of HD included
  integer, public, parameter        :: nvxhd  = 17                 ! Highest vibrational level (X)
  integer, public, parameter        :: nvbhd  = 39                 ! Highest vibrational level (B)
  integer, public, parameter        :: nvchd  = 15                 ! Highest vibrational level (C)

! H, D, CO & HCO+ related dimensions

  integer, public, parameter        :: nmrh   = 199                ! Number of H Lyman lines included
  integer, public, parameter        :: nmrd   = 20                 ! Number of D Lyman lines included
  integer, public, parameter        :: nmrco  = 4000               ! Max number of CO electronic lines
  integer, public, parameter        :: njco   = 30                 ! Highest rotational level of CO included
  integer, public, parameter        :: njhcop = 20                 ! Highest rotational level of HCO+ included

  integer, public, parameter        :: npmg   = 12                 ! Number of points in Gaussian integration over
                                                                   ! The grain size distrib. (note: instable if nmpg > 12)
end module constantes

!-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

module inoutput

  use constantes

! Replace "user" by your user ID, and change the "len" parameter accordingly
!                                        '0--------1---------2---------3---------4---------5---------6'
!                                        '123456789012345678901234567890123456789012345678901234567890'
  character (len=8), public :: data_dir='../data/'
  character (len=7), public :: out_dir ='../out/'

  character (len=255), public :: fichier
  character (len=255), public :: file_bin

end module inoutput
