
! Copyright (C) 2002 - Observatoire de Paris/Meudon
!                      Universite Paris 7
!                      CNRS (France)
!                      Contact : Jacques.Lebourlot@obspm.fr
!
! This program is free software; you can redistribute it and/or
! modify it under the terms of the GNU General Public License version 2
! as published by the Free Software Foundation.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


module auxiliaries

  use constantes

  private

  public :: lev_H2
  public :: grparam

!  Computed in lev_H2:

  integer , public                             :: mlevh2  !  Nombre de nivaux de H2 effectivement utilises
  integer , public                             :: mlph2   !  Nombre de nivaux de H2 pairs effectivement utilises
  integer , public                             :: mlih2   !  Nombre de nivaux de H2 impairs effectivement utilises
  integer , public, allocatable, dimension (:) :: levinc  !  Liste (ordonnee) de ces niveaux
  integer , public, allocatable, dimension (:) :: levpc   !  Liste (ordonnee) de ces niveaux pairs
  integer , public, allocatable, dimension (:) :: levic   !  Liste (ordonnee) de ces niveaux impairs

  real (kind=dp), public                       :: wlrwid = 5000.0_dp !  Width (in reduced units) of lines
                                                                     !  included in complete transfer

!  H UV lines

  integer , public                             :: nraih   !  Number of H lines
  integer , public, dimension(nmrh)            :: iraih   !  Lower level identification
  real (kind=dp), public, dimension(nmrh,3)    :: praih   !  Line parameters (real values)
                                                          !       1: oscilator strength
                                                          !       2: wave length (Angstrom)
                                                          !       3: inverse life time (s-1)

!  D UV lines

  integer , public                             :: nraid   !  Number of H lines
  integer , public, dimension(nmrd)            :: iraid   !  Lower level identification
  real (kind=dp), public, dimension(nmrd,3)    :: praid   !  Line parameters (real values)

!  H2 UV lines

  integer , public                                    :: nrh2q   !  Number of H2 quadrupole lines
  integer , public                                    :: nrh2bt  !  Number of H2 lines - Lyman transitions
  integer , public                                    :: nrh2ct  !  Number of H2 lines - Werner transitions
  integer , public, dimension(:,:), allocatable       :: irah2b  !  Line parameters (integer values - Lyman)
  integer , public, dimension(:,:), allocatable       :: irah2c  !  Line parameters (integer values - Werner)
                                                                 !       2: v lower level
                                                                 !       3: J lower level
                                                                 !       4: v upper level
                                                                 !       5: Delta J (J_up = J_low + Delta J)
  real (kind=dp), public, dimension(:,:), allocatable :: prah2b  !  Line parameters (real values - Lyman)
  real (kind=dp), public, dimension(:,:), allocatable :: prah2c  !  Line parameters (real values - Werner)
                                                                 !       1: oscilator strength
                                                                 !       2: wave length (Angstrom)
                                                                 !       3: inverse life time (s-1)
                                                                 !       4: Upper level dissociation probability

!  HD UV lines

  integer , public                                    :: nrhdq   !  Number of HD dipole and quadrupole lines (X-X)
  integer , public                                    :: nrhdbt  !  Number of HD lines - Lyman transitions
  integer , public                                    :: nrhdct  !  Number of HD lines - Werner transitions
  integer , public, dimension(:,:), allocatable       :: irahdb  !  Line parameters (integer values - Lyman)
  integer , public, dimension(:,:), allocatable       :: irahdc  !  Line parameters (integer values - Werner)
                                                                 !       2: v lower level
                                                                 !       3: J lower level
                                                                 !       4: v upper level
                                                                 !       5: Delta J (J_up = J_low + Delta J)
  real (kind=dp), public, dimension(:,:), allocatable :: prahdb  !  Line parameters (real values - Lyman)
  real (kind=dp), public, dimension(:,:), allocatable :: prahdc  !  Line parameters (real values - Werner)
                                                                 !       1: oscilator strength
                                                                 !       2: wave length (Angstrom)
                                                                 !       3: inverse life time (s-1)
                                                                 !       4: Upper level dissociation probability

!  CO UV lines

  integer, public                                     :: nrco0   !  Number of CO lines
  integer, public, dimension(nmrco)                   :: irco0   !  Line parameters (integer values)
  real (kind=dp), public, dimension(nmrco,4)          :: prco0   !  Line parameters (real values)
  integer, public                                     :: nrco1   !  Number of 13CO lines
  integer, public, dimension(nmrco)                   :: irco1   !  Line parameters (integer values)
  real (kind=dp), public, dimension(nmrco,4)          :: prco1   !  Line parameters (real values)
  integer, public                                     :: nrco2   !  Number of C18O lines
  integer, public, dimension(nmrco)                   :: irco2   !  Line parameters (integer values)
  real (kind=dp), public, dimension(nmrco,4)          :: prco2   !  Line parameters (real values)
  integer, public                                     :: nrco3   !  Number of 13C18O lines
  integer, public, dimension(nmrco)                   :: irco3   !  Line parameters (integer values)
  real (kind=dp), public, dimension(nmrco,4)          :: prco3   !  Line parameters (real values)

!  Computed in grparam

  real (kind=dp), public :: xngr
  real (kind=dp), public :: agrnrm
  real (kind=dp), public :: signgr
  real (kind=dp), public :: bsitgr
  real (kind=dp), public :: rdeso
  real (kind=dp), public :: sngclb
  real (kind=dp), public :: vmaxw
  real (kind=dp), public :: fgrneu = 0.0_dp                 ! Fraction of neutral grains
  real (kind=dp), public :: fgrmoi = 1.0_dp                   ! Fraction of neg charged grains
  real (kind=dp), public :: fgrplu = 0.0_dp                 ! Fraction of pos charged grainso

! Integration on grain size distribution is done by Gaussian integration.
!    absissa and weight of the orthogonal polynomials associated to
!    the weight function x**(-alp) are computed in subroutine my_gauss
!    The number of points npg depends on r_min and r_max
! All physical quantities F depending on grain size are computed at the
!    points adust (Fi) and weighted so that Sum Fi = Integral F dng

  integer, public        :: npg                           ! Number of point in gaussian integration

  real (kind=dp), public, allocatable, dimension (:)   :: adust   ! Absissa of gaussian integration
  real (kind=dp), public, allocatable, dimension (:)   :: wwmg    ! Weight of gaussian integration
  real (kind=dp), public, allocatable, dimension (:)   :: surfgr  ! Surface of grains of size i
  real (kind=dp), public, allocatable, dimension (:)   :: x_nc    ! Number of C atom in grains of size i
  real (kind=dp), public, allocatable, dimension (:)   :: bt_fy   ! Ionisation enhancement factor (Bakes & Tielens)
  real (kind=dp), public, allocatable, dimension (:,:) :: tdust   ! Dust temperature
  real (kind=dp), public, allocatable, dimension (:,:) :: zmoy    ! Mean charge

! parameter related to the grain size distribution

  real (kind=dp), public, parameter   :: r_lim_PAH = 1.25e-7_dp   ! PAH size : from rgrmin to 12.5 Angstrom
  real (kind=dp), public, parameter   :: r_lim_VSG = 1.0e-6_dp    ! VSG size : from r_lim_PAH to 100 Angstrom
                                                                  ! BG above (Emilie Habart dixit...)

! Radiation field and related quantities (i.e. depending on wavelength)

  real (kind=dp), public, dimension (0:noptm)          :: fluph      ! Total flux of photons at tau
  real (kind=dp), public, dimension (0:noptm)          :: int_rad    ! Total radiative energy density (erg cm-3)
  real (kind=dp), public, allocatable, dimension (:,:) :: intJm      ! dust atenuation from "minus" side
  real (kind=dp), public, allocatable, dimension (:,:) :: intJp      ! dust atenuation from "plus" side
  real (kind=dp), public, allocatable, dimension (:,:) :: bbspm      ! dust atenuation from "minus" side
  real (kind=dp), public, allocatable, dimension (:,:) :: bbspp      ! dust atenuation from "plus" side

!------------------------------------------------------------------------

contains

subroutine lev_H2 (l0h2)

   use H2_DATA

   integer, intent (in)                    :: l0h2

   mlevh2 = l0h2
   allocate (levinc(mlevh2))
   levinc = (/ (i, i=1,l0h2) /)

   mlph2 = count( mod(njlev(levinc),2) == 0)
   allocate (levpc(mlph2))
   levpc = pack ( levinc, mask=(mod(njlev(levinc),2) == 0))

   mlih2 = count( mod(njlev(levinc),2) == 1)
   allocate (levic(mlih2))
   levic = pack ( levinc, mask=(mod(njlev(levinc),2) == 1))

end subroutine lev_H2

!------------------------------------------------------------------------

subroutine grparam

  use constantes
  USE PROFIL
  USE SP_DATA

  real (kind=dp) :: f1, f2, f3, f4, f5
  real (kind=dp) :: f1sf4, f3sf4, f2sf3, f15s34
  real (kind=dp) :: rgazgr, rapgr, fcolgg
  real (kind=dp), parameter   :: x_la = 1.0e-6_dp         ! Photon attenuation length (Bakes & Tielens)
  real (kind=dp), parameter   :: x_le = 1.0e-7_dp         ! Photon attenuation length (Bakes & Tielens)
  real (kind=dp) :: aalp, fy, zetph

  rapgr = rgrmax / rgrmin
  if (rgrmax == rgrmin) then
    alpgr = 0.0_dp
  endif

!  Auxiliary quantities. We use an expansion for R-1 < 0.1

! fi = Integral from Rmin to Rmax of (x**(i-1) x**(-alpgr) dx)

  if (rapgr > 1.1d0) then

    f1 = (rgrmax**(1.0_dp-alpgr)     - rgrmin**(1.0_dp-alpgr))     / (1.0_dp-alpgr)
    f2 = (rgrmax**(2.0_dp-alpgr)   - rgrmin**(2.0_dp-alpgr))   / (2.0_dp-alpgr)
    f3 = (rgrmax**(3.0_dp-alpgr)  - rgrmin**(3.0_dp-alpgr))  / (3.0_dp-alpgr)
    f4 = (rgrmax**(4.0_dp-alpgr) - rgrmin**(4.0_dp-alpgr)) / (4.0_dp-alpgr)
    f5 = (rgrmax**(5.0_dp-alpgr)   - rgrmin**(5.0_dp-alpgr))   / (5.0_dp-alpgr)
    f1sf4  = f1 / f4
    f3sf4  = f3 / f4
    f2sf3  = f2 / f3
    f15s34 = (f1 * f5) / (f3 * f4)

  else

    f1 = 1.0_dp
    f3sf4 = (3.0_dp - rapgr) / (2.0_dp * rgrmin)
    f1sf4 = (5.0_dp - 3.0_dp * rapgr) / (2.0_dp * rgrmin**3)
    f2sf3 = f3sf4
    f15s34 = f3sf4

  endif

! rgazgr : Volume of grain per nucleon (cm3)
  rgazgr = 1.4d0 * xmh * gratio / rhogr

! xngr : Numerical relative density of grains
  xngr = 3.0_dp * rgazgr * f1sf4 / (4.0_dp * xpi)

! agrnrm : Normalisation factor
  agrnrm = xngr / f1

  print *, " <4 pi a**2> =", 4.0_dp * xpi * f3sf4 / f1sf4

! signgr : mean of ngr * sigma over nH (cm2)
  signgr = 3.0_dp * rgazgr * f3sf4 / 4.0_dp

! bsitgr : mean number of site per grain
  bsitgr = 4.0_dp * signgr / (dsite * dsite)

! fcolgr : Geometrical factor used for grain-grain collisions (cm-1)
  fcolgg = f3sf4 + 2.0_dp * f2sf3 + f15s34

! rdeso : desorption velocity per grain-grain collision
  rdeso = 1.5d0 * (vturb / 10.0_dp) * rgazgr * fcolgg

! sngclb : Coulomb interaction caracteristic constant
  sngclb = 2.0_dp * xqe2 * f2sf3 / (3.0_dp * xk)

! vmaxw : numerical constant in mean velocity computation
!  vbar = vmaxw * sqrt(temp/m)
  vmaxw = sqrt(8.0_dp * xk / (xmh *xpi))

!  Compute weight and absissas of Gaussian integration
!  over size distribution

  if (rapgr > 1.0d0) then
    call my_gauss
  else
    npg = 1
    allocate (adust(npg), wwmg(npg))
    adust(1) = rgrmin
    wwmg(1) = 1.0_dp
  endif

  allocate (surfgr(npg),tdust(npg,0:noptm), zmoy(npg,0:noptm))
  allocate (x_nc(npg), bt_fy(npg))

! grains initial temperature (for first iteration only)

  tdust(:,:) = min(15.0d0, 15.0d0 * (1.0_dp + log10(radm)))

  do mm=1,npg
    x_nc(mm) = 4.0_dp * xpi * adust(mm)**3 * rhogr &
             / (3.0_dp * xmh * 12.0_dp)
  end do

!  Eq (16) of Bakes & Tielens

  do mm=1,npg
    aalp = adust(mm) / x_la + adust(mm) / x_le
    zetph = adust(mm) / x_la
    fy = aalp*aalp - 2.0_dp * aalp + 2.0_dp - 2.0_dp * exp(-aalp)
    fy = fy / (zetph*zetph - 2.0_dp * zetph + 2.0_dp - 2.0_dp * exp(-zetph))
    bt_fy(mm) = fy * (zetph / aalp)**2
  end do

  surfgr = agrnrm * 4.0_dp * xpi * wwmg(1:npg) * adust(1:npg)*adust(1:npg)
! toto = sum(surfgr)

end subroutine grparam

!------------------------------------------------------------------------

end module auxiliaries

