
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

c      %%%%%%%%%%%%%%%%%%%%%%%%
       subroutine FGKTR2 (iopt)
c      %%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE RF_TOOLS
       USE SP_DATA
       USE H2_DATA
       USE HD_DATA
       USE STR_DATA
       USE PROFIL

c---------------------------------------------------------------------
c       Computes line absorption using FGK approximation
c       Radiation on both hsize of the cloud
c       The radiation field is defined in stv1y.f90 and transfer is solved in stv2z.f90
c---------------------------------------------------------------------

        implicit double precision (a-h, o-z)

c  Used for interpolation

        real (KIND=dp) cdh2f(nlevh2)
        real (KIND=dp) cdhdf(nlevhd)
        real (KIND=dp) cdc0f(nlvco0)
        real (KIND=dp) cdc1f(nlvco0)
        real (KIND=dp) cdc2f(nlvco0)
        real (KIND=dp) cdc3f(nlvco0)
        real (KIND=dp) cdh2r(nlevh2)
        real (KIND=dp) cdhdr(nlevhd)
        real (KIND=dp) cdc0r(nlvco0)
        real (KIND=dp) cdc1r(nlvco0)
        real (KIND=dp) cdc2r(nlvco0)
        real (KIND=dp) cdc3r(nlvco0)
        real (KIND=dp) cdtau(0:noptm)
        real (KIND=dp) cdinf

        real (KIND=dp) interpcd, ress
        real (KIND=dp) ajdf, ajrf
        real (KIND=dp) vt2, aux1, aux2
        real (KIND=dp) nu0, nu

        real (KIND=dp)                    :: avoi
        real (KIND=dp), dimension(:), allocatable :: xwl, yvoi, xsum
        real (kind=dp), dimension(0:nwlg) :: intJ1, intJ2
        real (kind=dp), dimension(0:nwlg) :: bbsp1, bbsp2

        TYPE (RF_TYPE)  :: rf_tmp

        save

        data  ff1, ff2, ff3   / 3.02_dp, 1.0d3, 6.4d-2 /

c  Interpolation location in previous grid

         do j=1,npo
           if (tau(iopt) <= tau_o(j)) then
             icd_o = j-1
             exit
           endif
         end do

         raptau = (tau(iopt) - tau_o(icd_o))
     1           / (tau_o(icd_o+1) - tau_o(icd_o))

c  Computes radiation field attenuation between the edge of the
c  cloud and present position.
c  See Flannery, Roberge, Rybicki, Ap J, 236, 598 (1980)
c   intJm and abpspp computed in stv1c.f and stv2z.f90

       vt2 = vturb * vturb

       intJ1 = 0.0_dp
       intJ2 = 0.0_dp
       bbsp1 = 0.0_dp
       bbsp2 = 0.0_dp
       do iwlg=0,nwlg
         intJ1(iwlg) = interpcd(raptau, intJm(0:npo,iwlg), icd_o, npo)
         intJ2(iwlg) = interpcd(raptau, intJp(0:npo,iwlg), icd_o, npo)
         rf_u%Val(iwlg) = (intJ1(iwlg) + intJ2(iwlg)) * J_to_u
c        print *, iwlg, rf_u%Val(iwlg)
         bbsp1(iwlg) = interpcd(raptau, bbspm(0:npo,iwlg), icd_o, npo)
         bbsp2(iwlg) = interpcd(raptau, bbspp(0:npo,iwlg), icd_o, npo)
       end do

       call rf_create(rf_tmp, rf_wl%lower, rf_wl%upper)
       rf_tmp%Ord = rf_u%Ord
       do mm=1,npg
         rf_tmp%Val = rf_u%Val
         call sect_pri(sigabs(mm), rf_tmp, wl_pointer, si_, nwlg)
         si_QU(mm,:) = si_
         rf_tmp%Val = rf_tmp%Val * rf_wl%Val
         call sect_pri(sigabs(mm), rf_tmp, wl_pointer, si_, nwlg)
         si_QUl(mm,:) = si_
         rf_tmp%Val = rf_tmp%Val * rf_wl%Val
         call sect_pri(sigabs(mm), rf_tmp, wl_pointer, si_, nwlg)
         si_QUl2(mm,:) = si_
       end do
       call rf_free (rf_tmp)

c     *********************************************************************
c               ct1= pi e^2/mc = 2.654057d-2
c                  =(section d absorption totale integree sur les frequences)
c                    en cm^2
c               vturb donne en cm s-1
c               ct2= 2k/mua   = 1.662902e8  en cgs
c               bdop:  dimension vitesse cm s-1 a
c      *****************************************************************

c  CDUNIT Gas to dust ratio : NH = CDUNIT * E(B-V)
c  RRR Extinction to color index ratio : Av = RRR * E(B-V)

       ct1 = xpi * xqe2 / xme / clum
       ct2 = 2.0_dp * xk / amu

c  Computes Doppler width weigheted H2 column density for each level

c  If "one side", then the true optical depth towards "dark" side is high

       if (isoneside == 0) then
         cdinf = 1.0_dp
       else
         cdinf = 10.0_dp
       endif

c  Ratio of turbulent Doppler width to total Doppler width
c  This ratio does not depend on the line because lambda disapears!

       do lev=1,l0h2
         cdtau(0:npo) = cdth2_o(lev,:)
         ress = interpcd(raptau, cdtau(0:npo), icd_o, npo)
         cdh2f(lev) = ress
         cdh2r(lev) = cdinf * cdtau(npo) - ress
       end do

c  Computes Doppler width weigheted HD column density for each level

       if (i_hd /= 0) then
         do lev=1,l0hd
           cdtau(0:npo) = cdthd_o(lev,:)
           ress = interpcd(raptau, cdtau(0:npo), icd_o, npo)
           cdhdf(lev) = ress
           cdhdr(lev) = cdinf * cdtau(npo) - ress
         end do
       endif

c  Computes Doppler width weigheted CO column density for each level

       do j=1,nlvco0
         cdtau(0:npo) = cdtc0_o(j,:)
         ress = interpcd(raptau, cdtau(0:npo), icd_o, npo)
         cdc0f(j) = ress
         cdc0r(j) = cdinf * cdtau(npo) - ress
       end do

c  Computes Doppler width weigheted C*O column density for each level

       if (i_c13 /= 0) then

         do j=1,nlvco0
           cdtau(0:npo) = cdtc1_o(j,:)
           ress = interpcd(raptau, cdtau(0:npo), icd_o, npo)
           cdc1f(j) = ress
           cdc1r(j) = cdinf * cdtau(npo) - ress
         end do

       endif

c  Computes Doppler width weigheted CO* column density for each level

       if (i_o18 /= 0) then

         do j=1,nlvco0
           cdtau(0:npo) = cdtc2_o(j,:)
           ress = interpcd(raptau, cdtau(0:npo), icd_o, npo)
           cdc2f(j) = ress
           cdc2r(j) = cdinf * cdtau(npo) - ress
         end do

       endif

c  Computes Doppler width weigheted C*O* column density for each level

       if (i_c13 /= 0 .and. i_o18 /= 0) then

         do j=1,nlvco0
           cdtau(0:npo) = cdtc3_o(j,:)
           ress = interpcd(raptau, cdtau(0:npo), icd_o, npo)
           cdc3f(j) = ress
           cdc3r(j) = cdinf * cdtau(npo) - ress
         end do

       endif

c  Reminder:
c     irah2(bc) 2:       lower level v
c               3:       lower level J
c               4:       upper level v
c               5:       delta J (Jup = Jlo + delj)
c     prah2(bc) 1:       Oscillator strength
c               2:       Wavelength (in A)
c               3:       Inverse radiative lifetime of upper level
c               4:       Upper level dissociation probability

c  Computes H2 line profiles

       bdop = 1.0d8 * sqrt(vt2 + ct2 * tgaz(iopt) / amh2)

       do i=1,nrh2bt
         njl = irah2b(i,2)
         nvl = irah2b(i,1)
         lev = level(nvl,njl)
         xlam0 = prah2b(i,2)
         gama = prah2b(i,3) / (4.0_dp * xpi)
         betas = bdop / xlam0

         if (nvl > 0 .OR. njl >= jfgkh2) then
           betas0 = 1.0d8 * vturb / xlam0
           r     = gama / (sqpi * betas)
           t1    = ff1 * (ff2 * r) ** (-ff3)
           opt0  = ct1 * prah2b(i,1) / (sqpi * betas0)
           taudf = opt0 * cdh2f(lev)
           u1f   = sqrt(taudf * r) / t1
           ajrf  = (r / t1) / sqrt(u1f * u1f + xpi / 4.0_dp)
           taudfr = opt0 * cdh2r(lev)
           u1fr   = sqrt(taudfr * r) / t1
           ajrfr  = (r / t1) / sqrt(u1fr * u1fr + xpi / 4.0_dp)

           fgkrf(i) = min(ajdf(taudf) + ajrf, 1.0_dp)
           fgkrfr(i) = min(ajdf(taudfr) + ajrfr, 1.0_dp)
         else
           nu0 = 1.0e8_dp * clum / xlam0
           avoi = prah2b(i,3) / (4.0_dp * xpi * betas)
           wlmin = 1.0e8_dp * clum / (nu0 + wlrwid * betas)
           wlmax = 1.0e8_dp * clum / (nu0 - wlrwid * betas)
           call rf_locate (rf_wl, wlmin, ilmin)
           call rf_locate (rf_wl, wlmax, ilmax)
           nbil = ilmax-ilmin
           allocate (xwl(0:nbil), yvoi(0:nbil), xsum(0:nbil))
           do j=0,nbil
             nu = 1.0e8_dp * clum / rf_wl%Val(j+ilmin)
             xwl(j) = (nu0 - nu) / betas
           end do

           call humlik (nbil+1, xwl, avoi, yvoi)

           xsum = yvoi * bbsp1(ilmin:ilmax)
           call simpint(xwl, xsum, nbil, tata)
           fgkrf(i) = tata

           xsum = yvoi * bbsp2(ilmin:ilmax)
           call simpint(xwl, xsum, nbil, tata)
           fgkrfr(i) = tata

           deallocate (xwl, yvoi, xsum)
         endif

       end do

       krai = nrh2bt
 
       do i=1,nrh2ct
         njl = irah2c(i,2)
         nvl = irah2c(i,1)
         lev = level(nvl,njl)
         xlam0 = prah2c(i,2)
         gama = prah2c(i,3) / (4.0_dp * xpi)
         betas = bdop / xlam0
         ikh2 = krai + i

         if (nvl > 0 .OR. njl >= jfgkh2) then
           betas0 = 1.0d8 * vturb / xlam0
           r     = gama / (sqpi * betas)
           t1    = ff1 * (ff2 * r) ** (-ff3)
           opt0  = ct1 * prah2c(i,1) / (sqpi * betas0)
           taudf = opt0 * cdh2f(lev)
           u1f   = sqrt(taudf * r) / t1
           ajrf  = (r / t1) / sqrt(u1f * u1f + xpi / 4.0_dp)
           taudfr = opt0 * cdh2r(lev)
           u1fr   = sqrt(taudfr * r) / t1
           ajrfr  = (r / t1) / sqrt(u1fr * u1fr + xpi / 4.0_dp)

c          ikh2 = krai + i                 ! FLP le 8 VII 2004
           fgkrf(ikh2) = min(ajdf(taudf) + ajrf, 1.0_dp)
           fgkrfr(ikh2) = min(ajdf(taudfr) + ajrfr, 1.0_dp)
         else
           nu0 = 1.0e8_dp * clum / xlam0
           avoi = prah2c(i,3) / (4.0_dp * xpi * betas)
           wlmin = 1.0e8_dp * clum / (nu0 + wlrwid * betas)
           wlmax = 1.0e8_dp * clum / (nu0 - wlrwid * betas)
           call rf_locate (rf_wl, wlmin, ilmin)
           call rf_locate (rf_wl, wlmax, ilmax)
           nbil = ilmax-ilmin
           allocate (xwl(0:nbil), yvoi(0:nbil), xsum(0:nbil))
           do j=0,nbil
             nu = 1.0e8_dp * clum / rf_wl%Val(j+ilmin)
             xwl(j) = (nu0 - nu) / betas
           end do

           call humlik (nbil+1, xwl, avoi, yvoi)

           xsum = yvoi * bbsp1(ilmin:ilmax)
           call simpint(xwl, xsum, nbil, tata)
           fgkrf(ikh2) = tata

           xsum = yvoi * bbsp2(ilmin:ilmax)
           call simpint(xwl, xsum, nbil, tata)
           fgkrfr(ikh2) = tata

           deallocate (xwl, yvoi, xsum)
         endif

       end do

       krai = krai + nrh2ct
 
c  Computes HD line profiles
 
       if (i_hd /= 0) then
 
       bdop = 1.0d8 * sqrt(vt2 + ct2 * tgaz(iopt) / amhd)
 
       do i=1,nrhdbt
         njl = irahdb(i,2)
         nvl = irahdb(i,1)
         lev = levhd(nvl,njl)
         xlam0 = prahdb(i,2)
         gama = prahdb(i,3) / (4.0_dp * xpi)
         betas = bdop / xlam0
         betas0 = 1.0d8 * vturb / xlam0
         r     = gama / (sqpi * betas)
         t1    = ff1 * (ff2 * r) ** (-ff3)
         opt0  = ct1 * prahdb(i,1) / (sqpi * betas0)
c        taudf = opt0 * cdthd(lev,iopt)
         taudf = opt0 * cdhdf(lev)
         u1f   = sqrt(taudf * r) / t1
         ajrf  = (r / t1) / sqrt(u1f * u1f + xpi / 4.0_dp)
         taudfr = opt0 * cdhdr(lev)
         u1fr   = sqrt(taudfr * r) / t1
         ajrfr  = (r / t1) / sqrt(u1fr * u1fr + xpi / 4.0_dp)
 
         ikhd = krai + i
         fgkrf(ikhd) = min(ajdf(taudf) + ajrf, 1.0_dp)
         fgkrfr(ikhd) = min(ajdf(taudfr) + ajrfr, 1.0_dp)
 
       end do
 
         krai = krai + nrhdbt

       do i=1,nrhdct
         njl = irahdc(i,2)
         nvl = irahdc(i,1)
         lev = levhd(nvl,njl)
         xlam0 = prahdc(i,2)
         gama = prahdc(i,3) / (4.0_dp * xpi)
         betas = bdop / xlam0
         betas0 = 1.0d8 * vturb / xlam0
         r     = gama / (sqpi * betas)
         t1    = ff1 * (ff2 * r) ** (-ff3)
         opt0  = ct1 * prahdc(i,1) / (sqpi * betas0)
c        taudf = opt0 * cdthd(lev,iopt)
         taudf = opt0 * cdhdf(lev)
         u1f   = sqrt(taudf * r) / t1
         ajrf  = (r / t1) / sqrt(u1f * u1f + xpi / 4.0_dp)
         taudfr = opt0 * cdhdr(lev)
         u1fr   = sqrt(taudfr * r) / t1
         ajrfr  = (r / t1) / sqrt(u1fr * u1fr + xpi / 4.0_dp)
 
         ikhd = krai + i
         fgkrf(ikhd) = min(ajdf(taudf) + ajrf, 1.0_dp)
         fgkrfr(ikhd) = min(ajdf(taudfr) + ajrfr, 1.0_dp)
 
       end do
 
         krai = krai + nrhdct

       endif

c  Computes CO line profiles

       if (i_co0 /= 0) then

       bdop  = 1.0d8 * sqrt(vt2 + ct2 * tgaz(iopt) / amco0)

       do i=1, nrco0
         njj   = irco0(i) + 1
         gama  = prco0(i,3) / (4.0_dp * xpi)
         xlam0 = prco0(i,2)

         betas = bdop / xlam0
         betas0 = 1.0d8 * vturb / xlam0
         r     = gama / (sqpi * betas)
         t1    = ff1 * (ff2*r) ** (-ff3)
         opt0  = ct1 * prco0(i,1) / (sqpi * betas0)
         taudf = opt0 * cdc0f(njj)
         u1f   = sqrt(taudf * r) / t1
         ajrf  = (r / t1) / sqrt(u1f * u1f + xpi / 4.0_dp)
         taudfr = opt0 * cdc0r(njj)
         u1fr   = sqrt(taudfr * r) / t1
         ajrfr  = (r / t1) / sqrt(u1fr * u1fr + xpi / 4.0_dp)
c
         ikco = krai + i
         fgkrf(ikco) = min(ajdf(taudf) + ajrf, 1.0_dp)
         fgkrfr(ikco) = min(ajdf(taudfr) + ajrfr, 1.0_dp)
       end do

       krai = krai + nrco0

       endif

c  Computes C*O line profiles

       if (i_co1 /= 0) then

       bdop = 1.0d8 * sqrt(vt2 + ct2 * tgaz(iopt) / amco1)

       do i=1, nrco1
         njj   = irco1(i) + 1
         gama  = prco1(i,3) / (4.0_dp * xpi)
         xlam0  = prco1(i,2)

         betas  = bdop / xlam0
         betas0 = 1.0d8 * vturb / xlam0
         r      = gama / (sqpi*betas)
         t1     = ff1 * (ff2*r) ** (-ff3)
         opt0   = ct1 * prco1(i,1) / (sqpi * betas0)
         taudf  = opt0 * cdc1f(njj)
         u1f    = sqrt(taudf * r) / t1
         ajrf   = (r / t1) / sqrt(u1f * u1f + xpi / 4.0_dp)
         taudfr = opt0 * cdc1r(njj)
         u1fr   = sqrt(taudfr * r) / t1
         ajrfr  = (r / t1) / sqrt(u1fr * u1fr + xpi / 4.0_dp)

         ikco1 = krai + i
         fgkrf(ikco1) = min(ajdf(taudf) + ajrf, 1.0_dp)
         fgkrfr(ikco1) = min(ajdf(taudfr) + ajrfr, 1.0_dp)
       end do

       krai = krai + nrco1

       endif

c  Computes CO* line profiles

       if (i_co2 /= 0) then

       bdop = 1.0d8 * sqrt(vt2 + ct2 * tgaz(iopt) / amco2)
 
       do i=1, nrco2
         njj   = irco2(i) + 1
         gama  = prco2(i,3) / (4.0_dp * xpi)
         xlam0  = prco2(i,2)

         betas  = bdop / xlam0
         betas0 = 1.0d8 * vturb / xlam0
         r      = gama / (sqpi*betas)
         t1     = ff1 * (ff2*r) ** (-ff3)
         opt0   = ct1 * prco2(i,1) / (sqpi * betas0)
         taudf  = opt0 * cdc2f(njj)
         u1f    = sqrt(taudf * r) / t1
         ajrf   = (r / t1) / sqrt(u1f * u1f + xpi / 4.0_dp)
         taudfr = opt0 * cdc2r(njj)
         u1fr   = sqrt(taudfr * r) / t1
         ajrfr  = (r / t1) / sqrt(u1fr * u1fr + xpi / 4.0_dp)

         ikco2 = krai + i
         fgkrf(ikco2) = min(ajdf(taudf) + ajrf, 1.0_dp)
         fgkrfr(ikco2) = min(ajdf(taudfr) + ajrfr, 1.0_dp)
       end do

       krai = krai + nrco2

       endif

c  Computes C*O* line profiles

       if (i_co3 /= 0) then

       bdop = 1.0d8 * sqrt(vt2 + ct2 * tgaz(iopt) / amco3)
 
       do i=1, nrco3
         njj   = irco3(i) + 1
         gama  = prco3(i,3) / (4.0_dp * xpi)
         xlam0  = prco3(i,2)

         betas  = bdop / xlam0
         betas0 = 1.0d8 * vturb / xlam0
         r      = gama / (sqpi*betas)
         t1     = ff1 * (ff2*r) ** (-ff3)
         opt0   = ct1 * prco3(i,1) / (sqpi * betas0)
         taudf  = opt0 * cdc3f(njj)
         u1f    = sqrt(taudf * r) / t1
         ajrf   = (r / t1) / sqrt(u1f * u1f + xpi / 4.0_dp)
         taudfr = opt0 * cdc3r(njj)
         u1fr   = sqrt(taudfr * r) / t1
         ajrfr  = (r / t1) / sqrt(u1fr * u1fr + xpi / 4.0_dp)

         ikco3 = krai + i
         fgkrf(ikco3) = min(ajdf(taudf) + ajrf, 1.0_dp)
         fgkrfr(ikco3) = min(ajdf(taudfr) + ajrfr, 1.0_dp)
       end do

       endif

c  Computes radiation field in slab

       do iwlg=1,nrh2bt
         nvl = irah2b(iwlg,1)
         njl = irah2b(iwlg,2)
         if (nvl > 0 .OR. njl >= jfgkh2) then
           xlam0 = fgkwl(iwlg)
           call rf_locate(rf_wl, xlam0, jwl)
           rapwl = (xlam0 - rf_wl%Val(jwl))
     1           / (rf_wl%Val(jwl+1) - rf_wl%Val(jwl))
           aux1 = intJ1(jwl) + (intJ1(jwl+1) - intJ1(jwl)) * rapwl
           aux2 = intJ2(jwl) + (intJ2(jwl+1) - intJ2(jwl)) * rapwl
           fgkgrd(iwlg) = aux1 * fgkrf(iwlg) + aux2 * fgkrfr(iwlg)
           fgkgrd(iwlg) = fgkgrd(iwlg) * J_to_u
         endif
       end do
       do iwlg=nrh2bt+1,nrh2bt+nrh2ct
         nvl = irah2c(iwlg-nrh2bt,1)
         njl = irah2c(iwlg-nrh2bt,2)
         if (nvl > 0 .OR. njl >= jfgkh2) then
           xlam0 = fgkwl(iwlg)
           call rf_locate(rf_wl, xlam0, jwl)
           rapwl = (xlam0 - rf_wl%Val(jwl))
     1           / (rf_wl%Val(jwl+1) - rf_wl%Val(jwl))
           aux1 = intJ1(jwl) + (intJ1(jwl+1) - intJ1(jwl)) * rapwl
           aux2 = intJ2(jwl) + (intJ2(jwl+1) - intJ2(jwl)) * rapwl
           fgkgrd(iwlg) = aux1 * fgkrf(iwlg) + aux2 * fgkrfr(iwlg)
           fgkgrd(iwlg) = fgkgrd(iwlg) * J_to_u
         endif
       end do
       do iwlg=nrh2bt+nrh2ct+1,mfgk
         xlam0 = fgkwl(iwlg)
         call rf_locate(rf_wl, xlam0, jwl)
         rapwl = (xlam0 - rf_wl%Val(jwl))
     1         / (rf_wl%Val(jwl+1) - rf_wl%Val(jwl))
         aux1 = intJ1(jwl) + (intJ1(jwl+1) - intJ1(jwl)) * rapwl
         aux2 = intJ2(jwl) + (intJ2(jwl+1) - intJ2(jwl)) * rapwl
         fgkgrd(iwlg) = aux1 * fgkrf(iwlg) + aux2 * fgkrfr(iwlg)
         fgkgrd(iwlg) = fgkgrd(iwlg) * J_to_u
       end do

       return
       end

c      %%%%%%%%%%%%%%%%%%%%%%%%
       subroutine FGKDAT (iopt)
c      %%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE RF_TOOLS
       USE SP_DATA
       USE H2_DATA
       USE HD_DATA
       USE STR_DATA
       USE PROFIL
       USE CHEM_DATA

c  Prepares data for H2, HD and CO detailed balance
c  Computes H2, HD and CO photodissociation rates
c  Computes C and S ionisation rates

        implicit double precision (a-h,o-z)

        TYPE (RF_TYPE)  :: rf_tmp, rf_tmp1

        real (kind=dp)  :: coefb
        real (KIND=dp), dimension(:), allocatable :: xwl, yvoi, xsum
        real (KIND=dp) nu0, nu

        save

c  Numerical coeficient : pi e**2 / (me * h * c**2) * 1.0e-16 (for Angstrom)

        coefb = 1.0e-16_dp * xpi * xqe2 / (xme * xhp * clum*clum)
        vt2 = vturb * vturb

c  Compute photon flux (used in chemistry)
        call rf_create(rf_tmp, 1, 2)
        call rf_locate(rf_wl, wl_6ev, i2)
        rf_tmp%Val(2) = 1.0_dp
        rf_tmp%Ord(2) = rf_wl%Ord(i2)
        int_rad(iopt) = sect_int(rf_tmp, rf_u, wl_pointer, wl_6ev)
        call rf_free (rf_tmp)

        call rf_create(rf_tmp1, rf_wl%lower, rf_wl%upper)
        rf_tmp1%Val = rf_u%Val * rf_wl%Val
        fluph(iopt) = rf_integration (rf_wl,rf_tmp1) * 1.0d-8 / xhp
        call rf_free (rf_tmp1)

c     calcul du taux de photoionisation de toutes les especes 
c     a partir des sections de photodestruction contenues dans 
c     les fichiers data/section/*.dat : Mars 2006
        CALL rf_create (rf_tmp1, 0, nwlg)
        rf_tmp1%Val = rf_wl%Val * rf_u%Val
        do k = 1, nphot_i - i_mandat
           pdiesp(k,iopt) = sect_int (phdest(k)%rf_esp, rf_tmp1,
     1          wl_pointer, phdest(k)%wl_cut_esp) / (1.0e8_DP * xhp)
        end do
        call rf_free(rf_tmp1)

c  End modif - 14 Jan au 12 Fev 2002

        ct2 = 2.0_dp * xk / amu

c  Radiative pumping
c    H2

        evrdbc(1:l0h2,1:l0h2) = 0.0_dp
        if (i_hd /= 0) then
          evrdhd(1:l0hd,1:l0hd) = 0.0_dp
        endif

        eqrco0 = 0.0_dp
        eqrco1 = 0.0_dp
        eqrco2 = 0.0_dp
        eqrco3 = 0.0_dp

        dumh2 = 0.0_dp
        dumhd = 0.0_dp
        dumy = 0.0_dp
        dumco = 0.0_dp
        dumco1 = 0.0_dp
        dumco2 = 0.0_dp
        dumco3 = 0.0_dp
        pdish2(iopt) = 0.0_dp
        pdh2vJ(:,iopt) = 0.0_dp
        sdh2vJ = 0.0_dp
        pdishd(iopt) = 0.0_dp
        pdico0(iopt) = 0.0_dp
        pdico1(iopt) = 0.0_dp
        pdico2(iopt) = 0.0_dp
        pdico3(iopt) = 0.0_dp

c-----------------------------
c  H2 dissociation probability
c-----------------------------

        bdop = 1.0d8 * sqrt(vt2 + ct2 * tgaz(iopt) / amh2)

        do ik=1,nrh2bt
          nvl = irah2b(ik,1)
          njl = irah2b(ik,2)
          lev = level(nvl,njl)
          xlam0 = prah2b(ik,2)

          if (nvl > 0 .OR. njl >= jfgkh2) then
            chrai = fgkgrd(ik)
          else
            nu0 = 1.0e8_dp * clum / xlam0
            betas = bdop / xlam0
            avoi = prah2b(ik,3) / (4.0_dp * xpi * betas)
            wlmin = 1.0e8_dp * clum / (nu0 + wlrwid * betas)
            wlmax = 1.0e8_dp * clum / (nu0 - wlrwid * betas)
            call rf_locate (rf_wl, wlmin, ilmin)
            call rf_locate (rf_wl, wlmax, ilmax)
            nbil = ilmax-ilmin
            allocate (xwl(0:nbil), yvoi(0:nbil), xsum(0:nbil))
            do j=0,nbil
              nu = 1.0e8_dp * clum / rf_wl%Val(j+ilmin)
              xwl(j) = (nu0 - nu) / betas
            end do

            call humlik (nbil+1, xwl, avoi, yvoi)

            xsum = yvoi * rf_u%Val(ilmin:ilmax)
     1           * 1.0e8 * clum / (betas * rf_wl%Val(ilmin:ilmax)**2)
            call simpint(rf_wl%Val(ilmin:ilmax), xsum, nbil, tata)
            chrai = tata
            deallocate (xwl, yvoi, xsum)
          endif

          nvu = irah2b(ik,3)
          ndj = irah2b(ik,4)
          nju = njl + ndj

          probdi = prah2b(ik,4)
          beinst = coefb * prah2b(ik,1) * (xlam0**3)
          absorb = beinst * chrai
          disici = absorb * probdi
          dumh2 = dumh2 + disici * xreh2(lev,iopt)
          pdh2vJ(lev,iopt) = pdh2vJ(lev,iopt)
     1                     + disici * xreh2(lev,iopt)
          if (iopt == 0) then
            sdh2vJ(lev) = sdh2vJ(lev) + beinst * probdi
          endif

          ! calcul explicite des populations de la bande B
          if ((nvu <= nvbc) .and. (nju <= njbc)) then
            h2bnua(nvu,nju,iopt) = h2bnua(nvu,nju,iopt) +
     1                            absorb * xreh2(lev,iopt)
          endif

c  Radiative equilibrium matrix

          evrdbc(lev,lev) = evrdbc(lev,lev) + absorb

          dumy = absorb * (1.0_dp - probdi)
          if (mod(njl,2) == 0) then
            evrdbc(levpc,lev) = evrdbc(levpc,lev)
     1                       - dumy * bhmbx(nvu,nju,levpc)
          else
            evrdbc(levic,lev) = evrdbc(levic,lev)
     1                        - dumy * bhmbx(nvu,nju,levic)
          endif
        end do

c  End of upper levels population computation
        do ivu=0,nvbc
          do iju=0, njbc
             h2bnua(ivu,iju,iopt) = h2bnua(ivu,iju,iopt)
     1                                / tmunh2bc(1,ivu,iju)
          enddo
        enddo

        krai = nrh2bt
 
        do ik=1,nrh2ct
          ikh2 = krai + ik
          nvl = irah2c(ik,1)
          njl = irah2c(ik,2)
          lev = level(nvl,njl)
          xlam0 = prah2c(ik,2)

          if (nvl > 0 .OR. njl >= jfgkh2) then
            chrai = fgkgrd(ikh2)
          else
            nu0 = 1.0e8_dp * clum / xlam0
            betas = bdop / xlam0
            avoi = prah2c(ik,3) / (4.0_dp * xpi * betas)
            wlmin = 1.0e8_dp * clum / (nu0 + wlrwid * betas)
            wlmax = 1.0e8_dp * clum / (nu0 - wlrwid * betas)
            call rf_locate (rf_wl, wlmin, ilmin)
            call rf_locate (rf_wl, wlmax, ilmax)
            nbil = ilmax-ilmin
            allocate (xwl(0:nbil), yvoi(0:nbil), xsum(0:nbil))
            do j=0,nbil
              nu = 1.0e8_dp * clum / rf_wl%Val(j+ilmin)
              xwl(j) = (nu0 - nu) / betas
            end do

            call humlik (nbil+1, xwl, avoi, yvoi)

            xsum = yvoi * rf_u%Val(ilmin:ilmax)
     1           * 1.0e8 * clum / (betas * rf_wl%Val(ilmin:ilmax)**2)
            call simpint(rf_wl%Val(ilmin:ilmax), xsum, nbil, tata)
            chrai = tata
            deallocate (xwl, yvoi, xsum)
          endif

          nvu = irah2c(ik,3)
          ndj = irah2c(ik,4)
          nju = njl + ndj

          probdi = prah2c(ik,4)
          beinst = coefb * prah2c(ik,1) * (xlam0**3)
          absorb = beinst * chrai
          disici = absorb * probdi
          dumh2 = dumh2 + disici * xreh2(lev,iopt)
          pdh2vJ(lev,iopt) = pdh2vJ(lev,iopt)
     1                     + disici * xreh2(lev,iopt)
          if (iopt == 0) then
            sdh2vJ(lev) = sdh2vJ(lev) + beinst * probdi
          endif

          ! calcul explicite des populations de la bande C
          if ((nvu <= nvbc) .and. (nju <= njbc)) then
            h2cnua(nvu,nju,iopt) = h2cnua(nvu,nju,iopt) +
     1                            absorb * xreh2(lev,iopt)
          endif

          ! Construction de la matrice d'equilibre radiatif
          evrdbc(lev,lev) = evrdbc(lev,lev) + absorb

          dumy = absorb * (1.0_dp - probdi)
          if (mod(njl,2) == 0) then
            evrdbc(levpc,lev) = evrdbc(levpc,lev)
     1                        - dumy * bhmcx(nvu,nju,levpc)
          else
            evrdbc(levic,lev) = evrdbc(levic,lev)
     1                        - dumy * bhmcx(nvu,nju,levic)
          endif

        end do

        ! Fin du calcul des populations des niveaux hauts 
        do ivu=0,nvbc
          do iju=0, njbc
            if (tmunh2bc(2,ivu,iju) > 1.0e-30_dp) then
             h2cnua(ivu,iju,iopt) = h2cnua(ivu,iju,iopt)
     1                              / tmunh2bc(2,ivu,iju)
            else
             h2cnua(ivu,iju,iopt) = -1.0_dp
            endif
          enddo
        enddo

        
        pdish2(iopt) = dumh2
        k = nphot_i - i_mandat + 1
        pdiesp(k,iopt) = dumh2       

        krai = krai + nrh2ct
 
c----------------------------------------------
c  HD dissociation probability
c-----------------------------------------------
 
        if (i_hd /= 0) then
 
        do ik=1,nrhdbt
          ikhd = krai + ik
          nvl = irahdb(ik,1)
          njl = irahdb(ik,2)
          lev = levhd(nvl,njl)
          xlam0 = prahdb(ik,2)
          chrai = fgkgrd(ikhd)
          if (lev > l0hd .or. chrai <= 0.0_dp) cycle
 
          nvu = irahdb(ik,3)
          ndj = irahdb(ik,4)
          nju = njl + ndj
 
          probdi = prahdb(ik,4)
          beinst = coefb * prahdb(ik,1) * (xlam0**3)
          absorb = beinst * chrai
          disici = absorb * probdi
          dumhd = dumhd + disici * xrehd(lev,iopt)
 
          evrdhd(lev,lev) = evrdhd(lev,lev) + absorb

          do levl=1,l0hd
            nj2 = njhd(levl)
            if (nj2 <= mjhd) then
              dumy = absorb * (1.0_dp - probdi)
     1             * bhdbx(nvu,nju,levl)
              evrdhd(levl,lev) = evrdhd(levl,lev) - dumy
            endif
          end do
        end do
 
        krai = krai + nrhdbt

        do ik=1,nrhdct
          ikhd = krai + ik
          nvl = irahdc(ik,1)
          njl = irahdc(ik,2)
          lev = levhd(nvl,njl)
          xlam0 = prahdc(ik,2)
          chrai = fgkgrd(ikhd)
          if (lev > l0hd .or. chrai <= 0.0_dp) cycle
 
          nvu = irahdc(ik,3)
          ndj = irahdc(ik,4)
          nju = njl + ndj
 
          probdi = prahdc(ik,4)
          beinst = coefb * prahdc(ik,1) * (xlam0**3)
          absorb = beinst * chrai
          disici = absorb * probdi
          dumhd = dumhd + disici * xrehd(lev,iopt)
 
          evrdhd(lev,lev) = evrdhd(lev,lev) + absorb
 
          do levl=1,l0hd
            nj2 = njhd(levl)
            if (nj2 <= mjhd) then
             if (ndj == 0) then
              dumy = absorb * (1.0_dp - probdi)
     1             * bhdcmx(nvu,nju,levl)
              evrdhd(levl,lev) = evrdhd(levl,lev) - dumy
             else
              dumy = absorb * (1.0_dp - probdi)
     1             * bhdcpx(nvu,nju,levl)
              evrdhd(levl,lev) = evrdhd(levl,lev) - dumy
            endif
            endif
          end do
        end do
 
        pdishd(iopt) = dumhd
 
        k = k + 1
        pdiesp(k,iopt) = dumhd

        krai = krai + nrhdct

        endif

c------------------------------------------------------
c  CO dissociation probability
c------------------------------------------------------

        do ik=1,nrco0
          njl = irco0(ik)
          probdi = prco0(ik,4)
          xlam0 = prco0(ik,2)

c  coeff num: (pi*e**2)/(m*h*c**2)=1.336d14
c     * 1.0d-16 (pour tenir compte des longueurs d onde en a).

          beinst = coefb * prco0(ik,1) * (xlam0**3)

          ikco = krai + ik
          chrai = fgkgrd(ikco)

          absorb = beinst * chrai
          dumco = dumco + absorb * probdi * xreco0(njl+1,iopt)
          eqrco0(njl+1) = eqrco0(njl+1) + absorb * probdi
        end do

c introduction de la protection de co par h2 (Evelyne)
c  Now obsolete with full transfer
c         dumco = dumco * 0.8457_dp
c    1          * exp(-5.6d-22*sum(cdth2(1:l0h2,iopt)))

        pdico0(iopt) = dumco
        k = k + 1
        pdiesp(k,iopt) = dumco
        krai = krai + nrco0

c  C*O dissociation probability
c  ----------------------------

        if (i_co1 /= 0) then

        do ik=1,nrco1
          njl = irco1(ik)
          probdi = prco1(ik,4)
          xlam0 = prco1(ik,2)

          beinst = coefb * prco1(ik,1) * (xlam0**3)

          ikco1 = krai + ik
          chrai = fgkgrd(ikco1)

          absorb = beinst * chrai
          dumco1 = dumco1 + absorb * probdi * xreco1(njl+1,iopt)
          eqrco1(njl+1) = eqrco1(njl+1) + absorb * probdi
        end do

        pdico1(iopt) = dumco1
        k = k + 1
        pdiesp(k,iopt) = dumco1
        krai = krai + nrco1

        endif

c  CO* dissociation probability
c  ---------------------------------

        if (i_co2 /= 0) then

        do ik=1,nrco2
          njl = irco2(ik)
          probdi = prco2(ik,4)
          xlam0 = prco2(ik,2)

          beinst = coefb * prco2(ik,1) * (xlam0**3)

          ikco2 = krai + ik
          chrai = fgkgrd(ikco2)

          absorb = beinst * chrai
          dumco2 = dumco2 + absorb * probdi * xreco2(njl+1,iopt)
          eqrco2(njl+1) = eqrco2(njl+1) + absorb * probdi
        end do

        pdico2(iopt) = dumco2
        k = k + 1
        pdiesp(k,iopt) = dumco2
        krai = krai + nrco2

        endif

c  C*O* dissociation probability
c  ---------------------------------

        if (i_co3 /= 0) then

        do ik=1,nrco3
          njl = irco3(ik)
          probdi = prco3(ik,4)
          xlam0 = prco3(ik,2)

          beinst = coefb * prco3(ik,1) * (xlam0**3)

          ikco3 = krai + ik
          chrai = fgkgrd(ikco3)

          absorb = beinst * chrai
          dumco3 = dumco3 + absorb * probdi * xreco3(njl+1,iopt)
          eqrco3(njl+1) = eqrco3(njl+1) + absorb * probdi
        end do

        pdico3(iopt) = dumco3
        k = k + 1
        pdiesp(k,iopt) = dumco3

        endif

        PRINT *,"------------------------------"
        PRINT *,"--- Photodestruction (s-1) ---"
        DO k=1,nphot_i
           PRINT *,k, phdest(k)%r1,"+ phot = ",phdest(k)%p1,phdest(k)%p2
     1              , phdest(k)%p3," P(s-1) = ",pdiesp(k,iopt)
        ENDDO
        PRINT *,"------------------------------"


        return
        end

c--------------------------------------------------------------------------

c  Interpolation function, utilisee pour determinee la colonne densitee
c           entre le point courant et le centre du nuage

       double precision function interpcd(rapp, cd_o, i, npo)

         use constantes
         integer i, npo
         real (KIND=dp) rapp, cd_o(0:npo)

         real (KIND=dp) cd_c

         cd_c = cd_o(i) + (cd_o(i+1) - cd_o(i)) * rapp
         interpcd = cd_c

       return
       end

c--------------------------------------------------------------------------

       double precision function ajdf(taudf)

         use constantes
         real (KIND=dp) taudf

         if (taudf < 2.0_dp) then
           ajdf = exp(-2.0_dp * taudf / 3.0_dp)
         else if (taudf < 10.0_dp) then
           ajdf = 0.638_dp * taudf**(-1.25_dp)
         else if (taudf < 1.0d2) then
           ajdf = 0.505_dp * taudf**(-1.15_dp)
         else
           ajdf = 0.344_dp * taudf**(-1.0667_dp)
         endif

       return
       end

c--------------------------------------------------------------------------

       subroutine simpint(x, y, n, som)

c  Compute the sum of y over x by Simpson rule
c  with uneven spacing of the x's

c  Formula established by Pierre Hily-Blant - 8 Sept 2003

c  The sum is then normalised by sqrt(pi)

       use constantes
       implicit none

       integer, intent (in)                        :: n 
       real (kind=dp), intent (in), dimension(0:n) :: x, y
       real (kind=dp), intent (out)                :: som

       integer        :: i
       real (kind=dp) :: x1, x2, x3, y1, y2, y3

       real (kind=dp) :: aux1
       real (kind=dp) :: t1, t2, t3
       real (kind=dp) :: a31, a12, a23

       som = 0.0_dp
       x3 = x(0)
       y3 = y(0)
       do i=1,n-1,2
         x1 = x3
         x2 = x(i)
         x3 = x(i+1)
         y1 = y3
         y2 = y(i)
         y3 = y(i+1)

         a12 = x1 - x2
         a23 = x2 - x3
         a31 = x3 - x1

         aux1 = a12*a12 + a23*a23 + x2 * (x2-x1-x3) + x1*x3
         t1 = -y1 * (3.0_dp * a12*a12 - aux1) / (6.0_dp * a12)
         t2 =  y2 * a31*a31*a31 / (6.0_dp * a12 * a23)
         t3 = -y3 * (3.0_dp * a23*a23 - aux1) / (6.0_dp * a23)

         som = som + t1 + t2 + t3
       end do
       som = som / sqrt(xpi)

       return
       end

