
! Copyright (C) 2002 - Observatoire de Paris/Meudon
!                      Universite Paris 7
!                      CNRS (France)
!                      Contact : Jacques.Lebourlot@obspm.fr
!
! This program is free software; you can redistribute it and/or
! modify it under the terms of the GNU General Public License version 2
! as published by the Free Software Foundation.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


MODULE TRANSFER
! +new formulation for the non-embedded case
!  but incuding R' effects   (spring 2005) jlb
! +theory & code revision (june-july 2005) jrg
! +embedded sources preliminary coding (oct 2005) jlb+jrg

!*****************************************************************
! Radiation Field manipulations tools
!*****************************************************************

  USE constantes

  IMPLICIT NONE

! Extinction computation related dimensions
! (Flannery et al. dev on Legendre polynomials)

  !Effective order of Legendre development
  integer, public                   :: lleg  = 19
! integer, public                   :: lleg  = 11

  integer, public                   :: mleg  = 10   ! mleg = (lleg + 1) / 2
! integer, public                   :: mleg  = 6

 !Local gaz line absorption
 REAL (kind=dp), dimension (:,:), public, allocatable  :: g_absL
 !Integrated gaz line absorption
 REAL (kind=dp), dimension (:,:), public, allocatable  :: g_absI

! Roots of Legendre polynomials

  real (kind=dp), public, dimension(10,10)             :: xmui

! L = 1 -> M = 1
       data xmui(1,1) / 0.577350269189626_dp /

! L = 3 -> M = 2
       data xmui(2,1) / 0.339981043584856_dp /
       data xmui(2,2) / 0.861136311594053_dp /

! L = 5 -> M = 3
       data xmui(3,1) / 0.238619186083197_dp /
       data xmui(3,2) / 0.661209386466265_dp /
       data xmui(3,3) / 0.932469514203152_dp /

! L = 7 -> M = 4
       data xmui(4,1) / 0.183434642495650_dp /
       data xmui(4,2) / 0.525532409916329_dp /
       data xmui(4,3) / 0.796666477413627_dp /
       data xmui(4,4) / 0.960289856497536_dp /

! L = 9 -> M = 5
       data xmui(5,1) / 0.148874338981631_dp /
       data xmui(5,2) / 0.433395394129247_dp /
       data xmui(5,3) / 0.679409568299024_dp /
       data xmui(5,4) / 0.865063366688985_dp /
       data xmui(5,5) / 0.973906528517172_dp /

! L = 11 -> M = 6
       data xmui(6,1) / 0.125233408511469_dp /
       data xmui(6,2) / 0.367831498998180_dp /
       data xmui(6,3) / 0.587317954286617_dp /
       data xmui(6,4) / 0.769902674194305_dp /
       data xmui(6,5) / 0.904117256370475_dp /
       data xmui(6,6) / 0.981560634246719_dp /

! L = 15 -> M = 8
       data xmui(8,1) / 0.095012509837637_dp /
       data xmui(8,2) / 0.281603550779258_dp /
       data xmui(8,3) / 0.458016777657227_dp /
       data xmui(8,4) / 0.617876244402643_dp /
       data xmui(8,5) / 0.755404408355003_dp /
       data xmui(8,6) / 0.865631202387831_dp /
       data xmui(8,7) / 0.944575023073232_dp /
       data xmui(8,8) / 0.989400934991649_dp /

! L = 19 -> M = 10
       data xmui(10,1) / 0.076526521133497_dp /
       data xmui(10,2) / 0.227785851141645_dp /
       data xmui(10,3) / 0.373706088715419_dp /
       data xmui(10,4) / 0.510867001950827_dp /
       data xmui(10,5) / 0.636053680726515_dp /
       data xmui(10,6) / 0.746331906460150_dp /
       data xmui(10,7) / 0.839116971822218_dp /
       data xmui(10,8) / 0.912234428251325_dp /
       data xmui(10,9) / 0.963971927277913_dp /
       data xmui(10,10) / 0.993128599185094_dp /


  private

  public :: etin90, etin901s

!------------------------------------------------------------------------

contains

SUBROUTINE etin90

 USE constantes
 USE auxiliaries
 USE inoutput
 USE RF_TOOLS
 USE STR_DATA
 USE PROFIL
 USE CHEM_DATA

! Radiative transfer computation. See file transfer_S.pdf for definitions
! and comments. Index conventions:
!       l goes from 0 to lleg (first Legendre Pol. index)
!       m goes from +/- 1 to +/- mleg (second Legendre Pol. index)
!       n goes from 0 to npo (optical depth grid)

 IMPLICIT NONE
 INTEGER                                     :: l, m, n, n1, n3

! Radiative quantities

 INTEGER                                       :: iwlg
 REAL (kind=dp)                                :: coef2
 REAL (kind=dp), dimension(:,:),   allocatable :: tl       ! Full optical depth
 REAL (kind=dp), dimension(:),     allocatable :: dtl      ! Full optical depth
 REAL (kind=dp), dimension(:),     allocatable :: sigl     ! Phase function

 REAL (kind=dp), dimension(:,:),   allocatable :: albed    ! Reduced albedo (including gas)

 REAL (kind=dp), dimension(:),     allocatable :: kap_dus
 REAL (kind=dp), dimension(:),     allocatable :: kap_gas
 REAL (kind=dp), dimension(:),     allocatable :: sca_dus
 REAL (kind=dp), dimension(:),     allocatable :: amp      ! Legendre amplitudes

 type(rf_type)                                 :: rf_tmp, rf_tmp1

! Legendre polynomial related variables
 REAL (kind=dp), dimension(:),     allocatable :: xmunl    ! -1 power l  = (-1)**l
 REAL (kind=dp), dimension(:,:),   allocatable :: pli      ! P_l(mu_i)
 REAL (kind=dp), dimension(:),     allocatable :: p_0
 REAL (kind=dp), dimension(:),     allocatable :: q_0

! Matrix A coefficients
 REAL (kind=dp), dimension(:,:),   allocatable :: hl
 REAL (kind=dp), dimension(:),     allocatable :: wr, wi
 REAL (kind=dp), dimension(:,:),   allocatable :: xkm      ! Legendre absissa
 REAL (kind=dp), dimension(:,:),   allocatable :: Smk      ! integrated
 REAL (kind=dp), dimension(:,:),   allocatable :: Emk_m    ! Exp(-Smk) 0 -> t
 REAL (kind=dp), dimension(:,:),   allocatable :: Emk_p    ! Exp(-Smk) t -> t_max
 REAL (kind=dp), dimension(:,:,:), allocatable :: Emij     ! Exp(-(Smk(i)-Smk(j))) t_i -> t_j
 REAL (kind=dp), dimension(:,:,:), allocatable :: rlm      ! Eigenvectors matrix
 REAL (kind=dp), dimension(:,:,:), allocatable :: rlmp     ! derivative
 REAL (kind=dp), dimension(:,:,:), allocatable :: rlmi     ! inverse of Rlm(t)

 !Matrix product of R^-1 x R'
 REAL (kind=dp), dimension(:,:,:), allocatable :: rlmip
 REAL (kind=dp), dimension(:,:),   allocatable :: Srm

! Source function contributions

 !Source function expansion coeficients (as function of l and m respectively)
 REAL (kind=dp), dimension(:,:),   allocatable :: sfl, sfm, sfl_g, sfl_d
 REAL (kind=dp), dimension(:,:),   allocatable :: ggm      ! Inhomogeneous contribution (g vector in notes)
 REAL (kind=dp), dimension(:,:),   allocatable :: Git      ! Product R^-1*A^-1*g vector, same dimensions as Qit


! Boundary conditions related variables

 REAL (kind=dp), dimension(:,:),   allocatable :: Tim      ! Auxiliary
 REAL (kind=dp), dimension(:,:),   allocatable :: Bim      ! BC matrix
 REAL (kind=dp), dimension(:),     allocatable :: Iinc     ! Incident radiation field

 REAL (kind=dp)                                :: wl, wlo, geom, bbstar
 REAL (kind=dp)                                :: fluxm, fluxp

! Iteration related variables

 REAL (kind=dp), dimension(:,:),   allocatable :: Qit
 REAL (kind=dp), dimension(:,:),   allocatable :: Qit_o
 REAL (kind=dp), dimension(:,:),   allocatable :: Dit
 REAL (kind=dp), dimension(:,:),   allocatable :: Yit
 REAL (kind=dp), dimension(:,:),   allocatable :: Yit_o
 REAL (kind=dp), dimension(:),     allocatable :: Hit

! LAPACK related variables

 !Eigenvalues only
 CHARACTER (LEN=1)                             :: JOBZ  = 'N'
 REAL (kind=dp), dimension(:),     allocatable :: WORK           ! Real work array
 INTEGER,        dimension(:),     allocatable :: IWORK          ! Work array
 REAL (kind=dp), dimension(:,:),   allocatable :: aam, bbm
 REAL (kind=dp), dimension(:,:),   allocatable :: aamf, bbmf
 INTEGER                                       :: info
 CHARACTER (LEN=1)                             :: FACT  = 'E'    ! Equilibrate system
 CHARACTER (LEN=1)                             :: TRANS = 'N'    ! Transpose
 INTEGER                                       :: nrhs
 CHARACTER (LEN=1)                             :: EQUED          ! Kind of equilibration
 REAL (kind=dp), dimension(:), allocatable     :: R, C           ! Equilibration factors
 REAL (kind=dp), dimension(:), allocatable     :: FERR, BERR     ! Errors (estimates)
 REAL (kind=dp)                                :: RCOND          ! Condition number
 INTEGER, dimension(:), allocatable            :: indx

! Auxiliaries
 INTEGER                                       :: nl, nnit
 INTEGER                                       :: i, it
 INTEGER                                       :: i1, i2
 REAL (kind=dp)                                :: ht, ymui
 REAL (kind=dp)                                :: enor_m, enor_p
 REAL (kind=dp)                                :: flu_rad0, flu_rad1, flu_rad2
 REAL (kind=dp)                                :: taux1, taux2
 REAL (kind=dp)                                :: taux3, taux4
 REAL (kind=dp)                                :: wlth, wlthcm, extpou
 REAL (kind=dp)                                :: deriv_d
 REAL (kind=dp), dimension(:), allocatable     :: argu
 REAL (kind=dp), dimension(-mleg:mleg)         :: auxil
 INTEGER, dimension(:,:), allocatable          :: indmi, indmj

!Thermalized (black body) emission of dust and gas
 REAL (kind=dp)                                :: bbdust
 REAL(kind=dp)                                 :: bbgas

!Others
 REAL (kind=dp)                                :: wl_cut_C

!-----------------------------------------------------------------------------

! Number of Legendred coefficient (used for loops that run from 1 to lleg+1)

  nl = lleg + 1

       if ( ALLOCATED(g_absI)) then
         deallocate (g_absI)
         deallocate (g_absL)
       endif

       if ( ALLOCATED(intJm)) then
         deallocate (intJm, intJp)
         deallocate (bbspm, bbspp)
       endif


  allocate (xmunl(0:lleg))
  allocate (pli(0:lleg,-mleg:mleg))
  allocate (p_0(0:lleg), q_0(0:lleg))
  allocate (hl(0:lleg,0:npo))
  allocate (wr(lleg+1), wi(lleg+1))
  allocate (xkm(-mleg:mleg,0:npo))
  allocate (Smk(mleg,0:npo))
  allocate (Emk_m(mleg,0:npo))
  allocate (Emk_p(mleg,0:npo))
  allocate (Emij(mleg,0:npo,0:npo))
  allocate (rlm(0:lleg,-mleg:mleg,0:npo))
  allocate (rlmp(0:lleg,-mleg:mleg,0:npo))
  allocate (rlmi(0:lleg,-mleg:mleg,0:npo))
  allocate (rlmip(0:lleg,-mleg:mleg,0:npo))
  allocate (sfm(-mleg:mleg,0:npo))
  allocate (sfl(0:lleg,0:npo))
  allocate (sfl_g(0:lleg,0:npo),sfl_d(0:lleg,0:npo))
  allocate (ggm(-mleg:mleg,0:npo))
  allocate (Srm(mleg,0:npo))
  allocate (sigl(0:lleg))
  allocate (dtl(0:npo))
  allocate (tl(0:npo,0:nwlg))
  allocate (albed(0:npo,0:nwlg))
  allocate (kap_dus(0:npo))
  allocate (kap_gas(0:npo))
  allocate (sca_dus(0:npo))
  allocate (g_absI(0:npo,0:nwlg))
  allocate (g_absL(0:npo,0:nwlg))
  allocate (intJm(0:npo,0:nwlg))
  allocate (intJp(0:npo,0:nwlg))
  allocate (bbspm(0:npo,0:nwlg))
  allocate (bbspp(0:npo,0:nwlg))
  allocate (R(nl), C(nl))
  allocate (FERR(1), BERR(1))
  allocate (WORK(4*nl))
  allocate (IWORK(nl))
  allocate (aam(nl,nl), bbm(nl,1))
  allocate (aamf(nl,nl), bbmf(nl,1))
  allocate (indx(nl))
  allocate (Tim(-mleg:mleg,-mleg:mleg))
  allocate (Bim(-mleg:mleg,-mleg:mleg))
  allocate (Iinc(-mleg:mleg))
  allocate (Qit(-mleg:mleg,0:npo))
  allocate (Qit_o(-mleg:mleg,0:npo))
  allocate (Git(-mleg:mleg,0:npo))
  allocate (Dit(-mleg:mleg,0:npo))
  allocate (Yit(-mleg:mleg,0:npo))
  allocate (Yit_o(-mleg:mleg,0:npo))
  allocate (Hit(-mleg:mleg))
  allocate (amp(-mleg:mleg))
  allocate (argu(0:npo))
  allocate (indmi(mleg,0:npo), indmj(mleg,0:npo))

  p_0 = 0.0_dp
  q_0 = 0.0_dp
  sigl = 0.0_dp
  Tim = 0.0_dp
  Bim = 0.0_dp
  Iinc = 0.0_dp
  xkm = 0.0_dp
  Qit = 0.0_dp
  Qit_o = 0.0_dp

! This is just (-1)**l

  xmunl(0) = 1.0_dp
  do l=1,lleg
    xmunl(l) = - xmunl(l-1)
  end do

! STEP 0 - Initialise radiative quantities
!  (boundary conditions and optical depth grid)

!  uvspec.out is the total absorption through the cloud. multiply by
!             incident radition field to compute spectrum.

  fichier = TRIM(out_dir)//TRIM(modele)//'.uv'
  open (37, file=fichier, status="unknown")
  fichier = TRIM(out_dir)//TRIM(modele)//'.flin'
  open (60, file=fichier, status="unknown")

  coef2 = tautav * cdunit / rrr

!  Initialise illuminating star parameters, if any...
!  geom is (1 - cos(theta_0)).
!  A second order development is adequate in most cases.

  if (d_sour /= 0.0_dp) then
    if (srcpp(1:2) == 'F_') then
      nbpt = 1
      wlo = wlth0
      wl = rf_wl%Val(wl_pointer(rf_exter%Ord(2)))
    else
      call STAR
    endif
    geom = 0.5_dp * (rstar/(d_sour*pccm))**2
  endif

!  sigl = gg**l (Formule 48, Henyey-Greenstein phase function)

  sigl(0) = 1.0_dp
  do l=1,lleg
    sigl(l) = sigl(l-1) * gg
  end do

! Compute absorption by gas

  g_absI = 0.0_dp
  call ABSGAZ

! STEP 1 - Compute P_l(mu_i) - see recurence in Num Rec

  pli(0,:) = 1.0_dp
  do m=1,mleg
    ymui = xmui(mleg,m)
    pli(1,-m) = -ymui
    pli(1,m) = ymui
    do l=1,lleg-1
      pli(l+1,-m) = (-ymui*(2.0_dp*l+1.0_dp) * pli(l,-m) - l * pli(l-1,-m)) / (l+1.0_dp)
      pli(l+1,m) = xmunl(l+1) * pli(l+1,-m)
    end do
  end do


  p_0(0) = 1.0_dp
  q_0(1) = p_0(0) / 2.0_dp
  do l=1,lleg/2
    p_0(2*l) = - (2.0_dp*l-1) * p_0(2*l-2) / (2.0_dp*l)
    q_0(2*l+1) = p_0(2*l) / (2.0_dp*l+2.0_dp)
    !write(*,*) l,2*l+1,  p_0(2*l), q_0(2*l+1)  !Flag
  end do

  do iwlg=0,nwlg

! a - Reset to 0 most variables

    hl = 0.0_dp
    wr = 0.0_dp
    wi = 0.0_dp
    tl(:,iwlg) = 0.0_dp
    dtl = 0.0_dp
    albed(:,iwlg) = 0.0_dp
    kap_dus = 0.0_dp
    kap_gas = 0.0_dp
    sca_dus = 0.0_dp

    wlth = rf_wl%Val(iwlg)
    wlthcm = wlth * 1.0e-8_dp
    extpou = rf_extpou%Val(iwlg)

! Compute local values ob absorption coef cor dust and gas and dust scatering coef
! Compute effective albedo ! ??? Problems with effective albedo definition, jrg > jlb

    do n=0,npo
      kap_dus(n) = (1.0_dp - alb) * extpou * dens_o(n) / coef2
      kap_gas(n) = g_absL(n,iwlg)
      sca_dus(n) = alb * extpou * dens_o(n) / coef2
      albed(n,iwlg) = sca_dus(n) / (kap_dus(n)+kap_gas(n)+sca_dus(n))
    end do

! b - Compute optical depth grid
!     Convert Delta Tau_v (dust only at V) to
!     Delta tau_lambda (dust + gaz at Lambda)

    tl(0,iwlg) = 0.0_dp
    do n=0,npo-1
      ht = ((kap_dus(n)+kap_gas(n)+sca_dus(n))/dens_o(n) &
         + (kap_dus(n+1)+kap_gas(n+1)+sca_dus(n+1))/dens_o(n+1)) &
         * 0.5_dp * coef2 *(tau_o(n+1) - tau_o(n))
      dtl(n) = ht
      tl(n+1,iwlg) = tl(n,iwlg) + ht
!     print*, n+1, tau_o(n+1), tl(n+1,iwlg), ht    !Flag : point/tau_V/tau_d+g
    end do

! STEP 2a - Compute matrix A(tau)

    do n=0,npo

! Formulae A2 and A3 of Roberge:
      hl(0,n) = 1.0_dp - albed(n,iwlg)
      do l=1,lleg
        hl(l,n) = (2.0_dp*l+1.0_dp) * (1.0_dp - albed(n,iwlg) * sigl(l))
      end do
      do i=1,lleg
        wi(i) = i / sqrt(hl(i-1,n)*hl(i,n))
      end do
      wr = 0.0_dp

!  Compute eigenvalues (tridiagonal symetric matrix) - LAPACK

      call DSTEVD(JOBZ, nl, wr, wi, aam, nl, WORK, 4*nl, IWORK, nl, info )

!  order eigenvalues by decreasing order

      wi = wr
      do i=1,nl
        wr(i) = wi(nl+1-i)
      end do

!  km = 1 / lamda_m in increasing order

      do m=1,mleg
        xkm(m,n) = 1.0_dp / wr(m)
        xkm(-m,n) = -xkm(m,n)
      end do

! Calcul des Rlm: Formule A9 de Roberge 1983
! Eigenvectors computed only from recurrence relations.

      do m=1,mleg
        rlm(0,m,n)  = 1.0_dp
        rlm(0,-m,n) = 1.0_dp
        rlm(1,m,n)  = (1.0_dp - albed(n,iwlg)) / xkm(m,n)
        rlm(1,-m,n) = -rlm(1,m,n)                             !jrg
        !rlm(1,-m,n) = -(1.0_dp - albed(n,iwlg)) / xkm(m,n)   !jlb
        do l=2,lleg
          rlm(l,m,n) = (hl(l-1,n) * rlm(l-1,m,n) &
                     - (l-1) * xkm(m,n) * rlm(l-2,m,n)) / (l*xkm(m,n))
          rlm(l,-m,n) = xmunl(l) * rlm(l,m,n)
        end do
      end do

    end do   ! end loop in npo

! STEP 2c - Compute Srm - used for mean intensity

    call aux_Srm(q_0, rlm, Srm)

! STEP 3 - Compute Smk = Sum (xkm). Simple Simpson rule

    Smk(:,0) = 0.0_dp
    do n=1,npo
      do m=1,mleg
        Smk(m,n) = Smk(m,n-1) + 0.5_dp * dtl(n-1) * (xkm(m,n) + xkm(m,n-1))
      end do
    end do

    Emk_m = exp(-Smk)   !Asignation of the whole matrix !!
    do m=1,mleg         !Compute Emk_p(m,t') to any computed t'
      Emk_p(m,:) = exp(-(Smk(m,npo)-Smk(m,:)))
    end do

    call aux_Emij(Smk, Emij, indmi, indmj)

! STEP 4 - Compute R**-1    !Loop to all cloud opacity points

    call aux_Rlmi(rlm, hl, rlmi)

! STEP 5 - Compute source function contribution g~ = R^-1*A^-1*g (in notes)
!          JLB : Check Javier correction (not yet done here)

    do n=0,npo

      !Ex. Black body emission of dust 
      bbdust = 2.0_dp*xhp*clum**2 / (wlthcm)**5
      bbdust = bbdust * 1.0e-8_dp / (exp (hcsurk/(wlthcm*tdust(1,n))) -1.0_dp)

      !Ex. Black body emission of thermalized gas 
      bbgas = 2.0_dp*xhp*clum**2 / (wlthcm)**5
      bbgas = bbgas * 1.0e-8_dp / (exp (hcsurk/(wlthcm*tgaz(n))) -1.0_dp)
!     bbgas = bbgas * 1.0e-8_dp / (exp (hcsurk/(wlthcm*3.55e3_dp)) -1.0_dp)

! 5a - Asign source function, isotropic  (PROVISIONAL)

      !METHOD B1: aprox. to continuum thermal isotropic emission
      sfl_d(0,n) = bbdust * kap_dus(n) / (kap_dus(n)+kap_gas(n)+sca_dus(n))

      !METHOD B2: aprox .to line thermal isotropic emission
      sfl_g(0,n) = bbgas * kap_gas(n) / (kap_dus(n)+kap_gas(n)+sca_dus(n))
      !print*, 'GAS SF:', sfl_g(0,n), 'DUST SF:', sfl_d(0,n) !Flag

      !Add both contributions
      sfl(0,n) = sfl_g(0,n) + sfl_d(0,n)
!     sfl(0,n) = 0.0_dp

      sfl(1:lleg,n) = 0.0_dp
    enddo

    !Put them in m,n desired order
    sfm(-mleg:-1,:) = sfl(0:mleg-1,:)
    sfm(1:mleg,:) = sfl(mleg:lleg,:)

! 5b - Compute vector g

    do n=0,npo
       do m=-mleg,-1
          l = mleg + m
          ggm(m,n) = -sfm(m,n) / ( 1.0_dp - albed(n,iwlg) * sigl(l) )
       enddo
       do m=1,mleg
          l = mleg + m - 1
          ggm(m,n) = -sfm(m,n) / ( 1.0_dp - albed(n,iwlg) * sigl(l) )
       enddo
     enddo

! 5d - Compute matrix product  R^-1*A^-1*g = Git

   call aux_Git(rlmi, ggm, xkm, Git)

! STEP 6 - Compute derivatives of Rlm

! 6c - derivative of Rlm

    do n=2,npo-2
      do m=1,mleg
        rlmp(0,m,n) = 0.0_dp
        rlmp(0,-m,n) = 0.0_dp
        n1 = n - 2
        n3 = n + 2
        do l=1,lleg
!  Direct variable
          deriv_d = rlm(l,m,n1) * (2.0_dp*tl(n,iwlg) - tl(n,iwlg) - tl(n3,iwlg))   &
                                / ((tl(n1,iwlg)-tl(n,iwlg)) * (tl(n1,iwlg)-tl(n3,iwlg))) &
                  + rlm(l,m,n)  * (2.0_dp*tl(n,iwlg) - tl(n1,iwlg) - tl(n3,iwlg)) &
                                / ((tl(n,iwlg)-tl(n3,iwlg)) * (tl(n,iwlg)-tl(n1,iwlg))) &
                  + rlm(l,m,n3) * (2.0_dp*tl(n,iwlg) - tl(n1,iwlg) - tl(n,iwlg))   &
                                / ((tl(n3,iwlg)-tl(n1,iwlg)) * (tl(n3,iwlg)-tl(n,iwlg)))
          rlmp(l,m,n) = deriv_d
          rlmp(l,-m,n) = xmunl(l) * rlmp(l,m,n)
        end do
      end do
    end do

!  Boundary condition. Replace par linear extrapolation (TODO)

    do m=1,mleg
      rlmp(0,m,0) = 0.0_dp
      rlmp(0,-m,0) = 0.0_dp
      rlmp(0,m,1) = 0.0_dp
      rlmp(0,-m,1) = 0.0_dp
      rlmp(0,m,npo-1) = 0.0_dp
      rlmp(0,-m,npo-1) = 0.0_dp
      rlmp(0,m,npo) = 0.0_dp
      rlmp(0,-m,npo) = 0.0_dp
      do l=1,lleg
        rlmp(l,m,0) = rlmp(l,m,2)
        rlmp(l,-m,0) = xmunl(l) * rlmp(l,m,0)
        rlmp(l,m,1) = rlmp(l,m,2)
        rlmp(l,-m,1) = xmunl(l) * rlmp(l,m,1)
        rlmp(l,m,npo-1) = rlmp(l,m,npo-2)
        rlmp(l,-m,npo-1) = xmunl(l) * rlmp(l,m,npo-1)
        rlmp(l,m,npo) = rlmp(l,m,npo-2)
        rlmp(l,-m,npo) = xmunl(l) * rlmp(l,m,npo-2)
      end do
    end do

! 6d - Multiply by R**-1

    call aux_Rlmip(rlmi, rlmp, rlmip)

! STEP 7 - Compute Tim

    do m=1,mleg
      do i=1,mleg
        taux1 = 0.0_dp
        taux2 = 0.0_dp
        taux3 = 0.0_dp
        taux4 = 0.0_dp
        do l=0,lleg
          taux1 = taux1 + (2.0_dp * l + 1.0_dp) * rlm(l,-m,0) * pli(l,-i)
          taux2 = taux2 + (2.0_dp * l + 1.0_dp) * rlm(l,m,0) * pli(l,-i)
          taux3 = taux3 + (2.0_dp * l + 1.0_dp) * rlm(l,-m,npo) * pli(l,i)
          taux4 = taux4 + (2.0_dp * l + 1.0_dp) * rlm(l,m,npo) * pli(l,i)
        end do
        Tim(-i,-m) = taux1
        Tim(-i,m)  = taux2
        Tim(i,-m)  = taux3
        Tim(i,m)   = taux4
      end do
    end do

! STEP 8 - Compute Bim (Boundary conditions matrix)

    do m=1,mleg
      do i=1,mleg
        Bim(-i,-m) = Tim(-i,-m)
        Bim(-i,m) = Tim(-i,m) * Emk_m(m,npo)
        Bim(i,-m) = Tim(i,-m) * Emk_m(m,npo)
        Bim(i,m) = Tim(i,m)
      end do
    end do

! STEP 9 - Set boundary conditions

!  Compute total absorption by dust + gas through whole cloud

    taux1 = taumax * extpou + g_absI(npo,iwlg)
    write (37,"(1p,3e17.7E3)") wlth, exp(-taux1), taux1

!  Add potential // source (star)
!  Warning: lambda : 4 times in cm and 1 in Angstrom...

    if (d_sour < 0.0_dp) then

      if (srcpp(1:2) == 'F_') then

        do
          if (wlth > rf_wl%Val(wl_pointer(rf_exter%Ord(nbpt+1)))) then
            nbpt = nbpt + 1
            wlo = wl
            wl = rf_wl%Val(wl_pointer(rf_exter%Ord(nbpt+1)))
          else
            exit
          endif
        end do
        if (wlo == wlth) then
          bbstar = rf_exter%Val(nbpt)
        else if (wl == wlth) then
          bbstar = rf_exter%Val(nbpt+1)
        else
          bbstar = rf_exter%Val(nbpt) + (rf_exter%Val(nbpt+1)-rf_exter%Val(nbpt)) &
                 * (wlth - wlo) / (wl - wlo)
        endif
      else
        bbstar = 1.0e-8_dp*2.0_dp*xhp*clum*clum / wlthcm**5
        bbstar = bbstar / (exp(hcsurk/(teff*wlthcm)) - 1.0_dp)
      endif

      fluxm = 2.0_dp * geom * bbstar / (2.0_dp - xmui(mleg,mleg) - xmui(mleg,mleg-1))
      fluxp = 0.0_dp

      write (60, '(1p,4e15.6)') wlth, bbstar, fluxm, rf_incI%Val(iwlg)

    else if (d_sour > 0.0_dp) then

      bbstar = 1.0e-8_dp*2.0_dp*xhp*clum*clum / wlthcm**5
      bbstar = bbstar / (exp(hcsurk/(teff*wlthcm)) - 1.0_dp)

      fluxm = 0.0_dp
      fluxp = 2.0_dp * geom * bbstar / (2.0_dp - xmui(mleg,mleg) - xmui(mleg,mleg-1))

      write (60, '(1p,4e15.6)') wlth, bbstar, fluxp, rf_incI%Val(iwlg)

    else

      fluxm = 0.0_dp
      fluxp = 0.0_dp
      write (60, '(1p,4e15.6)') wlth, rf_incI%Val(iwlg)

    endif

! 19 XII 2005 - Use "real" units (specific intensity)

    Iinc(-mleg:-1) = radm0 * rf_incI%Val(iwlg)
    Iinc(1:mleg)   = radp0 * rf_incI%Val(iwlg)

! mu_i = +/- 1 is on +/- mleg (not +/- 1)

    Iinc(-mleg) = radm0 * rf_incI%Val(iwlg) + fluxm
    Iinc(mleg)  = radp0 * rf_incI%Val(iwlg) + fluxp

! STEP 10 - Start iteration with an appropriate guess for Y and Q

! 10a - Compute Y_o = R^-1 x F_approx
!       F_approx ~ sfm / 1 - albed + Iexp(kt), then it IS wavelength and tau dependent

     call aux_Yit1(Iinc, xkm, tl(:,iwlg), albed(:,iwlg), rlmi, sfm, Yit_o)

! 10b - Compute Q_o = R^-1 x R' x Y_o

     call aux_Qit(rlmip, Yit_o, Qit)

     nnit = 200

     !Git = 0.0_dp !old procedure if Git = 0 and Qit = 0
     !Qit = 0.0_dp !tests javi

     do it=1,nnit ! begin loop :: 101

       ! STEP 11 - Compute D  - Simple trapezoidal rule
       Dit(-mleg:-1,0) = 0.0_dp
       do n=1,npo
         do m=-mleg,-1
           argu = 0.0_dp
           do i=indmi(-m,n),n
             argu(i) = (Qit(m,i) - Git(m,i)) * Emij(-m,n,i)
           end do
           taux1 = SUM( (/ (dtl(i-1) * (argu(i-1)+argu(i)), i=indmi(-m,n)+1,n) /) )
           Dit(m,n) = taux1 * 0.5_dp
         end do
       end do

       Dit(1:mleg,npo) = 0.0_dp
       do n=0,npo-1
         do m=1,mleg
           argu = 0.0_dp
           do i=n,indmj(m,n)
             argu(i) = (Qit(m,i) - Git(m,i)) * Emij(m,i,n)
           end do
           taux1 = SUM( (/ (dtl(i) * (argu(i+1)+argu(i)), i=n,indmj(m,n)-1) /) )
           Dit(m,n) = taux1 * 0.5_dp
         end do
       end do

! STEP 12 - Compute Hit

       Hit = 0.0_dp
       do i=1,mleg
         taux1 = 0.0_dp
         taux2 = 0.0_dp
         do m=1,mleg
           taux1 = taux1 + Dit(-m,npo) * Tim(i,-m)
           taux2 = taux2 + Dit(m,0) * Tim(-i,m)
           !write(*,*) Dit(-m,npo), Dit(m,0) !Flag
         end do
         Hit(-i) = Iinc(-i) - taux2
         Hit(i) = Iinc(i) + taux1
         !write(*,*) 'i, Hi:', i, Hit(-i), Hit(i)  !Flag
       end do

! STEP 13 - solve using LAPACK

       aam(1:mleg,1:mleg) = Bim(-mleg:-1,-mleg:-1)
       aam(1:mleg,mleg+1:2*mleg) = Bim(-mleg:-1,1:mleg)
       aam(mleg+1:nl,1:mleg) = Bim(1:mleg,-mleg:-1)
       aam(mleg+1:nl,mleg+1:nl) = Bim(1:mleg,1:mleg)

       bbm(1:mleg,1) = Hit(-mleg:-1)
       bbm(mleg+1:nl,1) = Hit(1:mleg)

       nrhs = 1
       call dgesvx (FACT, TRANS, nl, nrhs, aam, nl, aamf, nl, indx, EQUED, &
            R, C, bbm, nl, bbmf, nl, RCOND, FERR, BERR, WORK, IWORK, info)

       amp(-mleg:-1) = bbmf(1:mleg,1)
       amp(1:mleg) = bbmf(mleg+1:nl,1)

! STEP 14 - Compute Y

       do n=0,npo
         do m=1,mleg
           Yit(-m,n) = amp(-m) * Emk_m(m,n) - Dit(-m,n)
           Yit(m,n)  = amp(m)  * Emk_p(m,n) + Dit(m,n)
         end do
       end do

! STEP 15 - Compute new Q

       Qit_o = Qit
       !write(*,*) Qit_o  !Flag

       call aux_Qit(rlmip, Yit, Qit)

! STEP 16 - convergence ?

       auxil(0) = 0.0_dp
       taux2 = 0.0_dp
       do n=0,npo
         do m=-mleg,-1
           if (Qit(m,n) /= 0.0_dp) then
             auxil(m) = abs ((Qit(m,n)-Qit_o(m,n))/Qit(m,n))
           else
             auxil(m) = 0.0_dp
           endif
         end do
         do m=1,mleg
           if (Qit(m,n) /= 0.0_dp) then
             auxil(m) = abs ((Qit(m,n)-Qit_o(m,n))/Qit(m,n))
           else
             auxil(m) = 0.0_dp
           endif
         end do
         taux2 = taux2 + SUM(auxil)
       end do

       if (taux2 < 1.0e-3_dp) then
!        print*, iwlg, 'qm converges in iter.', it  !
         exit      !exits sub-iteration loop
       endif

     end do        !Exits loop to find q  :: 101
!    print *, " "

! STEP 17 - Compute radiation field - mean intensity
!           intJm is 0.5 sum(-1->0) of I
!           intJp is 0.5 sum(0->1) of I
!           Thus intJm + intJp = J in erg cm-2 s-1 A-1 sterad-1
!           one gets u in erg cm-3 A-1 by u = J * 4 pi / c

     do n=0,npo
       taux1 = 0.0_dp
       taux2 = 0.0_dp
       do m=1,mleg
         taux1 = taux1 + Yit(-m,n) * (1.0_dp + Srm(m,n)) &
                       + Yit(m,n) * (1.0_dp - Srm(m,n))
         taux2 = taux2 + Yit(-m,n) * (1.0_dp - Srm(m,n)) &
                       + Yit(m,n) * (1.0_dp + Srm(m,n))
       enddo
! 19 XII 2005 - Change normalisation

       intJm(n,iwlg) = 0.5_dp * taux1
       intJp(n,iwlg) = 0.5_dp * taux2
     end do

! Compute escape probabilities
! Half system only

     do n=0,npo        !position
       call aux_bp(rlm, pli, n, aam)
       bbm(1:mleg,1) = 1.0_dp

       call dgesvx (FACT, TRANS, mleg, nrhs, aam, nl, aamf, nl, indx, EQUED, &
                    R, C, bbm, nl, bbmf, nl, RCOND, FERR, BERR, WORK, IWORK, info)

       taux1 = 0.0_dp
       do m=mleg,1,-1
         taux1 = taux1 + bbmf(m,1) * Emk_p(m,n)
       end do
       bbspp(n,iwlg) = taux1
     end do

     do n=0,npo
       call aux_bm(rlm, pli, n, aam)
       bbm(1:mleg,1) = 1.0_dp

       call dgesvx (FACT, TRANS, mleg, nrhs, aam, nl, aamf, nl, indx, EQUED, &
                    R, C, bbm, nl, bbmf, nl, RCOND, FERR, BERR, WORK, IWORK, info)

       taux1 = 0.0_dp
       do m=mleg,1,-1
         taux1 = taux1 + bbmf(m,1) * Emk_m(m,n)
       end do
       bbspm(n,iwlg) = taux1
     end do

     enor_m = bbspm(0,iwlg)
     enor_p = bbspp(npo,iwlg)
     if (enor_p == 0.0_dp) then
       enor_p = 1.0_dp
     endif

     do n=0,npo
       bbspm(n,iwlg) = bbspm(n,iwlg) / enor_m
       bbspp(n,iwlg) = bbspp(n,iwlg) / enor_p
     end do

     if (mod(iwlg,100) == 0) then
       PRINT *,"iwlg = ",iwlg, "wlth = ", wlth, "intJm(0,iwlg) = ", intJm(0,iwlg), &
                                                "intJm(100,iwlg) = ", intJm(100,iwlg)
     endif

   enddo    !end wavelenght loop :: 99  !too slow !!

   close (37)
   close (60)

! Free memory

  deallocate (xmunl)
  deallocate (pli)
  deallocate (p_0, q_0)
  deallocate (hl)
  deallocate (wr, wi)
  deallocate (xkm)
  deallocate (Emk_m, Emk_p, Smk)
  deallocate (Emij)
  deallocate (rlm, rlmp, rlmi, rlmip)
  deallocate (sfl, sfm, sfl_g, sfl_d)
  deallocate (Srm)
  deallocate (sigl)
  deallocate (tl, dtl)
  deallocate (albed)
  deallocate (kap_dus, kap_gas, sca_dus)
  deallocate (g_absI, g_absL)
  deallocate (aam, bbm)
  deallocate (aamf, bbmf)
  deallocate (Tim, Bim)
  deallocate (Iinc)
  deallocate (ggm)
  deallocate (Qit, Qit_o, Git, Dit, Yit, Yit_o)
  deallocate (Hit)
  deallocate (R, C)
  deallocate (FERR, BERR)
  deallocate (WORK, IWORK)
  deallocate (amp)
  deallocate (indx)
  deallocate (argu)
  deallocate (indmi, indmj)

! stop !Flag


!=-=-=-=-=-=-=-=-=-=-=-= NOT CHECKED FROM HERE jrg =-=-=-=-=-=-=-=-=-=-=-=-=

! 20 V 2003 - Compute here the effective radiation field scaling

       call rf_create(rf_tmp,1,2)
       call rf_locate(rf_wl, wlth0, i1)
       rf_tmp%Val(1) = 1.0_dp
       rf_tmp%Ord(1) = rf_wl%Ord(i1)
       print *, " "
       print *, " Standard Draine field in free space:"

       call rf_create(rf_tmp1,0,nwlg)
       rf_tmp1%Val = rf_wl%Val * rf_incI%Val

! A - cut-off at 6 eV:

       call rf_locate(rf_wl, wl_6ev, i2)
       rf_tmp%Val(2) = 1.0_dp
       rf_tmp%Ord(2) = rf_wl%Ord(i2)
!      print *, rf_tmp%Ord(1), rf_tmp%Ord(2)
       int_rad0 = sect_int(rf_tmp, rf_incI, wl_pointer, wl_6ev)
       flu_rad0 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_6ev) / (1.0e8_dp * xhp)
       int_rad0 = int_rad0 * J_to_u
       flu_rad0 = flu_rad0 * J_to_u

       write (iscrn,'("  U0:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
              int_rad0, wl_6ev
       write (iscrn,'("  F0:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
              flu_rad0, wl_6ev
       write (iwrit2,'("  U0:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
              int_rad0, wl_6ev
       write (iwrit2,'("  F0:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
              flu_rad0, wl_6ev

! B - cut-off at 2400 A:

       call rf_locate(rf_wl, wl_hab, i2)
       rf_tmp%Val(2) = 1.0_dp
       rf_tmp%Ord(2) = rf_wl%Ord(i2)
!      print *, rf_tmp%Ord(1), rf_tmp%Ord(2)
       int_rad0 = sect_int(rf_tmp, rf_incI, wl_pointer, wl_hab)
       flu_rad0 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_hab) / (1.0e8_dp * xhp)
       int_rad0 = int_rad0 * J_to_u
       flu_rad0 = flu_rad0 * J_to_u

       write (iscrn,'("  U0:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
              int_rad0, wl_hab
       write (iscrn,'("  F0:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
              flu_rad0, wl_hab
       write (iwrit2,'("  U0:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
              int_rad0, wl_hab
       write (iwrit2,'("  F0:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
              flu_rad0, wl_hab
       print *, " "
       print *, " Scaled Draine field at cloud edge"

       int_rad1 = 0.0_dp
       int_rad2 = 0.0_dp
       if (isoneside == 1) then
         do iwlg=0,nwlg
           rf_u%Val(iwlg) = (intJm(0,iwlg) + intJp(0,iwlg)) * J_to_u
         end do
         rf_tmp1%Val = rf_wl%Val * rf_u%Val

         DO i = 1, nphot_i
            IF (TRIM(phdest(i)%nom) == "C") THEN
               wl_cut_C = phdest(i)%wl_cut_esp
            ENDIF
         ENDDO
         call rf_locate(rf_wl, wl_cut_C, i2)
         rf_tmp%Val(2) = 1.0_dp
         rf_tmp%Ord(2) = rf_wl%Ord(i2)
         int_rad1 = sect_int(rf_tmp, rf_u, wl_pointer, wl_cut_C)
         flu_rad1 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_cut_C) / (1.0e8_dp * xhp)
         write (iscrn,'("  U(C):",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
                int_rad1, wl_cut_C
         write (iscrn,'("  F(C):",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
                flu_rad1, wl_cut_C
         write (iwrit2,'("  U(C):",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
                int_rad1, wl_cut_C
         write (iwrit2,'("  F(C):",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
                flu_rad1, wl_cut_C

         
         call rf_locate(rf_wl, wl_6ev, i2)
         rf_tmp%Val(2) = 1.0_dp
         rf_tmp%Ord(2) = rf_wl%Ord(i2)
         int_rad1 = sect_int(rf_tmp, rf_u, wl_pointer, wl_6ev)
         flu_rad1 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_6ev) / (1.0e8_dp * xhp)
         write (iscrn,'("  U1:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
                int_rad1, wl_6ev
         write (iscrn,'("  F1:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
                flu_rad1, wl_6ev
         write (iwrit2,'("  U1:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
                int_rad1, wl_6ev
         write (iwrit2,'("  F1:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
                flu_rad1, wl_6ev

         call rf_locate(rf_wl, wl_hab, i2)
         rf_tmp%Val(2) = 1.0_dp
         rf_tmp%Ord(2) = rf_wl%Ord(i2)
         int_rad1 = sect_int(rf_tmp, rf_u, wl_pointer, wl_hab)
         flu_rad1 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_hab) / (1.0e8_dp * xhp)
         write (iscrn,'("  U1:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
                int_rad1, wl_hab
         write (iscrn,'("  F1:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
                flu_rad1, wl_hab
         write (iwrit2,'("  U1:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
                int_rad1, wl_hab
         write (iwrit2,'("  F1:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
                flu_rad1, wl_hab

         radm = int_rad1 / int_rad0
         rfield(0,:) = rf_u%Val(:)
       else
         do iwlg=0,nwlg
           rf_u%Val(iwlg) = (intJm(npo,iwlg) + intJp(npo,iwlg)) * J_to_u
         end do
         rf_tmp1%Val = rf_wl%Val * rf_u%Val

         call rf_locate(rf_wl, wl_6ev, i2)
         rf_tmp%Val(2) = 1.0_dp
         rf_tmp%Ord(2) = rf_wl%Ord(i2)
         int_rad2 = sect_int(rf_tmp, rf_u, wl_pointer, wl_6ev)
         flu_rad2 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_6ev) / (1.0e8_dp * xhp)
         write (iscrn,'("  U2:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
                int_rad2, wl_6ev
         write (iscrn,'("  F2:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
                flu_rad2, wl_6ev
         write (iwrit2,'("  U2:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
                int_rad2, wl_6ev
         write (iwrit2,'("  F2:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
                flu_rad2, wl_6ev

         call rf_locate(rf_wl, wl_hab, i2)
         rf_tmp%Val(2) = 1.0_dp
         rf_tmp%Ord(2) = rf_wl%Ord(i2)
         int_rad2 = sect_int(rf_tmp, rf_u, wl_pointer, wl_hab)
         flu_rad2 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_hab) / (1.0e8_dp * xhp)
         write (iscrn,'("  U2:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
                int_rad2, wl_hab
         write (iscrn,'("  F2:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
                flu_rad2, wl_hab
         write (iwrit2,'("  U2:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
                int_rad2, wl_hab
         write (iwrit2,'("  F2:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
                flu_rad2, wl_hab

         radp = int_rad2 / int_rad0
         rfield(n_imp,:) = rf_u%Val(:)

         do iwlg=0,nwlg
           rf_u%Val(iwlg) = (intJm(0,iwlg) + intJp(0,iwlg)) * J_to_u
         end do
         rf_tmp1%Val = rf_wl%Val * rf_u%Val

         call rf_locate(rf_wl, wl_6ev, i2)
         rf_tmp%Val(2) = 1.0_dp
         rf_tmp%Ord(2) = rf_wl%Ord(i2)
         int_rad1 = sect_int(rf_tmp, rf_u, wl_pointer, wl_6ev)
         flu_rad1 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_6ev) / (1.0e8_dp * xhp)
         write (iscrn,'("  U1:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
                int_rad1, wl_6ev
         write (iscrn,'("  F1:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
                flu_rad1, wl_6ev
         write (iwrit2,'("  U1:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
                int_rad1, wl_6ev
         write (iwrit2,'("  F1:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
                flu_rad1, wl_6ev

         call rf_locate(rf_wl, wl_hab, i2)
         rf_tmp%Val(2) = 1.0_dp
         rf_tmp%Ord(2) = rf_wl%Ord(i2)
         int_rad1 = sect_int(rf_tmp, rf_u, wl_pointer, wl_hab)
         flu_rad1 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_hab) / (1.0e8_dp * xhp)
         write (iscrn,'("  U1:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
                int_rad1, wl_hab
         write (iscrn,'("  F1:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
                flu_rad1, wl_hab
         write (iwrit2,'("  U1:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
                int_rad1, wl_hab
         write (iwrit2,'("  F1:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
                flu_rad1, wl_hab

         radm = int_rad1 / int_rad0
         rfield(0,:) = rf_u%Val(:)
       endif

       print *, " "
       write (iscrn,'("  G_0m :",1pe12.4," (in Habing units)")') int_rad1/5.6e-14_dp
       write (iscrn,'("  G_0p :",1pe12.4," (in Habing units)")') int_rad2/5.6e-14_dp
       write(iwrit2,'(/," U0:",1pe13.4, " erg cm-3")') int_rad0
       write(iwrit2,'(" G_0m =", 1pe12.3, " in Habing units",/)') int_rad1/5.6e-14_dp
       write(iwrit2,'(" G_0p =", 1pe12.3, " in Habing units",/)') int_rad2/5.6e-14_dp

! set here wavelength threshold for integrated intensity.

       call rf_locate(rf_wl, wl_6ev, i2)
       rf_tmp%Val(2) = 1.0_dp
       rf_tmp%Ord(2) = rf_wl%Ord(i2)
       int_rad(0) = sect_int(rf_tmp, rf_u, wl_pointer, wl_6ev)

       print *, " "
       print *, " Effective Radiation field strength AT the edge:"
       write (iscrn,'("  left :",1pe12.4," (Draine)")') radm
       write (iscrn,'(" right :",1pe12.4," (Draine)")') radp
       write (iwrit2,'("  left :",1pe12.4," (Draine)")') radm
       write (iwrit2,'(" right :",1pe12.4," (Draine)")') radp
       print *, " "

       call rf_free (rf_tmp)
       call rf_free (rf_tmp1)

END SUBROUTINE etin90

!---------------------------------------------------------------------------

SUBROUTINE etin901s

 USE constantes
 USE auxiliaries
 USE inoutput
 USE RF_TOOLS
 USE STR_DATA
 USE PROFIL
 USE CHEM_DATA

! Radiative transfer computation. See file transfer_S.pdf for definitions
! and comments. Index conventions:
!       l goes from 0 to lleg (first Legendre Pol. index)
!       m goes from +/- 1 to +/- mleg (second Legendre Pol. index)
!       n goes from 0 to npo (optical depth grid)
! 1 side version: tmax is infinity and and radiation impiging from mu_i > 0

 INTEGER                                     :: l, m, n, n1, n3

! Radiative quantities

 INTEGER                                       :: iwlg
 REAL (kind=dp)                                :: coef2
 !Full optical depth
 REAL (kind=dp), dimension(:,:),   allocatable :: tl
 !Full optical depth
 REAL (kind=dp), dimension(:),     allocatable :: dtl
 !Phase function
 REAL (kind=dp), dimension(:),     allocatable :: sigl
 !Reduced albedo (including gas)
 REAL (kind=dp), dimension(:,:),   allocatable :: albed
 REAL (kind=dp), dimension(:),     allocatable :: kap_dus, kap_gas, sca_dus
 !Legendre amplitudes
 REAL (kind=dp), dimension(:),     allocatable :: amp
 type(rf_type)                                 :: rf_tmp, rf_tmp1

! Legendre polynomial related variables

 !-1 power l  = (-1)**l
 REAL (kind=dp), dimension(:),     allocatable :: xmunl
 !P_l(mu_i)
 REAL (kind=dp), dimension(:,:),   allocatable :: pli
 REAL (kind=dp), dimension(:),     allocatable :: p_0
 REAL (kind=dp), dimension(:),     allocatable :: q_0

! Matrix A coefficients

 REAL (kind=dp), dimension(:,:),   allocatable :: hl
 !Derivatives of hl
 REAL (kind=dp), dimension(:),     allocatable :: wr, wi
 !Legendre absissa
 REAL (kind=dp), dimension(:,:),   allocatable :: xkm
 !integrated
 REAL (kind=dp), dimension(:,:),   allocatable :: Smk
 !Exp(-Smk) 0 -> t
 REAL (kind=dp), dimension(:,:),   allocatable :: Emk_m
 !Exp(-Smk) t -> t_max
 REAL (kind=dp), dimension(:,:),   allocatable :: Emk_p
 !Exp(-(Smk(i)-Smk(j))) t_i -> t_j
 REAL (kind=dp), dimension(:,:,:), allocatable :: Emij
 !Eigenvectors matrix
 REAL (kind=dp), dimension(:,:,:), allocatable :: rlm
 !derivative
 REAL (kind=dp), dimension(:,:,:), allocatable :: rlmp
 !inverse of Rlm(t)
 REAL (kind=dp), dimension(:,:,:), allocatable :: rlmi
 !Matrix product of R^-1 x R'
 REAL (kind=dp), dimension(:,:,:), allocatable :: rlmip
 REAL (kind=dp), dimension(:,:),   allocatable :: Srm

! Source function contributions

 !Source function expansion coeficients (as function of l and m respectively)
 REAL (kind=dp), dimension(:,:),   allocatable :: sfl, sfm, sfl_g, sfl_d
 !Inhomogeneous contribution (g vector in notes)
 REAL (kind=dp), dimension(:,:),   allocatable :: ggm
 !Product R^-1*A^-1*g vector, same dimensions as Qit
 REAL (kind=dp), dimension(:,:),   allocatable :: Git

! Boundary conditions related variables

 !Auxiliary
 REAL (kind=dp), dimension(:,:),   allocatable :: Tim
 !BC matrix
 REAL (kind=dp), dimension(:,:),   allocatable :: Bim
 !Incident radiation field
 REAL (kind=dp), dimension(:),     allocatable :: Iinc
 REAL (kind=dp)                                :: wl, wlo, geom, bbstar
 REAL (kind=dp)                                :: fluxm

! Iteration related variables

 REAL (kind=dp), dimension(:,:),   allocatable :: Qit
 REAL (kind=dp), dimension(:,:),   allocatable :: Qit_o
 REAL (kind=dp), dimension(:,:),   allocatable :: Dit
 REAL (kind=dp), dimension(:,:),   allocatable :: Yit
 REAL (kind=dp), dimension(:,:),   allocatable :: Yit_o
 REAL (kind=dp), dimension(:),     allocatable :: Hit

! LAPACK related variables

 !Eigenvalues only
 CHARACTER (LEN=1)                             :: JOBZ  = 'N'
 !Real work array
 REAL (kind=dp), dimension(:),     allocatable :: WORK
 !Work array
 INTEGER,        dimension(:),     allocatable :: IWORK
 REAL (kind=dp), dimension(:,:),   allocatable :: aam, bbm
 REAL (kind=dp), dimension(:,:),   allocatable :: aamf, bbmf
 INTEGER                                       :: info
 !Equilibrate system
 CHARACTER (LEN=1)                             :: FACT  = 'E'
 !Transpose
 CHARACTER (LEN=1)                             :: TRANS = 'N'
 INTEGER                                       :: nrhs
 !Kind of equilibration
 CHARACTER (LEN=1)                             :: EQUED
 !Equilibration factors
 REAL (kind=dp), dimension(:), allocatable     :: R, C
 !Errors (estimates)
 REAL (kind=dp), dimension(:), allocatable     :: FERR, BERR
 !Condition number
 REAL (kind=dp)                                :: RCOND
 INTEGER, dimension(:), allocatable            :: indx

! Auxiliaries

 INTEGER                                       :: nl, nnit
 INTEGER                                       :: i, it
 INTEGER                                       :: i1, i2
 REAL (kind=dp)                                :: ht, ymui
 REAL (kind=dp)                                :: enor_m, enor_p
 REAL (kind=dp)                                :: flu_rad0, flu_rad1
 REAL (kind=dp)                                :: taux1, taux2
 REAL (kind=dp)                                :: taux4
 REAL (kind=dp)                                :: wlth, wlthcm, extpou
 REAL (kind=dp)                                :: deriv_d
 REAL (kind=dp), dimension(:), allocatable     :: argu
 REAL (kind=dp), dimension(-mleg:mleg)         :: auxil
 INTEGER, dimension(:,:), allocatable          :: indmi, indmj

!Thermalized (black body) emission of dust and gas
 REAL (kind=dp) :: bbdust, bbgas

!Others
 REAL (kind=dp)                                :: wl_cut_C


!-----------------------------------------------------------------------------

! Number of Legendred coefficient (used for loops that run from 1 to lleg+1)

  nl = lleg + 1

       if ( ALLOCATED(g_absI)) then
         deallocate (g_absI)
         deallocate (g_absL)
       endif

       if ( ALLOCATED(intJm)) then
         deallocate (intJm, intJp)
         deallocate (bbspm, bbspp)
       endif


  allocate (xmunl(0:lleg))
  allocate (pli(0:lleg,-mleg:mleg))
  allocate (p_0(0:lleg), q_0(0:lleg))
  allocate (hl(0:lleg,0:npo))
  allocate (wr(lleg+1), wi(lleg+1))
  allocate (xkm(-mleg:mleg,0:npo))
  allocate (Smk(mleg,0:npo))
  allocate (Emk_m(mleg,0:npo))
  allocate (Emk_p(mleg,0:npo))
  allocate (Emij(mleg,0:npo,0:npo))
  allocate (rlm(0:lleg,-mleg:mleg,0:npo))
  allocate (rlmp(0:lleg,-mleg:mleg,0:npo))
  allocate (rlmi(0:lleg,-mleg:mleg,0:npo))
  allocate (rlmip(0:lleg,-mleg:mleg,0:npo))
  allocate (sfm(-mleg:mleg,0:npo))
  allocate (sfl(0:lleg,0:npo))
  allocate (sfl_g(0:lleg,0:npo),sfl_d(0:lleg,0:npo))
  allocate (ggm(-mleg:mleg,0:npo))
  allocate (Srm(mleg,0:npo))
  allocate (sigl(0:lleg))
  allocate (dtl(0:npo))
  allocate (tl(0:npo,0:nwlg))
  allocate (albed(0:npo,0:nwlg))
  allocate (kap_dus(0:npo))
  allocate (kap_gas(0:npo))
  allocate (sca_dus(0:npo))
  allocate (g_absI(0:npo,0:nwlg))
  allocate (g_absL(0:npo,0:nwlg))
  allocate (intJm(0:npo,0:nwlg))
  allocate (intJp(0:npo,0:nwlg))
  allocate (bbspm(0:npo,0:nwlg))
  allocate (bbspp(0:npo,0:nwlg))
  allocate (R(nl), C(nl))
  allocate (FERR(1), BERR(1))
  allocate (WORK(4*nl))
  allocate (IWORK(nl))
  allocate (aam(nl,nl), bbm(nl,1))
  allocate (aamf(nl,nl), bbmf(nl,1))
  allocate (indx(nl))
  allocate (Tim(-mleg:mleg,-mleg:mleg))
  allocate (Bim(-mleg:mleg,-mleg:mleg))
  allocate (Iinc(-mleg:mleg))
  allocate (Qit(-mleg:mleg,0:npo))
  allocate (Qit_o(-mleg:mleg,0:npo))
  allocate (Git(-mleg:mleg,0:npo))
  allocate (Dit(-mleg:mleg,0:npo))
  allocate (Yit(-mleg:mleg,0:npo))
  allocate (Yit_o(-mleg:mleg,0:npo))
  allocate (Hit(-mleg:mleg))
  allocate (amp(-mleg:mleg))
  allocate (argu(0:npo))
  allocate (indmi(mleg,0:npo), indmj(mleg,0:npo))

  p_0 = 0.0_dp
  q_0 = 0.0_dp
  sigl = 0.0_dp
  Tim = 0.0_dp
  Bim = 0.0_dp
  Iinc = 0.0_dp
  xkm = 0.0_dp

! This is just (-1)**l

  xmunl(0) = 1.0_dp
  do l=1,lleg
    xmunl(l) = - xmunl(l-1)
  end do

! STEP 0 - Initialise radiative quantities
!  (boundary conditions and optical depth grid)

!  uvspec.out is the total absorption through the cloud. multiply by
!             incident radition field to compute spectrum.

  fichier = TRIM(out_dir)//TRIM(modele)//'.uv'
  open (37, file=fichier, status="unknown")
  fichier = TRIM(out_dir)//TRIM(modele)//'.flin'
  open (60, file=fichier, status="unknown")

  coef2 = tautav * cdunit / rrr

!  Initialise illuminating star parameters, if any...
!  geom is (1 - cos(theta_0)).
!  A second order development is adequate in most cases.

  if (d_sour /= 0.0_dp) then
    if (srcpp(1:2) == 'F_') then
      nbpt = 1
      wlo = wlth0
      wl = rf_wl%Val(wl_pointer(rf_exter%Ord(2)))
    else
      call STAR
    endif
    geom = 0.5_dp * (rstar/(d_sour*pccm))**2
  endif

!  sigl = gg**l (Formule 48, Henyey-Greenstein phase function)

  sigl(0) = 1.0_dp
  do l=1,lleg
    sigl(l) = sigl(l-1) * gg
  end do

! Compute absorption by gas

  g_absI = 0.0_dp
  call ABSGAZ

! STEP 1 - Compute P_l(mu_i) - see recurence in Num Rec

  pli(0,:) = 1.0_dp
  do m=1,mleg
    ymui = xmui(mleg,m)
    pli(1,-m) = -ymui
    pli(1,m) = ymui
    do l=1,lleg-1
      pli(l+1,-m) = (-ymui*(2.0_dp*l+1.0_dp) * pli(l,-m) - l * pli(l-1,-m)) / (l+1.0_dp)
      pli(l+1,m) = xmunl(l+1) * pli(l+1,-m)
    end do
  end do


  p_0(0) = 1.0_dp
  q_0(1) = p_0(0) / 2.0_dp
  do l=1,lleg/2
    p_0(2*l) = - (2.0_dp*l-1) * p_0(2*l-2) / (2.0_dp*l)
    q_0(2*l+1) = p_0(2*l) / (2.0_dp*l+2.0_dp)
    !write(*,*) l,2*l+1,  p_0(2*l), q_0(2*l+1)  !Flag
  end do

  do iwlg=0,nwlg

! a - Reset to 0 most variables

    hl = 0.0_dp
    wr = 0.0_dp
    wi = 0.0_dp
    tl(:,iwlg) = 0.0_dp
    dtl = 0.0_dp
    albed(:,iwlg) = 0.0_dp
    kap_dus = 0.0_dp
    kap_gas = 0.0_dp
    sca_dus = 0.0_dp

    wlth = rf_wl%Val(iwlg)
    wlthcm = wlth * 1.0e-8_dp
    extpou = rf_extpou%Val(iwlg)

! Compute local values ob absorption coef cor dust and gas and dust scatering coef
! Compute effective albedo ! ??? Problems with effective albedo definition, jrg > jlb

    do n=0,npo
      kap_dus(n) = (1.0_dp - alb) * extpou * dens_o(n) / coef2
      kap_gas(n) = g_absL(n,iwlg)
      sca_dus(n) = alb * extpou * dens_o(n) / coef2
      albed(n,iwlg) = sca_dus(n) / (kap_dus(n)+kap_gas(n)+sca_dus(n))
    end do

! b - Compute optical depth grid
!     Convert Delta Tau_v (dust only at V) to
!     Delta tau_lambda (dust + gaz at Lambda)

    tl(0,iwlg) = 0.0_dp
    do n=0,npo-1
      ht = ((kap_dus(n)+kap_gas(n)+sca_dus(n))/dens_o(n) &
         + (kap_dus(n+1)+kap_gas(n+1)+sca_dus(n+1))/dens_o(n+1)) &
         * 0.5_dp * coef2 *(tau_o(n+1) - tau_o(n))
      dtl(n) = ht
      tl(n+1,iwlg) = tl(n,iwlg) + ht
!     print*, n+1, tau_o(n+1), tl(n+1,iwlg), ht    !Flag : point/tau_V/tau_d+g
    end do

! STEP 2a - Compute matrix A(tau)

    do n=0,npo

! Formulae A2 and A3 of Roberge:
      hl(0,n) = 1.0_dp - albed(n,iwlg)
      do l=1,lleg
        hl(l,n) = (2.0_dp*l+1.0_dp) * (1.0_dp - albed(n,iwlg) * sigl(l))
      end do
      do i=1,lleg
        wi(i) = i / sqrt(hl(i-1,n)*hl(i,n))
      end do
      wr = 0.0_dp

!  Compute eigenvalues (tridiagonal symetric matrix) - LAPACK

      call DSTEVD(JOBZ, nl, wr, wi, aam, nl, WORK, 4*nl, IWORK, nl, info )

!  order eigenvalues by decreasing order

      wi = wr
      do i=1,nl
        wr(i) = wi(nl+1-i)
      end do

!  km = 1 / lamda_m in increasing order

      do m=1,mleg
        xkm(m,n) = 1.0_dp / wr(m)
        xkm(-m,n) = -xkm(m,n)
      end do

! Calcul des Rlm: Formule A9 de Roberge 1983
! Eigenvectors computed only from recurrence relations.

      do m=1,mleg
        rlm(0,m,n)  = 1.0_dp
        rlm(0,-m,n) = 1.0_dp
        rlm(1,m,n)  = (1.0_dp - albed(n,iwlg)) / xkm(m,n)
        rlm(1,-m,n) = -rlm(1,m,n)                             !jrg
        !rlm(1,-m,n) = -(1.0_dp - albed(n,iwlg)) / xkm(m,n)   !jlb
        do l=2,lleg
          rlm(l,m,n) = (hl(l-1,n) * rlm(l-1,m,n) &
                     - (l-1) * xkm(m,n) * rlm(l-2,m,n)) / (l*xkm(m,n))
          rlm(l,-m,n) = xmunl(l) * rlm(l,m,n)
        end do
      end do

    end do   ! end loop in npo

! STEP 2c - Compute Srm - used for mean intensity

    call aux_Srm(q_0, rlm, Srm)

! STEP 3 - Compute Smk = Sum (xkm). Simple Simpson rule

    Smk(:,0) = 0.0_dp
    do n=1,npo
      do m=1,mleg
        Smk(m,n) = Smk(m,n-1) + 0.5_dp * dtl(n-1) * (xkm(m,n) + xkm(m,n-1))
      end do
    end do

    Emk_m = exp(-Smk)   !Asignation of the whole matrix !!
    Emk_p = 0.0_dp      ! Tau_max = infty

    call aux_Emij(Smk, Emij, indmi, indmj)

! STEP 4 - Compute R**-1    !Loop to all cloud opacity points

    call aux_Rlmi(rlm, hl, rlmi)

! STEP 5 - Compute source function contribution g~ = R^-1*A^-1*g (in notes)
!          JLB : Check Javier correction (not yet done here)

    do n=0,npo

      !Ex. Black body emission of dust 
      bbdust = 2.0_dp*xhp*clum**2 / (wlthcm)**5
      bbdust = bbdust * 1.0e-8_dp / (exp (hcsurk/(wlthcm*tdust(1,n))) -1.0_dp)

      !Ex. Black body emission of thermalized gas 
      bbgas = 2.0_dp*xhp*clum**2 / (wlthcm)**5
      bbgas = bbgas * 1.0e-8_dp / (exp (hcsurk/(wlthcm*tgaz(n))) -1.0_dp)
!     bbgas = bbgas * 1.0e-8_dp / (exp (hcsurk/(wlthcm*3.55e3_dp)) -1.0_dp)

! 5a - Asign source function, isotropic  (PROVISIONAL)

      !METHOD B1: aprox. to continuum thermal isotropic emission
      sfl_d(0,n) = bbdust * kap_dus(n) / (kap_dus(n)+kap_gas(n)+sca_dus(n))

      !METHOD B2: aprox .to line thermal isotropic emission
      sfl_g(0,n) = bbgas * kap_gas(n) / (kap_dus(n)+kap_gas(n)+sca_dus(n))
      !print*, 'GAS SF:', sfl_g(0,n), 'DUST SF:', sfl_d(0,n) !Flag

      !Add both contributions
      sfl(0,n) = sfl_g(0,n) + sfl_d(0,n)
!     sfl(0,n) = 0.0_dp

      sfl(1:lleg,n) = 0.0_dp
    enddo

    !Put them in m,n desired order
    sfm(-mleg:-1,:) = sfl(0:mleg-1,:)
    sfm(1:mleg,:) = sfl(mleg:lleg,:)

! 5b - Compute vector g

    do n=0,npo
       do m=-mleg,-1
          l = mleg + m
          ggm(m,n) = -sfm(m,n) / ( 1.0_dp - albed(n,iwlg) * sigl(l) )
       enddo
       do m=1,mleg
          l = mleg + m - 1
          ggm(m,n) = -sfm(m,n) / ( 1.0_dp - albed(n,iwlg) * sigl(l) )
       enddo
     enddo

! 5d - Compute matrix product  R^-1*A^-1*g = Git

   call aux_Git(rlmi, ggm, xkm, Git)

! STEP 6 - Compute derivatives of Rlm

! 6c - derivative of Rlm

    do n=2,npo-2
      do m=1,mleg
        rlmp(0,m,n) = 0.0_dp
        rlmp(0,-m,n) = 0.0_dp
        n1 = n - 2
        n3 = n + 2
        do l=1,lleg
!  Direct variable
          deriv_d = rlm(l,m,n1) * (2.0_dp*tl(n,iwlg) - tl(n,iwlg) - tl(n3,iwlg))   &
                                / ((tl(n1,iwlg)-tl(n,iwlg)) * (tl(n1,iwlg)-tl(n3,iwlg))) &
                  + rlm(l,m,n)  * (2.0_dp*tl(n,iwlg) - tl(n1,iwlg) - tl(n3,iwlg)) &
                                / ((tl(n,iwlg)-tl(n3,iwlg)) * (tl(n,iwlg)-tl(n1,iwlg))) &
                  + rlm(l,m,n3) * (2.0_dp*tl(n,iwlg) - tl(n1,iwlg) - tl(n,iwlg))   &
                                / ((tl(n3,iwlg)-tl(n1,iwlg)) * (tl(n3,iwlg)-tl(n,iwlg)))
          rlmp(l,m,n) = deriv_d
          rlmp(l,-m,n) = xmunl(l) * rlmp(l,m,n)
        end do
      end do
    end do

!  Boundary condition. Replace par linear extrapolation (TODO)

    do m=1,mleg
      rlmp(0,m,0) = 0.0_dp
      rlmp(0,-m,0) = 0.0_dp
      rlmp(0,m,1) = 0.0_dp
      rlmp(0,-m,1) = 0.0_dp
      rlmp(0,m,npo-1) = 0.0_dp
      rlmp(0,-m,npo-1) = 0.0_dp
      rlmp(0,m,npo) = 0.0_dp
      rlmp(0,-m,npo) = 0.0_dp
      do l=1,lleg
        rlmp(l,m,0) = rlmp(l,m,2)
        rlmp(l,-m,0) = xmunl(l) * rlmp(l,m,0)
        rlmp(l,m,1) = rlmp(l,m,2)
        rlmp(l,-m,1) = xmunl(l) * rlmp(l,m,1)
        rlmp(l,m,npo-1) = rlmp(l,m,npo-2)
        rlmp(l,-m,npo-1) = xmunl(l) * rlmp(l,m,npo-1)
        rlmp(l,m,npo) = rlmp(l,m,npo-2)
        rlmp(l,-m,npo) = xmunl(l) * rlmp(l,m,npo-2)
      end do
    end do

! 6d - Multiply by R**-1

    call aux_Rlmip(rlmi, rlmp, rlmip)

! STEP 7 - Compute Tim

    do m=1,mleg
      do i=1,mleg
        taux1 = 0.0_dp
        taux4 = 0.0_dp
        do l=0,lleg
          taux1 = taux1 + (2.0_dp * l + 1.0_dp) * rlm(l,-m,0) * pli(l,-i)
          taux4 = taux4 + (2.0_dp * l + 1.0_dp) * rlm(l,m,npo) * pli(l,i)
        end do
        Tim(-i,-m) = taux1
        Tim(i,m)   = taux4
      end do
    end do

! STEP 8 - Compute Bim (Boundary conditions matrix)

    do m=1,mleg
      do i=1,mleg
        Bim(-i,-m) = Tim(-i,-m)
        Bim(-i,m) = 0.0_dp
        Bim(i,-m) = 0.0_dp
        Bim(i,m) = Tim(i,m)
      end do
    end do

! STEP 9 - Set boundary conditions

!  Compute total absorption by dust + gas through whole cloud

    taux1 = taumax * extpou + g_absI(npo,iwlg)
    write (37,"(1p,3e17.7E3)") wlth, exp(-taux1), taux1

!  Add potential // source (star)
!  Warning: lambda : 4 times in cm and 1 in Angstrom...

    if (d_sour < 0.0_dp) then

      if (srcpp(1:2) == 'F_') then

        if (wlth > rf_wl%Val(wl_pointer(rf_exter%Ord(nbpt+1)))) then
          nbpt = nbpt + 1
          wlo = wl
          wl = rf_wl%Val(wl_pointer(rf_exter%Ord(nbpt+1)))
        endif
        if (wlo == wlth) then
          bbstar = rf_exter%Val(nbpt)
        else if (wl == wlth) then
          bbstar = rf_exter%Val(nbpt+1)
        else
          bbstar = rf_exter%Val(nbpt) + (rf_exter%Val(nbpt+1)-rf_exter%Val(nbpt)) &
                 * (wlth - wlo) / (wl - wlo)
        endif
      else
        bbstar = 1.0e-8_dp*2.0_dp*xhp*clum*clum / wlthcm**5
        bbstar = bbstar / (exp(hcsurk/(teff*wlthcm)) - 1.0_dp)
      endif

      fluxm = 2.0_dp * geom * bbstar / (2.0_dp - xmui(mleg,mleg) - xmui(mleg,mleg-1))

      write (60, '(1p,4e15.6)') wlth, bbstar, fluxm, rf_incI%Val(iwlg)

    else

      fluxm = 0.0_dp
      write (60, '(1p,4e15.6)') wlth, rf_incI%Val(iwlg)

    endif

! 19 XII 2005 - Use "real" units (specific intensity)

    Iinc(-mleg:-1) = radm0 * rf_incI%Val(iwlg)
    Iinc(1:mleg)   = 0.0_dp

! mu_i = +/- 1 is on +/- mleg (not +/- 1)

    Iinc(-mleg) = radm0 * rf_incI%Val(iwlg) + fluxm
    Iinc(mleg)  = 0.0_dp

! STEP 10 - Start iteration with an appropriate guess for Y and Q

! 10a - Compute Y_o = R^-1 x F_approx
!       F_approx ~ sfm / 1 - albed + Iexp(kt), then it IS wavelength and tau dependent

     call aux_Yit1(Iinc, xkm, tl(:,iwlg), albed(:,iwlg), rlmi, sfm, Yit_o)

! 10b - Compute Q_o = R^-1 x R' x Y_o

     call aux_Qit(rlmip, Yit_o, Qit)

     nnit = 200

     !Git = 0.0_dp !old procedure if Git = 0 and Qit = 0
     !Qit = 0.0_dp !tests javi

     do it=1,nnit ! begin loop :: 101

       ! STEP 11 - Compute D  - Simple trapezoidal rule
       Dit(-mleg:-1,0) = 0.0_dp
       do n=1,npo
         do m=-mleg,-1
           do i=indmi(-m,n),n
             argu(i) = (Qit(m,i) - Git(m,i)) * Emij(-m,n,i)
           end do
           taux1 = SUM( (/ (dtl(i-1) * (argu(i-1)+argu(i)), i=indmi(-m,n)+1,n) /) )
           Dit(m,n) = taux1 * 0.5_dp
         end do
       end do

       Dit(1:mleg,:) = 0.0_dp

! STEP 12 - Compute Hit

       Hit = 0.0_dp
       do i=1,mleg
         taux1 = 0.0_dp
         taux2 = 0.0_dp
         do m=1,mleg
           taux1 = taux1 + Dit(-m,npo) * Tim(i,-m)
           !write(*,*) Dit(-m,npo), Dit(m,0) !Flag
         end do
         Hit(-i) = Iinc(-i) - taux2
         Hit(i) = 0.0_dp + taux1
         !write(*,*) 'i, Hi:', i, Hit(-i), Hit(i)  !Flag
       end do

! STEP 13 - solve using LAPACK

       aam(1:mleg,1:mleg) = Bim(-mleg:-1,-mleg:-1)
       aam(1:mleg,mleg+1:2*mleg) = Bim(-mleg:-1,1:mleg)
       aam(mleg+1:nl,1:mleg) = Bim(1:mleg,-mleg:-1)
       aam(mleg+1:nl,mleg+1:nl) = Bim(1:mleg,1:mleg)

       bbm(1:mleg,1) = Hit(-mleg:-1)
       bbm(mleg+1:nl,1) = Hit(1:mleg)

       nrhs = 1
       call dgesvx (FACT, TRANS, nl, nrhs, aam, nl, aamf, nl, indx, EQUED, &
            R, C, bbm, nl, bbmf, nl, RCOND, FERR, BERR, WORK, IWORK, info)

       amp(-mleg:-1) = bbmf(1:mleg,1)
       amp(1:mleg) = bbmf(mleg+1:nl,1)

! STEP 14 - Compute Y

       do n=0,npo
         do m=1,mleg
           Yit(-m,n) = amp(-m) * Emk_m(m,n) - Dit(-m,n)
           Yit(m,n)  = 0.0_dp
         end do
       end do


! STEP 15 - Compute new Q

       Qit_o = Qit
       !write(*,*) Qit_o  !Flag
       call aux_Qit(rlmip, Yit, Qit)

! STEP 16 - convergence ?

       auxil(0) = 0.0_dp
       taux2 = 0.0_dp
       do n=0,npo
         do m=-mleg,-1
           if (Qit(m,n) /= 0.0_dp) then
             auxil(m) = abs ((Qit(m,n)-Qit_o(m,n))/Qit(m,n))
           else
             auxil(m) = 0.0_dp
           endif
         end do
         do m=1,mleg
           if (Qit(m,n) /= 0.0_dp) then
             auxil(m) = abs ((Qit(m,n)-Qit_o(m,n))/Qit(m,n))
           else
             auxil(m) = 0.0_dp
           endif
         end do
         taux2 = taux2 + SUM(auxil)
       end do

       if (taux2 < 1.0e-3_dp) then
!        print*, iwlg, 'qm converges in iter.', it  !
         exit      !exits sub-iteration loop
       endif

     end do        !Exist loop to find q  :: 101
!    print *, " "

! STEP 17 - Compute radiation field - mean intensity
!           intJm is 0.5 sum(-1->0) of I
!           intJp is 0.5 sum(0->1) of I
!           Thus intJm + intJp = J in erg cm-2 s-1 A-1 sterad-1
!           one gets u in erg cm-3 A-1 by u = J * 4 pi / c

     do n=0,npo
       taux1 = 0.0_dp
       taux2 = 0.0_dp
       do m=1,mleg
         taux1 = taux1 + Yit(-m,n) * (1.0_dp + Srm(m,n))
         taux2 = taux2 + Yit(-m,n) * (1.0_dp - Srm(m,n))
       enddo
! 19 XII 2005 - Change normalisation

       intJm(n,iwlg) = 0.5_dp * taux1
       intJp(n,iwlg) = 0.5_dp * taux2
     end do

! Compute escape probabilities
! Half system only

     do n=0,npo        !position
       call aux_bp(rlm, pli, n, aam)
       bbm(1:mleg,1) = 1.0_dp

       call dgesvx (FACT, TRANS, mleg, nrhs, aam, nl, aamf, nl, indx, EQUED, &
                    R, C, bbm, nl, bbmf, nl, RCOND, FERR, BERR, WORK, IWORK, info)

       taux1 = 0.0_dp
       do m=mleg,1,-1
         taux1 = taux1 + bbmf(m,1) * Emk_p(m,n)
       end do
       bbspp(n,iwlg) = taux1
     end do

     do n=0,npo
       call aux_bm(rlm, pli, n, aam)
       bbm(1:mleg,1) = 1.0_dp

       call dgesvx (FACT, TRANS, mleg, nrhs, aam, nl, aamf, nl, indx, EQUED, &
                    R, C, bbm, nl, bbmf, nl, RCOND, FERR, BERR, WORK, IWORK, info)

       taux1 = 0.0_dp
       do m=mleg,1,-1
         taux1 = taux1 + bbmf(m,1) * Emk_m(m,n)
       end do
       bbspm(n,iwlg) = taux1
     end do

     enor_m = bbspm(0,iwlg)
     enor_p = bbspp(npo,iwlg)
     if (enor_p == 0.0_dp) then
       enor_p = 1.0_dp
     endif

     do n=0,npo
       bbspm(n,iwlg) = bbspm(n,iwlg) / enor_m
       bbspp(n,iwlg) = bbspp(n,iwlg) / enor_p
     end do

     if (mod(iwlg,100) == 0) then
       write(*,*) iwlg, wlth, intJm(0,iwlg), intJm(100,iwlg)
     endif

   enddo    !end wavelenght loop :: 99  !too slow !!

   close (37)
   close (60)

! Free memory

  deallocate (xmunl)
  deallocate (pli)
  deallocate (p_0, q_0)
  deallocate (hl)
  deallocate (wr, wi)
  deallocate (xkm)
  deallocate (Emk_m, Emk_p, Smk)
  deallocate (Emij)
  deallocate (rlm, rlmp, rlmi, rlmip)
  deallocate (sfl, sfm, sfl_g, sfl_d)
  deallocate (Srm)
  deallocate (sigl)
  deallocate (tl, dtl)
  deallocate (albed)
  deallocate (kap_dus, kap_gas, sca_dus)
  deallocate (g_absI, g_absL)
  deallocate (aam, bbm)
  deallocate (aamf, bbmf)
  deallocate (Tim, Bim)
  deallocate (Iinc)
  deallocate (ggm)
  deallocate (Qit, Qit_o, Git, Dit, Yit, Yit_o)
  deallocate (Hit)
  deallocate (R, C)
  deallocate (FERR, BERR)
  deallocate (WORK, IWORK)
  deallocate (amp)
  deallocate (indx)
  deallocate (argu)
  deallocate (indmi, indmj)

! stop !Flag


!=-=-=-=-=-=-=-=-=-=-=-= NOT CHECKED FROM HERE jrg =-=-=-=-=-=-=-=-=-=-=-=-=

! 20 V 2003 - Compute here the effective radiation field scaling

       call rf_create(rf_tmp,1,2)
       call rf_locate(rf_wl, wlth0, i1)
       rf_tmp%Val(1) = 1.0_dp
       rf_tmp%Ord(1) = rf_wl%Ord(i1)
       print *, " "
       print *, " Standard Draine field in free space:"

       call rf_create(rf_tmp1,0,nwlg)
       rf_tmp1%Val = rf_wl%Val * rf_incI%Val

! A - cut-off at 6 eV:

       call rf_locate(rf_wl, wl_6ev, i2)
       rf_tmp%Val(2) = 1.0_dp
       rf_tmp%Ord(2) = rf_wl%Ord(i2)
!      print *, rf_tmp%Ord(1), rf_tmp%Ord(2)
       int_rad0 = sect_int(rf_tmp, rf_incI, wl_pointer, wl_6ev)
       flu_rad0 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_6ev) / (1.0e8_dp * xhp)
       int_rad0 = int_rad0 * J_to_u
       flu_rad0 = flu_rad0 * J_to_u

       write (iscrn,'("  U0:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
              int_rad0, wl_6ev
       write (iscrn,'("  F0:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
              flu_rad0, wl_6ev
       write (iwrit2,'("  U0:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
              int_rad0, wl_6ev
       write (iwrit2,'("  F0:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
              flu_rad0, wl_6ev

! B - cut-off at 2400 A:

       call rf_locate(rf_wl, wl_hab, i2)
       rf_tmp%Val(2) = 1.0_dp
       rf_tmp%Ord(2) = rf_wl%Ord(i2)
!      print *, rf_tmp%Ord(1), rf_tmp%Ord(2)
       int_rad0 = sect_int(rf_tmp, rf_incI, wl_pointer, wl_hab)
       flu_rad0 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_hab) / (1.0e8_dp * xhp)
       int_rad0 = int_rad0 * J_to_u
       flu_rad0 = flu_rad0 * J_to_u

       write (iscrn,'("  U0:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
              int_rad0, wl_hab
       write (iscrn,'("  F0:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
              flu_rad0, wl_hab
       write (iwrit2,'("  U0:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
              int_rad0, wl_hab
       write (iwrit2,'("  F0:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
              flu_rad0, wl_hab
       print *, " "
       print *, " Scaled Draine field at cloud edge"

       int_rad1 = 0.0_dp
       int_rad2 = 0.0_dp

       do iwlg=0,nwlg
         rf_u%Val(iwlg) = (intJm(0,iwlg) + intJp(0,iwlg)) * J_to_u
       end do
       rf_tmp1%Val = rf_wl%Val * rf_u%Val


       DO i = 1, nphot_i
          IF (TRIM(phdest(i)%nom) == "C") THEN
             wl_cut_C = phdest(i)%wl_cut_esp
          ENDIF
       ENDDO
       call rf_locate(rf_wl, wl_cut_C, i2)
       rf_tmp%Val(2) = 1.0_dp
       rf_tmp%Ord(2) = rf_wl%Ord(i2)
       int_rad1 = sect_int(rf_tmp, rf_u, wl_pointer, wl_cut_C)
       flu_rad1 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_cut_C) / (1.0e8_dp * xhp)
       write (iscrn,'("  U(C):",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
              int_rad1, wl_cut_C
       write (iscrn,'("  F(C):",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
              flu_rad1, wl_cut_C
       write (iwrit2,'("  U(C):",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
              int_rad1, wl_cut_C
       write (iwrit2,'("  F(C):",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
              flu_rad1, wl_cut_C

       call rf_locate(rf_wl, wl_6ev, i2)
       rf_tmp%Val(2) = 1.0_dp
       rf_tmp%Ord(2) = rf_wl%Ord(i2)
       int_rad1 = sect_int(rf_tmp, rf_u, wl_pointer, wl_6ev)
       flu_rad1 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_6ev) / (1.0e8_dp * xhp)
       write (iscrn,'("  U1:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
              int_rad1, wl_6ev
       write (iscrn,'("  F1:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
              flu_rad1, wl_6ev
       write (iwrit2,'("  U1:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
              int_rad1, wl_6ev
       write (iwrit2,'("  F1:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
              flu_rad1, wl_6ev

       call rf_locate(rf_wl, wl_hab, i2)
       rf_tmp%Val(2) = 1.0_dp
       rf_tmp%Ord(2) = rf_wl%Ord(i2)
       int_rad1 = sect_int(rf_tmp, rf_u, wl_pointer, wl_hab)
       flu_rad1 = sect_int(rf_tmp, rf_tmp1, wl_pointer, wl_hab) / (1.0e8_dp * xhp)
       write (iscrn,'("  U1:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
              int_rad1, wl_hab
       write (iscrn,'("  F1:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
              flu_rad1, wl_hab
       write (iwrit2,'("  U1:",1pe12.4," erg cm-3,     ","from 912 to",0pf9.2," Angstrom")') &
              int_rad1, wl_hab
       write (iwrit2,'("  F1:",1pe12.4," phot cm-2 s-1,","from 912 to",0pf9.2," Angstrom")') &
              flu_rad1, wl_hab

       radm = int_rad1 / int_rad0
       rfield(0,:) = rf_u%Val(:)

       print *, " "
       write (iscrn,'("  G_0m :",1pe12.4," (in Habing units)")') int_rad1/5.6e-14_dp
       write (iscrn,'("  G_0p :",1pe12.4," (in Habing units)")') int_rad2/5.6e-14_dp
       write(iwrit2,'(/," U0:",1pe13.4, " erg cm-3")') int_rad0
       write(iwrit2,'(" G_0m =", 1pe12.3, " in Habing units",/)') int_rad1/5.6e-14_dp
       write(iwrit2,'(" G_0p =", 1pe12.3, " in Habing units",/)') int_rad2/5.6e-14_dp

! set here wavelength threshold for integrated intensity.

       call rf_locate(rf_wl, wl_6ev, i2)
       rf_tmp%Val(2) = 1.0_dp
       rf_tmp%Ord(2) = rf_wl%Ord(i2)
       int_rad(0) = sect_int(rf_tmp, rf_u, wl_pointer, wl_6ev)

       print *, " "
       print *, " Effective Radiation field strength AT the edge:"
       write (iscrn,'("  left :",1pe12.4," (Draine)")') radm
       write (iscrn,'(" right :",1pe12.4," (Draine)")') radp
       write (iwrit2,'("  left :",1pe12.4," (Draine)")') radm
       write (iwrit2,'(" right :",1pe12.4," (Draine)")') radp
       print *, " "

       call rf_free (rf_tmp)
       call rf_free (rf_tmp1)

END SUBROUTINE etin901s

!-------- Auxiliary subroutine for ETIN -----------------------------------

SUBROUTINE aux_Srm(q_0, rlm, Srm)

  USE constantes
  USE PROFIL

  implicit none
  REAL (kind=dp), dimension(0:lleg),                  intent (in)  :: q_0
  REAL (kind=dp), dimension(0:lleg,-mleg:mleg,0:npo), intent (in)  :: rlm
  REAL (kind=dp), dimension(mleg,0:npo),              intent (out) :: Srm
  INTEGER                                                          :: n, m, l, ll
  REAL (kind=dp), dimension(mleg)                                  :: qq
  REAL (kind=dp), dimension(mleg,mleg)                             :: rr
  REAL (kind=dp), dimension(:), allocatable                        :: ss

  CHARACTER (LEN=1) :: TRANS = 'N'
  REAL (KIND=dp)    :: ALPHA = 1.0_dp
  REAL (KIND=dp)    :: BETA = 0.0_dp
  INTEGER           :: INCX = 1
  INTEGER           :: INCY = 1

  allocate (ss(mleg))

  do l=1,mleg
    ll = 2 * l - 1
    qq(l) = (2*ll+1) * q_0(ll)
  end do

  do n=0,npo
    do l=1,mleg
      ll = 2 * l - 1
      do m=1,mleg
        rr(m,l) = rlm(ll,m,n)
      end do
    end do
    call dgemv (TRANS, mleg, mleg, ALPHA, rr, mleg, qq, INCX, BETA, ss, INCY)
    Srm(:,n) = ss
  end do

  deallocate (ss)

END SUBROUTINE aux_Srm

SUBROUTINE aux_Rlmi(rlm, hl, rlmi)

  USE constantes
  USE PROFIL

  implicit none
  REAL (kind=dp), dimension(0:lleg,-mleg:mleg,0:npo), intent (in)  :: rlm
  REAL (kind=dp), dimension(0:lleg,0:npo), intent (in)             :: hl
  REAL (kind=dp), dimension(0:lleg,-mleg:mleg,0:npo), intent (out) :: rlmi

  INTEGER                                                          :: m, l, n
  INTEGER                                                          :: i, j
  REAL (kind=dp)                                                   :: taux1
  REAL (kind=dp), dimension(lleg+1)                                :: eigJ
  REAL (kind=dp), dimension(lleg+1,lleg+1)                         :: rr_n
  REAL (kind=dp), dimension(lleg+1,lleg+1)                         :: rr_t

! rr_n = 0.0_dp
! eigJ = 0.0_dp

  do n=0,npo
 
! 4a - Compute inverse norm of J

    do m=1,mleg
      taux1 = SUM( (/ (rlm(l,m,n)*rlm(l,m,n)*hl(l,n), l=0,lleg) /) )
      eigJ(mleg+m) = 1.0_dp / taux1
      eigJ(mleg-m+1) = 1.0_dp / taux1
    end do

    rr_n(1:lleg+1,1:mleg) = rlm(0:lleg,-mleg:-1,n)
    rr_n(1:lleg+1,mleg+1:2*mleg) = rlm(0:lleg,1:mleg,n)

!  4c - Transpose and Left multiply by J**-1  ! jrg: rows are multiplied

    do i=1,lleg+1
      do j=1,lleg+1
        rr_t(i,j) = eigJ(i) * rr_n(j,i)
      end do
    end do

!  4d - Right mutiply by D**2  (G**2 in notes jrg)

    do j=1,lleg+1
      rr_t(:,j) = rr_t(:,j) * hl(j-1,n)
    end do

!  4e - Compute R**-1

    rlmi(0:lleg,-mleg:-1,n) = rr_t(1:lleg+1,1:mleg)
    rlmi(0:lleg,1:mleg,n) = rr_t(1:lleg+1,mleg+1:2*mleg)

  end do

END SUBROUTINE aux_Rlmi

SUBROUTINE aux_Emij(Smk, Emij, indmi, indmj)

  USE constantes
  USE PROFIL

  implicit none
  REAL (kind=dp), dimension(mleg,0:npo), intent (in)        :: Smk
  REAL (kind=dp), dimension(mleg,0:npo,0:npo), intent (out) :: Emij
  INTEGER, dimension(mleg,0:npo), intent (out)              :: indmi
  INTEGER, dimension(mleg,0:npo), intent (out)              :: indmj
  INTEGER                                                   :: m, i, j
  INTEGER                                                   :: jp, jb, jm

  REAL (kind=dp), dimension(:,:), allocatable               :: AAij

  allocate (AAij(0:npo,0:npo))
  AAij = 0.0_dp
  do m=1,mleg
    do i=0,npo
      do j=0,i
        AAij(i,j) = Smk(m,i) - Smk(m,j)
!       print *, i, j, AAij(i,j)
      end do
    end do

    indmi(m,:) = 0
    do i=0,npo
      jm = 0
      jb = i / 2
      jp = i
      do
        if (AAij(i,jb) > 35.0_dp) then
          jm = jb
        else
          jp = jb
        endif
        jb = (jp + jm) / 2
        if (jp-jm <= 1) exit
      end do
      indmi(m,i) = jb
!     print *, i, indmi(m,i)
    end do

    if (isoneside /= 1) then
    indmj(m,:) = npo + 1
    do i=0,npo
      j = indmi(m,i)
      indmj(m,j) = i
    end do
    do j=0,npo
      if (indmj(m,j) > npo) then
        indmj(m,j) = indmj(m,j-1)
      endif
!     print *, j, indmj(m,j)
    end do
    endif

    do i=0,npo
      do j=indmi(m,i),i
        Emij(m,i,j) = exp(-AAij(i,j))
      end do
    end do
  end do

  deallocate (AAij)

END SUBROUTINE aux_Emij

SUBROUTINE aux_Git(rlmi, ggm, xkm, Git)

  USE constantes
  USE PROFIL

  implicit none
  REAL (kind=dp), dimension(0:lleg,-mleg:mleg,0:npo), intent (in) :: rlmi
  REAL (kind=dp), dimension(-mleg:mleg,0:npo), intent (in)        :: ggm
  REAL (kind=dp), dimension(-mleg:mleg,0:npo), intent (in)        :: xkm
  REAL (kind=dp), dimension(-mleg:mleg,0:npo), intent (out)       :: Git

  REAL (kind=dp), dimension(lleg+1,2*mleg)                        :: R_aux
  REAL (kind=dp), dimension(2*mleg)                               :: g_aux
  REAL (kind=dp), dimension(2*mleg)                               :: GG_aux
  INTEGER                                                         :: n

  CHARACTER (LEN=1) :: TRANS = 'N'
  REAL (KIND=dp)    :: ALPHA = 1.0_dp
  REAL (KIND=dp)    :: BETA = 0.0_dp
  INTEGER           :: INCX = 1
  INTEGER           :: INCY = 1

! 5d - Compute matrix product  R^-1*A^-1*g = Git

   Git(0,:) = 0.0_dp
   do n=0,npo
     R_aux(1:lleg+1,1:mleg) = rlmi(0:lleg,-mleg:-1,n)
     R_aux(1:lleg+1,mleg+1:2*mleg) = rlmi(0:lleg,1:mleg,n)
     g_aux(1:mleg) = ggm(-mleg:-1,n)
     g_aux(mleg+1:2*mleg) = ggm(1:mleg,n)

     call dgemv (TRANS, 2*mleg, 2*mleg, ALPHA, R_aux, 2*mleg, g_aux, INCX, BETA, GG_aux, INCY)

     Git(-mleg:-1,n) = GG_aux(1:mleg)
     Git(1:mleg,n) = GG_aux(mleg+1:2*mleg)

   end do

   Git = Git * xkm

END SUBROUTINE aux_Git

SUBROUTINE aux_Rlmip(rlmi, rlmp, rlmip)

  USE constantes
  USE PROFIL

  implicit none
  REAL (kind=dp), dimension(0:lleg,-mleg:mleg,0:npo), intent (in)  :: rlmi
  REAL (kind=dp), dimension(0:lleg,-mleg:mleg,0:npo), intent (in)  :: rlmp
  REAL (kind=dp), dimension(0:lleg,-mleg:mleg,0:npo), intent (out) :: rlmip

  INTEGER                                                          :: n
  REAL (kind=dp), dimension(lleg+1,2*mleg)                         :: Ri_aux
  REAL (kind=dp), dimension(lleg+1,2*mleg)                         :: Rp_aux
  REAL (kind=dp), dimension(lleg+1,2*mleg)                         :: Rip_aux

  CHARACTER (LEN=1) :: TRANS = 'N'
  REAL (KIND=dp)    :: ALPHA = 1.0_dp
  REAL (KIND=dp)    :: BETA = 0.0_dp

  !Matrix product  R^-1 x dR/dt = rlmip to keep with rlmi stored

!  rlmip = 0.0_dp

   do n=0,npo

     Ri_aux(1:lleg+1,1:mleg) = rlmi(0:lleg,-mleg:-1,n)
     Ri_aux(1:lleg+1,mleg+1:2*mleg) = rlmi(0:lleg,1:mleg,n)
     Rp_aux(1:lleg+1,1:mleg) = rlmp(0:lleg,-mleg:-1,n)
     Rp_aux(1:lleg+1,mleg+1:2*mleg) = rlmp(0:lleg,1:mleg,n)

     call dgemm (TRANS, TRANS, lleg+1, 2*mleg, lleg+1, ALPHA, Ri_aux, lleg+1, Rp_aux, lleg+1, BETA, Rip_aux, lleg+1)

     rlmip(0:lleg,-mleg:-1,n) = Rip_aux(1:lleg+1,1:mleg)
     rlmip(0:lleg,1:mleg,n) = Rip_aux(1:lleg+1,mleg+1:2*mleg)

   end do

! Patch idiot
   rlmip(:,:,0) = 0.0_dp
   rlmip(:,:,npo) = 0.0_dp

END SUBROUTINE aux_Rlmip

SUBROUTINE aux_Yit1(Iinc, xkm, tl_wl, al_wl, rlmi, sfm, Yit_o)

  USE constantes
  USE PROFIL

  implicit none
  REAL (kind=dp), dimension(-mleg:mleg), intent (in)              :: Iinc
  REAL (kind=dp), dimension(-mleg:mleg,0:npo), intent (in)        :: xkm
  REAL (kind=dp), dimension(0:npo), intent (in)                   :: tl_wl  ! = tl(:,iwlg)
  REAL (kind=dp), dimension(0:npo), intent (in)                   :: al_wl  ! = albed(:,iwlg)
  REAL (kind=dp), dimension(0:lleg,-mleg:mleg,0:npo), intent (in) :: rlmi
  REAL (kind=dp), dimension(-mleg:mleg,0:npo), intent (in)        :: sfm
  REAL (kind=dp), dimension(-mleg:mleg,0:npo), intent (out)       :: Yit_o
  REAL (kind=dp)                                                  :: taux2
  INTEGER                                                         :: l, n

  REAL (kind=dp), dimension(lleg+1,2*mleg)                         :: R_aux
  REAL (kind=dp), dimension(2*mleg)                                :: S_aux
  REAL (kind=dp), dimension(2*mleg)                                :: Y_aux

  CHARACTER (LEN=1) :: TRANS = 'N'
  REAL (KIND=dp)    :: ALPHA = 1.0_dp
  INTEGER           :: INCX = 1
  INTEGER           :: INCY = 1

! 10a - Compute Y_o = R^-1 x F_approx
!       F_approx ~ sfm / 1 - albed + Iexp(kt), then it IS wavelength and tau dependent

     Yit_o(0,:) = 0.0_dp
     do n=0,npo
       R_aux(1:lleg+1,1:mleg) = rlmi(0:lleg,-mleg:-1,n)
       R_aux(1:lleg+1,mleg+1:2*mleg) = rlmi(0:lleg,1:mleg,n)
       S_aux(1:mleg) = sfm(-mleg:-1,n) / (1.0_dp - al_wl(n))
       S_aux(mleg+1:2*mleg) = sfm(1:mleg,n) / (1.0_dp - al_wl(n))

!  BUg 26 IV 06 : lleg+1 and not lleg
       do l=1,lleg+1
         Y_aux(l) = SUM(R_aux(l,:))
       end do

       taux2 = Iinc(-1)*exp(-xkm(1,n)*tl_wl(n)) + Iinc(+1)*exp(xkm(-1,n)*(tl_wl(npo)-tl_wl(n)))
       call dgemv (TRANS, 2*mleg, 2*mleg, ALPHA, R_aux, 2*mleg, S_aux, INCX, taux2, Y_aux, INCY)

       Yit_o(-mleg:-1,n) = Y_aux(1:mleg)
       Yit_o(1:mleg,n) = Y_aux(mleg+1:2*mleg)
     end do

END SUBROUTINE aux_Yit1

SUBROUTINE aux_Qit(rlmip, Yit_o, Qit)

  USE constantes
  USE PROFIL

  implicit none
  REAL (kind=dp), dimension(0:lleg,-mleg:mleg,0:npo), intent (in) :: rlmip
  REAL (kind=dp), dimension(-mleg:mleg,0:npo), intent (in)        :: Yit_o
  REAL (kind=dp), dimension(-mleg:mleg,0:npo), intent (out)       :: Qit
  INTEGER                                                         :: n

  REAL (kind=dp), dimension(lleg+1,2*mleg)                         :: R_aux
  REAL (kind=dp), dimension(2*mleg)                                :: Y_aux
  REAL (kind=dp), dimension(2*mleg)                                :: Q_aux

  CHARACTER (LEN=1) :: TRANS = 'N'
  REAL (KIND=dp)    :: ALPHA = 1.0_dp
  REAL (KIND=dp)    :: BETA = 0.0_dp
  INTEGER           :: INCX = 1
  INTEGER           :: INCY = 1

! 10b - Compute Q_o = R^-1 x R' x Y_o

     do n=0,npo
       R_aux(1:lleg+1,1:mleg) = rlmip(0:lleg,-mleg:-1,n)
       R_aux(1:lleg+1,mleg+1:2*mleg) = rlmip(0:lleg,1:mleg,n)
       Y_aux(1:mleg) = Yit_o(-mleg:-1,n)
       Y_aux(mleg+1:2*mleg) = Yit_o(1:mleg,n)

       call dgemv (TRANS, 2*mleg, 2*mleg, ALPHA, R_aux, 2*mleg, Y_aux, INCX, BETA, Q_aux, INCY)

       Qit(-mleg:-1,n) = Q_aux(1:mleg)
       Qit(1:mleg,n) = Q_aux(mleg+1:2*mleg)
     end do

END SUBROUTINE aux_Qit

SUBROUTINE aux_bp(rlm, pli, n, aam)

  USE constantes
  USE PROFIL

  implicit none
  REAL (kind=dp), dimension(0:lleg,-mleg:mleg,0:npo), intent (in) :: rlm
  REAL (kind=dp), dimension(0:lleg,-mleg:mleg), intent (in)       :: pli
  INTEGER, intent (in)                                            :: n
  REAL (kind=dp), dimension(lleg+1,lleg+1), intent (out)          :: aam

  REAL (kind=dp), dimension(mleg,lleg+1)                          :: R_aux
  REAL (kind=dp), dimension(lleg+1,mleg)                          :: P_aux
  REAL (kind=dp), dimension(mleg,mleg)                            :: A_aux
  REAL (kind=dp)                                                  :: g_st
  INTEGER                                                         :: l, m, i

  CHARACTER (LEN=1) :: TRANS = 'T'
  REAL (KIND=dp)    :: ALPHA = 1.0_dp
  REAL (KIND=dp)    :: BETA = 0.0_dp

  do l=0,lleg
    g_st = (2.0_dp*l+1.0_dp)
    do m=1,mleg
      R_aux(mleg+1-m,l+1) = g_st * rlm(l,-m,n)
    end do
    do i=1,mleg
      P_aux(l+1,mleg+1-i) = pli(l,-i)
    end do
  end do

  call dgemm (TRANS, TRANS, mleg, mleg, lleg+1, ALPHA, P_aux, lleg+1, R_aux, mleg, BETA, A_aux, mleg)

  aam(1:mleg,1:mleg) = A_aux

END SUBROUTINE aux_bp

SUBROUTINE aux_bm(rlm, pli, n, aam)

  USE constantes
  USE PROFIL

  implicit none
  REAL (kind=dp), dimension(0:lleg,-mleg:mleg,0:npo), intent (in) :: rlm
  REAL (kind=dp), dimension(0:lleg,-mleg:mleg), intent (in)       :: pli
  INTEGER, intent (in)                                            :: n
  REAL (kind=dp), dimension(lleg+1,lleg+1), intent (out)          :: aam

  REAL (kind=dp), dimension(mleg,lleg+1)                          :: R_aux
  REAL (kind=dp), dimension(lleg+1,mleg)                          :: P_aux
  REAL (kind=dp), dimension(mleg,mleg)                            :: A_aux
  REAL (kind=dp)                                                  :: g_st
  INTEGER                                                         :: l, m, i

  CHARACTER (LEN=1) :: TRANS = 'T'
  REAL (KIND=dp)    :: ALPHA = 1.0_dp
  REAL (KIND=dp)    :: BETA = 0.0_dp

  do l=0,lleg
    g_st = (2.0_dp*l+1.0_dp)
    do m=1,mleg
      R_aux(m,l+1) = g_st * rlm(l,m,n)
    end do
    do i=1,mleg
      P_aux(l+1,i) = pli(l,i)
    end do
  end do

  call dgemm (TRANS, TRANS, mleg, mleg, lleg+1, ALPHA, P_aux, lleg+1, R_aux, mleg, BETA, A_aux, mleg)

  aam(1:mleg,1:mleg) = A_aux

END SUBROUTINE aux_bm

!---------------------------------------------------------------------------

END MODULE TRANSFER
