
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%%%%%%%
       subroutine MNEWT (iopt)
c      %%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE STR_DATA
       USE CHEM_DATA
       USE SP_DATA
       USE PROFIL

c      voir numerical recipe p 272.

       implicit double precision (a-h,o-z)

c  Numerical parameters (unused outside MNEWT, used to be defined as parameter)

       INTEGER         :: ntr1 = 25         ! Max number of global iteration in N-R algorithm
                                            !     1 global iter = change conservation equation
       INTEGER         :: ntr2 = 3          ! Max number of iteration with the same
                                            !     conservation equation
       REAL (KIND=dp)  :: tolx = 1.0e-6_dp  ! Total relative error for chemistry

       real (kind=dp), allocatable, dimension (:,:) :: am, bbmn
       real (kind=dp), dimension (nes) :: abold, abt
       real (kind=dp), dimension(0:natom) :: ratiab
       real (kind=dp)                     :: ratlim = 1.0_dp
       dimension indx(nes)

       allocate (am(nspec,nspec), bbmn(nspec,1))

       abold(1:nspec) = ab(1:nspec)

       abt = 0
       do i=1,nspec
          duma = densh(iopt)
          do j=1,natom-1
            kspec = ael(i,j)
            if (kspec == 0) then
              cycle
            else
              duma = min(duma,densh(iopt)*abtot(j)/kspec)
            endif
          end do
          abt(i) = duma
       end do

       do k1=1,ntr1

       damp = 0.7d0

       do k2=1,ntr2

c  usrfun computes the Jacobian and the function to nullify

         call USRFUN (iopt, am, bbmn, nspec)

         abold(1:nspec) = ab(1:nspec)

c  Switch to LAPACK

         nrhs = 1
         call dgesv (nspec, nrhs, am, nspec, indx, bbmn, nspec, info)

         dum = sum(abs(bbmn(1:nspec,1)))
         if (dum > densh(iopt)) then
           bbmn(1:nspec,1) = bbmn(1:nspec,1) * densh(iopt) / dum
         endif

         errx = 0.0_dp

         bad = 0.0_dp
         mauvai = 0
         do i=1,nspec
           if (ab(i) > max(istiny,-bbmn(i,1))
     1         .OR. bbmn(i,1) > 0.0_dp) then
             tt1 = abs(bbmn(i,1)) / (abold(i) + istiny)
             if (ab(i) > 1.0e10_dp*istiny) then
               errx = errx + tt1
               if (bad < tt1) then
                 bad = tt1
                 mauvai = i
               endif
             endif

c  *** Patch: enhanced robustness ***

             if (lcon(i) == 0) then
               ab(i) = ab(i) + bbmn(i,1) * damp
             else
               ab(i) = ab(i) + bbmn(i,1)
             endif
             if (ab(i) > abt(i)) then
               ab(i) = (abt(i) + abold(i)) * 0.5_dp
             else if (ab(i) < 0.0_dp) then
               ab(i) = abold(i) / 10.0_dp
             endif
           else
             ab(i) = istiny
           endif
         end do

         if (mod(k2,2) == 1) then
           do i=1,nspec
             ab(i) = (ab(i) + abold(i)) * 0.5_dp
           end do
         endif
c  *** end patch ***

         abelec = sum(ab(nn+1:nspec-1) * ael(nn+1:nspec-1,natom))

         errx = errx / damp
         if (errx <= tolx .and. mod(k2,2) == 0) then
           deallocate (am, bbmn)
           return
         endif
       end do

c  Modify conservation equation in case of bad convergence

       do i = 1,natom-1
         top1 = 0.0_dp
         top2 = 0.0_dp
         icx(i,1) = 0
         icx(i,2) = 0

         do l = 1,nspec-1
           dum = ael(l,i)
           if (dum == 0.0_dp) cycle
           if (lcon(l) == i) then
             lcon(l) = 0
           endif
           ddumm = ab(l) * dum
           if (ddumm > top1) then
             top2 = top1
             top1 = ddumm
             icx(i,2) = icx(i,1)
             icx(i,1) = l
           else if (ddumm > top2) then
             top2 = ddumm
             icx(i,2) = l
           endif
         end do
         if (top1 /= 0.0_dp) then
           ratiab(i) = top2 / top1
         endif
       end do

       do i = 1,natom-1
         ici1 = icx(i,1)
         ici2 = icx(i,2)
         if (ici1 == 0) cycle

c  Patch recombination

         if (ratiab(i) > ratlim) then
           if (ael(ici1,natom) == 1
     1         .and. ael(ici2,natom) == 0) then
             jjj = ici1
             ici1 = ici2
             ici2 = jjj
           endif
         endif

         icc = 0
         do ii = 1,i-1
           if (lcon(ici1) == ii) icc = ii
         end do

         if (icc == 0) then
           lcon(ici1) = i
         else
           if (ratiab(i) > ratiab(icc)) then
             lcon(ici2) = i
           else
             lcon(ici1) = i
             lcon(icx(icc,2)) = icc
           endif
         endif

       end do

       end do

       k2 = k2 - 1
c      ratlim = 0.35d0
       ratlim = 0.20d0

       deallocate (am, bbmn)

       return
       end

c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine USRFUN (iopt, am, bbmn, nsp)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE STR_DATA
       USE SP_DATA
       USE CHEM_DATA
       USE PROFIL

c  Computes the function (-beta) to nullify and its derivatives
c  (alpha) at point ab, using conservation equations

       implicit double precision (a-h,o-z)

       real (kind=dp), dimension (nsp,nsp) :: am
       real (kind=dp), dimension (nsp,1) :: bbmn

       dimension ab1(nqm), ab2(nqm), ab3(nqm)
       dimension j1(nqm), j2(nqm), j3(nqm)
       dimension dumb(0:nes)

c  Chemical equations include only polynomials in the abundances

c  If lcon(i) = 0 : this is an ordinary eqaution
c               d[Xi] / dt = 0
c  If lcon(i) = j /= 0 : this is a conservation equation of atom j

       do i=1,nspec-1

         jatom = lcon(i)

         if (jatom == 0) then
           nrxi = ikmat(i,0)

           do k=1,nrxi
             ind = ikmat(i,k)
             j1(k) = react(ind,1)
             j2(k) = react(ind,2)
             j3(k) = react(ind,3)
             ab1(k) = ab(j1(k))
             ab2(k) = ab(j2(k))
             ab3(k) = ab(j3(k))
           end do

c  Computes residuals

           duma = sum(xkmat(i,1:nrxi) * ab1(1:nrxi) * ab2(1:nrxi)
     1          * ab3(1:nrxi))

           bbmn(i,1) = - duma

c  Computes partial derivatives

           dumb(1:nspec) = 0.0_dp

           do k=1,nrxi
             dumb(j1(k)) = dumb(j1(k)) + xkmat(i,k) * ab2(k) * ab3(k)
             dumb(j2(k)) = dumb(j2(k)) + xkmat(i,k) * ab1(k) * ab3(k)
             dumb(j3(k)) = dumb(j3(k)) + xkmat(i,k) * ab1(k) * ab2(k)
           end do

           am(i,1:nspec) = dumb(1:nspec)

         else

           duma = abtot(jatom) * densh(iopt)
           do j=1,nspec-1
             kspec = ael(j,jatom)
             duma = duma - kspec * ab(j)
             am(i,j) = - kspec
           end do

           bbmn(i,1) = - duma
           am(i,nspec) = 0.0_dp

         endif

       end do

         duma = ab(nspec)
         do j=1,nspec-1
           kspec = ael(j,natom)
           duma = duma - kspec * ab(j)
           am(nspec,j) = kspec
         end do

         bbmn(i,1) = duma
         am(nspec,nspec) = - 1.0_dp

       return
       end

