
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine EQLH2V (ttry, iopt)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       use RF_TOOLS
       use H2_DATA
       use SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

c  Computes equilibrium matrix of H2 levels and 
c  solves for detailed balace

       implicit double precision (a-h,o-z)

       dimension create(nlevh2)
       dimension aev(nlevh2,nlevh2), indx(nlevh2)
       dimension bev(nlevh2,1)

       real (KIND=dp), DIMENSION (:), ALLOCATABLE :: t_res
       real (KIND=dp), DIMENSION (:), ALLOCATABLE :: frac_c

c  TIMH2 useful datas for caracteristic time scales of H2 (in s-1 cm-3)
c      TIMH2(iopt,1) : FABRIC : H2 formation by gas phase chemistry
c      TIMH2(iopt,2) : DESTRU : H2 destructruction by gas phase chemistry
c      TIMH2(iopt,3) : RATEGR : H2 formation on grains
c      TIMH2(iopt,4) : DESPHO : H2 destruction by photons
c      TIMH2(iopt,5) :        : H2 desorption from grains

c      dimension TOTAL(nlevh2)

       IF ( .NOT. ALLOCATED(t_res)) THEN
         ALLOCATE (t_res(npg), frac_c(npg))
       ENDIF

       xnprot = ab(nn+1)

       evcolh(1:l0h2,1:l0h2) = 0.0_dp
       evltot(1:l0h2,1:l0h2) = 0.0_dp
       create(1:l0h2) = 0.0_dp

c  Collisional processes
c      evcolh is built in COLH2V

       call COLH2V (xnprot, ttry, iopt)

c  Quadrupole transitions
c  evlqua is initialised in init1

c  Pure chemical contributions

       fabric = 0.0_dp
       destru = 0.0_dp

c  We must exclude
c                  Formation on grains
c                  Photodissociation
c                  Adsorption - desorption reactions
c  which have a specific treatment
c  Endothermal reactions (type 6) are also dealed with elsewhere

       nrxi = ikmat(i_h2,0)

       do k=1,nrxi
         ind = ikmat(i_h2,k)
         if (ind == ifh2gr .or.
     1       ind == idh2ph .or.
     2       itype(ind) > 110 .or.
     3       itype(ind) == 6) cycle
         k1 = react(ind,1)
         k2 = react(ind,2)
         k3 = react(ind,3)
         taux = xkmat(i_h2,k)
         if (taux > 0.0_dp) then
           fabric = fabric + taux * ab(k1) * ab(k2) * ab(k3)
         else
           destru = destru + taux * ab(k1) * ab(k2) * ab(k3)
         endif
       end do

c  H2 formation on grains

       rate = gamm(ifh2gr) * 2.0_dp * tm1h(i_hgr) / xkhydr

       timh21(iopt) = fabric
       timh22(iopt) = - destru
       timh23(iopt) = rate * ab(i_hgr) * ab(i_hgr)
       timh24(iopt) = pdish2(iopt) * ab(i_h2)

c  H2 desorption from grains
c  The last reactions are artificial (to enforce a single adsorbed layer)
c  and thus are not included here
c  The numbering is the same for all "fractions"

       nrxigr = ikmat(i_h2gr,0)

       t_25(1:npg,iopt) = 0.0_dp
       dd1 = 0.0_dp
       dd2 = 0.0_dp
       do mm=1,npg
         do k=1,nrxigr
           ind = ikmat(i_h2gr+mm-1,k)
           ity = itype(ind)
           if (ity == 115 .or.
     1         ity == 117 .or.
     2         ity == 118) then
             taux = xkmat(i_h2gr+mm-1,k)
             t_25(mm,iopt) = t_25(mm,iopt) - taux * ab(i_h2gr+mm-1)
           else if (ity == 113) then
             taux = xkmat(i_h2gr+mm-1,k)
             dd1 = dd1 + taux * ab(i_h2)
c            destru = destru - dd1
           else if (ity == 123) then
             taux = xkmat(i_h2gr+mm-1,k)
             dd2 = dd2 + taux * ab(i_h2) * ab(i_h2gr+mm-1)
c            destru = destru - dd2
           endif
         end do
       end do

       dd1 = dd1 + dd2

c      print *, fabric, rate, ab(i_hgr)
c      print *, destru, pdish2(iopt), ab(i_h2)

       dumy = 0.0_dp
c      do lev=1,nlevh2
c  10 mai 2004 ?
       do lev=1,l0h2
         ij = njlev(lev)
         iv = nvlev(lev)
c        print *, iv, ij, pophm(iv,ij), elhm(lev)
         dumy = dumy + pophm(iv,ij)
       end do
c      print *, '  normalisation de pophm:', dumy
       echpop = dumy

c  8 Juillet 93, Secondary electrons excitation (CR)
c     from: A Dalgarno, private comm. (Computation by R. Gredel)

c  Modify O/P ratio for secondary electrons excitation

       corpar = h2para / paretl
       corort = h2orth / ortetl
       crnorm = zeta * densh(iopt) * ab(i_h2) / (ab(i_h) + ab(i_h2))

       do lev=1,l0h2
         ij = njlev(lev)
         jjj = mod(ij,2)
         corosp = jjj * corort + (1 - jjj) * corpar
         iv = nvlev(lev)
         dumy = 0.0_dp
         dumy2 = 0.0_dp
         do levu=l0h2+1,nlevh2
           iju = njlev(levu)
           ivu = nvlev(levu)
           dumy = dumy + pophm(ivu,iju) * cashm(levu,lev)
     1          / echpop
           dumy2 = dumy2 + entry(ivu,iju) * cashm(levu,lev)
         end do
         create(lev) = (dumy + pophm(iv,ij) / echpop)
     1               * rate * ab(i_hgr) * ab(i_hgr)
     2               + (dumy2 + entry(iv,ij)) * corosp
     3               * crnorm
       end do

c...............................................................................

c  Distribution of H2 formed in the gas phase

c  This is an ad hoc hypothesys
c      2/3 on j = 0, 1/3 on j = 1
c      because rxn h3+ + x -> h2 + xh+ dominates

c  Test: H2 adsorbed on grains may be converted to pure
c        para-H2 (J = 0) with a caracteristic time scale of
c        t_o2p = 1 minute or 1 hour

       t_o2p = 60.0_dp
c      t_o2p = 3600.0_dp
       if (t_25(1,iopt) /= 0.0_dp) then
         do mm=1,npg
           t_res(mm) = ab(i_h2gr+mm-1) / t_25(mm,iopt)
         end do
         frac_c = exp(-t_o2p/t_res)
       else
         frac_c = 0.0_dp
       endif

       leva = level(0,0)
       create(leva) = create(leva) + 2.0_dp * fabric / 3.0_dp
     1        + sum(t_25(1:npg,iopt) * (h2para + frac_c * h2orth))
       levb = level(0,1)
       create(levb) = create(levb) + fabric / 3.0_dp
     1        + sum(t_25(1:npg,iopt) * (1.0_dp - frac_c) * h2orth)

c  or: all on j = 0

c      leva = level(0,0)
c      create(leva) = create(leva) + fabric

c...............................................................................

c  Build full matrix

       do i2=1,l0h2
         do i1=1,l0h2
           evltot(i1,i2) = (evcolh(i1,i2) + evlqua(i1,i2)
     1                   + evrdbc(i1,i2)) * ab(i_h2)
         end do
       end do

       do ii=1,l0h2
         evltot(ii,ii) = evltot(ii,ii) - destru + dd1
       end do

c Add endothermal reactions (direct destruction of a specific level)

       do k=1,nrxi
         ind = ikmat(i_h2,k)
         if (itype(ind) /= 6) cycle
         k1 = react(ind,1)
         k2 = react(ind,2)
         do ii=1,l0h2
           endo_rxn = max(bet(ind) - elhm(ii), 0.0_dp)
           rat_l = gamm(ind) * (ttry/300.0_dp)**alpha(ind)
     1         * exp(-endo_rxn/ttry)
           evltot(ii,ii) = evltot(ii,ii) + rat_l * ab(k1) * ab(k2)
         end do
       end do

c  Substract from diagonal electronic excitation form RC
c     popetl, H2 repartition postulated by
c     Gredel (ETL at 30 K, 8 july 93)
c  10 mai 2004 : To be changed completely !!!

c      do jj=0,njh2
c        lev = level(0,jj)
c        if (lev > l0h2) cycle
c        jjj = mod(njlev(lev),2)
c        corosp = jjj * corort + (1 - jjj) * corpar
c        evltot(lev,lev) = evltot(lev,lev)
c    1                   + (popetl(jj) / xreh2(lev,iopt))
c    2                   * sentry * corosp * crnorm
c      end do

c   ****** DEBUG seulement

c       do jcol=1,l0h2
c         jj = njlev(jcol)
c         TOTAL(jcol) = 0.0_dp
c         do jlin=1,l0h2
c           TOTAL(jcol) = TOTAL(jcol) + evcolh(jlin,jcol)
c         end do
c         if (abs(TOTAL(jcol)) >= 1.0e-15) then
c           print *, jcol, '  ', TOTAL(jcol)
c         endif
c       end do

c       do jcol=1,l0h2
c         jj = njlev(jcol)
c         TOTAL(jcol) = 0.0_dp
c         do jlin=1,l0h2
c           TOTAL(jcol) = TOTAL(jcol) + evlqua(jlin,jcol)
c         end do
c         if (abs(TOTAL(jcol)) >= 1.0e-15) then
c           print *, jcol, '  ', TOTAL(jcol)
c         endif
c       end do

c       dumy = 0.0_dp
c       do jcol=1,l0h2
c         jj = njlev(jcol)
c         TOTAL(jcol) = 0.0_dp
c         do jlin=1,l0h2
c           TOTAL(jcol) = TOTAL(jcol) + evrdbc(jlin,jcol)
c         end do
cc        print *, jcol, '  ', TOTAL(jcol)
c         dumy = dumy + TOTAL(jcol) * xreh2(jcol,iopt)
c       end do
cc      print *, dumy, pdish2(iopt)

c  ****** end DEBUG

       aev(1:mlevh2,1:mlevh2) = evltot(levinc,levinc)
       bev(1:mlevh2,1) = create(levinc)

c   switch to LAPACK

         nrhs = 1
         call dgesv (mlevh2, nrhs, aev, nlevh2, indx, bev, nlevh2, info)

       ceqh2 = 0.0_dp
       do ii=1,mlevh2
         ceqh2 = ceqh2 + bev(ii,1)
       end do

       do ii=1,mlevh2
         bev(ii,1) = bev(ii,1) / ceqh2
       end do

       ceqh2 = ceqh2 - 1.0_dp
       if (abs(ceqh2) > 1.0d-2) then
         write(iscrn,1001) ceqh2
       endif

c      print *, '  conservation H2: ', ceqh2

       j2 = 0
       h2para = 0.0_dp
       h2orth = 0.0_dp
       do lev=1,l0h2
         jj = njlev(lev)
         j2 = j2 + 1
         xreh2(lev,iopt) = bev(j2,1)
         if (mod(jj,2) == 0) then
           h2para = h2para + bev(j2,1)
         else
           h2orth = h2orth + bev(j2,1)
         endif
       end do
c      print *, "  O/P:", h2para, h2orth, h2orth/h2para

 1001  format (5x,'normalisation H2:',1pd15.6)

       return
       end

c---------------------------------------------------------------------

c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine COLH2V (xnprot, ttry, iopt)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       use H2_DATA
       use SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

c---------------------------------------------------------------------
c      xnprot is no. density of protons (cm-3)
c      ttry is temperature of neutral fluid

c      called from sub. EQLH2V
c
c      collision rates for H2 + H come from Martin & Mandy.
c      New fits by Jacques Le Bourlot (july 1996)
c
c          or
c
c      from D. Flower (cf MNRAS)
c
c      Collision rates for H2 + He, H2 + H2o, H2 + H2p
c      come from D. Flower
c
c---------------------------------------------------------------------

       implicit double precision (a-h,o-z)

c  Data from Gerlich for ortho-para transfer with h+.

       dimension gerlic(30)

c  With David's prescription for O/P balance (3/1)

       data gerlic / 2.024d-10, 9.575d-10, 2.729d-10, 7.910d-10
     1       , 2.246d-10, 6.223d-10, 1.828d-10, 4.975d-10, 1.484d-10
     2       , 3.000d-10, 1.000d-10, 3.000d-10, 1.000d-10, 3.000d-10
     3       , 1.000d-10, 3.000d-10, 1.000d-10, 3.000d-10, 1.000d-10
     4       , 3.000d-10, 1.000d-10, 3.000d-10, 1.000d-10, 3.000d-10
     5       , 1.000d-10, 3.000d-10, 1.000d-10, 3.000d-10, 1.000d-10
     6       , 3.000d-10 /

       data tddh2 / 4500.0_dp /

c  Up to now collisions with H2 and He are used only for v = 0
c  Educated guesses may obtain from collisions with H
c  WARNING: c: desexcitation, d: excitation
c            h2h: H2 + H
c            h2h2: H2 + H2
c            h2he: H2 + He
c            ophp: H2 + H+ (ortho - para)

       if (ichh2 == 0) then
         nqc = 4
       else
         nqc = 3
       endif
       if (ichh2 <= 1) then
         xchh2 = 0.0_dp
       else
         xchh2 = 1.0_dp
       endif

       do levu=2,l0h2
         ju = njlev(levu)
         ivu = nvlev(levu)

         do levl=1,levu-1
           jl = njlev(levl)
           ivl = nvlev(levl)

c  ctod is the Boltzmann factor used to compute
c     excitation rates from desexcitation rates

           ndej = abs(ju - jl)
           fac = (elhm(levu) - elhm(levl)) / ttry
           ctod = exp(-fac) * (2.0_dp*ju+1.0_dp) / (2.0_dp*jl+1.0_dp)
           if (mod(ju,2) == 1) then
             ctod = ctod * 3.0_dp
           endif
           if (mod(jl,2) == 1) then
             ctod = ctod / 3.0_dp
           endif

c  Fit of H2 + H rates. The reduced variable tdd is offset by a
c  different value for reactive (resp. non-reactive) collisions.

           if (mod(ndej,2) == 0) then
             decal = 1.0_dp
           else
             decal = 0.3_dp
           endif
           tdd = decal + ttry * 1.0d-3
           tdd2 = tdd*tdd

c  Reactive and non-reactive collisions are mixed in Martin and Mandy

           gamma = qh1h2(1,levu,levl)
     1           + qh1h2(2,levu,levl) / tdd
     2           + qh1h2(3,levu,levl) / tdd2
           if (nqc == 4) then
             gamma = gamma +  qh1h2(4,levu,levl) * tdd
           endif

           ch2h_nr = 10.0_dp**gamma
           ch2h_r  = 0.0_dp

c  Add reactive collisions (David's rule) if ichh2 = 3

           if (ichh2 == 3 .and. tau(iopt) <= 1.0_dp) then

             if (mod(ndej,2) == 0) then
               ch2h_r = 10.0_dp**gamma *
     1                exp(-max(0.0_dp,(3900.0_dp
     2              - elhm(levu) + elhm(levl))/ttry))
             else
               if (jl > 0) then
                 lev1 = level(ivl,jl-1)
               else
                 lev1 = level(ivl,jl+1)
               endif
               lev2 = level(ivl,jl+1)
               if (lev2 == 0) then
                 lev2 = level(ivl,jl-1)
               endif
               ga1 = qh1h2(1,levu,lev1)
     1             + qh1h2(2,levu,lev1) / tdd
     2             + qh1h2(3,levu,lev1) / tdd2
               if (nqc == 4) then
                 ga1 = ga1 + qh1h2(4,levu,lev1) * tdd
               endif
               ga2 = qh1h2(1,levu,lev2)
     1             + qh1h2(2,levu,lev2) / tdd
     2             + qh1h2(3,levu,lev2) / tdd2
               if (nqc == 4) then
                 ga2 = ga2 + qh1h2(4,levu,lev2) * tdd
               endif
               ch2h_r = (10.0_dp**ga1 + 10.0_dp**ga2) * 0.5 *
     1                exp(-max(0.0_dp,(3900.0_dp
     2              - elhm(levu) + elhm(levl))/ttry))

               if (mod(ju,2) == 1) then
                 ch2h_r = ch2h_r / 3.0_dp
               endif

             endif
           endif

c  Ortho/Para transfer with H from Schofield
c  plus David's rule (for a few delta v = 0)

           if (ichh2 >= 2 .and.
     1       ivu == ivl .and. ndej < 3) then
             if (mod(ju,2) == 1 .and. mod(ndej,2) == 1) then
               ch2h_r = 8.0d-11 * exp(-3900.0_dp/ttry) / 3.0_dp
             else
               ch2h_r = 8.0d-11 * exp(-3900.0_dp/ttry)
             endif
           endif

c          if (ichh2 <= 1) then
c            ch2h = ch2h_nr
c          else
c            ch2h = ch2h_nr + ch2h_r
c          endif

           ch2h = ch2h_nr + xchh2 * ch2h_r

c  He - H2

           gamma = qheh2(1,levu,levl)
     1           + qheh2(2,levu,levl) / tdd
     2           + qheh2(3,levu,levl) / tdd2

           ch2he = 10.0_dp**gamma

c  pH2 - H2

           gamma = qph2h2(1,levu,levl)
     1           + qph2h2(2,levu,levl) / tdd
     2           + qph2h2(3,levu,levl) / tdd2

           ch2ph2 = 10.0_dp**gamma

c  oH2 - H2

           gamma = qoh2h2(1,levu,levl)
     1           + qoh2h2(2,levu,levl) / tdd
     2           + qoh2h2(3,levu,levl) / tdd2

           ch2oh2 = 10.0_dp**gamma

c  c- Ortho to Para transfer withions (Gerlich), only for v = 0

c          if (ivu == 0 .and. ndej == 1) then
           if (jl == ju-1) then
             cophp = gerlic(ju)
           else
             cophp = 0.0_dp
           endif

c  Bringing it all back home

           c = ch2ph2 * ab(i_h2) * h2para
     1       + ch2oh2 * ab(i_h2) * h2orth
     2       + ch2he * ab(i_he)
     3       + cophp * (xnprot + ab(i_h3p))
     4       + ch2h * ab(i_h)
           d = c * ctod

           evcolh(levl,levu) = evcolh(levl,levu) - c
           evcolh(levu,levu) = evcolh(levu,levu) + c
           evcolh(levu,levl) = evcolh(levu,levl) - d
           evcolh(levl,levl) = evcolh(levl,levl) + d

         end do
       end do

       return
       end

