
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine EQLHDV (ttry, iopt)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE RF_TOOLS
       USE HD_DATA
       USE SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

c  HD detailed balance

       implicit double precision (a-h,o-z)

       dimension evltot(nlevhd,nlevhd), create(nlevhd)
       dimension aev(nlevhd,nlevhd), bev(nlevhd,1),
     1           indx(nlevhd)

c  TIMHD contains informations on caracteristic time scales for HD (en s-1 cm-3)
c      TIMHD(iopt,1) : FABRIC : HD formation by gas phase chemistry
c      TIMHD(iopt,2) : DESTRU : HD destruction by gas phase chemistry
c      TIMHD(iopt,3) : RATEGR : HD formation on grains
c      TIMHD(iopt,4) : DESPHO : HD destruction by photons

c      dimension TOTAL(nlevhd)

       evcohd(1:l0hd,1:l0hd) = 0.0_dp
       evltot(1:l0hd,1:l0hd) = 0.0_dp
       create(1:l0hd) = 0.0_dp

c  Collisional processes

       call COLHD (ttry)

c  Dipole and quadrupole transitions in X
c  evdihd is built in init1

       fabric = 0.0_dp
       destru = 0.0_dp

c  Chemical destruction and formation of HD
c      formation on grains (k=1) and photodissociations are excluded here

       nrxi = ikmat(i_hd,0)

       do k=1,nrxi
         ind = ikmat(i_hd,k)
         if (ind == ifhdgr .or.
     1       ind == idhdph) cycle
         k1 = react(ind,1)
         k2 = react(ind,2)
         k3 = react(ind,3)
         taux = xkmat(i_hd,k)
         if (taux > 0.0_dp) then
           fabric = fabric + taux * ab(k1) * ab(k2)
     1            * ab(k3)
         else
           destru = destru + taux * ab(k1) * ab(k2)
     1            * ab(k3)
         endif
       end do

c------------------------
c  HD formation on grains
c------------------------

       rate = gamm(ifhdgr) * (tm1h(i_dgr) + tm1h(i_hgr)) / xkhydr
       timhd3(iopt) = rate * ab(i_hgr) * ab(i_dgr)
       rateff = rate * ab(i_hgr) * ab(i_dgr)

       timhd1(iopt) = fabric
       timhd2(iopt) = - destru
       timhd4(iopt) = pdishd(iopt) * ab(i_hd)

c      print *, fabric, rate, ab(i_hgr), ab(i_dgr)
c      print *, destru, pdishd(iopt), ab(i_hd)

       dumy = 0.0_dp
       do lev=1,nlevhd
         ij = njhd(lev)
         if (ij <= mjhd) then
           iv = nvhd(lev)
c          print *, iv, ij, pophd(iv,ij), elhd(lev)
           dumy = dumy + pophd(iv,ij)
         endif
       end do
c      print *, '  normalisation de pophd:', dumy
       echpod = dumy


       do lev=1,l0hd
         ij = njhd(lev)
         if (ij <= mjhd) then
           iv = nvhd(lev)
           dumy = 0.0_dp
           do levu=l0hd+1,nlevhd
             iju = njhd(levu)
             if (iju <= mjhd) then
               ivu = nvhd(levu)
               dumy = dumy + pophd(ivu,iju) * cashd(levu,lev)
     1              / echpod
             endif
           end do
           create(lev) = (dumy + pophd(iv,ij) / echpod)
     1                 * rateff
         endif
       end do

c...............................................................................

c  Which repartition upon formation?
c  Equipartition:

c      fabric = fabric / (mjhd + 1)

c      do jj=0,mjhd
c        lev = levhd(0,jj)
c        create(lev) = create(lev) + fabric
c      end do


c  or: all on j = 0

      leva = levhd(0,0)
      create(leva) = create(leva) + fabric

c...............................................................................

c  bild full matrix

       do i2=1,l0hd
         do i1=1,l0hd
           evltot(i1,i2) = (evcohd(i1,i2) + evdihd(i1,i2)
     1                   + evrdhd(i1,i2)) * ab(i_hd)
         end do
       end do

       do ii=1,l0hd
         evltot(ii,ii) = evltot(ii,ii) - destru
       end do


c   ****** DEBUG only

c      do jcol=1,l0hd
c        jj = njhd(jcol)
c        if (jj > mjhd) cycle
c        TOTAL(jcol) = 0.0_dp
c        do jlin=1,l0hd
c          TOTAL(jcol) = TOTAL(jcol) + evcohd(jlin,jcol)
c        end do
c        if (abs(TOTAL(jcol)) >= 1.0e-15) then
c          print *, jcol, '  ', TOTAL(jcol)
c        endif
c      end do
c      pause

c      do jcol=1,l0hd
c        jj = njhd(jcol)
c        if (jj > mjhd) cycle
c        TOTAL(jcol) = 0.0_dp
c        do jlin=1,l0hd
c          TOTAL(jcol) = TOTAL(jcol) + evdihd(jlin,jcol)
c        end do
c        print *, jcol, '  ', TOTAL(jcol)
c      end do
c      pause

c      dumy = 0.0_dp
c      do jcol=1,l0hd
c        jj = njhd(jcol)
c        if (jj > mjhd) cycle
c        TOTAL(jcol) = 0.0_dp
c        do jlin=1,l0hd
c          TOTAL(jcol) = TOTAL(jcol) + evrdhd(jlin,jcol)
c        end do
c        print *, jcol, '  ', TOTAL(jcol)
c        dumy = dumy + TOTAL(jcol) * xrehd(jcol,iopt)
c      end do
c      print *, dumy, pdishd(iopt)

c      pause

c  ****** end DEBUG

       j2 = 0
       do i2=1,l0hd
         jj = njhd(i2)
         if (jj > mjhd) cycle
         j2 = j2 + 1
         j1 = 0
         do i1=1,l0hd
           jj = njhd(i1)
           if (jj > mjhd) cycle
           j1 = j1 + 1
           aev(j1,j2) = evltot(i1,i2)
         end do
         bev(j2,1) = create(i2)
       end do

       nat = j2

       nrhs = 1
       call dgesv (l0hd, nrhs, aev, nlevhd, indx, bev, nlevhd, info)

       ceqhd = 0.0_dp
       do ii=1,nat
         ceqhd = ceqhd + bev(ii,1)
       end do

       do ii=1,nat
         bev(ii,1) = bev(ii,1) / ceqhd
       end do

       ceqhd = ceqhd - 1.0_dp
       if (abs(ceqhd) > 1.0d-2) then
         write(iscrn,"(5x,'normalisation HD:',1pd15.6)") ceqhd
       endif

c      print *, '  conservation HD: ', ceqhd

       j2 = 0
       do lev=1,l0hd
         jj = njhd(lev)
         if (jj > mjhd) cycle
         j2 = j2 + 1
         xrehd(lev,iopt) = bev(j2,1)
       end do

       return
       end

c-------------------------------------------------------------

c       %%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLHD (ttry)
c       %%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE HD_DATA
       USE H2_DATA
       USE SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

c---------------------------------------------
c        ttry  : cloud temperature in the slab
c---------------------------------------------

       implicit double precision (a-h, o-z)

c  Warning: evcohd(levl,levu) < 0
c           evcohd(levu,levu) > 0

c     New fits (Jacques (1999))
       deche = 1.0_dp / 3.0_dp
       tddhe = deche + ttry * 1.0d-3
       dech1 = 2.0_dp / 3.0_dp
       tddh1 = dech1 + ttry * 1.0d-3
       dech2 = 0.5_dp
       tddh2 = dech2 + ttry * 1.0d-3

       do ju=1,mjhd    
         do jl=0,ju-1

            levu = levhd(0,ju)
            levl = levhd(0,jl)
            fac = (elhd(levu) - elhd(levl)) / ttry
            ctod = exp(-fac) * (2.0_dp*ju+1.0_dp) / (2.0_dp*jl+1.0_dp)

c Collision HD - He

            chdhe = 10.0_dp**(qhehd(1,levu,levl)
     1                     + qhehd(2,levu,levl) / tddhe
     2                     + qhehd(3,levu,levl) / (tddhe*tddhe))

c Collision HD - H
 
            chdh = 10.0_dp**(qh1hd(1,levu,levl)
     1                    + qh1hd(2,levu,levl) / tddh1
     2                    + qh1hd(3,levu,levl) / (tddh1*tddh1))

c Collision HD - H2 para
 
            chdph2 = 10.0_dp**(qph2hd(1,levu,levl)
     1                    + qph2hd(2,levu,levl) / tddh2
     2                    + qph2hd(3,levu,levl) / (tddh2*tddh2))

C collisions HD - H2 ortho

            chdoh2 = 10.0_dp**(qoh2hd(1,levu,levl)
     1                      + qoh2hd(2,levu,levl) / tddh2
     2                      + qoh2hd(3,levu,levl) / (tddh2*tddh2))

            c = chdhe * ab(i_he)
     1        + chdh * ab(i_h)
     2        + chdph2 * ab(i_h2) * h2para
     3        + chdoh2 * ab(i_h2) * h2orth
            d = c * ctod

            evcohd(levl,levu) = evcohd(levl,levu) - c
            evcohd(levu,levu) = evcohd(levu,levu) + c
            evcohd(levu,levl) = evcohd(levu,levl) - d
            evcohd(levl,levl) = evcohd(levl,levl) + d

           end do
         end do

c  Collisions with electrons -------------------------------------
c  cf Dickinson et al.  A&A 54,645(1977)       

       dip = 8.4d-4
       betas = 11600.0_dp / ttry
       b0 = 45.655_dp
       do levl=1,l0hd-1
         nj = njhd(levl)
         if (nj < mjhd) then
           nv = nvhd(levl)
           levu = levhd(nv,nj+1)
           deltaE = 2.48d-04 * b0 * (nj+1)
           c = 9.08e3_dp / (b0*(nj+1))
           taux = (3.56d-06 * (dip**2) / sqrt(ttry))
     1          * (nj+1) / (2*nj+1)
           taux = taux * exp(-betas*deltaE)
           taux = taux * log(c*deltaE+(c/betas)
     1          * exp(-0.577_dp / (1.0_dp+2.0_dp*betas*deltaE)))
           taux = taux * ab(nspec)
           evcohd(levu,levl) = evcohd(levu,levl) - taux
           evcohd(levl,levl) = evcohd(levl,levl) + taux
           fac = (elhd(levu) - elhd(levl)) / ttry
           ctod = exp(fac) * (2.0_dp*nj+1.0_dp) / (2.0_dp*nj+3.0_dp)
           taux = taux * ctod
           evcohd(levl,levu) = evcohd(levl,levu) - taux
           evcohd(levu,levu) = evcohd(levu,levu) + taux
         endif
       end do

       return
       end

