
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine BILTHM (iopt, itn, ifaf)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE STR_DATA
       USE PROFIL

       implicit double precision (a-h, o-z)

       real (kind=dp)                     :: t_lim_do
       real (kind=dp), parameter          :: t_lim_up = 1.0e6_dp

       save

       data eps / 0.9_dp /

c  Computes new temperature thru thermal balance
c  First 2 calls are used as initialisation

c   TB is current "best" estimate of T
c   TA and TC are lower and upper bounds at this iteration
c         (ie, with current chemistry)
c   TAP and TCP is a wider bounding with previous iterations

c   one has always TA < TB < TC   et TAP < TB < TCP

       t_lim_do = tcmb

       if (iopt > 0) then
         oldte = tgaz(iopt-1)
       else
         oldte = tg_init
       endif

       if (itn == 1) then

         ta = temp
         tb(0) = temp
         tc = temp

         call LADENS(temp, iopt)
         call FROID (temp, iopt, ifaf)
         call CHAUD (temp, iopt, itn)
         bthb = chotot(iopt) - reftot(iopt)

         btha = bthb
         bthc = bthb

         tcp = temp * 1.1_dp
         tap = max(temp * 0.9_dp, t_lim_do)

c  o w bound the solution ?

  10     continue
         if (bthc > 0.0_dp .and. tc < t_lim_up) then
           ta = tc
           btha = bthc
           tc = tc * 1.02_dp
           call LADENS(tc, iopt)
           call FROID (tc, iopt, ifaf)
           call CHAUD (tc, iopt, itn)
           bthc = chotot(iopt) - reftot(iopt)
           goto 10
         else if (btha < 0.0_dp .and. ta > t_lim_do) then
           tc = ta
           bthc = btha
           ta = max(ta * 0.98_dp, t_lim_do)
           call LADENS(ta, iopt)
           call FROID (ta, iopt, ifaf)
           call CHAUD (ta, iopt, itn)
           btha = chotot(iopt) - reftot(iopt)
           goto 10
         endif

         tb(1) = 0.5_dp * (ta + tc)

         temp = tb(1)

         call LADENS(temp, iopt)
         call FROID (temp, iopt, ifaf)
         call CHAUD (temp, iopt, itn)
         bthb = chotot(iopt) - reftot(iopt)
         energi = chotot(iopt) + reftot(iopt)
         bthrel = bthb/energi

         tap = min(tap,ta)
         tcp = max(tcp,tc)

         write(iscrn,*) 
         write(iscrn,*) '--------------- thermal balance --------------'
         write (iscrn,"(2x,5f10.2,2x,1p,3e12.3)")
     1          tap, ta, tb(itn), tc, tcp, btha, bthrel, bthc

       else if (itn >= 2 .and. itn <= 5) then

         call LADENS(ta, iopt)
         call FROID (ta, iopt, ifaf)
         call CHAUD (ta, iopt, itn)
         btha = chotot(iopt) - reftot(iopt)

         call LADENS(tc, iopt)
         call FROID (tc, iopt, ifaf)
         call CHAUD (tc, iopt, itn)
         bthc = chotot(iopt) - reftot(iopt)

         call LADENS(temp, iopt)
         call FROID (temp, iopt, ifaf)
         call CHAUD (temp, iopt, itn)
         bthb = chotot(iopt) - reftot(iopt)

c   update external bounds

         if (bthb > 0.0_dp) then
           tap = min(tap,temp)
         else
           tcp = max(tcp,temp)
         endif

  20     continue
         if (btha*bthc < 0.0_dp) then
           tb(itn) = ta - btha * (tc - ta) / (bthc - btha)
         else if (btha < 0.0_dp .and. ta > oldte * 0.7_dp
     1             .and. ta > t_lim_do) then
           ta = max(ta * 0.98_dp, t_lim_do)
           call LADENS(ta, iopt)
           call FROID (ta, iopt, ifaf)
           call CHAUD (ta, iopt, itn)
           btha = chotot(iopt) - reftot(iopt)
           goto 20
         else if (bthc > 0.0_dp .and. tc < oldte * 1.3_dp) then
           tc = tc * 1.02_dp
           call LADENS(tc, iopt)
           call FROID (tc, iopt, ifaf)
           call CHAUD (tc, iopt, itn)
           bthc = chotot(iopt) - reftot(iopt)
           goto 20
         else
           tb(itn) = 0.5_dp * (ta + tc)
         endif

         if (tb(itn) <= t_lim_up .and. tb(itn) >= t_lim_do) then
           continue
         else
           if (iopt > 0) then
             tb(itn) = tgaz(iopt-1)
           else
             tb(itn) = tg_init
           endif
         endif

         temp = tb(itn)

         tap = min(tap,ta)
         tcp = max(tcp,tc)

         call LADENS(temp, iopt)
         call FROID (temp, iopt, ifaf)
         call CHAUD (temp, iopt, itn)
         bthb = chotot(iopt) - reftot(iopt)
         energi = chotot(iopt) + reftot(iopt)
         bthrel = bthb/energi

         write(iscrn,*) 
         write(iscrn,*) '--------------- thermal balance --------------'
         write (iscrn,"(2x,5f10.2,2x,1p,3e12.3)")
     1          tap, ta, temp, tc, tcp, btha, bthrel, bthc

       else

c   general case

       call LADENS(ta, iopt)
       call FROID (ta, iopt, ifaf)
       call CHAUD (ta, iopt, itn)
       btha = chotot(iopt) - reftot(iopt)

       call LADENS(tc, iopt)
       call FROID (tc, iopt, ifaf)
       call CHAUD (tc, iopt, itn)
       bthc = chotot(iopt) - reftot(iopt)

       call LADENS(temp, iopt)
       call FROID (temp, iopt, ifaf)
       call CHAUD (temp, iopt, itn)
       bthb = chotot(iopt) - reftot(iopt)

c   Update external bounds

       if (bthb > 0.0_dp) then
         tap = max(tap,temp)
       else
         tcp = min(tcp, temp)
       endif

       if (btha*bthc < 0.0_dp) then

         if (bthb > 0.0_dp) then
           ta = temp
           btha = bthb
         else
           tc = temp
           bthc = bthb
         endif

         tb(itn) = ta - btha * (tc - ta) / (bthc - btha)

c  Various tries to accelerate convergence

         tb(itn) = tb(itn-1) * (1.0_dp - eps) + eps * tb(itn)

         if (tb(itn) > tb(itn-1)) then
           if ((tb(itn-2) > tb(itn))
     1         .and. (tb(itn-3) < tb(itn-1))) then
               tb(itn) = 0.5_dp * (tb(itn) + tb(itn-1))
           endif
         else
           if ((tb(itn-2) < tb(itn))
     1         .and. (tb(itn-3) > tb(itn-1))) then
               tb(itn) = 0.5_dp * (tb(itn) + tb(itn-1))
           endif
         endif

         if ((tb(itn) < tb(itn-1))
     1       .and. (tb(itn-1) < tb(itn-2))
     2       .and. (tb(itn-2) < tb(itn-3))) then
           tb(itn) = 0.5_dp * (tb(itn) + tap)
         endif

         if (tb(itn) <= t_lim_up .and. tb(itn) >= t_lim_do) then
           continue
         else
           tb(itn) = tgaz(iopt-1)
         endif

         temp = tb(itn)

         if (ta > temp) then
           ta = temp
         endif
         if (tc < temp) then
           tc = temp
         endif

         call LADENS(ta, iopt)
         call FROID (ta, iopt, ifaf)
         call CHAUD (ta, iopt, itn)
         btha = chotot(iopt) - reftot(iopt)

         call LADENS(tc, iopt)
         call FROID (tc, iopt, ifaf)
         call CHAUD (tc, iopt, itn)
         bthc = chotot(iopt) - reftot(iopt)

         call LADENS(temp, iopt)
         call FROID (temp, iopt, ifaf)
         call CHAUD (temp, iopt, itn)
         bthb = chotot(iopt) - reftot(iopt)
         energi = chotot(iopt) + reftot(iopt)
         bthrel = bthb/energi

       else

         tb(itn) = 0.5_dp * (tap + tcp)

         if (tb(itn) > tb(itn-1)) then
           if ((tb(itn-2) > tb(itn))
     1         .and. (tb(itn-3) < tb(itn-1))) then
               tb(itn) = 0.5_dp * (tb(itn) + tb(itn-1))
           endif
         else
           if ((tb(itn-2) < tb(itn))
     1         .and. (tb(itn-3) > tb(itn-1))) then
               tb(itn) = 0.5_dp * (tb(itn) + tb(itn-1))
           endif
         endif

         if ((tb(itn) < tb(itn-1))
     1       .and. (tb(itn-1) < tb(itn-2))
     2       .and. (tb(itn-2) < tb(itn-3))) then
           tb(itn) = 0.5_dp * (tb(itn) + tap)
         endif

         if (tb(itn) <= t_lim_up .and. tb(itn) >= t_lim_do) then
           continue
         else
           if (iopt > 0) then
             tb(itn) = tgaz(iopt-1)
           else
             tb(itn) = tg_init
           endif
         endif

         temp = tb(itn)

         if (ta > temp) then
           ta = temp
         endif
         if (tc < temp) then
           tc = temp
         endif

         ta = max(ta, tap)
         tc = min(tc, tcp)

         call LADENS(ta, iopt)
         call FROID (ta, iopt, ifaf)
         call CHAUD (ta, iopt, itn)
         btha = chotot(iopt) - reftot(iopt)

         call LADENS(tc, iopt)
         call FROID (tc, iopt, ifaf)
         call CHAUD (tc, iopt, itn)
         bthc = chotot(iopt) - reftot(iopt)

         call LADENS(temp, iopt)
         call FROID (temp, iopt, ifaf)
         call CHAUD (temp, iopt, itn)
         bthb = chotot(iopt) - reftot(iopt)
         energi = chotot(iopt) + reftot(iopt)
         bthrel = bthb/energi

         if ((btha*bthc > 0.0_dp)
     1        .and. (abs(tcp - tap) < 0.01_dp)) then
           if (bthb > 0.0_dp) then
             tcp = tcp * 1.1_dp
           else
             tap = tap * 0.9_dp
           endif
         endif

       endif

       write(iscrn,*) 
       write(iscrn,*) '--------------- thermal balance --------------'
       write (iscrn,"(2x,5f10.2,2x,1p,3e12.3)")
     1        tap, ta, tb(itn), tc, tcp, btha, bthrel, bthc

       endif

       return
       end

c-------------------------------------------------------------------

       subroutine LADENS(tabc, iopt)

       use constantes
       use auxiliaries
       use SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

c   *** correction for pressure, 9 Aout 92: and 21 janv 94

       implicit double precision (a-h, o-z)

       save

       if (ifisob /= 2) then
         return
       endif

       hsurh2 = ab(i_h) / ab(i_h2)
       divi = (1.0_dp + hsurh2) / (2.0_dp + hsurh2)
       xdens = (presse / tabc) / (abtot(5) + divi)

       do ii=1,nspec
         ab(ii) = ab(ii) * xdens / densh(iopt)
       end do
       densh(iopt) = xdens

       return
       end
