
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine FROID (ttry, iopt, ifaf)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE RF_TOOLS
       USE SP_DATA
       USE H2_DATA
       USE HD_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

c  Excitation of various species
c  and cooling induced
       
       IMPLICIT NONE
c       implicit double precision (a-h,o-z)
       INTEGER, INTENT(IN) :: iopt           ! position index in the cloud
       INTEGER, INTENT(IN) :: ifaf           ! number of iteration
       REAL (KIND=dp), INTENT(IN) :: ttry    ! Temperature for try
       
       REAL (KIND=dp) :: tempo1
       REAL (KIND=dp) :: tempo2
       REAL (KIND=dp) :: enerhi
       REAL (KIND=dp) :: raptau
       REAL (KIND=dp) :: raptin
       REAL (KIND=dp) :: fabco
       REAL (KIND=dp) :: desco

       INTEGER :: lev1, lev2
       INTEGER :: nju
       INTEGER :: j, k
       INTEGER :: icd_o, jcd_o
       INTEGER :: i_spec
       INTEGER :: idsp
       
       SAVE

c     *********************************************************************
c     ct2= 2 k / amu   = 1.662902e8  in cgs
c     bdop:  velocity cm s-1
c     hcsurk = h c / k
c     ******************************************************************

c  FROID must be called before CHAUD for initialisation

       reftot(iopt) = 0.0_dp
       chotot(iopt) = 0.0_dp

c  I - Cooling by H2
c  ------------------------

c   WARNING: evrdbc and evlqua are NEGATIVES

       rfjh2t(iopt) = 0.0_dp

       call EQLH2V (ttry, iopt)

       tempo1 = 0.0_dp
       do lev2=1,l0h2
         nju = njlev(lev2)
         tempo2 = 0.0_dp
         enerhi = elhm(lev2)
         do lev1=1,l0h2
           tempo2 = tempo2 + (enerhi - elhm(lev1))
     1       * (evlqua(lev1,lev2) + evrdbc(lev1,lev2))
         end do
         tempo1 = tempo1 + tempo2 * xreh2(lev2,iopt)
       end do

       rfjh2t(iopt) = - tempo1 * ab(i_h2) * xk

c  I bis - Cooling by HD
c  ------------------------
 
       rfjhdt(iopt) = 0.0_dp
 
       if (i_hd /= 0) then

         call EQLHDV (ttry, iopt)
 
         tempo1 = 0.0_dp
         do lev2=1,l0hd
           nju = njhd(lev2)
           if (nju > mjhd) cycle
           tempo2 = 0.0_dp
           enerhi = elhd(lev2)
           do lev1=1,l0hd
             tempo2 = tempo2 + (enerhi - elhd(lev1))
     1         * (evdihd(lev1,lev2) + evrdhd(lev1,lev2))
           end do
           tempo1 = tempo1 + tempo2 * xrehd(lev2,iopt)
         end do
 
         rfjhdt(iopt) = - tempo1 * ab(i_hd) * xk
 
       endif

c  Interpolation in previous grid
 
       do j=1,npo
         if (tau(iopt) < tau_o(j)) then
           icd_o = j-1
           exit
         endif
       end do
 
       raptau = (tau(iopt) - tau_o(icd_o))
     1        / (tau_o(icd_o+1) - tau_o(icd_o))

       do j=1,npo
         if (tau_o(j) == tau_imp(n_imp-1)) then
           jcd_o = j
           exit
         endif
       end do

       raptin = (max(2.0_dp*tau_o(npo),100.0_dp) - tau_o(jcd_o))
     1        / (tau_o(npo) - tau_o(jcd_o))

c  Cooling by CO
c  ----------------------

       call prepar (iopt, ifaf, i_co0, idco0p, ntrco0, nlvco0
     1            , amco0, opdco0_o, jcd_o, raptin, raptau
     2            , aijco0, wlkco0, iupco0, jloco0, opdco0, emico0
     3            , gstco0, elkco0, xreco0, ttry, refco0(iopt)
     4            , fabco, desco, icd_o)

c  Cooling by C*O
c  --------------

       if (i_co1 == 0) then
         i_spec = 1001
       else
         i_spec = i_co1
       endif

       call prepar (iopt, ifaf, i_spec, idco1p, ntrco0, nlvco0
     1            , amco1, opdco1_o, jcd_o, raptin, raptau
     2            , aijco1, wlkco1, iupco1, jloco1, opdco1, emico1
     3            , gstco1, elkco1, xreco1, ttry, refco1(iopt)
     4            , fabco, desco, icd_o)

c  Cooling by CO*
c  --------------

       if (i_co2 == 0) then
         i_spec = 1002
       else
         i_spec = i_co2
       endif

       call prepar (iopt, ifaf, i_spec, idco2p, ntrco0, nlvco0
     1            , amco2, opdco2_o, jcd_o, raptin, raptau
     2            , aijco2, wlkco2, iupco2, jloco2, opdco2, emico2
     3            , gstco2, elkco2, xreco2, ttry, refco2(iopt)
     4            , fabco, desco, icd_o)

c  Cooling by C*O*
c  ---------------

       if (i_co3 == 0) then
         i_spec = 1003
       else
         i_spec = i_co3
       endif

       call prepar (iopt, ifaf, i_spec, idco3p, ntrco0, nlvco0
     1            , amco3, opdco3_o, jcd_o, raptin, raptau
     2            , aijco3, wlkco3, iupco3, jloco3, opdco3, emico3
     3            , gstco3, elkco3, xreco3, ttry, refco3(iopt)
     4            , fabco, desco, icd_o)

c  Cooling by C
c  ------------

       idsp = 0
       call prepar (iopt, ifaf, i_c, idsp, ntrcat, nlvcat
     1            , amcat, opdcat_o, jcd_o, raptin, raptau
     2            , aijcat, wlkcat, iupcat, jlocat, opdcat, emicat
     3            , gstcat, elkcat, xrecat, ttry, refcat(iopt)
     4            , fabco, desco, icd_o)

c  Cooling by N
c  ------------

        if (i_n /= 0) then
       idsp = 0
       call prepar (iopt, ifaf, i_n, idsp, ntrnat, nlvnat
     1            , amnat, opdnat_o, jcd_o, raptin, raptau
     2            , aijnat, wlknat, iupnat, jlonat, opdnat, eminat
     3            , gstnat, elknat, xrenat, ttry, refnat(iopt)
     4            , fabco, desco, icd_o)

        endif

c  Cooling by O
c  ----------------

       idsp = 0
       call prepar (iopt, ifaf, i_o, idsp, ntroat, nlvoat
     1            , amoat, opdoat_o, jcd_o, raptin, raptau
     2            , aijoat, wlkoat, iupoat, jlooat, opdoat, emioat
     3            , gstoat, elkoat, xreoat, ttry, refoat(iopt)
     4            , fabco, desco, icd_o)

c  Cooling by S
c  ----------------

        if (i_s /= 0) then
       idsp = 0
       call prepar (iopt, ifaf, i_s, idsp, ntrsat, nlvsat
     1            , amsat, opdsat_o, jcd_o, raptin, raptau
     2            , aijsat, wlksat, iupsat, jlosat, opdsat, emisat
     3            , gstsat, elksat, xresat, ttry, refsat(iopt)
     4            , fabco, desco, icd_o)

        endif

c  Cooling by Si
c  ----------------

        if (i_si /= 0) then
       idsp = 0
       call prepar (iopt, ifaf, i_si, idsp, ntrsiat, nlvsiat
     1            , amsiat, opdsiat_o, jcd_o, raptin, raptau
     2            , aijsiat, wlksiat, iupsiat, jlosiat, opdsiat, emisiat
     3            , gstsiat, elksiat, xresiat, ttry, refsiat(iopt)
     4            , fabco, desco, icd_o)

        endif

c  Cooling by C+
c  ----------------

       idsp = 0
       call prepar (iopt, ifaf, i_cp, idsp, ntrcpl, nlvcpl
     1            , amcpl, opdcpl_o, jcd_o, raptin, raptau
     2            , aijcpl, wlkcpl, iupcpl, jlocpl, opdcpl, emicpl
     3            , gstcpl, elkcpl, xrecpl, ttry, refcpl(iopt)
     4            , fabco, desco, icd_o)

c  Cooling by N+
c  ----------------

        if (i_np /= 0) then
       idsp = 0
       call prepar (iopt, ifaf, i_np, idsp, ntrnpl, nlvnpl
     1            , amnpl, opdnpl_o, jcd_o, raptin, raptau
     2            , aijnpl, wlknpl, iupnpl, jlonpl, opdnpl, eminpl
     3            , gstnpl, elknpl, xrenpl, ttry, refnpl(iopt)
     4            , fabco, desco, icd_o)

        endif

c  Cooling by O+
c  ----------------

       idsp = 0
       call prepar (iopt, ifaf, i_op, idsp, ntropl, nlvopl
     1            , amopl, opdopl_o, jcd_o, raptin, raptau
     2            , aijopl, wlkopl, iupopl, jloopl, opdopl, emiopl
     3            , gstopl, elkopl, xreopl, ttry, refopl(iopt)
     4            , fabco, desco, icd_o)

c  Cooling by S+
c  ----------------

        if (i_sp /= 0) then

       idsp = 0
       call prepar (iopt, ifaf, i_sp, idsp, ntrspl, nlvspl
     1            , amspl, opdspl_o, jcd_o, raptin, raptau
     2            , aijspl, wlkspl, iupspl, jlospl, opdspl, emispl
     3            , gstspl, elkspl, xrespl, ttry, refspl(iopt)
     4            , fabco, desco, icd_o)

        endif

c  Cooling by Si+
c  -----------------

        if (i_sip /= 0) then

       idsp = 0
       call prepar (iopt, ifaf, i_sip, idsp, ntrsip, nlvsip
     1            , amsip, opdsip_o, jcd_o, raptin, raptau
     2            , aijsip, wlksip, iupsip, jlosip, opdsip, emisip
     3            , gstsip, elksip, xresip, ttry, refsip(iopt)
     4            , fabco, desco, icd_o)

        endif

c  Cooling by OH (empirical fits)
c  ------------------------------
c  From Guillaume

       refoh(iopt) = 1.0d-26 * ttry * exp(-120.0_dp / ttry)
     1             * ab(i_oh) * (10.0_dp * ab(i_h)
     2             + ab(i_h2) / sqrt(2.0_dp))

c  Cooling by Fe+

       reffep(iopt) = 1.1d-22 * (exp(-554.0_dp/ttry) + 1.4_dp
     1         * exp(-961.0_dp/ttry)) * ab(i_fep) * (ab(i_h) + ab(i_h2))
c  Add electrons contribution (Dalgarno et Mc Cray)
       reffep(iopt) = reffep(iopt)
     1              + 1.1d-18 * ab(i_fep) * (exp(-554._dp/ttry)
     2              + 1.3_dp * exp(-961._dp/ttry))
     3              * ab(nspec) / sqrt(ttry)

c  Cooling by CS
c  -------------

        if (i_cs /= 0) then

       idsp = 0
!        PRINT *,"CS population a iopt = ",iopt
!        PRINT *,"Avant prepar : "
!        PRINT *,"E    : ", (elkcs0(k), k=1, nlvcs0)
!        PRINT *,"g    : ", (gstcs0(k), k=1, nlvcs0)
!        PRINT *,"Aij  : ", (aijcs0(k), k=1, ntrcs0)
!        PRINT *,"Wlk  : ", (wlkcs0(k), k=1, ntrcs0)
!        PRINT *,"iUp  : ", (iupcs0(k), k=1, ntrcs0)
!        PRINT *,"jLo  : ", (jlocs0(k), k=1, ntrcs0)
!        PRINT *,"Po   : ", (opdcs0(k,iopt), k=1, ntrcs0)
!        PRINT *,"Po_o : ", (opdcs0_o(k,iopt), k=1, ntrcs0)
!        PRINT *,"Emi  : ", (emics0(k,iopt), k=1, ntrcs0)
       call prepar (iopt, ifaf, i_cs, idsp, ntrcs0, nlvcs0
     1            , amcs0, opdcs0_o, jcd_o, raptin, raptau
     2            , aijcs0, wlkcs0, iupcs0, jlocs0, opdcs0, emics0
     3            , gstcs0, elkcs0, xrecs0, ttry, refcs0(iopt)
     4            , fabco, desco, icd_o)

        endif

c  Cooling by HCO+
c  ------------------------
 
        if (i_hcop /= 0) then
 
       idsp = 0
       call prepar (iopt, ifaf, i_hcop, idsp, ntrhcop, nlvhcop
     1            , amhcop, opdhcop_o, jcd_o, raptin, raptau
     2            , aijhcop, wlkhcop, iuphcop, jlohcop, opdhcop, emihcop
     3            , gsthcop, elkhcop, xrehcop, ttry, refhcop(iopt)
     4            , fabco, desco, icd_o)

        endif

c  Cooling by H2O
c  --------------

        if (i_h2o /= 0) then

       idsp = 0
       call prepar (iopt, ifaf, i_h2o, idsp, ntrh2o, nlvh2o
     1            , amh2o, opdh2o_o, jcd_o, raptin, raptau
     2            , aijh2o, wlkh2o, iuph2o, jloh2o, opdh2o, emih2o
     3            , gsth2o, elkh2o, xreh2o, ttry, refh2o(iopt)
     4            , fabco, desco, icd_o)

        endif
        
c  Cooling by H3+
c  ---------------

        if (i_h3p /= 0) then

        idsp = 0
!        PRINT *,"H3 population a iopt = ",iopt
!        PRINT *,"Avant prepar : "
!        PRINT *,"E    : ", (elkh3p(k), k=1, nlvh3p)
!        PRINT *,"g    : ", (gsth3p(k), k=1, nlvh3p)
!        PRINT *,"Aij  : ", (aijh3p(k), k=1, ntrh3p)
!        PRINT *,"Wlk  : ", (wlkh3p(k), k=1, ntrh3p)
!        PRINT *,"iUp  : ", (iuph3p(k), k=1, ntrh3p)
!        PRINT *,"jLo  : ", (jloh3p(k), k=1, ntrh3p)
!        PRINT *,"Po   : ", (opdh3p(k,iopt), k=1, ntrh3p)
!        PRINT *,"Po_o : ", (opdh3p_o(k,iopt), k=1, ntrh3p)
!        PRINT *,"Emi  : ", (emih3p(k,iopt), k=1, ntrh3p)        
!        PRINT *,"x : ",(xreh3p(k,iopt), k=1,nlvh3p)
        

        call prepar (iopt, ifaf, i_h3p, idsp, ntrh3p, nlvh3p
     1             , amh3p, opdh3p_o, jcd_o, raptin, raptau
     2             , aijh3p, wlkh3p, iuph3p, jloh3p, opdh3p, emih3p
     3             , gsth3p, elkh3p, xreh3p, ttry, refh3p(iopt)
     4             , fabco, desco, icd_o)
!        PRINT *,"Apres prepar : " 
!        PRINT *,"x : ",(xreh3p(k,iopt), k=1,nlvh3p)
!        PAUSE
        endif
    

c  Sum everything

       reftot(iopt) = refcat(iopt) + refnat(iopt) + refoat(iopt)
     1              + refsat(iopt) + refsiat(iopt) + refcpl(iopt)
     2              + refnpl(iopt) + refopl(iopt) + refspl(iopt)
     3              + refsip(iopt) + refh2o(iopt) + refoh(iopt)
     4              + reffep(iopt) + refco1(iopt) + refco2(iopt)
     5              + refco3(iopt) + refco0(iopt) + refcs0(iopt)
     6              + refhcop(iopt)

       if (rfjh2t(iopt) >= 0.0_dp ) then
         reftot(iopt) = reftot(iopt) + rfjh2t(iopt)
       else
         chotot(iopt) = chotot(iopt) - rfjh2t(iopt)
       endif

       if (rfjhdt(iopt) >= 0.0_dp ) then
         reftot(iopt) = reftot(iopt) + rfjhdt(iopt)
       else
         chotot(iopt) = chotot(iopt) - rfjhdt(iopt)
       endif

       return
       end

c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine DETBAL (iopt, abici, fabchi, deschi, ttry
     1           , bet_ij, bep_ij, bb_int, bb_ext, on, refr
     2           , xrepre, xreici, ntr, nlv, eqrspe
     3           , aij, wlk, iup, jlo, elk, gst)
c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        use RF_TOOLS
        use STR_DATA
        use PROFIL

        implicit double precision (a-h,o-z)

        integer                                      :: ntr, nlv
        real (KIND=dp), dimension (ntr)              :: bet_ij
        real (KIND=dp), dimension (ntr)              :: bep_ij
        real (KIND=dp), dimension (ntr)              :: bb_int
        real (KIND=dp), dimension (ntr)              :: bb_ext
        real (KIND=dp), dimension (:,:), allocatable :: eqmat
        real (KIND=dp), dimension (:,:), allocatable :: bbab
        integer       , dimension (:),   allocatable :: indx
        REAL (KIND=dp), dimension (ntr), intent (in) :: aij, wlk
        REAL (KIND=dp), dimension (nlv), intent (in) :: gst, elk
        REAL (KIND=dp), dimension (nlv), intent (in) :: eqrspe
        INTEGER, dimension (ntr), intent (in)        :: iup, jlo

        real (KIND=dp), dimension (:), allocatable   :: bfact
        real (KIND=dp)            :: rien = 1.0e-50_dp
        real (KIND=dp)            :: refr

        real (KIND=dp)            :: fct = 46.0_dp
        real (KIND=dp)            :: on
        integer                   :: nrhs = 1
        CHARACTER (LEN=1)                            :: FACT  = 'E'
        CHARACTER (LEN=1)                            :: TRANS = 'N'
        CHARACTER (LEN=1)                            :: EQUED
        real (KIND=dp), dimension (:,:), allocatable :: AF
        REAL (kind=dp), dimension(:),    allocatable :: R, C
        real (KIND=dp), dimension (:,:), allocatable :: bbax
        REAL (kind=dp), dimension(:),    allocatable :: FERR, BERR
        REAL (kind=dp)                               :: RCOND
        REAL (kind=dp), dimension(:),    allocatable :: WORK
        INTEGER,        dimension(:),    allocatable :: IWORK
        REAL (kind=dp), DIMENSION (nlv) :: xrepre, xreici


c  Computes detailed balance and line transfer (approx)
c  From de Jong, Dalgarno & Boland, A&A 91, 68 (1980)

        allocate (eqmat(nlv,nlv))
        allocate (indx(nlv))
        allocate (bbab(nlv,1))
        allocate (bbax(nlv,1))
        allocate (bfact(nlv))

        allocate (AF(nlv,nlv))
        allocate (R(nlv), C(nlv))
        allocate (FERR(1), BERR(1))
        allocate (WORK(4*nlv))
        allocate (IWORK(nlv))

c  18 V 2006
c  Equations are in fact LINEAR !!!

        sumetl = 0.0_dp
        do ju=1,nlv
          bfact(ju) = gst(ju) * exp(-elk(ju) / ttry)
          sumetl = sumetl + bfact(ju)
        end do

        eqmat = 0.0_dp
        bbab = 0.0_dp

        do i=1,ntr
          ju = iup(i)
          jl = jlo(i)
          if (ju > nlv) cycle
          gusgl = gst(ju) / gst(jl)

          if (wlk(i) < fct*tdust(npg,iopt)) then
            bb_int(i) = on / (exp(wlk(i) / tdust(npg,iopt)) - 1.0_dp)
          else
            bb_int(i) = 0.0_dp
          endif
          if (wlk(i) < fct*tcmb) then
            bb_ext(i) = 1.0_dp / (exp(wlk(i) / tcmb) - 1.0_dp)
          else
            bb_ext(i) = 0.0_dp
          endif

c         yy_ij = bep_ij(i) * bb_int(i)
c    1          + bet_ij(i) * bb_ext(i)

c 7 VIII 2006 - Keep Tielens (and Spaans) approximation for now

          yy_ij = bet_ij(i) * bb_int(i)
     1          + bet_ij(i) * bb_ext(i)
          xx_ij = bet_ij(i) + yy_ij
          xx_ij = xx_ij * aij(i)
          yy_ij = yy_ij * aij(i) * gusgl

          eqmat(ju,ju) = eqmat(ju,ju) + xx_ij
          eqmat(ju,jl) = eqmat(ju,jl) - yy_ij
          eqmat(jl,jl) = eqmat(jl,jl) + yy_ij
          eqmat(jl,ju) = eqmat(jl,ju) - xx_ij

        end do

        do ju=1,nlv
          do jl=1,nlv
            eqmat(ju,jl) = (eqmat(ju,jl) + eqcol(ju,jl)) * abici
          end do
          eqmat(ju,ju) = eqmat(ju,ju) - deschi
          eqmat(ju,ju) = eqmat(ju,ju) + eqrspe(ju) * abici
          bbab(ju,1) = fabchi * max(rien, bfact(ju) / sumetl)
        end do
!  Test 12 VII 2006 - JLB
!       bbab(1,1) = fabchi

c  use LAPACK

        call dgesvx (FACT, TRANS, nlv, nrhs, eqmat, nlv, AF, nlv, 
     1       indx, EQUED, R, C, bbab, nlv, bbax, nlv, RCOND,
     2       FERR, BERR, WORK, IWORK, info)

        dumy = 0.0_dp
        do ju=1,nlv
          if (bbax(ju,1) <= 0.0_dp) then
            bbax(ju,1) = rien
          endif
          dumy = dumy + bbax(ju,1)
        end do

        do ju=1,nlv
          bbax(ju,1) = bbax(ju,1) / dumy
        end do

        dumy = abs(dumy - 1.0_dp)
c       print *, dumy

        do ju=1,nlv
          xreici(ju) = bbax(ju,1)
        end do

c  Remember that off-diagonal elements of eqcol are negative!

        refr = 0.0_dp
        do ju=1,nlv
          do jl=1,nlv
            refr = refr - eqcol(ju,jl) * bbax(jl,1)
     1           * xk * (elk(ju) - elk(jl))
          end do
        end do
        refr = refr * abici

        deallocate (eqmat)
        deallocate (indx)
        deallocate (bbab)
        deallocate (bbax)
        deallocate (bfact)

        deallocate (AF)
        deallocate (R, C)
        deallocate (FERR, BERR)
        deallocate (WORK)
        deallocate (IWORK)

        return
        end subroutine DETBAL

c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine EMILINE (n, i_spec, velici, velpre, abpre, abici
     1                     , bet_ij, bep_ij, bb_int, bb_ext
     2                     , emi, opd , xrepre, xreici, ntr
     3                     , nlv , aij, wlk, iup, jlo, gst)
c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use auxiliaries
        use RF_TOOLS
        use STR_DATA
        use SP_DATA, only : speci
        use PROFIL

        implicit double precision (a-h,o-z)

        integer, intent(in)                            :: ntr, nlv
        integer, intent(in)                            :: i_spec
        real (KIND=dp), dimension (ntr), intent(in)    :: bet_ij
        real (KIND=dp), dimension (ntr), intent(in)    :: bep_ij
        real (KIND=dp), dimension (ntr), intent(in)    :: bb_int
        real (KIND=dp), dimension (ntr), intent(in)    :: bb_ext
        REAL (kind=dp), DIMENSION (nlv), intent(in)    :: xrepre, xreici
        REAL (KIND=dp), dimension (ntr), intent(inout) :: emi, opd
        REAL (KIND=dp), dimension (ntr), intent (in)   :: aij, wlk
        REAL (KIND=dp), dimension (nlv), intent (in)   :: gst
        INTEGER, dimension (ntr), intent (in)          :: iup, jlo

        real (KIND=dp)                 :: nhpre, nhici


        if (n > 0) then
          m = n - 1
        else
          m = n
        endif
        deltau = tau(n) - tau(m)
        taile = (cdunit * tautav / rrr) * 0.5_dp * deltau
        nhpre = densh(m)
        nhici = densh(n)

        emi(1:ntr) = 0.0_dp
        do i=1,ntr
          ju = iup(i)
          jl = jlo(i)
          if (ju > nlv) cycle
          gusgl = gst(ju) / gst(jl)
          waveln = hcsurk / wlk(i)
          aux1 = abpre * (xrepre(jl) * gusgl - xrepre(ju))
     1         / (nhpre * velpre)
          aux2 = abici * (xreici(jl) * gusgl - xreici(ju))
     1         / (nhici * velici)
          delopd = (aij(i) * waveln**3 / (8.0_dp * xpi)) * taile / sqpi
     1           * (aux1 + aux2)
          opd(i) = opd(i) + delopd
          souici = (xreici(jl) * gusgl / xreici(ju)) - 1.0_dp
          if (souici == 0.0_dp) then
            souici = 1.0e-10_dp
            print *, speci(i_spec), "ntr : ", i,"Ju : ", ju, 
     1               "Jl : ", jl," Population inversion!"
          endif
          f_emis = 1.0_dp + bet_ij(i) * bb_ext(i)
     1           + bep_ij(i) * bb_int(i) + (1.0_dp-bet_ij(i)) / souici

          emi(i) = xk * wlk(i) * abici * xreici(ju)
     1           * bet_ij(i) * aij(i) * f_emis
        end do

        return
        end subroutine EMILINE

c       %%%%%%%%%%%%%%%%%%%
        function escap(opd)
c       %%%%%%%%%%%%%%%%%%%

          use constantes
          implicit none
          real (kind=dp) :: opd
          real (kind=dp) :: escap
          real (kind=dp) :: bbetar

          if (opd < 0.0_dp) then
            bbetar = 0.5_dp
          else if (opd < 1.0e-2_dp) then
            bbetar = (1.0_dp - 1.17_dp * opd
     1             + 0.9126_dp * opd * opd) * 0.5_dp
          else if (opd < 7.0_dp) then
            bbetar = (1.0_dp - exp(-2.34_dp * opd))
     1             / (4.68_dp * opd)
          else
            bbetar = 1.0_dp / (4.0_dp * opd
     1             * sqrt(log(opd/sqpi)))
          endif

          escap = bbetar

        end function escap

c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine prepar(iopt, ifaf, i_spec, idsp, ntr, nlv
     1                  , am, opd_o, jcd_o, raptin, raptau
     2                  , aij, wlk, iup, jlo, opdspe, emispe
     3                  , gst, elk, xre, ttry, ref
     4                  , fabco, desco, icd_o)
c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE CHEM_DATA
       USE RF_TOOLS
       USE SP_DATA
       USE STR_DATA
       USE PROFIL
       implicit none

       REAL (KIND=dp), intent (in)    :: am, raptin, raptau
       REAL (KIND=dp), intent (in)    :: ttry
       REAL (KIND=dp), intent (out)   :: ref
       REAL (KIND=dp), intent (inout) :: fabco, desco
       INTEGER, intent (in)           :: i_spec, idsp
       INTEGER, intent (in)           :: ntr, nlv
       INTEGER, intent (in)           :: icd_o, jcd_o, ifaf
       REAL (KIND=dp), dimension (ntr,0:npo), intent (in) :: opd_o
       REAL (KIND=dp), dimension (ntr), intent (in)            :: aij
       REAL (KIND=dp), dimension (ntr), intent (in)            :: wlk
       REAL (KIND=dp), dimension (nlv), intent (in)            :: gst
       REAL (KIND=dp), dimension (ntr), intent (in)            :: elk
       REAL (KIND=dp), dimension (ntr,0:noptm), intent (inout) :: opdspe
       REAL (KIND=dp), dimension (ntr,0:noptm), intent (inout) :: emispe
       REAL (KIND=dp), dimension (nlv,0:noptm), intent (inout) :: xre
       INTEGER, dimension (ntr), intent (in)              :: iup, jlo

       REAL (KIND=dp), dimension (ntr) :: emi, opd_t
       INTEGER        :: iopt, jopt
       REAL (KIND=dp) :: vt2, ct2
       REAL (KIND=dp) :: velpre, velici
       INTEGER        :: nrxi, ind, k, i
       INTEGER        :: k1, k2, k3, ju
       REAL (KIND=dp) :: oopd, oopdr
       REAL (KIND=dp) :: escap, tt, on
       REAL (kind=dp), DIMENSION (nlv) :: xrepre, xreici
       REAL (kind=dp) :: abpre, abici
       REAL (kind=dp) :: sumemi, opdin
       REAL (KIND=dp) :: interpcd
       REAL (KIND=dp) :: fabchi, deschi, taux

       real (KIND=dp), dimension (:), allocatable   :: bet_ij
       real (KIND=dp), dimension (:), allocatable   :: bep_ij
       real (KIND=dp), dimension (:), allocatable   :: bb_int
       real (KIND=dp), dimension (:), allocatable   :: bb_ext
       REAL (KIND=dp), dimension (:), allocatable   :: eqrspe

c  Mean dust absorption, from Draine's standard galactic model
c  valid only for T < 10**2.5
c  k_abs = 10**(a * t*t + b * t + c)
c        with t = log10(T)

       REAL (KIND=dp) :: aa_k = 0.194601_dp
       REAL (KIND=dp) :: bb_k = 1.3307_dp
       REAL (KIND=dp) :: cc_k = -28.0681_dp
       REAL (KIND=dp) :: tau_D ! Dust total optical depth
       REAL (KIND=dp) :: b_integ = 0.247574639011584_dp
       REAL (KIND=dp) :: dnud, toto

       if (iopt > 0) then
         jopt = iopt - 1
       else
         jopt = 0
       endif

       vt2 = vturb * vturb
       ct2 = 2.0_dp * xk / amu
       velpre = sqrt(vt2 + ct2 * tgaz(jopt) / am)
       velici = sqrt(vt2 + ct2 * tgaz(iopt) / am)

       fabchi = 0.0_dp
       deschi = 0.0_dp

       if (i_spec < 1000) then
         nrxi = ikmat(i_spec,0)
         do k=1,nrxi
           ind = ikmat(i_spec,k)
           if (ind == idsp) cycle   ! Do not count twice Photodissociation
           k1 = react(ind,1)
           k2 = react(ind,2)
           k3 = react(ind,3)
           taux = xkmat(i_spec,k)
           if (taux > 0.0_dp) then
             fabchi = fabchi + taux * ab(k1) * ab(k2) * ab(k3)
           else
             deschi = deschi + taux * ab(k1) * ab(k2) * ab(k3)
           endif
         end do
       else if (i_spec == 1001) then
         fabchi = fabco / 90.0_dp
         deschi = desco / 90.0_dp
       else if (i_spec == 1002) then
         fabchi = fabco / 499.0_dp
         deschi = desco / 499.0_dp
       else if (i_spec == 1003) then
         fabchi = fabco / (90.0_dp * 499.0_dp)
         deschi = desco / (90.0_dp * 499.0_dp)
       endif

       allocate (eqrspe(nlv))
       if (i_spec == i_co0) then
         timco1(iopt) = fabchi
         timco2(iopt) = - deschi
         timco3(iopt) = pdico0(iopt) * ab(i_spec)
       endif
       if (i_spec == i_co0 .OR. i_spec > 1000) then
         eqrspe = eqrco0
       else if (i_spec == i_co1) then
         eqrspe = eqrco1
       else if (i_spec == i_co2) then
         eqrspe = eqrco2
       else if (i_spec == i_co3) then
         eqrspe = eqrco3
       else
         eqrspe = 0.0_dp
       endif

       allocate (bet_ij(ntr))
       allocate (bep_ij(ntr))
       allocate (bb_int(ntr))
       allocate (bb_ext(ntr))

      do i=1,ntr
         if (isoneside == 0) then
           opdin = 0.0_dp
         else
           opdin = (opd_o(i,npo) - opd_o(i,jcd_o)) * raptin
         endif
         if (ifaf < 3) then
           oopd = opdspe(i,jopt)
           oopdr = opd_o(i,npo) + opdin
     1             - interpcd(raptau, opd_o(i,0:npo), icd_o, npo)
         else
           oopd = interpcd(raptau, opd_o(i,0:npo), icd_o, npo)
           oopdr = opd_o(i,npo) + opdin - oopd
         endif
         bet_ij(i) = escap(oopd) + escap(oopdr)
         tt = min(log10(wlk(i)), 2.5_dp)
         tau_D = NHtot * 10.0_dp**(aa_k * tt*tt + bb_k * tt + cc_k)
         dnud = (xk * wlk(i) / xhp) * (velici / clum)
         bep_ij(i) = b_integ + 0.5_dp
         toto = 0.0_dp
         if (oopd > 0.0_dp) then
           toto = toto + oopd * log(oopd/dnud)
         endif
         if (oopdr > 0.0_dp) then
           toto = toto + oopdr * log(oopdr/dnud)
         endif
         bep_ij(i) = bep_ij(i) - 0.5_dp * toto / (oopd + oopdr)
         bep_ij(i) = bep_ij(i) * tau_D
      end do

       do ju=1,nlv
         xrepre(ju) = xre(ju,jopt)
         xreici(ju) = xre(ju,jopt)
       end do

       ALLOCATE (eqcol(nlv,nlv))
       eqcol(:,:) = 0.0_dp
       call COLSPE (ttry, i_spec)

       if (i_spec < 1000) then
         abpre = abnua(i_spec,jopt)
         abici = abnua(i_spec,iopt)
       else if (i_spec == 1001) then
         abpre = abnua(i_co0,jopt) / 90.0_dp
         abici = abnua(i_co0,iopt) / 90.0_dp
       else  if (i_spec == 1002) then
         abpre = abnua(i_co0,jopt) / 499.0_dp
         abici = abnua(i_co0,iopt) / 499.0_dp
       else if (i_spec == 1003) then
         abpre = abnua(i_co0,jopt) / (90.0_dp * 499.0_dp)
         abici = abnua(i_co0,iopt) / (90.0_dp * 499.0_dp)
       endif

       if (ifaf < 3) then
         on = 0.0_dp
       else
         on = 1.0_dp
c        on = 0.0_dp
       endif
       call DETBAL (iopt, abici, fabchi, deschi, ttry
     1    , bet_ij, bep_ij, bb_int, bb_ext, on, ref
     2    , xrepre, xreici, ntr, nlv, eqrspe
     3    , aij, wlk, iup, jlo, elk, gst)

       do i=1,ntr
         opd_t(i) = opdspe(i,jopt) 
       end do

       call EMILINE (iopt, i_spec, velici, velpre, abpre, abici
     1             , bet_ij, bep_ij, bb_int, bb_ext, emi
     2             , opd_t, xrepre, xreici, ntr, nlv
     3             , aij, wlk, iup, jlo, gst)

       deallocate (bet_ij)
       deallocate (bep_ij)
       deallocate (bb_int)
       deallocate (bb_ext)
       deallocate (eqrspe)
       deallocate (eqcol)

       sumemi = 0.0_dp
       do i=1,ntr
         opdspe(i,iopt) = opd_t(i)
         emispe(i,iopt) = emi(i)
         sumemi = sumemi + emi(i)
       end do
       if (sumemi /= 0.0_dp) then
         sumemi = (sumemi - ref) / sumemi
       endif

       do ju=1,nlv
         xre(ju,iopt) = xreici(ju)
       end do

       if (i_spec == i_co0) then
         fabco = fabchi
         desco = deschi
       endif

       end subroutine prepar


c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        subroutine COLSPE (ttry,i_spec)
c       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        use constantes
        use SP_DATA
        use VERIF
        implicit none

        REAL (KIND=dp), intent (in) :: ttry
        INTEGER                     :: i_spec

        if (i_spec == i_co0) then
          call COLCO0 (ttry)
        else if (i_spec == 1001 .OR. i_spec == i_co1) then
          call COLCO1 (ttry)
        else if (i_spec == 1002 .OR. i_spec == i_co2) then
          call COLCO2 (ttry)
        else if (i_spec == 1003 .OR. i_spec == i_co3) then
          call COLCO3 (ttry)
        else if (i_spec == i_c) then
          call COLCAT (ttry)
        else if (i_spec == i_n) then
          call COLNAT (ttry)
        else if (i_spec == i_o) then
          call COLOAT (ttry)
        else if (i_spec == i_s) then
          call COLSAT (ttry)
        else if (i_spec == i_si) then
          call COLSiAT (ttry)
        else if (i_spec == i_cp) then
          call COLCPL (ttry)
        else if (i_spec == i_np) then
          call COLNPL (ttry)
        else if (i_spec == i_op) then
          call COLOPL (ttry)
        else if (i_spec == i_sp) then
          call COLSPL (ttry)
        else if (i_spec == i_sip) then
          call COLSIP (ttry)
        else if (i_spec == i_cs) then
          call COLCS0 (ttry)
        else if (i_spec == i_hcop) then
          call COLHCOP (ttry)
        else if (i_spec == i_h2o) then
          call COLH2O (ttry)
        else if (i_spec == i_h3p) then
          call COLH3P (ttry)
          IF (verif_eqcol_h3p == 1) THEN
          CALL VERIF_COLH3P(ttry)
          PRINT *,"Verification de eqcol H3+"
          PRINT *,"dans : verif_coll_out"
          PAUSE
          ENDIF
        endif

        end subroutine COLSPE
