
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine CHAUD (ttry, iopt, itn)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE RF_TOOLS
       USE H2_DATA
       USE HD_DATA
       USE SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

c 17 juin 2002 - JLB - All grain physics should changed and brought closer to Draine et al.
c                      Now:  close to Emilie Habart

c ref.1 Hollenbach, Takahashi et Tielens (1991) ApJ 377, 192
c ref.2 Verstraete, Leger, d'Hendecourt, Dutuit et Defourneau
c     (1990) AA 237, 436
c ref:3 Tielens & Hollenbach(1985) ApJ 291, 722

       implicit double precision (a-h,o-z)

c  bilthg : eating by photoelectric effect on grains
c  xkneut : ions neutralisation on grains
c  From Bakes & Tielens, Ap. J., 427, 822, (1994)

       real (KIND=dp) xkneut(nes)
       real (KIND=dp) xiig, s_gr

       common / cofbth / bilthg, xkneut

       save

       avr = Avmax - av

c  Grains temperature from Eq (1) of Hollenbach et al (ref.1)
c  with integration over OUR radiation field and using whatever Q we want


       tau100 = 4.0d-3
       ccff = 15.0_dp * hcsurk**4 / (32.0_dp * xpi**5 * xk * clum*clum)

c  td0 is dust temperature at the edge
c  tdust(:,0) is initialised in main before the first iteration
       do mm=1,npg
         xnu0 = clum / (2.0_dp * xpi * adust(mm))
         td0 = tdust(mm,0)

         s_gr = xpi * adust(mm)*adust(mm)

        xiig = sect_int(sigabs(mm), rf_u, wl_pointer
     1                , rf_wl%Val(nwlg))
        xiig = xiig * clum * x_nc(mm) / s_gr

c formulae  (6) and (7) of ref.1 lead to
c   with modified values from Burton, Hollenbach, Tielens
c   Ap J 365, 620, (1990)
c 1 mars 2000; bug: log10 instead of ln

         tgrain = ccff * xnu0 * xiig + tcmb**5
     1          + 3.4d-2 * (0.423 - log(3.45d-2 * tau100 * td0))
     2          * tau100 * td0**6

         tdust(mm,iopt) = tgrain**0.2_dp

c 3 April 2003 - Madrid
c        tdust(mm,iopt) = max(tdust(mm,iopt), 100.0_dp)
c    Temperature de grains elevee pour CRL618
c end
       end do

c   Photoelectric heating on PAH (ref.2)
c   Supressed and included in grains

c   H2 fraction (for secondary photons flux)

       frach2 = ab(i_h2) / (ab(i_h) + ab(i_h2))

c On tient compte du champ de rayonnement externe et des photons secondaires:
c    27 juin 2002 : Ce flux n'est plus inclus dans le calcul de chophe
c                   A remettre via le Champ de rayonnement!
c      radsec = frach2 * fphsec / 1.0d8

c-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

c  Photoelectric heating on grains from
c  Bakes & Tielens, Ap. J., 427, 822, (1994)
c  Gives charge distribution of grains as a function of size

c     if (mod(itn,3) == 1) then

c  Integration over distribution:

      ndeb = nh2for + ncrion + npsion + nasrad + nrest + nh2endo
     1    + nphot_f + nphot_i + n3body + ncpart + ngsurf + ngphot
     2    + ngads + 1
      nfin = ndeb + ngneut - 1

      chophe = 0.0_dp
      do i=ndeb,nfin
        ir1 = react(i,1)
        rxigr(ir1,iopt) = 0.0_dp
      end do

      do mm=1,npg
        call fbth(ttry, iopt, itn, mm)
c  bilthb may be negative if electron recombination is faster than ejection
c       bilthg = max(bilthg,0.0_dp)
        chophe = chophe + bilthg * wwmg(mm)
        j = 0
        do i=ndeb,nfin
          ir1 = react(i,1)
          j = j + 1
          rxigr(ir1,iopt) = rxigr(ir1,iopt) + xkneut(j) * wwmg(mm)
        end do
      end do

      chophg(iopt) = chophe

c-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

c  Heating by thermal balance with grains
c   Tielens & Hollenbach 1985 (ref.3) eq A19 : not used anymore

c  Burke & Hollenbach 1983 Ap J 265, 223 (Eq 9)
c    4.017e-12 = 2 k sqrt(8 k / pi mH)

        densr = ab(i_h) + ab(i_h2) + ab(i_he)
        xmu = 1.4_dp * densh(iopt) / densr
        vbar = vmaxw * sqrt(ttry / xmu)
        alphat = (ab(i_h) + ab(i_h2) * 0.5_dp * sqrt(2.0_dp)) * 0.8
        choeqg(iopt) = alphat * 0.5_dp * xk * vbar * densh(iopt)
     1          * sum(surfgr(1:npg) * (tdust(1:npg,iopt)-ttry))

       xkphot = 0.0_dp
       xkrc = 0.0_dp
       xkphs = 0.0_dp
       xkchim = 0.0_dp

c  Heating by photoionisation or photodissociation
c  1 eV for H2, HD, CO
c  de for atoms (C et S)
c  1/3 of de for others
c  photons have an "enthalpy" of 13.O eV

       ! Photodestruction 
       ideb = nh2for + ncrion + npsion + nasrad + nrest + nh2endo + 1
       ifin = ideb + nphot_f - 1

       do i=ideb,ifin
         ir1 = react(i,1)
         rate1 = gamm(i) * exp(-bet(i) * av) * radm
         rate2 = gamm(i) * exp(-bet(i) * avr) * radp
         rate = (rate1 + rate2) * de(i) / 3.0_dp
         xkphot = xkphot + ab(ir1) * rate
       end do

       ! Photodestruction by direct integration
       ideb = ifin + 1
       ifin = ifin + nphot_i

       do i=ideb,ifin
         ir1 = react(i,1)
         rate = pdiesp(i-ideb+1,iopt) * de(i)
         xkphot = xkphot + ab(ir1) * rate
       end do

c  Heating by cosmic rays ionisation

       ideb = nh2for + 1
       ifin = nh2for + ncrion
       do i=ideb,ifin
         ir1 = react(i,1)
         rate = gamm(i) * zeta
         xkrc = xkrc + ab(ir1) * rate
       end do

c  Heating by secondary photons ionisation

       ideb = nh2for + ncrion + 1
       ifin = ideb + npsion - 1
       do i=ideb,ifin
         ir1 = react(i,1)
         rate = gamm(i) * frach2 * zeta * (ttry / 300.0_dp)**alpha(i)
         xkphs = xkphs + ab(ir1) * rate * de(i)
       end do

c  Chemistry thermal balance

       ideb = nh2for + ncrion + npsion + nasrad + 1
       ifin = nh2for + ncrion + npsion + nasrad + nrest + nh2endo
       do i=ideb,ifin

         if (de(i) > 1.0e3_dp) then
           dde = 0.0_dp
           print *, i, ir1, ir2, de(i)
         else
           dde = de(i)
         endif

         ir1 = react(i,1)
         ir2 = react(i,2)
         ir3 = react(i,3)
         if (bet(i) < 0.0_dp) then
           expon = exp(-bet(i)/max(ttry,200.0_dp))
         else
           expon = exp(-bet(i)/ttry)
         endif
         rate = gamm(i) * expon
     1        * (ttry * 3.3333d-3)**alpha(i)
         xkchim = xkchim
     1          + ab(ir1) * ab(ir2) * ab(ir3) * rate * dde

       end do

c  WARNING: cross check photons energy (2 I 06)
c   photoionisation ou photodissociation: 1 eV pour H2, HD et CO, de(i) pour les autres
c   avec une energie moyenne de 10 eV pour les photons incidents.

       chopht(iopt) = xkphot * everg

c   4 ev per cosmray

c      chorc(iopt) = xkrc * everg * 6.0_dp
       chorc(iopt) = xkrc * everg * 4.0_dp

c   de(i) per secondary photon

       chophs(iopt) = xkphs * everg

c  Convert to erg

       chochi(iopt) = xkchim * everg

c  Heating by H2 formation on grains

c  Each formation releases 1/3 of formation enthalpy (if possible)

       rate = gamm(ifh2gr) * 2.0_dp * tm1h(i_hgr) / xkhydr

c  Include also internal energy since formation followed by
c  direct cascades are counted as cooling.
c  First term in "MIN" is the sum of energy not in the grain
c  (which has to be lower than 4.5 eV)

       toto = everg * de(ifh2gr)
       choh2g(iopt) = min(emoyh2 * xk  + toto / 3.0_dp, toto)

       choh2g(iopt) = choh2g(iopt) * rate * ab(i_hgr) * ab(i_hgr)

c   Heating by formation of HD on grains

       if (i_dgr /= 0) then
         rate = gamm(ifhdgr) *(tm1h(i_dgr) + tm1h(i_hgr)) / xkhydr
         rate = rate * ab(i_hgr) * ab(i_dgr)
       else
         rate = 0.0_dp
       endif

       if (i_hd /= 0) then
         toto = everg * de(ifhdgr)
         chohdg(iopt) = min(emoyhd * xk  + toto / 3.0_dp, toto)
         chohdg(iopt) = chohdg(iopt) * rate
       else
         chohdg(iopt) = 0.0_dp
       endif

c  Heating by dissip	ation of turbulence
c  Falgarone et Puget (1985)
c  Usually switched off

c      fv = 0.03_dp
c   On limite la taille inferieure d un clump, pour eviter la divergence de chotur
c      size = max(taille(iopt),0.1_dp)
c      chotur(iopt) = 2.0d-29 * densh(iopt) * (vturb / 1.0d5)**3
c    1              * (fv / 0.03_dp) / size

c   Other possibility : (Boland & de Jong, 1984)

c      ct2 = 2.0_dp * xk / amu
c      pmu = (ab(i_h) + 2.0_dp * ab(i_h2) + 4.0_dp * ab(i_he))
c    1     / (ab(i_h) + ab(i_h2) + ab(i_he))
c      alp = vturb / sqrt(ct2 * ttry / pmu)
c      chotur(iopt) = 2.7d-30 * alp**3 * densh(iopt) * ttry**1.5_dp
c    1              / (1.0_dp + alp*alp)

c   up to now:

       chotur(iopt) = 0.0_dp

c      print *, ct2, pmu, alp, chotur(iopt)

c   Total heating

       chotot(iopt) = chotot(iopt) + chophg(iopt)
c                   + chohdg(iopt)
     2              + choh2g(iopt) + chopht(iopt) + chorc(iopt)
     3              + chophs(iopt) + chochi(iopt) + chotur(iopt)

       if (choeqg(iopt) >= 0.0_dp) then
         chotot(iopt) = chotot(iopt) + choeqg(iopt)
       else
         reftot(iopt) = reftot(iopt) - choeqg(iopt)
       endif

       return
       end

c----------------------------------------------------------------------

       subroutine fbth(temper, iopt, itn, mm)

       use constantes
       use auxiliaries
       use SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL
       USE RF_TOOLS

       implicit none

c  Photoelectric heating and grain charge balance as a
c  function of size
c  Modified from Bakes & Tielens, Ap J 427, 822 (1994)

c     xkneut : vecteur resultat (taux de chauffage + taux de collisions)

      real (KIND=dp) xkneut(nes)

      common / cofbth / bilthg, xkneut

c     x_nc : Number of carbon atoms in the grain

      integer iopt, itn
      integer mm
c  Maximal possible charge

c  Charges...

c     real (KIND=dp) zmin
      real (KIND=dp) zmax
      integer izmin, izmax
      integer kzmin, kzmax
      real (KIND=dp) zzmoy
      integer, DIMENSION(:), ALLOCATABLE ::  jzmin
      integer, DIMENSION(:), ALLOCATABLE ::  jzmax

c  Gas parameters

c     temper : Temperature (in K)

      real (KIND=dp) temper

c  Integrals of Q * U * (1 or l or l2)  over lambda

      real (KIND=dp) s_QU, s_QUl, s_QUl2

c  Gas grain interactions are:
c     xjpe : photoemission rate
c     xje  : electrons accretion rate
c     xji  : individual ions accretion rate
c     xjit : total ions accretion rate
!--------------------------------------------------------
      integer izm0
      parameter (izm0=10000)

      real (KIND=dp) xjpe(-izm0:izm0), xje(-izm0:izm0)
c     real (KIND=dp) xji(0:nes,-izm0:izm0), xjit(-izm0:izm0)
      real (KIND=dp) xjit(-izm0:izm0)
      real (KIND=dp), allocatable, dimension(:,:) :: xji

c  ionisation fraction 

      real (KIND=dp) frac(-izm0:izm0)

c  Heating and Cooling

      real (KIND=dp) heat(-izm0:izm0), cool(-izm0:izm0)
      real (KIND=dp) bilth, bilthg

c     stic : electrons sticking coefficient

      real (KIND=dp) stic

c  Dummy variables

      real (KIND=dp) tauph, cte, x_nc_t
      real (KIND=dp) x_ip, thet, xjtm, xjtp, xjt0
      real (KIND=dp) xltm, xltp, xlt0, ziz
      real (KIND=dp) sup, titi, toto, tata, xsum, tier, tempo
      real (KIND=dp) cte_H, cte_J, cte_IP

c     TYPE(RF_TYPE)  :: rf_tmp
      real (KIND=dp) :: wl_cut_IP, IP_min
      integer        :: I_cut_IP

c  Dummy Indices

      integer i, j, iz, ndeb, nfin, ir1, izf, izd

      data stic / 1.0_dp /

      save jzmin, jzmax

c------------------------------------------------------------------------

      if (.NOT. ALLOCATED (xji)) then
        allocate(xji(0:nspec,-izm0:izm0))
      endif

      tier = 1.0_dp / 3.0_dp
      IF ( .NOT. ALLOCATED(jzmin)) THEN
        ALLOCATE (jzmin(npg), jzmax(npg))
        jzmin = 0
        jzmax = 0
      ENDIF

      IP_min = xhp * clum * 1.0e8_dp / (rf_wl%Val(nwlg) * everg)

c  Reduced parameter (from Eq 9). With qi = +/- 1

      tauph = adust(mm) * xk * temper / xqe2

c  Eq 9 (first part)

      cte = sqrt(8.0_dp * xk * temper * xpi) * adust(mm)**2

      x_nc_t = x_nc(mm)**tier

c  Maximum Charges (Eq 26 et 27):

      if (iopt == 0 .and. itn == 1) then

c  Charges can not be very negative at the edge...
c       zmin = 0.4_dp * x_nc_t - 0.5_dp
c       zmin =  max(-zmin,-90.0_dp*tauph)
c       izmin = zmin
        izmin = -1

        zmax = (hiop/everg - 4.4_dp) * x_nc_t / 11.1_dp - 0.5_dp
        izmax = zmax

c  We limit the maximum possible extension

        izmin = max(izmin,-izm0)
        izmax = min(izmax,izm0)

        if (iopt /= 0 .AND. jzmin(mm) < jzmax(mm)) then
          izmin = jzmin(mm) - 5
          izmax = jzmax(mm) + 5
        else if (iopt /= 0 .AND. mm > 1) then
          izmin = min(izmax-5,max(izmin,jzmin(mm-1)/2))
          izmax = max(izmin+5,min(izmax,jzmax(mm-1)+10))
        endif

        if (izmin >= izmax) then
          izmin = izmin - 2
          izmax = izmax + 2
        end if

      else

        izmin = jzmin(mm) - 2
        izmax = jzmax(mm) + 2

      endif

c  For neutralisation reactions

      ndeb = nh2for + ncrion + npsion + nasrad + nrest + nh2endo
     1    + nphot_f + nphot_i + n3body + ncpart + ngsurf + ngphot
     2    + ngads + 1
      nfin = ndeb + ngneut - 1

c  Loop over grain charge:
c  Three parts: negatives, neutrals, positives

      cte_J = x_nc(mm) * 0.14 * bt_fy(mm) * 1.0e-8_dp / xhp
      cte_H = x_nc(mm) * 0.14 * bt_fy(mm) * 0.5_dp * clum

      izf = min(izmax,-1)

      do iz=izmin,izf

c  Ionisation potentiel (en eV) (Eq 25)

        x_ip = max(IP_min,min(4.4_dp + (iz + 0.5_dp)
     1       * 11.1_dp / x_nc_t, hiop/everg))
        cte_IP = x_ip * everg * 1.0e-8_dp / (xhp * clum)
        wl_cut_IP = 1.0_dp / cte_IP
        CALL rf_locate (rf_wl, wl_cut_IP, I_cut_IP)

        s_QU   = si_QU(mm,I_cut_IP)
        s_QUl  = si_QUl(mm,I_cut_IP)
        s_QUl2 = si_QUl2(mm,I_cut_IP)

c  Ejection of one photo-electron (Eq 20):

        xjpe(iz) = cte_J * (s_QUl - cte_IP * s_QUl2)

c  photo-electric heating ( Eq 30):

        heat(iz) = cte_H * (s_QU - 2.0_dp * cte_IP * s_QUl
     1           + cte_IP*cte_IP * s_QUl2)
c       print *, xjpe(iz), heat(iz)

c  Collisions with gas phase particules. Eq 13:

        ziz = -iz
        thet = ziz / (1.0_dp + 1.0_dp/sqrt(ziz))

c  Eq 11:

        xjtm = (1.0_dp - iz / tauph)
     1       * (1.0_dp + sqrt(2.0_dp/(tauph - 2.0_dp * iz)))

c  Eq 12:

        xjtp = exp(-thet / tauph)
     1       * (1.0_dp + 1.0_dp / sqrt(4.0_dp * tauph
     2       - 3.0_dp * iz))**2

c  Eq 33:

        xltm = (2.0_dp - iz / tauph)
     1       * (1.0_dp + sqrt(1.0_dp/(tauph - iz)))

c  Eq 34:

        xltp = exp(-thet / tauph) * (2.0_dp - iz / tauph)
     1       * (1.0_dp + 1.0_dp / sqrt(3.0_dp /(2.0_dp*tauph)
     2       - 3.0_dp * iz))

c  Eq 9 (continued) et 31:

        xje(iz) = cte * stic * ab(nspec) * xjtp / sqrt(xme)
        titi = 0.0_dp
        toto = cte * xk * temper * ab(nspec) * xltp / sqrt(xme)
        tata = toto

        do i=ndeb,nfin
          ir1 = react(i,1)
          xji(ir1,iz) = cte * gamm(i) * ab(ir1)
     1              * xjtm / sqrt(xmh*mol(ir1))
          titi = titi + xji(ir1,iz)
          tata = tata + cte * gamm(i) * xk * temper
     1             * ab(ir1) * xltm / sqrt(xmh*mol(ir1))
        end do
        xjit(iz) = titi

c  If positive ions do not stick on grains, they do not contribute too cooling
c  Include only electrons contribution
c       cool(iz) = tata
        cool(iz) = toto

      end do

       if (izmin <= 0 .and. izmax >= 0) then

      iz = 0

      x_ip = max(IP_min,min(4.4_dp + (iz + 0.5_dp)
     1     * 11.1_dp / x_nc_t, hiop/everg))
      cte_IP = x_ip * everg * 1.0e-8_dp / (xhp * clum)
      wl_cut_IP = 1.0_dp / cte_IP
      CALL rf_locate (rf_wl, wl_cut_IP, I_cut_IP)

      s_QU   = si_QU(mm,I_cut_IP)
      s_QUl  = si_QUl(mm,I_cut_IP)
      s_QUl2 = si_QUl2(mm,I_cut_IP)

      xjpe(0) = cte_J * (s_QUl - cte_IP * s_QUl2)
      heat(0) = cte_H * (s_QU - 2.0_dp * cte_IP * s_QUl
     1        + cte_IP*cte_IP * s_QUl2)

      xjt0 = 1.0_dp + sqrt(xpi / (2.0_dp * tauph))

      xlt0 = 2.0_dp + 3.0_dp * sqrt(xpi/(2.0_dp*tauph)) /2.0_dp

      xje(0) = cte * ab(nspec) * xjt0 / sqrt(xme)
      titi= 0.0_dp
      toto = cte * xk * temper * ab(nspec) * xlt0 / sqrt(xme)
      tata = toto

      do i=ndeb,nfin
        xji(ir1,0) = cte * gamm(i) * ab(ir1)
     1             * xjt0 / sqrt(xmh*mol(ir1))
        titi= titi+ xji(ir1,0)
        tata = tata + cte * gamm(i)* xk * temper * xlt0
     1           * ab(ir1) / sqrt(xmh*mol(ir1))
      end do
      xjit(0) = titi
c  If positive ions do not stick on grains, they do not contribute too cooling
c     cool(0) = tata
      cool(0) = toto

      endif

      izd = max (izmin,1)

      do iz=izd,izmax

c  Ionisation potentiel (in eV) (Eq 25)

        x_ip = max(IP_min,min(4.4_dp + (iz + 0.5_dp)
     1       * 11.1_dp / x_nc_t, hiop/everg))
        cte_IP = x_ip * everg * 1.0e-8_dp / (xhp * clum)
        wl_cut_IP = 1.0_dp / cte_IP
        CALL rf_locate (rf_wl, wl_cut_IP, I_cut_IP)

        s_QU   = si_QU(mm,I_cut_IP)
        s_QUl  = si_QUl(mm,I_cut_IP)
        s_QUl2 = si_QUl2(mm,I_cut_IP)

        xjpe(iz) = cte_J * (s_QUl - cte_IP * s_QUl2)
        heat(iz) = cte_H * (s_QU - 2.0_dp * cte_IP * s_QUl
     1           + cte_IP*cte_IP * s_QUl2)

c  Eq 13:

        ziz = iz
        thet = ziz / (1.0_dp + 1.0_dp/sqrt(ziz))

c  Eq 11:

        xjtm = (1.0_dp + iz / tauph)
     1       * (1.0_dp + sqrt(2.0_dp/(tauph + 2.0_dp * iz)))

c  Eq 12:

        xjtp = exp(-thet / tauph)
     1       * (1.0_dp + 1.0_dp / sqrt(4.0_dp * tauph
     2       + 3.0_dp * iz))**2

c  Eq 33:

        xltm = (2.0_dp + iz / tauph)
     1       * (1.0_dp + sqrt(1.0_dp/(tauph + iz)))

c  Eq 34:

        xltp = exp(-thet / tauph) * (2.0_dp + iz / tauph)
     1       * (1.0_dp + 1.0_dp / sqrt(3.0_dp /(2.0_dp*tauph)
     2       + 3.0_dp * iz))

c  Eq 9 (continued) et 31:

        xje(iz) = cte * ab(nspec) * xjtm / sqrt(xme)
        titi = 0.0_dp
        toto = cte * xk * temper * ab(nspec) * xltm / sqrt(xme)
        tata = toto

        do i=ndeb,nfin
          ir1 = react(i,1)
          xji(ir1,iz) = cte * gamm(i) * ab(ir1)
     1              * xjtp / sqrt(xmh*mol(ir1))
          titi = titi + xji(ir1,iz)
          tata = tata + cte * gamm(i) * xk * temper
     1             * ab(ir1) * xltp / sqrt(xmh*mol(ir1))
        end do
        xjit(iz) = titi
c  If positive ions do not stick on grains, they do not contribute too cooling
c       cool(iz) = tata
        cool(iz) = toto

      end do

c  Ionisation fraction :

      sup = 0.0_dp

      if (izmin >= 0) then

        frac(izmin) = 0.0_dp

        toto = 0.0_dp
        do iz=izmin+1,izmax
          tempo = (xjpe(iz-1) + xjit(iz-1)) / xje(iz)
          if (tempo > 1.0e-40_dp) then
            toto = toto + log(tempo)
          else
            toto = toto - 92.1_dp
          endif
          frac(iz) = toto
          sup = max(sup,toto)
        end do

      else if (izmax <= 0) then

        frac(izmax) = 0.0_dp

        toto = 0.0_dp
        do iz=izf,izmin,-1
          tempo = xje(iz+1) / (xjpe(iz) + xjit(iz))
          if (tempo > 1.0e-40_dp) then
            toto = toto + log(tempo)
          else
            toto = toto - 92.1_dp
          endif
          frac(iz) = toto
          sup = max(sup,toto)
        end do

      else

        frac(0) = 0.0_dp

c  Eq 6:

        toto = 0.0_dp
        do iz=izd,izmax
          tempo = (xjpe(iz-1) + xjit(iz-1)) / xje(iz)
          if (tempo > 1.0e-40_dp) then
            toto = toto + log(tempo)
          else
            toto = toto - 92.1_dp
          endif
          frac(iz) = toto
          sup = max(sup,toto)
        end do

c  Eq 7:

        toto = 0.0_dp
        do iz=izf,izmin,-1
          tempo = xje(iz+1) / (xjpe(iz) + xjit(iz))
          if (tempo > 1.0e-40_dp) then
            toto = toto + log(tempo)
          else
            toto = toto - 92.1_dp
          endif
          frac(iz) = toto
          sup = max(sup,toto)
        end do

      endif

c  Eq 8 et 3 (part of it):

      bilth = 0.0_dp
      bilthg = 0.0_dp
      xsum = 0.0_dp
      do i=1,ngneut
        xkneut(i) = 0.0_dp
      end do
      do iz = izmin,izmax
        frac(iz) = exp(frac(iz) - sup)
        xsum = xsum + frac(iz)
      end do

      if (xsum == 0.0_dp) then
        print *, ' Pb dans fbth, xsum = 0.0'
      endif

      kzmin = izmax
      kzmax = izmin
      zzmoy = 0
      do iz = izmin,izmax
        frac(iz) = frac(iz) / xsum
        if (frac(iz) > 1.0e-5_dp) then
          kzmin = min(kzmin,iz)
          kzmax = max(kzmax,iz)
        endif
        zzmoy = zzmoy + iz * frac(iz)
        bilth = bilth + (heat(iz) - cool(iz)) * frac(iz)
      end do
      jzmin(mm) = kzmin
      jzmax(mm) = kzmax
      zmoy(mm,iopt) = zzmoy

c  Normalisation constant (See Guillaume's notes)

      bilthg = bilth * agrnrm * densh(iopt)

      j = 0
      do i=ndeb,nfin
        ir1 = react(i,1)
        j = j + 1
        do iz = izmin,izmax
          xkneut(j) = xkneut(j) + xji(ir1,iz) * frac(iz)
        end do
c  4 I 2006 : Bug (ER + FLP) : divide by ab(ir1)
c       xkneut(j) = xkneut(j) * agrnrm * densh(iopt)
        xkneut(j) = xkneut(j) * agrnrm * densh(iopt) / ab(ir1)
      end do

      deallocate (xji)

c     CALL rf_free (rf_tmp)

      return
      end

